@@ -1,6 +1,6 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 
-"""strdup isn't available on win32."""
+"""strdup isn't available on windows."""
 
 error_msg = 'Do not use strdup/strndup. Use std::string instead.'
 
