@@ -4,7 +4,7 @@
 """This catches some functions that are declared to have no parameters with the
 syntax "some_function(void)"."""
 
-regexp = """\( *(const)* *void *\)"""
+regexp = r"""\( *(const)* *void *\)"""
 error_msg = 'void f(void) is invalid. Use void f() instead!'
 
 forbidden = [
