@@ -3,13 +3,13 @@
 -- =========================
 
 function reebaud(title, text)
-   return speech("map:reebaud.png", "55BB55", title, text)
+   return speech("map:reebaud.png", styles.color("campaign_fri_reebaud"), title, text)
 end
 function hauke(title, text)
-   return speech("map:hauke.png", "7799BB", title, text)
+   return speech("map:hauke.png", styles.color("campaign_fri_hauke"), title, text)
 end
 function murilius(title, text)
-   return speech("map:murilius.png", "BB7755", title, text)
+   return speech("map:murilius.png", styles.color("campaign_fri_murilius"), title, text)
 end
 
 -- =======================================================================
@@ -273,14 +273,14 @@ supply_murilius_3 = {
 supply_murilius_4 = {
    title =_("We Are Not Alone"),
    body=reebaud(_("Strangers"),
-      -- TRANSLATORS: Reebaud – Supply Murilius 4. "Peace" is the traditional frisian greeting, nothing more.
+      -- TRANSLATORS: Reebaud – Supply Murilius 4. "Peace" is the traditional Frisian greeting, nothing more.
       _([[Peace, representative of the Empire! I am Reebaud, the chieftain of this tribe. All we want is to settle down here, build our home and live our lives in peace. We do not seek trouble, but we aren’t afraid to fight, either.]])),
 }
 supply_murilius_5 = {
    title =_("We Are Not Alone"),
    body=murilius(_("Salvete peregrini!"),
       -- TRANSLATORS: Murilius – Supply Murilius 5. "Reebaude" is the Latin vocative case (form of address) of Reebaud.
-      _([[In this case, Reebaude, I must inform you of the presence of an inhospitable, hostile tribe of barbarians in the vicinity of this very outpost of the Empire.]])
+      _([[In this case, Reebaude, I must inform you of the presence of an inhospitable, hostile tribe of Barbarians in the vicinity of this very outpost of the Empire.]])
       .. paragraphdivider() ..
       -- TRANSLATORS: Murilius – Supply Murilius 5
       _([[If you seek to live peacefully here, you are not faced with a choice whether to allow them to live here – you are required to assist the Empire in driving them out.]])),
@@ -307,7 +307,7 @@ supply_murilius_8 = {
       _([[If you fail to deliver these wares, the Empire shall treat you like enemies. And so as to prevent you wasting your precious metal, I forbid you to expand your territory from this moment on until you have given us your gift. Fail to obey, and the Empire shall treat you like enemies.]])
       .. paragraphdivider() ..
       -- TRANSLATORS: Murilius – Supply Murilius 8
-      _([[You will have to fight those barbarians if you intend to live in the area; surely you can see that fighting them together with the Empire is easier than alone? And you really do not want the Empire as your enemy. Barbarians though you are, I do believe you will realize the truth of my words after some consideration.]]))
+      _([[You will have to fight those Barbarians if you intend to live in the area; surely you can see that fighting them together with the Empire is easier than alone? And you really do not want the Empire as your enemy. Barbarians though you are, I do believe you will realize the truth of my words after some consideration.]]))
       .. new_objectives(obj_supply_murilius),
 }
 supply_murilius_9 = {
@@ -324,6 +324,8 @@ supply_murilius_9 = {
       .. new_objectives(obj_scout),
 }
 
+local indent = styles.get_size("campaign_fri02_poem_indent")
+
 expansion_hint = {
    title =_("The Scout Discovered Something"),
    body=hauke(_("Our scout returned"),
@@ -331,16 +333,16 @@ expansion_hint = {
       _([[Chieftain Reebaud! Our scout just returned. He has found a poem in the strange tongue this Proconsul speaks. With some difficulty, I have translated it into our language. If I did not mistranslate, it runs as follows:]])
       .. paragraphdivider() ..
       -- TRANSLATORS: Hauke – Expansion hint. A poem, verse 1
-      space(10) .. (_([[How many sites where soldiers stay,]]))
+      space(indent) .. (_([[How many sites where soldiers stay,]]))
       .. paragraphdivider() ..
       -- TRANSLATORS: Hauke – Expansion hint. A poem, verse 2
-      space(10) .. (_([[How many sites where thou hold’st sway,]]))
+      space(indent) .. (_([[How many sites where thou hold’st sway,]]))
       .. paragraphdivider() ..
       -- TRANSLATORS: Hauke – Expansion hint. A poem, verse 3
-      space(10) .. (_([[Though not their whereabouts sees he]]))
+      space(indent) .. (_([[Though not their whereabouts sees he]]))
       .. paragraphdivider() ..
       -- TRANSLATORS: Hauke – Expansion hint. A poem, verse 4
-      space(10) .. (_([[Who’s ordering around here thee!]]))
+      space(indent) .. (_([[Who’s ordering around here thee!]]))
       .. paragraphdivider() ..
       -- TRANSLATORS: Hauke – Expansion hint
       _([[I don’t know what it means, but I’m sure it’s important. Perhaps you can make some sense of it.]]))
@@ -373,7 +375,7 @@ defeat_both = {
       _([[Murilius has made it quite clear that he won’t leave us alone. I fear we must defeat him and destroy his colony if we ever want to have peace here.]])
       .. paragraphdivider() ..
       -- TRANSLATORS: Reebaud – defeat both enemies
-      _([[The barbarian tribe he mentioned is also a threat. I don’t want to share this land with anyone, as sharing only leads to conflicts and battles, as we saw in our old home. We must defeat the barbarians as well.]]))
+      _([[The Barbarian tribe he mentioned is also a threat. I don’t want to share this land with anyone, as sharing only leads to conflicts and battles, as we saw in our old home. We must defeat the Barbarians as well.]]))
       .. new_objectives(obj_defeat_both),
 }
 supply_murilius_thanks = {
@@ -384,7 +386,7 @@ supply_murilius_thanks = {
       _([[Strangers! I see you have kept your word. The Empire rewards its friends, therefore I shall reward you with the friendship of the Empire!]])
       .. paragraphdivider() ..
       -- TRANSLATORS: Murilius – Supplied Murilius. Murilius’s plan is to use the player as cannon fodder, while he himself only attacks when he has no choice
-      _([[Now I permit you to assist us in conquering the barbarians, who live in a sheltered valley northwest to us. I charge you, expand along the northern end of the mountains enclosing this outpost; you will find the barbarians there – attack them! We will fend off attacks launched by the enemy.]]))
+      _([[Now I permit you to assist us in conquering the Barbarians, who live in a sheltered valley northwest to us. I charge you, expand along the northern end of the mountains enclosing this outpost; you will find the Barbarians there – attack them! We will fend off attacks launched by the enemy.]]))
       .. new_objectives(obj_defeat_barbarians),
 }
 defeat_murilius_1 = {
