@@ -1,6 +1,6 @@
 push_textdomain("tribes")
 
-dirname = path.dirname (__file__)
+local dirname = path.dirname (__file__)
 
 wl.Descriptions():new_productionsite_type {
    name = "amazons_water_gatherers_hut",
@@ -25,7 +25,7 @@ wl.Descriptions():new_productionsite_type {
    },
 
    aihints = {
-      needs_water = true,
+      needs_water = true  -- is not a well but collects from bodies of water on the map
    },
 
    working_positions = {
@@ -37,9 +37,14 @@ wl.Descriptions():new_productionsite_type {
          -- TRANSLATORS: Completed/Skipped/Did not start working because ...
          descname = _("working"),
          actions = {
+            -- obtaining second bucket from hut takes takes 3.6 sec
+            -- time of worker: 8.2-40.6 sec, min+max average 24.4 sec
+            -- min. time total: 8.2 + 20 + 3.6 = 31.8 sec
+            -- max. time total: 40.6 + 20 + 3.6 = 64.2 sec
+            -- avg. time total: 24.4 + 20 + 3.6 = 48 sec
             "sleep=duration:20s",
             "callworker=fetch_water",
-            -- he carries 2 buckets so we need to create one now
+            -- she carries 2 buckets so we need to create one now
             "produce=water",
          }
       },
