@@ -1,6 +1,6 @@
 push_textdomain("tribes")
 
-dirname = path.dirname(__file__)
+local dirname = path.dirname(__file__)
 
 wl.Descriptions():new_productionsite_type {
    name = "atlanteans_ferry_yard",
@@ -56,10 +56,13 @@ wl.Descriptions():new_productionsite_type {
          -- TRANSLATORS: Completed/Skipped/Did not start working because ...
          descname = _("working"),
          actions = {
+            -- time of worker: 17.2-38.8 sec
+            -- min. time total: 20 + 17.2 = 37.2 sec
+            -- max. time total: 20 + 38.8 = 58.8 sec
             "sleep=duration:20s",
-            "callworker=buildferry_1",
+            "return=skipped unless fleet needs ferry",
             "consume=log:2 spidercloth",
-            "callworker=buildferry_2"
+            "callworker=buildferry",
          }
       },
    },
