@@ -1,6 +1,6 @@
 push_textdomain("tribes")
 
-dirname = path.dirname(__file__)
+local dirname = path.dirname(__file__)
 
 wl.Descriptions():new_productionsite_type {
    name = "empire_charcoal_kiln",
@@ -54,9 +54,10 @@ wl.Descriptions():new_productionsite_type {
          -- TRANSLATORS: Completed/Skipped/Did not start producing coal because ...
          descname = _("producing coal"),
          actions = {
+            -- time total: 30.4 + 90 + 3.6 = 124 sec
             "return=skipped unless economy needs coal",
             "consume=log:6",
-            "sleep=duration:30s",
+            "sleep=duration:30s400ms",
             "animate=working duration:1m30s", -- Charcoal fires will burn for some days in real life
             "produce=coal"
          }
