@@ -1,11 +1,12 @@
 push_textdomain("tribes")
 
-dirname = path.dirname (__file__)
+local dirname = path.dirname (__file__)
 
 wl.Descriptions():new_productionsite_type {
    name = "frisians_scouts_house",
    -- TRANSLATORS: This is a building name used in lists of buildings
    descname = pgettext ("frisians_building", "Scout’s House"),
+   animation_directory = dirname,
    icon = dirname .. "menu.png",
    size = "small",
 
@@ -22,19 +23,21 @@ wl.Descriptions():new_productionsite_type {
 
    spritesheets = {
       idle = {
-         directory = dirname,
-         basename = "idle",
          hotspot = {40, 66},
          frames = 10,
          columns = 5,
          rows = 2,
          fps = 10
+      },
+      build = {
+         hotspot = {40, 60},
+         frames = 4,
+         columns = 2,
+         rows = 2,
       }
    },
    animations = {
       unoccupied = {
-         directory = dirname,
-         basename = "unoccupied",
          hotspot = {40, 60}
       }
    },
@@ -54,6 +57,10 @@ wl.Descriptions():new_productionsite_type {
          -- TRANSLATORS: Completed/Skipped/Did not start scouting because ...
          descname = _("scouting"),
          actions = {
+            -- time of worker: 82.2-214.8 sec
+            -- min. time total: 30 + 82.2 = 112.2 sec
+            -- max. time total: 30 + 214.8 = 244.8 sec
+            -- avg. time total: (112.2 + 244.8) / 2 = 178.5 sec
             "consume=ration",
             "sleep=duration:30s",
             "callworker=scout"
