@@ -1,8 +1,8 @@
 push_textdomain("tribes")
 
-dirname = path.dirname(__file__)
+local dirname = path.dirname(__file__)
 
-descriptions:new_warehouse_type {
+wl.Descriptions():new_warehouse_type {
    name = "barbarians_headquarters_interim",
    -- TRANSLATORS: This is a building name used in lists of buildings
    descname = pgettext("barbarians_building", "Headquarters"),
@@ -10,11 +10,11 @@ descriptions:new_warehouse_type {
    size = "big",
    destructible = false,
 
-   -- The interim headquarters of the barbarians is a simple stone building thatched
+   -- The interim headquarters of the Barbarians is a simple stone building thatched
    -- with reed and coated with many different wares.
+   animation_directory = dirname,
    animations = {
       idle = {
-         pictures = path.list_files(dirname .. "idle_??.png"),
          hotspot = { 64, 106 }
       },
    },
@@ -23,6 +23,7 @@ descriptions:new_warehouse_type {
 
    heal_per_second = 170,
    conquers = 9,
+   max_garrison = 20,
 }
 
 pop_textdomain()
