@@ -1,11 +1,12 @@
 push_textdomain("tribes")
 
-dirname = path.dirname (__file__)
+local dirname = path.dirname (__file__)
 
 wl.Descriptions():new_warehouse_type {
    name = "frisians_warehouse",
    -- TRANSLATORS: This is a building name used in lists of buildings
    descname = pgettext ("frisians_building", "Warehouse"),
+   animation_directory = dirname,
    icon = dirname .. "menu.png",
    size = "medium",
 
@@ -24,20 +25,17 @@ wl.Descriptions():new_warehouse_type {
 
    spritesheets = {
       idle = {
-         directory = dirname,
-         basename = "idle",
          hotspot = {50, 69},
          frames = 10,
          columns = 5,
          rows = 2,
          fps = 10
-      }
-   },
-   animations = {
+      },
       build = {
-         directory = dirname,
-         basename = "build",
-         hotspot = {50, 69}
+         hotspot = {50, 69},
+         frames = 4,
+         columns = 2,
+         rows = 2,
       }
    },
 
