@@ -1,6 +1,6 @@
 push_textdomain("tribes")
 
-dirname = path.dirname(__file__)
+local dirname = path.dirname(__file__)
 
 wl.Descriptions():new_immovable_type {
    name = "reedfield_tiny",
@@ -19,9 +19,9 @@ wl.Descriptions():new_immovable_type {
    spritesheets = {
       idle = {
          frames = 5,
-         rows = 3,
-         columns = 2,
-         hotspot = { 24, 20 }
+         rows = 1,
+         columns = 5,
+         hotspot = { 25, 30 }
       },
    }
 }
