@@ -1,15 +1,15 @@
 push_textdomain("tribes")
 
-dirname = path.dirname(__file__) .. "../"
+local dirname = path.dirname(__file__) .. "../"
 
 include(dirname .. "../terrain_affinity.lua")
 include(dirname .. "../spritesheets.lua")
 
 wl.Descriptions():new_immovable_type{
    name = "ironwood_desert_amazons_old",
-   -- TRANSLATORS:  This is an amazon rare tree.
+   -- TRANSLATORS:  This is an Amazon rare tree.
    descname = _("Ironwood Tree (Old)"),
-   -- TRANSLATORS:  This is an amazon rare tree.
+   -- TRANSLATORS:  This is an Amazon rare tree.
    species = _("Ironwood Tree"),
    size = "small",
    terrain_affinity = terrain_affinity_desert,
