@@ -1,13 +1,13 @@
 push_textdomain("tribes")
 
-dirname = path.dirname(__file__) .. "../"
+local dirname = path.dirname(__file__) .. "../"
 
 include(dirname .. "../terrain_affinity.lua")
 include(dirname .. "../spritesheets.lua")
 
 wl.Descriptions():new_immovable_type{
    name = "rubber_amazons_pole",
-   -- TRANSLATORS:  This is an amazon rare tree.
+   -- TRANSLATORS:  This is an Amazon rare tree.
    descname = _("Rubber Tree (Pole)"),
    size = "small",
    terrain_affinity = terrain_affinity,
