@@ -1,11 +1,12 @@
 push_textdomain("tribes")
 
-dirname = path.dirname (__file__)
+local dirname = path.dirname (__file__)
 
 wl.Descriptions():new_worker_type {
    name = "frisians_claydigger",
    -- TRANSLATORS: This is a worker name used in lists of workers
    descname = pgettext ("frisians_worker", "Clay Digger"),
+   animation_directory = dirname,
    icon = dirname .. "menu.png",
    vision_range = 2,
 
@@ -16,6 +17,9 @@ wl.Descriptions():new_worker_type {
 
    programs = {
       dig = {
+         -- steps from building to pond: 2-7
+         -- min. worker time: 2 * 2 * 1.8 + 8 = 15.2 sec
+         -- max. worker time: 2 * 7 * 1.8 + 8 = 33.2 sec
          "findspace=size:any radius:4",
          "walk=coords",
          "animate=dig duration:8s",
@@ -28,8 +32,6 @@ wl.Descriptions():new_worker_type {
 
    spritesheets = {
       walk = {
-         directory = dirname,
-         basename = "walk",
          fps = 15,
          frames = 10,
          columns = 5,
@@ -38,8 +40,6 @@ wl.Descriptions():new_worker_type {
          hotspot = {11, 24}
       },
       walkload = {
-         directory = dirname,
-         basename = "walkload",
          fps = 15,
          frames = 10,
          columns = 5,
@@ -48,8 +48,6 @@ wl.Descriptions():new_worker_type {
          hotspot = {10, 26}
       },
       dig = {
-         directory = dirname,
-         basename = "dig",
          fps = 20,
          frames = 10,
          columns = 5,
@@ -59,8 +57,6 @@ wl.Descriptions():new_worker_type {
    },
    animations = {
       idle = {
-         directory = dirname,
-         basename = "idle",
          hotspot = {8, 23}
       },
    },
