@@ -1,19 +1,18 @@
 push_textdomain("tribes")
 
-dirname = path.dirname (__file__)
+local dirname = path.dirname (__file__)
 
 wl.Descriptions():new_ferry_type {
    name = "frisians_ferry",
    -- TRANSLATORS: This is a worker name used in lists of workers
    descname = pgettext ("frisians_worker", "Ferry"),
+   animation_directory = dirname,
    icon = dirname .. "menu.png",
    vision_range = 2,
 
    ware_hotspot = {0, 0},
    spritesheets = {
       idle = {
-         basename = "idle",
-         directory = dirname,
          hotspot = {21, 18},
          fps = 10,
          columns = 5,
@@ -22,7 +21,6 @@ wl.Descriptions():new_ferry_type {
       },
       walk = {
          basename = "sail",
-         directory = dirname,
          directional = true,
          hotspot = {21, 18},
          fps = 15,
@@ -32,7 +30,6 @@ wl.Descriptions():new_ferry_type {
       },
       walkload = {
          basename = "sail",
-         directory = dirname,
          directional = true,
          hotspot = {21, 18},
          fps = 15,
