@@ -1,11 +1,12 @@
 push_textdomain("tribes")
 
-dirname = path.dirname (__file__)
+local dirname = path.dirname (__file__)
 
 wl.Descriptions():new_worker_type {
    name = "frisians_stonemason",
    -- TRANSLATORS: This is a worker name used in lists of workers
    descname = pgettext ("frisians_worker", "Stonemason"),
+   animation_directory = dirname,
    icon = dirname .. "menu.png",
    vision_range = 2,
 
@@ -16,6 +17,10 @@ wl.Descriptions():new_worker_type {
 
    programs = {
       cut_granite = {
+         -- steps from building to rock: 2-9, min+max average 5.5
+         -- min. worker time: 2 * 2 * 1.8 + 17.5 = 24.7 sec
+         -- max. worker time: 2 * 9 * 1.8 + 17.5 = 49.9 sec
+         -- avg. worker time: 2 * 5.5 * 1.8 + 17.5 = 37.3 sec
          "findobject=attrib:rocks radius:6",
          "walk=object",
          "animate=hacking duration:17s500ms",
@@ -29,8 +34,6 @@ wl.Descriptions():new_worker_type {
 
    spritesheets = {
       walk = {
-         directory = dirname,
-         basename = "walk",
          fps = 15,
          frames = 10,
          columns = 5,
@@ -39,8 +42,6 @@ wl.Descriptions():new_worker_type {
          hotspot = {11, 24}
       },
       walkload = {
-         directory = dirname,
-         basename = "walkload",
          fps = 15,
          frames = 10,
          columns = 5,
@@ -49,8 +50,6 @@ wl.Descriptions():new_worker_type {
          hotspot = {10, 26}
       },
       hacking = {
-         directory = dirname,
-         basename = "hacking",
          fps = 10,
          frames = 10,
          columns = 5,
@@ -60,8 +59,6 @@ wl.Descriptions():new_worker_type {
    },
    animations = {
       idle = {
-         directory = dirname,
-         basename = "idle",
          hotspot = {8, 23}
       },
    },
