@@ -1,10 +1,10 @@
 push_textdomain("world")
 
-dirname = path.dirname(__file__)
+local dirname = path.dirname(__file__)
 
 wl.Descriptions():new_immovable_type{
    name = "deadtree2",
-   descname = _ "Dead Tree",
+   descname = _("Dead Tree"),
    size = "none",
    programs = {
       main = {
@@ -12,9 +12,10 @@ wl.Descriptions():new_immovable_type{
          "remove=chance:4.69%"
       }
    },
+   animation_directory = dirname,
+   icon = dirname .. "menu.png",
    animations = {
       idle = {
-         pictures = path.list_files(dirname .. "idle.png"),
          hotspot = { 27, 56 },
       },
    }
