@@ -1,6 +1,6 @@
 push_textdomain("world")
 
-dirname = path.dirname(__file__)
+local dirname = path.dirname(__file__)
 
 include(dirname .. "../terrain_affinity.lua")
 
@@ -8,6 +8,7 @@ wl.Descriptions():new_immovable_type{
    name = "larch_summer_mature",
    descname = _("Larch (Mature)"),
    size = "small",
+   animation_directory = dirname,
    terrain_affinity = terrain_affinity,
    programs = {
       main = {
@@ -18,7 +19,6 @@ wl.Descriptions():new_immovable_type{
    },
    spritesheets = {
       idle = {
-         directory = dirname,
          basename = "mature",
          fps = 8,
          frames = 4,
