@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2024 by the Widelands Development Team
+ * Copyright (C) 2006-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,7 +30,7 @@ namespace math {
 		try {                                                                                        \
 			return function(str);                                                                     \
 		} catch (...) {                                                                              \
-			throw WLWarning("", "Expected a " #type ", received: %s", str.c_str());                   \
+			throw WLWarning("", "Expected a " #type ", received: '%s'", str.c_str());                 \
 		}                                                                                            \
 	}
 
