@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2021-2024 by the Widelands Development Team
+ * Copyright (C) 2021-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -82,7 +82,7 @@ std::string join(const Container& words, const std::string& separator) {
 }
 
 /** Convert various types to string. Useful in templates where the parameter type is not fixed. */
-inline const std::string& as_string(const std::string& str) {
+inline std::string as_string(std::string str) {
 	return str;
 }
 inline std::string as_string(const char* str) {
