@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2022 by the Widelands Development Team
+ * Copyright (C) 2004-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,7 +20,9 @@
 #define WL_ECONOMY_ECONOMY_H
 
 #include <functional>
+#include <limits>
 #include <memory>
+#include <set>
 
 #include "base/macros.h"
 #include "economy/supply.h"
@@ -39,6 +41,7 @@ class ProductionSite;
 struct RSPairStruct;
 struct Route;
 struct Router;
+struct RoutingNode;
 class WorkerDescr;
 
 struct NoteEconomy {
@@ -54,8 +57,6 @@ struct NoteEconomy {
 	const Action action;
 };
 
-constexpr Quantity kEconomyTargetInfinity = std::numeric_limits<Quantity>::max();
-
 /**
  * Each Economy represents all building and flags, which are connected over the same
  * street network. In general a player can own multiple Economys, which
@@ -112,15 +113,15 @@ public:
 	explicit Economy(Player&, Serial serial, WareWorker);  // For saveloading
 	~Economy();
 
-	Serial serial() const {
+	[[nodiscard]] Serial serial() const {
 		return serial_;
 	}
 
-	Player& owner() const {
+	[[nodiscard]] Player& owner() const {
 		return owner_;
 	}
 
-	WareWorker type() const {
+	[[nodiscard]] WareWorker type() const {
 		return type_;
 	}
 
@@ -135,7 +136,7 @@ public:
 	                                  uint32_t cost_cutoff = 0,
 	                                  const WarehouseAcceptFn& acceptfn = WarehouseAcceptFn());
 
-	std::vector<Flag*>::size_type get_nrflags() const {
+	[[nodiscard]] std::vector<Flag*>::size_type get_nrflags() const {
 		return flags_.size();
 	}
 	void add_flag(Flag&);
@@ -153,7 +154,7 @@ public:
 
 	void add_warehouse(Warehouse&);
 	void remove_warehouse(Warehouse&);
-	const std::vector<Warehouse*>& warehouses() const {
+	[[nodiscard]] const std::vector<Warehouse*>& warehouses() const {
 		return warehouses_;
 	}
 
@@ -169,30 +170,31 @@ public:
 	}
 
 	/// Whether the economy needs more of this ware/worker type.
+	/// If a flag is given, consider only its district, otherwise the entire economy.
 	/// Productionsites may ask this before they produce, to avoid depleting a
 	/// ware type by overproducing another from it.
-	bool needs_ware_or_worker(DescriptionIndex) const;
+	[[nodiscard]] bool needs_ware_or_worker(DescriptionIndex, const Flag* flag) const;
 
-	const TargetQuantity& target_quantity(DescriptionIndex const i) const {
+	[[nodiscard]] const TargetQuantity& target_quantity(DescriptionIndex const i) const {
 		return target_quantities_[i];
 	}
 	TargetQuantity& target_quantity(DescriptionIndex const i) {
 		return target_quantities_[i];
 	}
 
-	void* get_options_window() const {
+	[[nodiscard]] void* get_options_window() const {
 		return options_window_;
 	}
 	void set_options_window(void* window) {
 		options_window_ = window;
 	}
 
-	const WareList& get_wares_or_workers() const {
+	[[nodiscard]] const WareList& get_wares_or_workers() const {
 		return wares_or_workers_;
 	}
 
 	// Checks whether this economy contains a building of the specified type
-	bool has_building(DescriptionIndex) const;
+	[[nodiscard]] bool has_building(DescriptionIndex) const;
 	/**
 	 * Of all occupied ProductionSites of the specified type in this economy,
 	 * find the one that is closest to the specified flag and return it.
@@ -210,8 +212,7 @@ public:
 		start_request_timer();
 	}
 
-protected:
-	static Serial last_economy_serial_;
+	[[nodiscard]] bool same_district(const PlayerImmovable& p1, const PlayerImmovable& p2) const;
 
 private:
 	// This structs is to store distance from supply to request(or), but to allow unambiguous
@@ -241,7 +242,10 @@ private:
 
 	void start_request_timer(const Duration& delta = Duration(200));
 
-	Supply* find_best_supply(Game&, const Request&, int32_t& cost);
+	Supply* find_best_supply(Game&,
+	                         const Request&,
+	                         int32_t& cost,
+	                         uint32_t search_radius = std::numeric_limits<uint32_t>::max());
 	void process_requests(Game&, RSPairStruct* supply_pairs);
 	void balance_requestsupply(Game&);
 	void handle_active_supplies(Game&);
@@ -250,6 +254,9 @@ private:
 
 	bool has_request(Request&) const;
 
+	void check_imports(Game& game);
+	void recalc_districts();
+
 	/*************/
 	/* Variables */
 	/*************/
@@ -263,12 +270,20 @@ private:
 	Flags flags_;
 	WareList wares_or_workers_;  ///< virtual storage with all wares/workers in this Economy
 	std::vector<Warehouse*> warehouses_;
+	uint32_t nr_districts_{0U};
 
 	WareWorker type_;  ///< whether we are a WareEconomy or a WorkerEconomy
 
 	RequestList requests_;  ///< requests
 	SupplyList supplies_;
 
+	/* Used in find_best_supply. These are only members to avoid
+	 * frequent memory allocation and freeing. */
+	std::vector<Supply*> possible_imports_;
+	std::vector<Supply*> possible_supplies_;
+	std::vector<Widelands::RoutingNode*> possible_flags_;
+	std::set<Widelands::Flag*> seen_flags_;
+
 	TargetQuantity* target_quantities_;
 	std::unique_ptr<Router> router_;
 
@@ -289,9 +304,6 @@ private:
 	// may change when merging while the window is open, so we have to keep track of it here.
 	void* options_window_;
 
-	// 'list' of unique providers
-	std::map<UniqueDistance, Supply*> available_supplies_;
-
 	// Helper function for `find_closest_occupied_productionsite()`
 	bool check_building_can_start_working(const ProductionSite&, bool check_inputqueues);
 
