@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2022 by the Widelands Development Team
+ * Copyright (C) 2006-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -43,7 +43,7 @@ class Worker;
 
 class ExpeditionBootstrap {
 public:
-	explicit ExpeditionBootstrap(PortDock* const portdock);
+	explicit ExpeditionBootstrap(PortDock* portdock);
 	virtual ~ExpeditionBootstrap();
 
 	// Start bootstrapping an expedition. This will request all wares and workers.
@@ -68,15 +68,18 @@ public:
 	void set_economy(Economy* economy, WareWorker);
 
 	// Returns the wares and workers currently waiting for the expedition.
-	std::vector<InputQueue*> queues(bool all) const;
+	[[nodiscard]] std::vector<InputQueue*> queues(bool all) const;
 
 	// Returns the matching input queue for the given index and type.
-	InputQueue& inputqueue(DescriptionIndex index, WareWorker type, bool) const;
-	InputQueue* inputqueue(size_t additional_index) const;
-	InputQueue& inputqueue(const Request&) const;
+	[[nodiscard]] InputQueue& inputqueue(DescriptionIndex index,
+	                                     WareWorker type,
+	                                     bool additional,
+	                                     uint32_t disambiguator_id) const;
+	[[nodiscard]] InputQueue* inputqueue(size_t additional_index) const;
+	[[nodiscard]] InputQueue& inputqueue(const Request&) const;
 
 	void demand_additional_item(Game&, WareWorker, DescriptionIndex, bool);
-	size_t count_additional_queues() const;
+	[[nodiscard]] size_t count_additional_queues() const;
 
 	// Tests if all wares for the expedition have arrived. If so, informs the portdock.
 	void check_is_ready(Game& game);
@@ -115,7 +118,7 @@ private:
 };
 
 struct NoteExpeditionCanceled {
-	CAN_BE_SENT_AS_NOTE(NoteId::NoteExpeditionCanceled)
+	CAN_BE_SENT_AS_NOTE(NoteId::ExpeditionCanceled)
 
 	ExpeditionBootstrap* bootstrap;
 
