@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2024 by the Widelands Development Team
+ * Copyright (C) 2004-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -174,6 +174,9 @@ bool Flag::get_passable() const {
 Flag& Flag::base_flag() {
 	return *this;
 }
+const Flag& Flag::base_flag() const {
+	return *this;
+}
 
 /**
  * Call this only from Economy code!
@@ -368,7 +371,7 @@ uint8_t Flag::nr_of_roads() const {
 	uint8_t counter = 0;
 	for (uint8_t road_id = WalkingDir::LAST_DIRECTION; road_id >= WalkingDir::FIRST_DIRECTION;
 	     --road_id) {
-		if (get_roadbase(road_id) != nullptr) {
+		if (get_road(road_id) != nullptr) {
 			++counter;
 		}
 	}
@@ -423,7 +426,7 @@ bool Flag::has_capacity() const {
  * The capacity queue is a simple FIFO queue.
  */
 void Flag::wait_for_capacity(Game& /* game */, Worker& bob) {
-	capacity_wait_.push_back(&bob);
+	capacity_wait_.emplace_back(&bob);
 }
 
 /**
@@ -977,6 +980,8 @@ void Flag::add_flag_job(Game& game, const FlagJob::Type t) {
 	case FlagJob::Type::kScout:
 		do_schedule_act(game, Duration(10));
 		break;
+	default:
+		NEVER_HERE();
 	}
 
 	flag_jobs_.push_back(j);
@@ -1015,11 +1020,25 @@ void Flag::log_general_info(const Widelands::EditorGameBase& egbase) const {
 
 	Widelands::PlayerImmovable::log_general_info(egbase);
 
+	if (district_center_[wwWARE] == nullptr) {
+		molog(egbase.get_gametime(), "No ware district\n");
+	} else {
+		molog(egbase.get_gametime(), "Ware district %u (%s)\n", district_center_[wwWARE]->serial(),
+		      district_center_[wwWARE]->get_warehouse_name().c_str());
+	}
+	if (district_center_[wwWORKER] == nullptr) {
+		molog(egbase.get_gametime(), "No worker district\n");
+	} else {
+		molog(egbase.get_gametime(), "Worker district %u (%s)\n",
+		      district_center_[wwWORKER]->serial(),
+		      district_center_[wwWORKER]->get_warehouse_name().c_str());
+	}
+
 	if (ware_filled_ != 0) {
 		molog(egbase.get_gametime(), "Wares at flag:\n");
 		for (int i = 0; i < ware_filled_; ++i) {
 			PendingWare& pi = wares_[i];
-			molog(egbase.get_gametime(), " %i/%i: %s(%i), nextstep %i, %s\n", i + 1, ware_capacity_,
+			molog(egbase.get_gametime(), " %i/%i: %s(%u), nextstep %u, %s\n", i + 1, ware_capacity_,
 			      pi.ware->descr().name().c_str(), pi.ware->serial(), pi.nextstep.serial(),
 			      pi.pending ? "pending" : "acked by carrier");
 		}
