@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2022 by the Widelands Development Team
+ * Copyright (C) 2004-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -40,11 +40,7 @@ InputQueue::InputQueue(PlayerImmovable& init_owner,
      index_(init_index),
      max_size_(init_max_size),
      max_fill_(init_max_size),
-     type_(init_type),
-     consume_interval_(0),
-     request_(nullptr),
-     callback_fn_(nullptr),
-     callback_data_(nullptr) {
+     type_(init_type) {
 }
 
 void InputQueue::update() {
@@ -78,7 +74,7 @@ void InputQueue::request_callback(Game& game,
 		type = wwWORKER;
 	}
 
-	InputQueue& iq = dynamic_cast<Building&>(target).inputqueue(index, type, &r);
+	InputQueue& iq = dynamic_cast<Building&>(target).inputqueue(index, type, &r, 0);
 
 	iq.entered(index, worker);
 
@@ -115,6 +111,8 @@ void InputQueue::set_max_fill(Quantity size) {
 	max_fill_ = size;
 
 	update();
+
+	owner_.inputqueue_max_fill_changed();
 }
 
 void InputQueue::set_consume_interval(const Duration& time) {
@@ -182,6 +180,8 @@ void InputQueue::write(FileWrite& fw, Game& game, MapObjectSaver& mos) {
 		fw.unsigned_8(1);
 		fw.c_string(owner().tribe().get_worker_descr(index_)->name().c_str());
 		break;
+	default:
+		NEVER_HERE();
 	}
 	fw.signed_32(max_size_);
 	fw.signed_32(max_fill_);
