@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2024 by the Widelands Development Team
+ * Copyright (C) 2004-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,6 +19,7 @@
 #ifndef WL_ECONOMY_ROUTING_NODE_H
 #define WL_ECONOMY_ROUTING_NODE_H
 
+#include "base/wexception.h"
 #include "logic/cookie_priority_queue.h"
 #include "logic/map_objects/tribes/wareworker.h"
 #include "logic/widelands_geometry.h"
@@ -86,6 +87,8 @@ public:
 		case wwWORKER:
 			mpf_cycle_worker = 0;
 			break;
+		default:
+			NEVER_HERE();
 		}
 	}
 
