@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2022 by the Widelands Development Team
+ * Copyright (C) 2011-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -44,21 +44,30 @@ namespace {
 // Every MapObject() needs to have a description. So we make a dummy one for
 // Fleet.
 ShipFleetDescr g_ship_fleet_descr("ship_fleet", "Ship Fleet");
+ShipFleetYardInterfaceDescr g_ship_fleet_yard_interface_descr("ship_fleet_yard_interface",
+                                                              "Ship Fleet Yard Interface");
 }  // namespace
 
 const ShipFleetDescr& ShipFleet::descr() const {
 	return g_ship_fleet_descr;
 }
 
+const ShipFleetYardInterfaceDescr& ShipFleetYardInterface::descr() const {
+	return g_ship_fleet_yard_interface_descr;
+}
+
+Bob& ShipFleetYardInterfaceDescr::create_object() const {
+	return *new ShipFleetYardInterface();
+}
+
 /**
  * Fleets are initialized empty.
  *
- * Intended use: @ref Ship and @ref PortDock, when created, create a new @ref ShipFleet
- * instance, then add themselves \em before calling the \ref init function.
- * The Fleet takes care of merging with existing fleets, if any.
+ * Intended use: @ref Ship, @ref ShipFleetYardInterface, and @ref PortDock, when created,
+ * create a new @ref ShipFleet instance, then add themselves \em before calling the
+ * \ref init function. The Fleet takes care of merging with existing fleets, if any.
  */
-ShipFleet::ShipFleet(Player* player)
-   : MapObject(&g_ship_fleet_descr), act_pending_(false), schedule_(*this) {
+ShipFleet::ShipFleet(Player* player) : MapObject(&g_ship_fleet_descr), schedule_(*this) {
 	owner_ = player;
 }
 
@@ -123,7 +132,8 @@ struct StepEvalFindShipFleet {
 		if ((from.field->nodecaps() & MOVECAPS_SWIM) == 0) {
 			// We are allowed to land on and leave the shore,
 			// but not in the middle of a path
-			if (fromcost > 0) {
+			// also prevent 1 step paths between 2 land nodes
+			if (fromcost > 0 || (fromcost == 0 && (to.field->nodecaps() & MOVECAPS_SWIM) == 0)) {
 				return -1;
 			}
 
@@ -144,6 +154,10 @@ bool ShipFleet::find_other_fleet(EditorGameBase& egbase) {
 		astar.push(temp_ship->get_position());
 	}
 
+	for (const ShipFleetYardInterface* temp_interface : interfaces_) {
+		astar.push(temp_interface->get_position());
+	}
+
 	for (const PortDock* temp_port : ports_) {
 		BaseImmovable::PositionList pos = temp_port->get_positions(egbase);
 
@@ -164,8 +178,7 @@ bool ShipFleet::find_other_fleet(EditorGameBase& egbase) {
 				if (dock->get_fleet() == nullptr) {
 					log_warn_time(egbase.get_gametime(), "The dock on %3dx%3d without a fleet!\n",
 					              dock->dockpoints_.front().x, dock->dockpoints_.front().y);
-				}
-				if (dock->get_fleet() != this && dock->get_owner() == get_owner()) {
+				} else if (dock->get_fleet() != this && dock->get_owner() == get_owner()) {
 					return dock->get_fleet()->merge(egbase, this);
 				}
 			}
@@ -179,6 +192,12 @@ bool ShipFleet::find_other_fleet(EditorGameBase& egbase) {
 				    ship->get_owner() == get_owner()) {
 					return ship->get_fleet()->merge(egbase, this);
 				}
+			} else if (type == MapObjectType::SHIP_FLEET_YARD_INTERFACE) {
+				upcast(ShipFleetYardInterface, interface, bob);
+				if (interface->get_fleet() != nullptr && interface->get_fleet() != this &&
+				    interface->get_owner() == get_owner()) {
+					return interface->get_fleet()->merge(egbase, this);
+				}
 			}
 		}
 	}
@@ -190,6 +209,11 @@ bool ShipFleet::find_other_fleet(EditorGameBase& egbase) {
 	return false;
 }
 
+void ShipFleet::set_ships_target(EditorGameBase& egbase, Quantity t) {
+	ships_target_ = t;
+	target_last_modified_ = egbase.get_gametime();
+}
+
 /**
  * Merge the @p other fleet into this fleet, and remove the other fleet.
  *
@@ -203,6 +227,11 @@ bool ShipFleet::merge(EditorGameBase& egbase, ShipFleet* other) {
 		return true;
 	}
 
+	if (target_last_modified_ < other->target_last_modified_) {
+		ships_target_ = other->ships_target_;
+		target_last_modified_ = other->target_last_modified_;
+	}
+
 	// TODO(Nordfriese): Currently fleets merge only directly after creation, so the fleet being
 	// merged has either no ports or no ships, and therefore should not yet have scheduled anything.
 	// Scripts could also enforce fleet merges by changing the terrain so that two previously
@@ -216,6 +245,12 @@ bool ShipFleet::merge(EditorGameBase& egbase, ShipFleet* other) {
 		add_ship(egbase, ship);
 	}
 
+	while (!other->interfaces_.empty()) {
+		ShipFleetYardInterface* interface = other->interfaces_.back();
+		other->interfaces_.pop_back();
+		add_interface(interface);
+	}
+
 	uint32_t old_nrports = ports_.size();
 	ports_.insert(ports_.end(), other->ports_.begin(), other->ports_.end());
 	port_paths_.insert(other->port_paths_.begin(), other->port_paths_.end());
@@ -236,6 +271,51 @@ bool ShipFleet::merge(EditorGameBase& egbase, ShipFleet* other) {
 	return false;
 }
 
+/**
+ * Split the fleet into two or more fleets, due to changed terrain.
+ *
+ */
+void ShipFleet::split(Game& game) {
+	// something changed the map, splitting fleets (Diker, maps scripts, etc.)
+	// we do not know which ports and ships are affected in the fleet
+	// therefore we need to remove them all from the fleet (effectively deleting the fleet)
+	// and afterwards reinit new fleets
+	std::vector<Ship*> fleetships = ships_;
+	std::vector<PortDock*> fleetports = ports_;
+	std::vector<ShipFleetYardInterface*> fleetinterfaces = interfaces_;
+	std::set<ProductionSite*> fleetshipyards;
+
+	for (auto& interface : fleetinterfaces) {
+		fleetshipyards.insert(interface->get_building());
+		remove_interface(game, interface);
+	}
+	verb_log_dbg_time(game.get_gametime(), "[Splitting ShipFleet]: interfaces removed");
+	for (auto& ship : fleetships) {
+		ship->set_destination(game, nullptr);
+		for (ShippingItem& si : ship->items_) {
+			si.destination_dock_ = nullptr;
+		}
+		remove_ship(game, ship);
+	}
+	verb_log_dbg_time(game.get_gametime(), "[Splitting ShipFleet]: ships removed");
+	for (auto& port : fleetports) {
+		remove_port(game, port);
+	}
+	verb_log_dbg_time(game.get_gametime(), "[Splitting ShipFleet]: ports removed");
+	for (auto& port : fleetports) {
+		port->init_fleet(game);
+	}
+	verb_log_dbg_time(game.get_gametime(), "[Splitting ShipFleet]: ports added");
+	for (auto& ship : fleetships) {
+		ship->init_fleet(game);
+	}
+	verb_log_dbg_time(game.get_gametime(), "[Splitting ShipFleet]: ships added");
+	for (const auto& yard : fleetshipyards) {
+		yard->init_yard_interfaces(game);
+	}
+	verb_log_dbg_time(game.get_gametime(), "[Splitting ShipFleet]: yardinterfaces added");
+}
+
 /**
  * If we have ports and ships, ensure that all ports belong to the same economy.
  */
@@ -274,6 +354,12 @@ void ShipFleet::cleanup(EditorGameBase& egbase) {
 		ships_.pop_back();
 	}
 
+	while (!interfaces_.empty()) {
+		ShipFleetYardInterface* interface = interfaces_.back();
+		interface->set_fleet(nullptr);
+		interfaces_.pop_back();
+	}
+
 	MapObject::cleanup(egbase);
 }
 
@@ -425,7 +511,7 @@ void ShipFleet::remove_ship(EditorGameBase& egbase, Ship* ship) {
 	if (ships_.empty()) {
 		if (empty()) {
 			remove(egbase);
-		} else {
+		} else if (ports_.size() > 1) {
 			Flag& base = ports_[0]->base_flag();
 			for (uint32_t i = 1; i < ports_.size(); ++i) {
 				// since two ports can be connected by land, it is possible that
@@ -441,8 +527,44 @@ void ShipFleet::remove_ship(EditorGameBase& egbase, Ship* ship) {
 	}
 }
 
+void ShipFleet::add_interface(ShipFleetYardInterface* interface) {
+	interfaces_.push_back(interface);
+	assert(std::count(interfaces_.begin(), interfaces_.end(), interface) == 1);
+	interface->set_fleet(this);
+}
+
+void ShipFleet::remove_interface(EditorGameBase& egbase, ShipFleetYardInterface* interface) {
+	auto it = std::find(interfaces_.begin(), interfaces_.end(), interface);
+	if (it == interfaces_.end()) {
+		log_warn_time(egbase.get_gametime(),
+		              "ShipFleet %u: Requested to remove interface %u which is not in this fleet\n",
+		              serial(), interface ? interface->serial() : 0);
+		return;
+	}
+	while (it != interfaces_.end()) {
+		interfaces_.erase(it);
+		it = std::find(interfaces_.begin(), interfaces_.end(), interface);
+		if (it != interfaces_.end()) {
+			log_warn_time(egbase.get_gametime(),
+			              "ShipFleet %u: Multiple instances of interface %u were in the Ship fleet",
+			              serial(), interface->serial());
+		}
+	}
+	assert(std::count(interfaces_.begin(), interfaces_.end(), interface) == 0);
+	interface->set_fleet(nullptr);
+
+	if (empty()) {
+		remove(egbase);
+	}
+}
+
 bool ShipFleet::empty() const {
-	return ships_.empty() && ports_.empty();
+	return ships_.empty() && interfaces_.empty() && ports_.empty();
+}
+
+bool ShipFleet::lacks_ship() const {
+	// If the target is explicitly set to 0, don't request any ships at all.
+	return ships_.size() < ships_target_;
 }
 
 struct StepEvalFindPorts {
@@ -460,7 +582,7 @@ struct StepEvalFindPorts {
 		return std::max(0, est - 5 * map.calc_cost(0));
 	}
 
-	int32_t stepcost(
+	[[nodiscard]] int32_t stepcost(
 	   const Map& map, FCoords from, int32_t /* fromcost */, WalkingDir dir, FCoords to) const {
 		if ((to.field->nodecaps() & MOVECAPS_SWIM) == 0) {
 			return -1;
@@ -537,7 +659,7 @@ void ShipFleet::connect_port(EditorGameBase& egbase, uint32_t idx) {
 			}
 
 			ppath.cost = cost;
-			ppath.path.reset(new Path);
+			ppath.path = std::make_shared<Path>();
 			astar.pathto(cur, *ppath.path);
 			if (reverse) {
 				ppath.path->reverse();
@@ -705,17 +827,58 @@ void ShipFleet::act(Game& game, uint32_t /*data*/) {
 void ShipFleet::log_general_info(const EditorGameBase& egbase) const {
 	MapObject::log_general_info(egbase);
 
-	molog(egbase.get_gametime(), "%" PRIuS " ships and %" PRIuS " ports\n", ships_.size(),
-	      ports_.size());
+	molog(egbase.get_gametime(),
+	      "%" PRIuS " ships and %" PRIuS " ports and %" PRIuS " shipyard interfaces\n",
+	      ships_.size(), ports_.size(), interfaces_.size());
 	molog(egbase.get_gametime(), "Schedule:\n");
 	schedule_.log_general_info(egbase);
 	molog(egbase.get_gametime(), "\n");
 }
 
-// Changelog of version 4 → 5: Added ShippingSchedule
-constexpr uint8_t kCurrentPacketVersion = 5;
+ShipFleetYardInterface::ShipFleetYardInterface() : Bob(g_ship_fleet_yard_interface_descr) {
+}
 
-void ShipFleet::Loader::load(FileRead& fr) {
+ShipFleetYardInterface*
+ShipFleetYardInterface::create(EditorGameBase& egbase, ProductionSite& ps, const Coords& pos) {
+	ShipFleetYardInterface* interface = new ShipFleetYardInterface();
+	interface->set_owner(ps.get_owner());
+	interface->set_position(egbase, pos);
+	interface->building_ = &ps;
+	interface->init(egbase);
+
+	ShipFleet* fleet = new ShipFleet(interface->get_owner());
+	fleet->add_interface(interface);
+	fleet->init(egbase);  // Calls interface->set_fleet()
+
+	return interface;
+}
+
+void ShipFleetYardInterface::init_auto_task(Game& game) {
+	start_task_idle(game, 0, -1);
+}
+
+void ShipFleetYardInterface::cleanup(EditorGameBase& egbase) {
+	building_->remove_fleet_interface(egbase, this);
+
+	if (fleet_ != nullptr) {
+		fleet_->remove_interface(egbase, this);
+	}
+
+	Bob::cleanup(egbase);
+}
+
+void ShipFleetYardInterface::log_general_info(const EditorGameBase& egbase) const {
+	Bob::log_general_info(egbase);
+	molog(egbase.get_gametime(), "Interface to building %u at %3dx%3d\n", building_->serial(),
+	      building_->get_position().x, building_->get_position().y);
+}
+
+// Changelog of version 4 → 5: Added ShippingSchedule
+// Changelog of version 5 → 6: Added ship yard interfaces
+constexpr uint8_t kCurrentPacketVersionShip = 6;
+constexpr uint8_t kCurrentPacketVersionInterface = 1;
+
+void ShipFleet::Loader::load(FileRead& fr, uint8_t packet_version) {
 	MapObject::Loader::load(fr);
 
 	ShipFleet& fleet = get<ShipFleet>();
@@ -726,6 +889,12 @@ void ShipFleet::Loader::load(FileRead& fr) {
 		ships_[i] = fr.unsigned_32();
 	}
 
+	const uint32_t nrinterfaces = packet_version >= 6 ? fr.unsigned_32() : 0;
+	interfaces_.resize(nrinterfaces);
+	for (uint32_t i = 0; i < nrinterfaces; ++i) {
+		interfaces_[i] = fr.unsigned_32();
+	}
+
 	const uint32_t nrports = fr.unsigned_32();
 	ports_.resize(nrports);
 	for (uint32_t i = 0; i < nrports; ++i) {
@@ -733,6 +902,9 @@ void ShipFleet::Loader::load(FileRead& fr) {
 	}
 
 	fleet.act_pending_ = (fr.unsigned_8() != 0u);
+	fleet.ships_target_ = packet_version >= 6 ? fr.unsigned_32() : kEconomyTargetInfinity;
+	fleet.target_last_modified_ = packet_version >= 6 ? Time(fr) : Time(0);
+
 	fleet.schedule_.load(fr);
 }
 
@@ -751,6 +923,12 @@ void ShipFleet::Loader::load_pointers() {
 		fleet.ships_.push_back(&map_object_loader.get<Ship>(temp_ship));
 		fleet.ships_.back()->set_fleet(&fleet);
 	}
+
+	for (const uint32_t& temp_interface : interfaces_) {
+		fleet.interfaces_.push_back(&mol().get<ShipFleetYardInterface>(temp_interface));
+		fleet.interfaces_.back()->set_fleet(&fleet);
+	}
+
 	for (const uint32_t& temp_port : ports_) {
 		fleet.ports_.push_back(&map_object_loader.get<PortDock>(temp_port));
 		fleet.ports_.back()->set_fleet(&fleet);
@@ -782,7 +960,8 @@ MapObject::Loader* ShipFleet::load(EditorGameBase& egbase, MapObjectLoader& mol,
 	try {
 		// The header has been peeled away by the caller
 		const uint8_t packet_version = fr.unsigned_8();
-		if (packet_version == kCurrentPacketVersion) {
+		// TODO(Nordfriese): Savegame compatibility v1.1
+		if (packet_version >= 5 && packet_version <= kCurrentPacketVersionShip) {
 			PlayerNumber owner_number = fr.unsigned_8();
 			if ((owner_number == 0u) || owner_number > egbase.map().get_nrplayers()) {
 				throw GameDataError("owner number is %u but there are only %u players", owner_number,
@@ -794,9 +973,9 @@ MapObject::Loader* ShipFleet::load(EditorGameBase& egbase, MapObjectLoader& mol,
 				throw GameDataError("owning player %u does not exist", owner_number);
 			}
 			loader->init(egbase, mol, *(new ShipFleet(owner)));
-			loader->load(fr);
+			loader->load(fr, packet_version);
 		} else {
-			throw UnhandledVersionError("ShipFleet", packet_version, kCurrentPacketVersion);
+			throw UnhandledVersionError("ShipFleet", packet_version, kCurrentPacketVersionShip);
 		}
 	} catch (const std::exception& e) {
 		throw wexception("loading ship fleet: %s", e.what());
@@ -807,7 +986,7 @@ MapObject::Loader* ShipFleet::load(EditorGameBase& egbase, MapObjectLoader& mol,
 
 void ShipFleet::save(EditorGameBase& egbase, MapObjectSaver& mos, FileWrite& fw) {
 	fw.unsigned_8(HeaderShipFleet);
-	fw.unsigned_8(kCurrentPacketVersion);
+	fw.unsigned_8(kCurrentPacketVersionShip);
 
 	fw.unsigned_8(owner_.load()->player_number());
 
@@ -817,14 +996,64 @@ void ShipFleet::save(EditorGameBase& egbase, MapObjectSaver& mos, FileWrite& fw)
 	for (const Ship* temp_ship : ships_) {
 		fw.unsigned_32(mos.get_object_file_index(*temp_ship));
 	}
+
+	fw.unsigned_32(interfaces_.size());
+	for (const ShipFleetYardInterface* temp_interface : interfaces_) {
+		fw.unsigned_32(mos.get_object_file_index(*temp_interface));
+	}
+
 	fw.unsigned_32(ports_.size());
 	for (const PortDock* temp_port : ports_) {
 		fw.unsigned_32(mos.get_object_file_index(*temp_port));
 	}
 
 	fw.unsigned_8(static_cast<uint8_t>(act_pending_));
+	fw.unsigned_32(ships_target_);
+	target_last_modified_.save(fw);
 
 	schedule_.save(egbase, mos, fw);
 }
 
+void ShipFleetYardInterface::Loader::load(FileRead& fr) {
+	Bob::Loader::load(fr);
+	building_ = fr.unsigned_32();
+}
+
+void ShipFleetYardInterface::Loader::load_pointers() {
+	Bob::Loader::load_pointers();
+
+	ShipFleetYardInterface& interface = get<ShipFleetYardInterface>();
+	interface.building_ = &mol().get<ProductionSite>(building_);
+}
+
+Bob::Loader*
+ShipFleetYardInterface::load(EditorGameBase& egbase, MapObjectLoader& mol, FileRead& fr) {
+	std::unique_ptr<Loader> loader(new Loader);
+
+	try {
+		// The header has been peeled away by the caller
+		uint8_t const packet_version = fr.unsigned_8();
+		if (packet_version == kCurrentPacketVersionInterface) {
+			loader->init(egbase, mol, *new ShipFleetYardInterface);
+			loader->load(fr);
+		} else {
+			throw UnhandledVersionError(
+			   "ShipFleetYardInterface", packet_version, kCurrentPacketVersionInterface);
+		}
+	} catch (const std::exception& e) {
+		throw wexception("loading ship fleet yard interface: %s", e.what());
+	}
+
+	return loader.release();
+}
+
+void ShipFleetYardInterface::save(EditorGameBase& egbase, MapObjectSaver& mos, FileWrite& fw) {
+	fw.unsigned_8(HeaderShipFleetInterface);
+	fw.unsigned_8(kCurrentPacketVersionInterface);
+
+	Bob::save(egbase, mos, fw);
+
+	fw.unsigned_32(mos.get_object_file_index(*building_));
+}
+
 }  // namespace Widelands
