@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007-2024 by the Widelands Development Team
+ * Copyright (C) 2007-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -78,8 +78,7 @@ void TestingRoutingNode::get_neighbours(Widelands::WareWorker type,
 	for (TestingRoutingNode* nb : neighbours_) {
 		// second parameter is walktime in ms from this flag to the neighbour.
 		// only depends on slope
-		n.push_back(Widelands::RoutingNodeNeighbour(
-		   nb, 1000 * ((type == Widelands::wwWARE) ? 1 + waitcost_ : 1)));
+		n.emplace_back(nb, 1000 * ((type == Widelands::wwWARE) ? 1 + waitcost_ : 1));
 	}
 }
 bool TestingRoutingNode::all_members_zeroed() const {
