@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2024 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -50,13 +50,16 @@
 #include "editor/ui_menus/tool_set_terrain_options_menu.h"
 #include "editor/ui_menus/tool_toolhistory_options_menu.h"
 #include "editor/ui_menus/toolsize_menu.h"
+#include "graphic/color_palette.h"
 #include "graphic/mouse_cursor.h"
 #include "graphic/playercolor.h"
 #include "graphic/text_layout.h"
 #include "logic/addons.h"
 #include "logic/generic_save_handler.h"
 #include "logic/map.h"
+#include "logic/map_objects/checkstep.h"
 #include "logic/map_objects/descriptions.h"
+#include "logic/map_objects/findnode.h"
 #include "logic/map_objects/map_object_type.h"
 #include "logic/map_objects/world/resource_description.h"
 #include "logic/mapregion.h"
@@ -192,6 +195,9 @@ EditorInteractive::EditorInteractive(Widelands::EditorGameBase& e)
 	set_display_flag(InteractiveBase::dfDebug, false);
 #endif
 
+	field_terrain_changed_subscriber_ = Notifications::subscribe<Widelands::NoteFieldTerrainChanged>(
+	   [this](const Widelands::NoteFieldTerrainChanged& /*note*/) { ocean_overlays_.reset(); });
+
 	map_view()->field_clicked.connect([this](const Widelands::NodeAndTriangle<>& node_and_triangle) {
 		map_clicked(node_and_triangle, false);
 	});
@@ -277,7 +283,10 @@ void EditorInteractive::main_menu_selected(MainMenuEntry entry) {
 	} break;
 	case MainMenuEntry::kExitEditor: {
 		exit((SDL_GetModState() & KMOD_CTRL) != 0);
+		break;
 	}
+	default:
+		NEVER_HERE();
 	}
 }
 
@@ -442,6 +451,8 @@ void EditorInteractive::tool_menu_selected(ToolMenuEntry entry) {
 	case ToolMenuEntry::kToolHistory:
 		tool_windows_.toolhistory.toggle();
 		break;
+	default:
+		NEVER_HERE();
 	}
 	toolmenu_.toggle();
 }
@@ -478,6 +489,16 @@ void EditorInteractive::rebuild_showhide_menu() {
 	                    "all immovables (trees, rocks, etc.) are removed"),
 	                  shortcut_string_for(KeyboardShortcut::kEditorShowhideMaximumBuildhelp, false));
 
+	showhidemenu_.add(get_display_flag(dfHeightHeatMap) ?
+	                     /** TRANSLATORS: An entry in the editor's show/hide menu to toggle whether
+	                     the map height heat map is shown */
+	                     _("Disable height heat map") :
+	                     _("Enable height heat map"),
+	                  ShowHideEntry::kHeightHeatMap,
+	                  g_image_cache->get("images/wui/menus/menu_toggle_height_heat_map.png"), false,
+	                  "",
+	                  shortcut_string_for(KeyboardShortcut::kEditorShowhideHeightHeatMap, false));
+
 	/** TRANSLATORS: An entry in the editor's show/hide menu to toggle whether the map grid is shown
 	 */
 	showhidemenu_.add(get_display_flag(dfShowGrid) ? _("Hide Grid") : _("Show Grid"),
@@ -485,6 +506,15 @@ void EditorInteractive::rebuild_showhide_menu() {
 	                  g_image_cache->get("images/wui/menus/menu_toggle_grid.png"), false, "",
 	                  shortcut_string_for(KeyboardShortcut::kEditorShowhideGrid, false));
 
+	/** TRANSLATORS: An entry in the editor's show/hide menu to toggle whether the oceans are shown
+	 */
+	showhidemenu_.add(get_display_flag(dfShowOceans) ? _("Hide Oceans") : _("Show Oceans"),
+	                  ShowHideEntry::kOceans,
+	                  g_image_cache->get("images/wui/menus/menu_toggle_oceans.png"), false,
+	                  _("Display separate water bodies with differently coloured overlays to see "
+	                    "which coasts can be connected by shipping routes"),
+	                  shortcut_string_for(KeyboardShortcut::kEditorShowhideOceans, false));
+
 	showhidemenu_.add(
 	   /** TRANSLATORS: An entry in the editor's show/hide menu to toggle whether immovables
 	    *  (trees, rocks etc.) are shown */
@@ -515,9 +545,15 @@ void EditorInteractive::showhide_menu_selected(ShowHideEntry entry) {
 	case ShowHideEntry::kMaximumBuildingSpaces: {
 		toggle_maximum_buildhelp();
 	} break;
+	case ShowHideEntry::kHeightHeatMap: {
+		toggle_height_heat_map();
+	} break;
 	case ShowHideEntry::kGrid: {
 		toggle_grid();
 	} break;
+	case ShowHideEntry::kOceans: {
+		toggle_oceans();
+	} break;
 	case ShowHideEntry::kImmovables: {
 		toggle_immovables();
 	} break;
@@ -527,6 +563,8 @@ void EditorInteractive::showhide_menu_selected(ShowHideEntry entry) {
 	case ShowHideEntry::kResources: {
 		toggle_resources();
 	} break;
+	default:
+		NEVER_HERE();
 	}
 	showhidemenu_.toggle();
 }
@@ -616,7 +654,6 @@ void EditorInteractive::exit(const bool force) {
 			return;
 		}
 	}
-	g_sh->change_music(Songset::kMenu, 200);
 	end_modal<UI::Panel::Returncodes>(UI::Panel::Returncodes::kBack);
 }
 
@@ -672,9 +709,15 @@ bool EditorInteractive::handle_mousepress(uint8_t btn, int32_t x, int32_t y) {
 }
 
 void EditorInteractive::draw(RenderTarget& dst) {
+	const bool show_oceans = get_display_flag(EditorInteractive::dfShowOceans);
+	if (show_oceans && ocean_overlays_ == nullptr) {
+		update_ocean_overlays();
+	}
+
 	const auto& ebase = egbase();
 	auto* fields_to_draw =
-	   map_view()->draw_terrain(ebase, nullptr, Workareas(), get_display_flag(dfShowGrid), &dst);
+	   map_view()->draw_terrain(ebase, nullptr, Workareas(), get_display_flag(dfHeightHeatMap),
+	                            get_display_flag(dfShowGrid), &dst);
 
 	const float scale = 1.f / map_view()->view().zoom;
 	const Time& gametime = ebase.get_gametime();
@@ -721,6 +764,7 @@ void EditorInteractive::draw(RenderTarget& dst) {
 
 	std::map<std::pair<Widelands::DescriptionIndex, Widelands::ResourceAmount>, const Image*>
 	   resource_images_cache;
+	std::map<uint32_t, std::pair<const Image*, float>> ocean_overlays_cache;
 
 	for (size_t idx = 0; idx < fields_to_draw->size(); ++idx) {
 		const FieldsToDraw::Field& field = fields_to_draw->at(idx);
@@ -731,8 +775,8 @@ void EditorInteractive::draw(RenderTarget& dst) {
 		if (get_display_flag(dfShowImmovables)) {
 			Widelands::BaseImmovable* const imm = field.fcoords.field->get_immovable();
 			if (imm != nullptr && imm->get_positions(ebase).front() == field.fcoords) {
-				imm->draw(
-				   gametime, InfoToDraw::kNone, field.rendertarget_pixel, field.fcoords, scale, &dst);
+				imm->draw(gametime, InfoToDraw::kShowBuildings, field.rendertarget_pixel, field.fcoords,
+				          scale, &dst);
 			}
 		}
 
@@ -798,6 +842,25 @@ void EditorInteractive::draw(RenderTarget& dst) {
 			}
 		}
 
+		// Show ocean overlay.
+		if (show_oceans) {
+			const unsigned ocean_number = ocean_overlays_->at(map.get_index(field.fcoords));
+			if (ocean_number != 0) {
+				auto it = ocean_overlays_cache.find(ocean_number);
+				if (it == ocean_overlays_cache.end()) {
+					it = ocean_overlays_cache
+					        .emplace(
+					           ocean_number, playercolor_image(kOceanColors.at(ocean_number - 1),
+					                                           "images/wui/overlays/ocean.png", scale))
+					        .first;
+				}
+				blit_field_overlay(
+				   &dst, field, it->second.first,
+				   Vector2i(it->second.first->width() / 2, it->second.first->height() / 2),
+				   scale / it->second.second);
+			}
+		}
+
 		// Draw the player starting position overlays.
 		const auto it = starting_positions.find(field.fcoords);
 		if (it != starting_positions.end()) {
@@ -851,6 +914,54 @@ void EditorInteractive::draw(RenderTarget& dst) {
 	}
 }
 
+void EditorInteractive::update_ocean_overlays() {
+	// Find all oceans on the map.
+	const Widelands::Map& map = egbase().map();
+	const Widelands::CheckStepDefault checkstep(Widelands::MOVECAPS_SWIM);
+	const Widelands::FindNodeAlwaysTrue functor;
+	const size_t nr_fields = map.max_index();
+
+	unsigned last_ocean = 0;
+	ocean_overlays_.reset(new std::vector<uint32_t>(nr_fields, 0));
+
+	for (size_t index = 0; index < nr_fields; ++index) {
+		if (ocean_overlays_->at(index) != 0) {
+			continue;  // Already visited
+		}
+
+		Widelands::Field& field = map[index];
+		if ((field.nodecaps() & Widelands::MOVECAPS_SWIM) == 0) {
+			continue;  // Not an ocean field
+		}
+
+		// New ocean found.
+		++last_ocean;
+
+		std::vector<Widelands::Coords> ocean_fields;
+		map.find_reachable_fields(egbase(),
+		                          Widelands::Area<Widelands::FCoords>(
+		                             map.get_fcoords(field), std::numeric_limits<uint16_t>::max()),
+		                          &ocean_fields, checkstep, functor);
+
+		// Store and validate the fields.
+		for (const Widelands::Coords& coords : ocean_fields) {
+			size_t i = map.get_index(coords);
+			if (ocean_overlays_->at(i) != 0 && ocean_overlays_->at(i) != last_ocean) {
+				throw wexception("Field #%" PRIuS " (%dx%d) belongs to two oceans #%u and #%u!", i,
+				                 coords.x, coords.y, ocean_overlays_->at(i), last_ocean);
+			}
+			ocean_overlays_->at(i) = last_ocean;
+		}
+
+		if (ocean_overlays_->at(index) != last_ocean) {
+			throw wexception("Field #%" PRIuS " does not belong to its own ocean #%u of size %" PRIuS
+			                 "!",
+			                 index, last_ocean, ocean_fields.size());
+		}
+	}
+	verb_log_dbg("Map has %u oceans.", last_ocean);
+}
+
 /// Needed to get freehand painting tools (hold down mouse and move to edit).
 void EditorInteractive::set_sel_pos(Widelands::NodeAndTriangle<> const sel) {
 	bool const target_changed = tools_->current().operates_on_triangles() ?
@@ -897,6 +1008,11 @@ void EditorInteractive::toggle_bobs() {
 	   EditorInteractive::dfShowBobs, !get_display_flag(EditorInteractive::dfShowBobs));
 }
 
+void EditorInteractive::toggle_height_heat_map() {
+	set_display_flag(
+	   EditorInteractive::dfHeightHeatMap, !get_display_flag(EditorInteractive::dfHeightHeatMap));
+}
+
 void EditorInteractive::toggle_grid() {
 	set_display_flag(
 	   EditorInteractive::dfShowGrid, !get_display_flag(EditorInteractive::dfShowGrid));
@@ -907,7 +1023,16 @@ void EditorInteractive::toggle_maximum_buildhelp() {
 	                 !get_display_flag(EditorInteractive::dfShowMaximumBuildhelp));
 }
 
+void EditorInteractive::toggle_oceans() {
+	set_display_flag(
+	   EditorInteractive::dfShowOceans, !get_display_flag(EditorInteractive::dfShowOceans));
+}
+
 bool EditorInteractive::handle_key(bool const down, SDL_Keysym const code) {
+	if (InteractiveBase::handle_key(down, code)) {
+		return true;
+	}
+
 	if (down) {
 		if (matches_shortcut(KeyboardShortcut::kCommonEncyclopedia, code)) {
 			menu_windows_.help.toggle();
@@ -989,10 +1114,18 @@ bool EditorInteractive::handle_key(bool const down, SDL_Keysym const code) {
 			tool_windows_.players.toggle();
 			return true;
 		}
+		if (matches_shortcut(KeyboardShortcut::kEditorShowhideHeightHeatMap, code)) {
+			toggle_height_heat_map();
+			return true;
+		}
 		if (matches_shortcut(KeyboardShortcut::kEditorShowhideGrid, code)) {
 			toggle_grid();
 			return true;
 		}
+		if (matches_shortcut(KeyboardShortcut::kEditorShowhideOceans, code)) {
+			toggle_oceans();
+			return true;
+		}
 		if (matches_shortcut(KeyboardShortcut::kEditorShowhideCritters, code)) {
 			toggle_bobs();
 			return true;
@@ -1073,7 +1206,7 @@ bool EditorInteractive::handle_key(bool const down, SDL_Keysym const code) {
 		}
 	}
 
-	return InteractiveBase::handle_key(down, code);
+	return false;
 }
 
 bool EditorInteractive::handle_mousewheel(int32_t x, int32_t y, uint16_t modstate) {
@@ -1087,7 +1220,7 @@ bool EditorInteractive::handle_mousewheel(int32_t x, int32_t y, uint16_t modstat
 	}
 
 	set_sel_radius_and_update_menu(
-	   std::max(0, std::min(static_cast<int32_t>(get_sel_radius()) + change_size, MAX_TOOL_AREA)),
+	   std::max(0, std::min(static_cast<int32_t>(get_sel_radius()) + change_size, kMaxToolArea)),
 	   std::max(0, std::min(static_cast<int32_t>(get_sel_gap_percent()) + change_gap, 100)));
 	return true;
 }
@@ -1131,6 +1264,9 @@ bool EditorInteractive::run_editor(UI::Panel* error_message_parent,
 		        "  FATAL EXCEPTION in editor: %s\n"
 		        "##############################\n",
 		        e.what());
+		if (g_fail_on_errors) {
+			abort();
+		}
 		if (error_message_parent == nullptr) {
 			return false;
 		}
@@ -1297,6 +1433,7 @@ void EditorInteractive::map_changed(const MapWas& action) {
 
 		set_need_save(false);
 		show_buildhelp(true);
+		ocean_overlays_.reset();
 
 		// Close all windows.
 		for (Panel* child = get_first_child(); child != nullptr; child = child->get_next_sibling()) {
@@ -1317,8 +1454,12 @@ void EditorInteractive::map_changed(const MapWas& action) {
 		break;
 
 	case MapWas::kResized:
+		ocean_overlays_.reset();
 		resize_minimap();
 		break;
+
+	default:
+		NEVER_HERE();
 	}
 }
 
@@ -1393,11 +1534,9 @@ UI::UniqueWindow::Registry& EditorInteractive::get_registry_for_window(WindowID
 		return tool_windows_.resources;
 	case WindowID::Resize:
 		return tool_windows_.resizemap;
-	case WindowID::Unset:
-		break;
+	default:
+		NEVER_HERE();
 	}
-
-	NEVER_HERE();
 }
 
 void EditorInteractive::set_sel_radius(uint32_t radius, uint16_t gap) {
@@ -1426,16 +1565,28 @@ void EditorInteractive::publish_map() {
 
 	/* Ask for confirmation. */
 	if ((SDL_GetModState() & KMOD_CTRL) == 0) {
-		UI::WLMessageBox w(
-		   this, UI::WindowStyle::kWui, _("Publish Map"),
+		std::string temp = richtext_escape(
 		   format(_("Are you certain that you want to publish this map online?\n\n"
 		            "Name: %1$s\n"
 		            "Internal name: %2$s\n"
 		            "Author: %3$s\n"
 		            "Description: %4$s\n"
-		            "Hint: %5$s\n"),
+		            "Hint: %5$s\n\n"),
 		          map_name, sanitized_name + kAddOnExtension, egbase().map().get_author(),
-		          egbase().map().get_description(), egbase().map().get_hint()),
+		          egbase().map().get_description(), egbase().map().get_hint()));
+		newlines_to_richtext(temp);
+
+		UI::WLMessageBox w(
+		   this, UI::WindowStyle::kWui, _("Publish Map"),
+		   format("<rt><p>%s</p><p>%s</p></rt>",
+		          g_style_manager->font_style(UI::FontStyle::kWuiLabel).as_font_tag(temp),
+		          g_style_manager->font_style(UI::FontStyle::kWuiInfoPanelParagraph)
+		             .as_font_tag(
+		                format(_("By uploading, you agree to publish the map under the terms of the "
+		                         "GNU General Public License (GPL) version 2 or any later version "
+		                         "(the same license under which Widelands itself is distributed). "
+		                         "For more information, see %s."),
+		                       as_url_hyperlink("https://www.gnu.org/licenses/")))),
 		   UI::WLMessageBox::MBoxType::kOkCancel);
 		if (w.run<UI::Panel::Returncodes>() != UI::Panel::Returncodes::kOk) {
 			return;
