@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2024 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,7 @@
 
 #include "editor/map_generator.h"
 
+#include <algorithm>
 #include <memory>
 
 #include "base/log.h"
@@ -95,14 +96,14 @@ void MapGenerator::generate_bobs(std::unique_ptr<uint32_t[]> const* random_bobs,
 
 	// Set bob according to bob area
 
-	if (set_immovable && ((num = bobCategory->num_immovables()) != 0u)) {
+	if (num = bobCategory->num_immovables(); set_immovable && num != 0u) {
 		egbase_.create_immovable_with_name(
 		   fc, bobCategory->get_immovable(static_cast<size_t>(rng.rand() / (kMaxElevation / num))),
 		   nullptr /* owner */, nullptr /* former_building_descr */
 		);
 	}
 
-	if (set_moveable && ((num = bobCategory->num_critters()) != 0u)) {
+	if (num = bobCategory->num_critters(); set_moveable && num != 0u) {
 		egbase_.create_critter(fc, egbase_.descriptions().critter_index(bobCategory->get_critter(
 		                              static_cast<size_t>(rng.rand() / (kMaxElevation / num)))));
 	}
@@ -214,8 +215,8 @@ uint8_t MapGenerator::make_node_elevation(double const elevation, const Coords&
 
 	//  Handle Map Border in island mode
 	if (map_info_.islandMode) {
-		int32_t const border_dist = std::min(
-		   std::min<int16_t>(c.x, map_info_.w - c.x), std::min<int16_t>(c.y, map_info_.h - c.y));
+		int32_t const border_dist = std::min({c.x, static_cast<int16_t>(map_info_.w - c.x), c.y,
+		                                      static_cast<int16_t>(map_info_.h - c.y)});
 		if (border_dist <= kIslandBorder) {
 			res_h = static_cast<uint8_t>(static_cast<double>(res_h) * border_dist /
 			                             static_cast<double>(kIslandBorder));
