@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -32,7 +32,12 @@ int32_t EditorDeleteImmovableTool::handle_click_impl(const Widelands::NodeAndTri
                                                      Widelands::Map* map) {
 	Widelands::MapRegion<Widelands::Area<Widelands::FCoords>> mr(
 	   *map, Widelands::Area<Widelands::FCoords>(map->get_fcoords(center.node), args->sel_radius));
+	auto gap_it = args->selection_gaps.cbegin();
 	do {
+		if (*gap_it++) {
+			continue;
+		}
+
 		if (upcast(Widelands::Immovable, immovable, mr.location().field->get_immovable())) {
 			args->old_immovable_types.push_back(immovable->descr().name());
 			immovable->remove(parent_.egbase());  //  Delete no buildings or stuff.
