@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,8 +29,7 @@ struct EditorIncreaseHeightTool : public EditorTool {
 	                         EditorSetHeightTool& the_set_tool)
 	   : EditorTool(parent, the_decrease_tool, the_set_tool),
 	     decrease_tool_(the_decrease_tool),
-	     set_tool_(the_set_tool),
-	     change_by_(1) {
+	     set_tool_(the_set_tool) {
 	}
 
 	int32_t handle_click_impl(const Widelands::NodeAndTriangle<>& center,
@@ -43,21 +42,21 @@ struct EditorIncreaseHeightTool : public EditorTool {
 
 	EditorActionArgs format_args_impl() override;
 
-	const Image* get_sel_impl() const override {
+	[[nodiscard]] const Image* get_sel_impl() const override {
 		return g_image_cache->get("images/wui/editor/fsel_editor_increase_height.png");
 	}
 
-	int32_t get_change_by() const {
+	[[nodiscard]] int32_t get_change_by() const {
 		return change_by_;
 	}
 	void set_change_by(const int32_t n) {
 		change_by_ = n;
 	}
 
-	EditorDecreaseHeightTool& decrease_tool() const {
+	[[nodiscard]] EditorDecreaseHeightTool& decrease_tool() const {
 		return decrease_tool_;
 	}
-	EditorSetHeightTool& set_tool() const {
+	[[nodiscard]] EditorSetHeightTool& set_tool() const {
 		return set_tool_;
 	}
 
@@ -79,7 +78,7 @@ struct EditorIncreaseHeightTool : public EditorTool {
 private:
 	EditorDecreaseHeightTool& decrease_tool_;
 	EditorSetHeightTool& set_tool_;
-	int32_t change_by_;
+	int32_t change_by_{1};
 };
 
 #endif  // end of include guard: WL_EDITOR_TOOLS_INCREASE_HEIGHT_TOOL_H
