@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,8 +26,8 @@
 #include "editor/ui_menus/main_menu_map_options.h"
 #include "logic/note_map_options.h"
 #include "ui_basic/button.h"
-#include "ui_basic/editbox.h"
 #include "ui_basic/textarea.h"
+#include "ui_basic/textinput.h"
 
 /**
  * Choose a filename and save your brand new created map
@@ -39,7 +39,7 @@ struct MainMenuSaveMap : public MainMenuLoadOrSaveMap {
 
 protected:
 	// Sets the current dir and updates labels.
-	void set_current_directory(const std::string& filename) override;
+	void set_current_directory(const std::vector<std::string>& filenames) override;
 	void layout() override;
 
 private:
