@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,14 +31,20 @@
 
 namespace Widelands {
 
-constexpr uint16_t kCurrentPacketVersion = 30;
+/* Changelog:
+ * Version 30: Release 1.1
+ * Version 31: Added warehouse names.
+ * Version 32: Added fleet statistics (in Game::read_statistics),
+ *             naval attack stats, and detected port spaces.
+ */
+constexpr uint16_t kCurrentPacketVersion = 32;
 
 void GamePlayerInfoPacket::read(FileSystem& fs, Game& game, MapObjectLoader* /* mol */) {
 	try {
 		FileRead fr;
 		fr.open(fs, "binary/player_info");
 		uint16_t const packet_version = fr.unsigned_16();
-		if (packet_version <= kCurrentPacketVersion && packet_version >= 22) {
+		if (packet_version <= kCurrentPacketVersion && packet_version >= 30) {
 			uint32_t const max_players = fr.unsigned_16();
 			for (uint32_t i = 1; i < max_players + 1; ++i) {
 				game.remove_player(i);
@@ -51,13 +57,9 @@ void GamePlayerInfoPacket::read(FileSystem& fs, Game& game, MapObjectLoader* /*
 						   "player number (%i) is out of range (1 .. %u)", plnum, kMaxPlayers);
 					}
 
-					// TODO(Nordfriese): Savegame compatibility, remove after v1.0
-					const uint8_t playercolor_r =
-					   packet_version >= 28 ? fr.unsigned_8() : kPlayerColors[i - 1].r;
-					const uint8_t playercolor_g =
-					   packet_version >= 28 ? fr.unsigned_8() : kPlayerColors[i - 1].g;
-					const uint8_t playercolor_b =
-					   packet_version >= 28 ? fr.unsigned_8() : kPlayerColors[i - 1].b;
+					const uint8_t playercolor_r = fr.unsigned_8();
+					const uint8_t playercolor_g = fr.unsigned_8();
+					const uint8_t playercolor_b = fr.unsigned_8();
 
 					Widelands::TeamNumber team = fr.unsigned_8();
 					char const* const tribe_name = fr.c_string();
@@ -70,21 +72,20 @@ void GamePlayerInfoPacket::read(FileSystem& fs, Game& game, MapObjectLoader* /*
 					player->set_see_all(see_all);
 
 					player->set_ai(fr.c_string());
+					player->set_random_tribe(fr.unsigned_8() != 0u);
 
-					if (packet_version >= 30) {
-						player->set_random_tribe(fr.unsigned_8() != 0u);
-					}
-
-					if (packet_version >= 23) {
-						player->forbid_attack_.clear();
-						uint8_t size = fr.unsigned_8();
-						for (uint8_t j = 0; j < size; ++j) {
-							player->forbid_attack_.emplace(fr.unsigned_8());
-						}
+					player->forbid_attack_.clear();
+					uint8_t size = fr.unsigned_8();
+					for (uint8_t j = 0; j < size; ++j) {
+						player->forbid_attack_.emplace(fr.unsigned_8());
 					}
 
 					player->read_statistics(fr, packet_version);
 					player->read_remaining_shipnames(fr);
+					if (packet_version >= 31) {
+						// TODO(Nordfriese): Savegame compatibility, remove after v1.2
+						player->read_remaining_warehousenames(fr);
+					}
 
 					player->casualties_ = fr.unsigned_32();
 					player->kills_ = fr.unsigned_32();
@@ -92,23 +93,35 @@ void GamePlayerInfoPacket::read(FileSystem& fs, Game& game, MapObjectLoader* /*
 					player->msites_defeated_ = fr.unsigned_32();
 					player->civil_blds_lost_ = fr.unsigned_32();
 					player->civil_blds_defeated_ = fr.unsigned_32();
+					player->naval_victories_ = packet_version >= 32 ? fr.unsigned_32() : 0;
+					player->naval_losses_ = packet_version >= 32 ? fr.unsigned_32() : 0;
 
-					// TODO(Nordfriese): Savegame compatibility, remove after v1.0
-					if (packet_version >= 24) {
-						for (size_t j = fr.unsigned_32(); j > 0; --j) {
-							player->muted_building_types_.insert(fr.unsigned_32());
+					for (uint32_t j = packet_version >= 32 ? fr.unsigned_32() : 0; j > 0; --j) {
+						std::unique_ptr<DetectedPortSpace> dps(new DetectedPortSpace(game));
+						dps->set_serial(game, fr.unsigned_32());
+						dps->coords.x = fr.signed_16();
+						dps->coords.y = fr.signed_16();
+						for (uint32_t k = fr.unsigned_32(); k > 0; --k) {
+							int16_t x = fr.signed_16();
+							int16_t y = fr.signed_16();
+							dps->dockpoints.emplace_back(x, y);
 						}
+						dps->owner = fr.unsigned_8();
+						dps->time_discovered = Time(fr);
+						dps->discovering_ship = fr.string();
+						dps->nearest_portdock = fr.string();
+						dps->direction_from_portdock = static_cast<CompassDir>(fr.unsigned_8());
+						dps->distance_to_portdock = fr.unsigned_32();
+						player->detected_port_spaces_.push_back(std::move(dps));
 					}
-					if (packet_version >= 25) {
-						player->is_picking_custom_starting_position_ = (fr.unsigned_8() != 0u);
-						player->initialization_index_ = fr.unsigned_8();
-					}
-					if (packet_version >= 26) {
-						player->allow_additional_expedition_items_ = (fr.unsigned_8() != 0u);
-					}
-					if (packet_version >= 27) {
-						player->hidden_from_general_statistics_ = (fr.unsigned_8() != 0u);
+
+					for (size_t j = fr.unsigned_32(); j > 0; --j) {
+						player->muted_building_types_.insert(fr.unsigned_32());
 					}
+					player->is_picking_custom_starting_position_ = (fr.unsigned_8() != 0u);
+					player->initialization_index_ = fr.unsigned_8();
+					player->allow_additional_expedition_items_ = (fr.unsigned_8() != 0u);
+					player->hidden_from_general_statistics_ = (fr.unsigned_8() != 0u);
 				}
 			}
 
@@ -121,10 +134,10 @@ void GamePlayerInfoPacket::read(FileSystem& fs, Game& game, MapObjectLoader* /*
 				status.result = static_cast<PlayerEndResult>(fr.unsigned_8());
 				status.time = Time(fr);
 				status.info = fr.c_string();
-				manager->set_player_end_status(status);
+				manager->add_player_end_status(status, true);
 			}
 
-			game.read_statistics(fr);
+			game.read_statistics(fr, packet_version);
 		} else {
 			throw UnhandledVersionError("GamePlayerInfoPacket", packet_version, kCurrentPacketVersion);
 		}
@@ -170,12 +183,33 @@ void GamePlayerInfoPacket::write(FileSystem& fs, Game& game, MapObjectSaver* /*
 
 		plr->write_statistics(fw);
 		plr->write_remaining_shipnames(fw);
+		plr->write_remaining_warehousenames(fw);
 		fw.unsigned_32(plr->casualties());
 		fw.unsigned_32(plr->kills());
 		fw.unsigned_32(plr->msites_lost());
 		fw.unsigned_32(plr->msites_defeated());
 		fw.unsigned_32(plr->civil_blds_lost());
 		fw.unsigned_32(plr->civil_blds_defeated());
+		fw.unsigned_32(plr->naval_victories());
+		fw.unsigned_32(plr->naval_losses());
+
+		fw.unsigned_32(plr->detected_port_spaces_.size());
+		for (const auto& dps : plr->detected_port_spaces_) {
+			fw.unsigned_32(dps->serial);
+			fw.signed_16(dps->coords.x);
+			fw.signed_16(dps->coords.y);
+			fw.unsigned_32(dps->dockpoints.size());
+			for (const Coords& c : dps->dockpoints) {
+				fw.signed_16(c.x);
+				fw.signed_16(c.y);
+			}
+			fw.unsigned_8(dps->owner);
+			dps->time_discovered.save(fw);
+			fw.string(dps->discovering_ship);
+			fw.string(dps->nearest_portdock);
+			fw.unsigned_8(static_cast<uint8_t>(dps->direction_from_portdock));
+			fw.unsigned_32(dps->distance_to_portdock);
+		}
 
 		fw.unsigned_32(plr->muted_building_types_.size());
 		for (const DescriptionIndex& di : plr->muted_building_types_) {
@@ -191,10 +225,11 @@ void GamePlayerInfoPacket::write(FileSystem& fs, Game& game, MapObjectSaver* /*
 	}
 
 	// Result screen
-	const std::vector<PlayerEndStatus>& end_status_list =
-	   game.player_manager()->get_players_end_status();
-	fw.unsigned_8(end_status_list.size());
-	for (const PlayerEndStatus& status : end_status_list) {
+	const std::map<Widelands::PlayerNumber, Widelands::PlayerEndStatus>& end_status_map =
+	   game.player_manager()->get_all_players_end_status();
+	fw.unsigned_8(end_status_map.size());
+	for (const auto& it : end_status_map) {
+		const Widelands::PlayerEndStatus& status = it.second;
 		fw.unsigned_8(status.player);
 		fw.unsigned_8(static_cast<uint8_t>(status.result));
 		status.time.save(fw);
