@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2024 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -97,8 +97,8 @@ void GamePreloadPacket::read(FileSystem& fs, Game& /* game */, MapObjectLoader*
 					   "Ignoring malformed add-on requirement substring '%s'\n", substring.c_str());
 				} else {
 					const std::string version = substring.substr(colonpos + 1);
-					required_addons_.push_back(std::make_pair(
-					   substring.substr(0, colonpos), AddOns::string_to_version(version)));
+					required_addons_.emplace_back(
+					   substring.substr(0, colonpos), AddOns::string_to_version(version));
 				}
 				if (commapos == std::string::npos) {
 					break;
