@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2024 by the Widelands Development Team
+ * Copyright (C) 2006-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,8 @@
 
 #include "graphic/align.h"
 
+#include "base/wexception.h"
+
 namespace UI {
 
 /**
@@ -59,6 +61,8 @@ Align mirror_alignment(Align alignment, bool is_rtl) {
 			break;
 		case Align::kCenter:
 			break;
+		default:
+			NEVER_HERE();
 		}
 	}
 	return alignment;
