@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2022 by the Widelands Development Team
+ * Copyright (C) 2010-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -33,7 +33,7 @@ void draw_border_markers(const FieldsToDraw::Field& field,
 	assert(field.owner != nullptr);
 
 	uint32_t const anim_idx = field.owner->tribe().frontier_animation();
-	if (field.seeing != Widelands::VisibleState::kUnexplored) {
+	if (field.seeing != Widelands::VisibleState::kUnexplored && !field.obscured_by_slope) {
 		dst->blit_animation(field.rendertarget_pixel, field.fcoords, scale, anim_idx, Time(0),
 		                    &field.owner->get_playercolor());
 	}
@@ -41,7 +41,8 @@ void draw_border_markers(const FieldsToDraw::Field& field,
 	                       fields_to_draw.at(field.brn_index)}) {
 		if ((field.seeing != Widelands::VisibleState::kUnexplored ||
 		     nf.seeing != Widelands::VisibleState::kUnexplored) &&
-		    nf.is_border && (field.owner == nf.owner || nf.owner == nullptr)) {
+		    nf.is_border && (field.owner == nf.owner || nf.owner == nullptr) &&
+		    !field.obscured_by_slope && !nf.obscured_by_slope) {
 			dst->blit_animation(middle(field.rendertarget_pixel, nf.rendertarget_pixel),
 			                    Widelands::Coords::null(), scale, anim_idx, Time(0),
 			                    &field.owner->get_playercolor());
@@ -54,6 +55,7 @@ void draw_terrain(uint32_t gametime,
                   const FieldsToDraw& fields_to_draw,
                   const float scale,
                   const Workareas& workarea,
+                  bool height_heat_map,
                   bool grid,
                   const Widelands::Player* player,
                   RenderTarget* dst) {
@@ -76,11 +78,12 @@ void draw_terrain(uint32_t gametime,
 	i.terrain_arguments.fields_to_draw = &fields_to_draw;
 	i.terrain_arguments.scale = scale;
 	i.terrain_arguments.player = player;
+	i.terrain_arguments.height_heat_map = height_heat_map;
 	RenderQueue::instance().enqueue(i);
 
-	// Enqueue the drawing of the dither layer.
-	i.program_id = RenderQueue::Program::kTerrainDither;
+	// Enqueue the drawing of the dither layer or height heat map layer.
 	i.blend_mode = BlendMode::UseAlpha;
+	i.program_id = RenderQueue::Program::kTerrainDitherOrHeightHeatMap;
 	RenderQueue::instance().enqueue(i);
 
 	if (!workarea.empty()) {
