@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2024 by the Widelands Development Team
+ * Copyright (C) 2006-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -41,8 +41,10 @@ DitherProgram::DitherProgram() {
 	u_terrain_texture_ = glGetUniformLocation(gl_program_.object(), "u_terrain_texture");
 	u_texture_dimensions_ = glGetUniformLocation(gl_program_.object(), "u_texture_dimensions");
 	u_z_value_ = glGetUniformLocation(gl_program_.object(), "u_z_value");
+}
 
-	dither_mask_.reset(new Texture(load_image_as_sdl_surface("world/pics/edge.png", g_fs), true));
+void DitherProgram::set_dither_mask(const std::string& filepath) {
+	dither_mask_.reset(new Texture(load_image_as_sdl_surface(filepath, g_fs), true));
 
 	Gl::State::instance().bind(GL_TEXTURE0, dither_mask_->blit_data().texture_id);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, static_cast<GLint>(GL_CLAMP_TO_EDGE));
@@ -78,6 +80,8 @@ void DitherProgram::add_vertex(const FieldsToDraw::Field& field,
 		back.dither_texture_x = 0.5;
 		back.dither_texture_y = 0.;
 		break;
+	default:
+		NEVER_HERE();
 	}
 }
 
@@ -117,6 +121,8 @@ void DitherProgram::maybe_add_dithering_triangle(
 }
 
 void DitherProgram::gl_draw(int gl_texture, float texture_w, float texture_h, const float z_value) {
+	assert(dither_mask_ != nullptr);
+
 	glUseProgram(gl_program_.object());
 
 	auto& gl_state = Gl::State::instance();
