@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2022 by the Widelands Development Team
+ * Copyright (C) 2006-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,27 +19,27 @@
 #ifndef WL_GRAPHIC_TEXT_TEXTSTREAM_H
 #define WL_GRAPHIC_TEXT_TEXTSTREAM_H
 
+#include <cstdint>
 #include <string>
 
 namespace RT {
 
 class TextStream {
 public:
-	explicit TextStream(const std::string& text)
-	   : text_(text), line_(1), col_(0), pos_(0), end_(text.size()) {
+	explicit TextStream(const std::string& text) : text_(text), end_(text.size()) {
 	}
 
-	size_t line() const {
+	[[nodiscard]] size_t line() const {
 		return line_;
 	}
-	size_t col() const {
+	[[nodiscard]] size_t col() const {
 		return col_;
 	}
-	size_t pos() const {
+	[[nodiscard]] size_t pos() const {
 		return pos_;
 	}
 
-	std::string peek(size_t, size_t = -1) const;
+	[[nodiscard]] std::string peek(size_t n, size_t at = std::string::npos) const;
 	void expect(std::string, bool = true);
 
 	std::string till_any(std::string);
@@ -57,8 +57,10 @@ public:
 private:
 	void consume(size_t);
 	std::string text_;
-	uint32_t line_, col_;
-	size_t pos_, end_;
+	uint32_t line_{1U};
+	uint32_t col_{0U};
+	size_t pos_{0U};
+	size_t end_;
 };
 }  // namespace RT
 #endif  // end of include guard: WL_GRAPHIC_TEXT_TEXTSTREAM_H
