@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -33,12 +33,12 @@ public:
 	explicit ZipFilesystem(const std::string&);
 	~ZipFilesystem() override = default;
 
-	bool is_writable() const override;
+	[[nodiscard]] bool is_writable() const override;
 
-	FilenameSet list_directory(const std::string& path) const override;
+	[[nodiscard]] FilenameSet list_directory(const std::string& path) const override;
 
-	bool is_directory(const std::string& path) const override;
-	bool file_exists(const std::string& path) const override;
+	[[nodiscard]] bool is_directory(const std::string& path) const override;
+	bool file_exists(const std::string& path) const override;  // NOLINT not nodicard
 
 	void* load(const std::string& fname, size_t& length) override;
 
@@ -79,7 +79,7 @@ private:
 		std::string strip_basename(const std::string& filename);
 
 		// Full path to the zip file.
-		const std::string& path() const;
+		[[nodiscard]] const std::string& path() const;
 
 		// Closes the file if it is open, reopens it for writing, and
 		// returns the minizip handle.
@@ -99,7 +99,7 @@ private:
 		// Closes 'path_' if it is opened.
 		void close();
 
-		State state_;
+		State state_{State::kIdle};
 
 		// E.g. "path/to/filename.zip"
 		std::string path_;
@@ -113,15 +113,15 @@ private:
 		std::string common_prefix_;
 
 		// File handles for zipping and unzipping.
-		zipFile write_handle_;
-		unzFile read_handle_;
+		zipFile write_handle_{nullptr};
+		unzFile read_handle_{nullptr};
 	};
 
 	struct ZipStreamRead : StreamRead {
 		explicit ZipStreamRead(const std::shared_ptr<ZipFile>& shared_data);
 		~ZipStreamRead() override = default;
 		size_t data(void* data, size_t bufsize) override;
-		bool end_of_file() const override;
+		[[nodiscard]] bool end_of_file() const override;
 
 	private:
 		std::shared_ptr<ZipFile> zip_file_;
@@ -130,7 +130,7 @@ private:
 	struct ZipStreamWrite : StreamWrite {
 		explicit ZipStreamWrite(const std::shared_ptr<ZipFile>& shared_data);
 		~ZipStreamWrite() override = default;
-		void data(const void* const data, size_t size) override;
+		void data(const void* data, size_t size) override;
 
 	private:
 		std::shared_ptr<ZipFile> zip_file_;
