@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2024 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -76,12 +76,13 @@ For general information about the format, see :ref:`map_object_programs_syntax`.
 Available actions are:
 
 - `animate`_
+- `playsound`_
+- `script`_
 - `transform`_
 - `grow`_
 - `remove`_
 - `seed`_
 - `construct`_
-- `playsound`_
 */
 
 ImmovableProgram::ImmovableProgram(const std::string& init_name,
@@ -111,6 +112,8 @@ ImmovableProgram::ImmovableProgram(const std::string& init_name,
 				   std::unique_ptr<Action>(new ActSeed(parseinput.arguments, immovable)));
 			} else if (parseinput.name == "playsound") {
 				actions_.push_back(std::unique_ptr<Action>(new ActPlaySound(parseinput.arguments)));
+			} else if (parseinput.name == "script") {
+				actions_.push_back(std::unique_ptr<Action>(new ActRunScript(parseinput.arguments)));
 			} else if (parseinput.name == "construct") {
 				actions_.push_back(
 				   std::unique_ptr<Action>(new ActConstruct(parseinput.arguments, immovable)));
@@ -170,6 +173,21 @@ void ImmovableProgram::ActPlaySound::execute(Game& game, Immovable& immovable) c
 
 /* RST
 
+script
+------
+Runs a Lua function. See :ref:`map_object_programs_script`.
+*/
+ImmovableProgram::ActRunScript::ActRunScript(const std::vector<std::string>& arguments)
+   : parameters(MapObjectProgram::parse_act_script(arguments)) {
+}
+
+void ImmovableProgram::ActRunScript::execute(Game& game, Immovable& immovable) const {
+	MapObjectProgram::do_run_script(game.lua(), &immovable, parameters.function);
+	return immovable.program_step(game);
+}
+
+/* RST
+
 transform
 ---------
 
