@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2024 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -38,6 +38,7 @@
 #include "logic/map_objects/descriptions.h"
 #include "logic/map_objects/immovable.h"
 #include "logic/map_objects/tribes/constructionsite.h"
+#include "logic/map_objects/tribes/market.h"
 #include "logic/map_objects/tribes/productionsite.h"
 #include "logic/map_objects/tribes/tribe_descr.h"
 #include "logic/map_objects/tribes/worker.h"
@@ -52,8 +53,9 @@ static const Duration kBuildingLeaveInterval = Duration(1000);
 BuildingDescr::BuildingDescr(const std::string& init_descname,
                              const MapObjectType init_type,
                              const LuaTable& table,
+                             const std::vector<std::string>& attribs,
                              Descriptions& descriptions)
-   : MapObjectDescr(init_type, table.get_string("name"), init_descname, table),
+   : MapObjectDescr(init_type, table.get_string("name"), init_descname, table, attribs),
      descriptions_(descriptions),
      buildable_(table.has_key("buildcost")),
      can_be_dismantled_(table.has_key("return_on_dismantle")),
@@ -215,7 +217,7 @@ Building& BuildingDescr::create(EditorGameBase& egbase,
 	if (immovable != INVALID_INDEX) {
 		// Remember that we're building on top of an immovable so we can put it back if the building
 		// gets removed
-		b.old_buildings_.push_back(std::make_pair(immovable, false));
+		b.old_buildings_.emplace_back(immovable, false);
 	}
 	for (const auto& pair : former_buildings) {
 		b.old_buildings_.push_back(pair);
@@ -361,6 +363,9 @@ bool Building::get_passable() const {
 Flag& Building::base_flag() {
 	return *flag_;
 }
+const Flag& Building::base_flag() const {
+	return *flag_;
+}
 
 /**
  * \return a bitfield of commands the owning player can issue for this building.
@@ -544,30 +549,71 @@ std::string Building::info_string(const InfoStringFormat& format) {
 	std::string result;
 	switch (format) {
 	case InfoStringFormat::kCensus:
-		if (upcast(ConstructionSite const, constructionsite, this)) {
+		switch (descr().type()) {
+		case MapObjectType::CONSTRUCTIONSITE: {
+			upcast(ConstructionSite const, constructionsite, this);
 			result = constructionsite->building().descname();
-		} else {
+		} break;
+
+		case MapObjectType::WAREHOUSE: {
+			upcast(Warehouse const, warehouse, this);
+			result = warehouse->warehouse_census_string();
+		} break;
+
+		case MapObjectType::MARKET: {
+			upcast(Market const, market, this);
+			result = market->market_census_string();
+		} break;
+
+		default:
 			result = descr().descname();
+			break;
 		}
+
 		break;
+
 	case InfoStringFormat::kStatistics:
 		result = update_and_get_statistics_string();
 		break;
+
 	case InfoStringFormat::kTooltip:
 		if (upcast(ProductionSite const, productionsite, this)) {
 			result = productionsite->production_result();
 		}
 		break;
+
+	default:
+		NEVER_HERE();
 	}
+
 	return result;
 }
 
+std::string Building::named_building_census_string(const std::string& icon_fmt,
+                                                   std::string name) const {
+	if (name.empty()) {
+		name = descr().descname();
+		// See explanation in set_warehouse_name().
+		// Needed because of e.g. Temple of Vesta in emp04
+		replace_all(name, " ", " ");
+	}
+	return format(icon_fmt, richtext_escape(name));
+}
+
 InputQueue& Building::inputqueue(DescriptionIndex const wi,
                                  WareWorker const /* type */,
-                                 const Request* /* req */) {
+                                 const Request* /* req */,
+                                 uint32_t /* disambiguator_id */) {
 	throw wexception("%s (%u) has no InputQueue for %u", descr().name().c_str(), serial(), wi);
 }
 
+bool Building::can_change_max_fill(DescriptionIndex /* di */,
+                                   WareWorker /* ww */,
+                                   const Request* /* r */,
+                                   uint32_t /* disambiguator_id */) {
+	return true;
+}
+
 /*
 ===============
 This function is called by workers in the buildingwork task.
@@ -611,7 +657,7 @@ bool Building::leave_check_and_wait(Game& game, Worker& w) {
 		schedule_act(game, leave_time_ - time);
 	}
 
-	leave_queue_.push_back(&w);
+	leave_queue_.emplace_back(&w);
 	return false;
 }
 
@@ -700,7 +746,7 @@ void Building::draw(const Time& gametime,
 			                    &get_owner()->get_playercolor());
 		} else {
 			dst->blit_animation(point_on_dst, coords, scale, was_immovable_->main_animation(), t,
-			                    nullptr, kBuildingSilhouetteOpacity);
+			                    nullptr, kImmovableSilhouetteOpacity);
 		}
 	}
 
@@ -708,7 +754,7 @@ void Building::draw(const Time& gametime,
 		dst->blit_animation(point_on_dst, coords, scale, anim_, t, &get_owner()->get_playercolor());
 	} else {
 		dst->blit_animation(
-		   point_on_dst, coords, scale, anim_, t, nullptr, kBuildingSilhouetteOpacity);
+		   point_on_dst, coords, scale, anim_, t, nullptr, kImmovableSilhouetteOpacity);
 	}
 
 	//  door animation?
@@ -733,10 +779,15 @@ void Building::draw_info(const InfoToDraw info_to_draw,
 	             point_on_dst, scale, dst);
 }
 
+uint32_t Building::get_priority_disambiguator_id(const Request* /*req*/) const {
+	return 0;
+}
+
 const WarePriority& Building::get_priority(const WareWorker type,
-                                           const DescriptionIndex ware_index) const {
+                                           const DescriptionIndex ware_index,
+                                           const uint32_t disambiguator_id) const {
 	if (type == wwWARE) {
-		const auto it = ware_priorities_.find(ware_index);
+		const auto it = ware_priorities_.find({ware_index, disambiguator_id});
 		if (it != ware_priorities_.end()) {
 			return it->second;
 		}
@@ -750,13 +801,15 @@ const WarePriority& Building::get_priority(const WareWorker type,
  */
 void Building::set_priority(const WareWorker type,
                             const DescriptionIndex ware_index,
-                            const WarePriority& new_priority) {
+                            const WarePriority& new_priority,
+                            const uint32_t disambiguator_id) {
 	if (type == wwWARE) {
 		// WarePriority is not default-constructible, so no [] access :(
-		if (ware_priorities_.count(ware_index) != 0u) {
-			ware_priorities_.at(ware_index) = new_priority;
+		std::pair<DescriptionIndex, uint32_t> key(ware_index, disambiguator_id);
+		if (ware_priorities_.count(key) != 0u) {
+			ware_priorities_.at(key) = new_priority;
 		} else {
-			ware_priorities_.emplace(ware_index, new_priority);
+			ware_priorities_.emplace(key, new_priority);
 		}
 	}
 }
@@ -772,9 +825,9 @@ void Building::log_general_info(const EditorGameBase& egbase) const {
 	      flag_->get_position().y);
 
 	molog(egbase.get_gametime(), "anim: %s\n", descr().get_animation_name(anim_).c_str());
-	molog(egbase.get_gametime(), "animstart: %i\n", animstart_.get());
+	molog(egbase.get_gametime(), "animstart: %u\n", animstart_.get());
 
-	molog(egbase.get_gametime(), "leave_time: %i\n", leave_time_.get());
+	molog(egbase.get_gametime(), "leave_time: %u\n", leave_time_.get());
 
 	molog(egbase.get_gametime(), "leave_queue.size(): %" PRIuS "\n", leave_queue_.size());
 	FORMAT_WARNINGS_OFF
