@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,6 +31,7 @@ class FileWrite;
 namespace Widelands {
 
 class Game;
+class MarketDescr;
 class MilitarySiteDescr;
 class ProductionSiteDescr;
 enum class StockPolicy;
@@ -42,8 +43,7 @@ struct BuildingSettings {
 	BuildingSettings(const std::string& name, const TribeDescr& tribe)
 	   : tribe_(tribe), descr_(name) {
 	}
-	virtual ~BuildingSettings() {
-	}
+	virtual ~BuildingSettings() = default;
 
 	static BuildingSettings* load(const Game&, const TribeDescr&, FileRead&);
 
@@ -62,8 +62,7 @@ private:
 
 struct ProductionsiteSettings : public BuildingSettings {
 	ProductionsiteSettings(const ProductionSiteDescr& descr, const TribeDescr& tribe);
-	~ProductionsiteSettings() override {
-	}
+	~ProductionsiteSettings() override = default;
 	void apply(const BuildingSettings&) override;
 
 	void save(const Game&, FileWrite&) const override;
@@ -76,13 +75,12 @@ struct ProductionsiteSettings : public BuildingSettings {
 	};
 	std::map<DescriptionIndex, InputQueueSetting> ware_queues;
 	std::map<DescriptionIndex, InputQueueSetting> worker_queues;
-	bool stopped;
+	bool stopped{false};
 };
 
 struct MilitarysiteSettings : public BuildingSettings {
 	MilitarysiteSettings(const MilitarySiteDescr&, const TribeDescr& tribe);
-	~MilitarysiteSettings() override {
-	}
+	~MilitarysiteSettings() override = default;
 	void apply(const BuildingSettings&) override;
 
 	void save(const Game&, FileWrite&) const override;
@@ -90,13 +88,12 @@ struct MilitarysiteSettings : public BuildingSettings {
 
 	const uint32_t max_capacity;
 	uint32_t desired_capacity;
-	bool prefer_heroes;
+	SoldierPreference soldier_preference;
 };
 
 struct TrainingsiteSettings : public ProductionsiteSettings {
 	TrainingsiteSettings(const TrainingSiteDescr&, const TribeDescr& tribe);
-	~TrainingsiteSettings() override {
-	}
+	~TrainingsiteSettings() override = default;
 	void apply(const BuildingSettings&) override;
 
 	void save(const Game&, FileWrite&) const override;
@@ -108,8 +105,7 @@ struct TrainingsiteSettings : public ProductionsiteSettings {
 
 struct WarehouseSettings : public BuildingSettings {
 	WarehouseSettings(const WarehouseDescr&, const TribeDescr& tribe);
-	~WarehouseSettings() override {
-	}
+	~WarehouseSettings() override = default;
 	void apply(const BuildingSettings&) override;
 
 	void save(const Game&, FileWrite&) const override;
@@ -118,7 +114,19 @@ struct WarehouseSettings : public BuildingSettings {
 	std::map<DescriptionIndex, StockPolicy> ware_preferences;
 	std::map<DescriptionIndex, StockPolicy> worker_preferences;
 	const bool launch_expedition_allowed;
-	bool launch_expedition;
+	bool launch_expedition{false};
+	uint32_t desired_capacity{0U};
+	const uint32_t max_garrison;
+	SoldierPreference soldier_preference{SoldierPreference::kAny};
+};
+
+struct MarketSettings : public BuildingSettings {
+	MarketSettings(const MarketDescr&, const TribeDescr& tribe);
+	~MarketSettings() override = default;
+	void apply(const BuildingSettings&) override;
+
+	void save(const Game&, FileWrite&) const override;
+	void read(const Game&, FileRead&) override;
 };
 
 }  // namespace Widelands
