@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,9 +24,6 @@
 #include "base/log.h"
 #include "base/macros.h"
 #include "economy/flag.h"
-#include "economy/request.h"
-#include "graphic/style_manager.h"
-#include "io/filesystem/layered_filesystem.h"
 #include "logic/editor_game_base.h"
 #include "logic/game.h"
 #include "logic/map_objects/findbob.h"
@@ -37,10 +34,6 @@
 #include "logic/message_queue.h"
 #include "logic/player.h"
 
-namespace {
-constexpr size_t kNoOfStatisticsStringCases = 4;
-}  // namespace
-
 namespace Widelands {
 
 std::vector<Soldier*> MilitarySite::SoldierControl::present_soldiers() const {
@@ -67,6 +60,18 @@ std::vector<Soldier*> MilitarySite::SoldierControl::stationed_soldiers() const {
 	return soldiers;
 }
 
+std::vector<Soldier*> MilitarySite::SoldierControl::associated_soldiers() const {
+	std::vector<Soldier*> soldiers = stationed_soldiers();
+	if (military_site_->soldier_request_manager_.get_request() != nullptr) {
+		for (const Transfer* t :
+		     military_site_->soldier_request_manager_.get_request()->get_transfers()) {
+			Soldier& s = dynamic_cast<Soldier&>(*t->get_worker());
+			soldiers.push_back(&s);
+		}
+	}
+	return soldiers;
+}
+
 Quantity MilitarySite::SoldierControl::min_soldier_capacity() const {
 	return 1;
 }
@@ -79,9 +84,9 @@ Quantity MilitarySite::SoldierControl::soldier_capacity() const {
 	return military_site_->capacity_;
 }
 
-void MilitarySite::SoldierControl::set_soldier_capacity(uint32_t const capacity) {
-	assert(min_soldier_capacity() <= capacity);
-	assert(capacity <= max_soldier_capacity());
+void MilitarySite::SoldierControl::set_soldier_capacity(uint32_t capacity) {
+	capacity = std::max(capacity, min_soldier_capacity());
+	capacity = std::min(capacity, max_soldier_capacity());
 	if (military_site_->capacity_ != capacity) {
 		military_site_->capacity_ = capacity;
 		military_site_->update_soldier_request();
@@ -301,11 +306,9 @@ AttackTarget::AttackResult MilitarySite::AttackTarget::attack(Soldier* enemy) co
  */
 MilitarySiteDescr::MilitarySiteDescr(const std::string& init_descname,
                                      const LuaTable& table,
+                                     const std::vector<std::string>& attribs,
                                      Descriptions& descriptions)
-   : BuildingDescr(init_descname, MapObjectType::MILITARYSITE, table, descriptions),
-     conquer_radius_(0),
-     num_soldiers_(0),
-     heal_per_second_(0) {
+   : BuildingDescr(init_descname, MapObjectType::MILITARYSITE, table, attribs, descriptions) {
 
 	conquer_radius_ = table.get_int("conquers");
 	num_soldiers_ = table.get_int("max_soldiers");
@@ -345,23 +348,21 @@ MilitarySite::MilitarySite(const MilitarySiteDescr& ms_descr)
    : Building(ms_descr),
      attack_target_(this),
      soldier_control_(this),
-     didconquer_(false),
+
      capacity_(ms_descr.get_max_number_of_soldiers()),
-     nexthealtime_(0),
-     soldier_preference_(ms_descr.prefers_heroes_at_start_ ? SoldierPreference::kHeroes :
-                                                             SoldierPreference::kRookies),
-     next_swap_soldiers_time_(Time(0)),
-     soldier_upgrade_try_(false),
-     doing_upgrade_request_(false),
-     // Initialize vector capacity for statistics string cache
-     statistics_string_cache_(kNoOfStatisticsStringCases) {
+     soldier_request_manager_(
+        *this,
+        ms_descr.prefers_heroes_at_start_ ? SoldierPreference::kHeroes :
+                                            SoldierPreference::kRookies,
+        MilitarySite::request_soldier_callback,
+        [this]() { return soldier_control_.soldier_capacity(); },
+        [this]() { return soldier_control_.stationed_soldiers(); }) {
 	set_attack_target(&attack_target_);
 	set_soldier_control(&soldier_control_);
 }
 
 MilitarySite::~MilitarySite() {
-	assert(!normal_soldier_request_);
-	assert(!upgrade_soldier_request_);
+	assert(soldier_request_manager_.get_request() == nullptr);
 }
 
 /**
@@ -370,46 +371,7 @@ Display number of soldiers.
 ===============
 */
 void MilitarySite::update_statistics_string(std::string* s) {
-	std::unique_ptr<i18n::GenericTextdomain> td(AddOns::create_textdomain_for_addon(
-	   owner().tribe().basic_info().addon, "tribes_encyclopedia"));
-	s->clear();
-	Quantity present = soldier_control_.present_soldiers().size();
-	Quantity stationed = soldier_control_.stationed_soldiers().size();
-
-	// military capacity strings
-	if (present == stationed) {
-		if (capacity_ > stationed) {
-			*s = format(
-			   npgettext(owner().tribe().get_soldier_context_string().c_str(),
-			             owner().tribe().get_soldier_capacity_strings_sg()[0].c_str(),
-			             owner().tribe().get_soldier_capacity_strings_pl()[0].c_str(), stationed),
-			   stationed, (capacity_ - stationed));
-		} else {
-			*s = format(
-			   npgettext(owner().tribe().get_soldier_context_string().c_str(),
-			             owner().tribe().get_soldier_capacity_strings_sg()[1].c_str(),
-			             owner().tribe().get_soldier_capacity_strings_pl()[1].c_str(), stationed),
-			   stationed);
-		}
-	} else {
-		if (capacity_ > stationed) {
-			*s = format(
-			   npgettext(owner().tribe().get_soldier_context_string().c_str(),
-			             owner().tribe().get_soldier_capacity_strings_sg()[2].c_str(),
-			             owner().tribe().get_soldier_capacity_strings_pl()[2].c_str(), stationed),
-			   present, (stationed - present), (capacity_ - stationed));
-		} else {
-			*s = format(
-			   npgettext(owner().tribe().get_soldier_context_string().c_str(),
-			             owner().tribe().get_soldier_capacity_strings_sg()[3].c_str(),
-			             owner().tribe().get_soldier_capacity_strings_pl()[3].c_str(), stationed),
-			   present, (stationed - present));
-		}
-	}
-
-	*s = StyleManager::color_tag(
-	   // Line break to make Codecheck happy.
-	   *s, g_style_manager->building_statistics_style().medium_color());
+	*s = soldier_control_.get_status_string(owner().tribe(), get_soldier_preference());
 }
 
 bool MilitarySite::init(EditorGameBase& egbase) {
@@ -444,13 +406,7 @@ Note that the workers are dealt with in the PlayerImmovable code.
 */
 void MilitarySite::set_economy(Economy* const e, WareWorker type) {
 	Building::set_economy(e, type);
-
-	if (normal_soldier_request_ && (e != nullptr) && type == normal_soldier_request_->get_type()) {
-		normal_soldier_request_->set_economy(e);
-	}
-	if (upgrade_soldier_request_ && (e != nullptr) && type == upgrade_soldier_request_->get_type()) {
-		upgrade_soldier_request_->set_economy(e);
-	}
+	soldier_request_manager_.set_economy(e, type);
 }
 
 /**
@@ -476,9 +432,6 @@ void MilitarySite::cleanup(EditorGameBase& egbase) {
 		--capacity_;
 	}
 	update_soldier_request();
-
-	normal_soldier_request_.reset();
-	upgrade_soldier_request_.reset();
 }
 
 /*
@@ -488,7 +441,8 @@ void MilitarySite::cleanup(EditorGameBase& egbase) {
 
 Soldier* MilitarySite::find_least_suited_soldier() {
 	const std::vector<Soldier*> present = soldier_control_.present_soldiers();
-	const int32_t multiplier = SoldierPreference::kHeroes == soldier_preference_ ? -1 : 1;
+	// Always kick out a rookie, unless rookies are preferred.
+	const int32_t multiplier = get_soldier_preference() == SoldierPreference::kRookies ? 1 : -1;
 	int worst_soldier_level = INT_MIN;
 	Soldier* worst_soldier = nullptr;
 	for (Soldier* sld : present) {
@@ -527,11 +481,14 @@ bool MilitarySite::drop_least_suited_soldier(bool new_soldier_has_arrived, Soldi
 		if (nullptr != newguy && nullptr != kickoutCandidate) {
 			int32_t old_level = kickoutCandidate->get_level(TrainingAttribute::kTotal);
 			int32_t new_level = newguy->get_level(TrainingAttribute::kTotal);
-			if (SoldierPreference::kHeroes == soldier_preference_ && old_level >= new_level) {
-				return false;
-			}
-			if (SoldierPreference::kRookies == soldier_preference_ && old_level <= new_level) {
-				return false;
+			if (get_soldier_preference() == SoldierPreference::kRookies) {
+				if (old_level <= new_level) {
+					return false;
+				}
+			} else {  // All policies except Prefer Rookies prefer to kick out rookies.
+				if (old_level >= new_level) {
+					return false;
+				}
 			}
 		}
 
@@ -580,147 +537,11 @@ void MilitarySite::request_soldier_callback(Game& game,
 	msite.soldier_control_.incorporate_soldier(game, s);
 }
 
-/**
- * Update the request for soldiers and cause soldiers to be evicted
- * as appropriate.
- */
-void MilitarySite::update_normal_soldier_request() {
-	std::vector<Soldier*> present = soldier_control_.present_soldiers();
-	Quantity const stationed = soldier_control_.stationed_soldiers().size();
+void MilitarySite::update_soldier_request(bool /* incd */) {
+	soldier_request_manager_.update();
 
-	if (stationed < capacity_) {
-		if (!normal_soldier_request_) {
-			normal_soldier_request_.reset(new Request(
-			   *this, owner().tribe().soldier(), MilitarySite::request_soldier_callback, wwWORKER));
-			normal_soldier_request_->set_requirements(soldier_requirements_);
-		}
-
-		normal_soldier_request_->set_count(capacity_ - stationed);
-	} else {
-		normal_soldier_request_.reset();
-	}
-
-	if (capacity_ < present.size()) {
-		Game& game = dynamic_cast<Game&>(get_owner()->egbase());
-		for (uint32_t i = 0; i < present.size() - capacity_; ++i) {
-			Soldier& soldier = *present[i];
-			soldier.reset_tasks(game);
-			soldier.start_task_leavebuilding(game, true);
-		}
-	}
-}
-
-/* There are two kinds of soldier requests: "normal", which is used whenever the military site needs
- * more soldiers, and "upgrade" which is used when there is a preference for either heroes or
- * rookies.
- *
- * In case of normal requests, the military site is filled. In case of upgrade requests, only one
- * guy
- * is exchanged at a time.
- *
- * There would be more efficient ways to get well trained soldiers. Now, new buildings appearing in
- * battle
- * field are more vulnerable at the beginning. This is intentional. The purpose of this upgrade
- * thing is
- * to reduce the benefits of site micromanagement. The intention is not to make gameplay easier in
- * other ways.
- */
-void MilitarySite::update_upgrade_soldier_request() {
-	bool reqch = update_upgrade_requirements();
-	if (!soldier_upgrade_try_) {
-		return;
-	}
-
-	bool do_rebuild_request = reqch;
-
-	if (upgrade_soldier_request_) {
-		if (!upgrade_soldier_request_->is_open()) {
-			// If a replacement is already walking this way, let's not change our minds.
-			do_rebuild_request = false;
-		}
-		if (0 == upgrade_soldier_request_->get_count()) {
-			do_rebuild_request = true;
-		}
-	} else {
-		do_rebuild_request = true;
-	}
-
-	if (do_rebuild_request) {
-		upgrade_soldier_request_.reset(new Request(
-		   *this, owner().tribe().soldier(), MilitarySite::request_soldier_callback, wwWORKER));
-
-		upgrade_soldier_request_->set_requirements(soldier_upgrade_requirements_);
-		upgrade_soldier_request_->set_count(1);
-	}
-}
-
-/*
- * I have update_soldier_request
- *        update_upgrade_soldier_request
- *        update_normal_soldier_request
- *
- * The first one handles state switching between site fill (normal more)
- * and grabbing soldiers with proper training (upgrade mode). The last
- * two actually make the requests.
- *
- * The input parameter incd is true, if we just incorporated a new soldier
- * as a result of an upgrade request. In such cases, we will re-arm the
- * upgrade request.
- */
-
-void MilitarySite::update_soldier_request(bool incd) {
-	const uint32_t capacity = soldier_control_.soldier_capacity();
-	const uint32_t stationed = soldier_control_.stationed_soldiers().size();
-
-	if (doing_upgrade_request_) {
-		if (incd && upgrade_soldier_request_)  // update requests always ask for one soldier at time!
-		{
-			upgrade_soldier_request_.reset();
-		}
-		if (capacity > stationed) {
-			// Somebody is killing my soldiers in the middle of upgrade
-			// or I have kicked out his predecessor already.
-			if (upgrade_soldier_request_ &&
-			    (upgrade_soldier_request_->is_open() || 0 == upgrade_soldier_request_->get_count())) {
-
-				// Economy was not able to find the soldiers I need.
-				// I can safely drop the upgrade request and go to fill mode.
-				upgrade_soldier_request_.reset();
-			}
-			if (!upgrade_soldier_request_) {
-				// phoo -- I can safely request new soldiers.
-				doing_upgrade_request_ = false;
-				update_normal_soldier_request();
-			}
-			// else -- ohno please help me! Player is in trouble -- evil grin
-		} else                        // military site is full or overfull
-		   if (capacity < stationed)  // player is reducing capacity
-		{
-			drop_least_suited_soldier(false, nullptr);
-		} else  // capacity == stationed size
-		{
-			if (upgrade_soldier_request_ && (!(upgrade_soldier_request_->is_open())) &&
-			    1 == upgrade_soldier_request_->get_count() && (!incd)) {
-				drop_least_suited_soldier(false, nullptr);
-			} else {
-				update_upgrade_soldier_request();
-			}
-		}
-	} else  // not doing upgrade request
-	{
-		if ((capacity != stationed) || (normal_soldier_request_)) {
-			update_normal_soldier_request();
-		}
-
-		if ((capacity == stationed) && (!normal_soldier_request_)) {
-			if (soldier_control_.present_soldiers().size() == capacity) {
-				doing_upgrade_request_ = true;
-				update_upgrade_soldier_request();
-			}
-			// Note -- if there are non-present stationed soldiers, nothing gets
-			// called. Therefore, I revisit this routine periodically without apparent
-			// reason, hoping that all stationed soldiers would be present.
-		}
+	if (soldier_control()->soldier_capacity() < soldier_control_.present_soldiers().size()) {
+		drop_least_suited_soldier(false, nullptr);
 	}
 }
 
@@ -739,10 +560,6 @@ void MilitarySite::act(Game& game, uint32_t const data) {
 	Building::act(game, data);
 
 	const Time& timeofgame = game.get_gametime();
-	if (normal_soldier_request_ && upgrade_soldier_request_) {
-		throw wexception(
-		   "MilitarySite::act: Two soldier requests are ongoing -- should never happen!\n");
-	}
 
 	// I do not get a callback when stationed, non-present soldier returns --
 	// Therefore I must poll in some occasions. Let's do that rather infrequently,
@@ -750,7 +567,7 @@ void MilitarySite::act(Game& game, uint32_t const data) {
 
 	// TODO(unknown): I would need two new callbacks, to get rid ot this polling.
 	if (timeofgame > next_swap_soldiers_time_) {
-		next_swap_soldiers_time_ = timeofgame + Duration(soldier_upgrade_try_ ? 20000 : 100000);
+		next_swap_soldiers_time_ = timeofgame + kSoldierSwapTime;
 		update_soldier_request();
 	}
 
@@ -833,7 +650,7 @@ bool MilitarySite::get_building_work(Game& game, Worker& worker, bool /*success*
 		bool stayhome;
 		if (MapObject* const enemy = pop_soldier_job(soldier, &stayhome)) {
 			if (upcast(Building, building, enemy)) {
-				soldier->start_task_attack(game, *building);
+				soldier->start_task_attack(game, *building, CF_RETREAT_WHEN_INJURED);
 				return true;
 			}
 			if (upcast(Soldier, opponent, enemy)) {
@@ -853,15 +670,6 @@ bool MilitarySite::get_building_work(Game& game, Worker& worker, bool /*success*
 	return false;
 }
 
-/**
- * \return \c true if the soldier is currently present and idle in the building.
- */
-bool MilitarySite::is_present(Soldier& soldier) const {
-	return soldier.get_location(get_owner()->egbase()) == this &&
-	       soldier.get_state() == soldier.get_state(Worker::taskBuildingwork) &&
-	       soldier.get_position() == get_position();
-}
-
 void MilitarySite::conquer_area(EditorGameBase& egbase) {
 	assert(!didconquer_);
 	egbase.conquer_area(PlayerArea<Area<FCoords>>(
@@ -872,7 +680,6 @@ void MilitarySite::conquer_area(EditorGameBase& egbase) {
 
 /// Initialises the militarysite after it was "conquered" (the old was replaced)
 void MilitarySite::reinit_after_conqueration(Game& game) {
-	clear_requirements();
 	conquer_area(game);
 	update_soldier_request();
 	start_animation(game, descr().get_animation("idle", this));
@@ -891,15 +698,14 @@ bool MilitarySite::military_presence_kept(Game& game) {
 	FCoords const fc = game.map().get_fcoords(get_position());
 	game.map().find_immovables(game, Area<FCoords>(fc, 3), &immovables);
 
-	for (const ImmovableFound& imm : immovables) {
-		if (upcast(MilitarySite const, militarysite, imm.object)) {
-			if (this != militarysite && &owner() == &militarysite->owner() &&
-			    get_size() <= militarysite->get_size() && militarysite->didconquer_) {
-				return true;
-			}
+	return std::any_of(immovables.begin(), immovables.end(), [this](const ImmovableFound& imm) {
+		if (imm.object->descr().type() != MapObjectType::MILITARYSITE) {
+			return false;
 		}
-	}
-	return false;
+		upcast(const MilitarySite, militarysite, imm.object);
+		return this != militarysite && &owner() == &militarysite->owner() &&
+		       get_size() <= militarysite->get_size() && militarysite->didconquer_;
+	});
 }
 
 /// Informs the player about an attack of his opponent.
@@ -913,30 +719,12 @@ void MilitarySite::notify_player(Game& game, bool const discovered) {
 	             Duration(60 * 1000), 5);
 }
 
-/*
-   MilitarySite::set_requirements
-
-   Easy to use, overwrite with given requirements.
-*/
-void MilitarySite::set_requirements(const Requirements& r) {
-	soldier_requirements_ = r;
-}
-
-/*
-   MilitarySite::clear_requirements
-
-   This should cancel any requirement pushed at this house
-*/
-void MilitarySite::clear_requirements() {
-	soldier_requirements_ = Requirements();
-}
-
 void MilitarySite::send_attacker(Soldier& soldier, Building& target) {
 	if (!is_present(soldier)) {
 		// The soldier may not be present anymore due to having been kicked out. Most of the time
 		// the function calling us will notice this, but there are cornercase where it might not,
 		// e.g. when a soldier was ordered to leave but did not physically quit the building yet.
-		log_warn_time(
+		verb_log_warn_time(
 		   owner().egbase().get_gametime(),
 		   "MilitarySite(%3dx%3d)::send_attacker: Not sending soldier %u because he left the "
 		   "building\n",
@@ -985,61 +773,12 @@ MapObject* MilitarySite::pop_soldier_job(Soldier* const soldier, bool* const sta
 	return nullptr;
 }
 
-/*
- * When upgrading soldiers, we do not ask for just any soldiers, but soldiers
- * that are better than what we already have. This routine sets the requirements
- * used by the request.
- *
- * The routine returns true if upgrade request thresholds have changed. This information could be
- * used to decide whether the soldier-Request should be upgraded.
- */
-bool MilitarySite::update_upgrade_requirements() {
-	int32_t soldier_upgrade_required_min = soldier_upgrade_requirements_.get_min();
-	int32_t soldier_upgrade_required_max = soldier_upgrade_requirements_.get_max();
-
-	// Find the level of the soldier that is currently least-suited.
-	Soldier* worst_guy = find_least_suited_soldier();
-	if (worst_guy == nullptr) {
-		// There could be no soldier in the militarysite right now. No reason to freak out.
-		return false;
-	}
-	int32_t wg_level = worst_guy->get_level(TrainingAttribute::kTotal);
-
-	// Micro-optimization: I assume that the majority of military sites have only level-zero
-	// soldiers and prefer rookies. Handle them separately.
-	soldier_upgrade_try_ = true;
-	if (SoldierPreference::kRookies == soldier_preference_) {
-		if (0 == wg_level) {
-			soldier_upgrade_try_ = false;
-			return false;
-		}
-	}
-
-	// Now I actually build the new requirements.
-	int32_t reqmin = SoldierPreference::kHeroes == soldier_preference_ ? 1 + wg_level : 0;
-	int32_t reqmax = SoldierPreference::kHeroes == soldier_preference_ ? SHRT_MAX : wg_level - 1;
-
-	bool maxchanged = reqmax != soldier_upgrade_required_max;
-	bool minchanged = reqmin != soldier_upgrade_required_min;
-
-	if (maxchanged || minchanged) {
-		if (upgrade_soldier_request_ && (upgrade_soldier_request_->is_open())) {
-			upgrade_soldier_request_.reset();
-		}
-		soldier_upgrade_requirements_ = RequireAttribute(TrainingAttribute::kTotal, reqmin, reqmax);
-
-		return true;
-	}
-
-	return false;
-}
-
 std::unique_ptr<const BuildingSettings> MilitarySite::create_building_settings() const {
 	std::unique_ptr<MilitarysiteSettings> settings(
 	   new MilitarysiteSettings(descr(), owner().tribe()));
 	settings->desired_capacity =
 	   std::min(settings->max_capacity, soldier_control_.soldier_capacity());
-	settings->prefer_heroes = soldier_preference_ == SoldierPreference::kHeroes;
+	settings->soldier_preference = get_soldier_preference();
 	// Prior to the resolution of a defect report against ISO C++11, local variable 'settings' would
 	// have been copied despite being returned by name, due to its not matching the function return
 	// type. Call 'std::move' explicitly to avoid copying on older compilers.
@@ -1050,7 +789,7 @@ std::unique_ptr<const BuildingSettings> MilitarySite::create_building_settings()
 // setters
 
 void MilitarySite::set_soldier_preference(SoldierPreference p) {
-	soldier_preference_ = p;
+	soldier_request_manager_.set_preference(p);
 	next_swap_soldiers_time_ = Time(0);
 }
 }  // namespace Widelands
