@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2024 by the Widelands Development Team
+ * Copyright (C) 2006-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -166,7 +166,11 @@ uint32_t PartiallyFinishedBuilding::get_built_per64k() const {
 
 	uint32_t ts = build_step_time().get();
 	if (working_) {
-		thisstep = ts - (work_steptime_.get() - time);
+		if (builder_ != nullptr) {
+			thisstep = ts - (workstep_completiontime_.get() - time);
+		} else {
+			thisstep = ts - last_remaining_time_.get();
+		}
 		// The check below is necessary because we drive construction via
 		// the construction worker in get_building_work(), and there can be
 		// a small delay between the worker completing his job and requesting
@@ -219,5 +223,25 @@ PartiallyFinishedBuilding is an exception.
 void PartiallyFinishedBuilding::add_worker(Worker& worker) {
 	Building::add_worker(worker);
 	set_seeing(true);
+	if (working_) {  // previously a builder had been evicted, we've just got a replacement
+		workstep_completiontime_ = owner().egbase().get_gametime() + last_remaining_time_;
+	} else {
+		// We clear the walking animation because if the builder got evicted while waiting for
+		// materials, then ConstructionSite::get_building_work() doesn't do it if there're still
+		// no materials.
+		worker.set_animation(owner().egbase(), worker.descr().get_animation("idle", &worker));
+	}
+}
+
+void PartiallyFinishedBuilding::notify_worker_evicted(Game& game, Worker& worker) {
+	Building::notify_worker_evicted(game, worker);
+	builder_ = nullptr;
+	const Time& gametime = game.get_gametime();
+	if (gametime < workstep_completiontime_) {
+		last_remaining_time_ = workstep_completiontime_ - gametime;
+	} else {
+		last_remaining_time_ = Duration(0);
+	}
+	request_builder(game);
 }
 }  // namespace Widelands
