@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2024 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -533,6 +533,23 @@ struct ProductionProgram : public MapObjectProgram {
 		PlaySoundParameters parameters;
 	};
 
+	/// Runs a Lua script.
+	///
+	/// Parameter syntax:
+	///    parameters ::= function
+	/// Parameter semantics:
+	///    function:
+	///       The name of the function to call.
+	///
+	/// Invokes the specified Lua function with this productionsite as the argument.
+	struct ActRunScript : public Action {
+		explicit ActRunScript(const std::vector<std::string>& arguments);
+		void execute(Game&, ProductionSite&) const override;
+
+	private:
+		RunScriptParameters parameters;
+	};
+
 	/// Sends a building worker to construct at an immovable.
 	///
 	/// Parameter syntax:
