@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,16 +22,20 @@
 #include <memory>
 
 #include "base/macros.h"
+#include "logic/map.h"
 #include "logic/map_objects/tribes/bill_of_materials.h"
 #include "logic/map_objects/tribes/building.h"
 #include "logic/map_objects/tribes/production_program.h"
 #include "logic/map_objects/tribes/program_result.h"
+#include "logic/map_objects/tribes/worker.h"
+#include "logic/map_objects/tribes/worker_descr.h"
 #include "scripting/lua_table.h"
 
 namespace Widelands {
 
+class FerryFleetYardInterface;
+class ShipFleetYardInterface;
 class Soldier;
-class WorkerDescr;
 
 enum class FailNotificationType { kDefault, kFull };
 
@@ -52,42 +56,44 @@ public:
 	ProductionSiteDescr(const std::string& init_descname,
 	                    MapObjectType type,
 	                    const LuaTable& t,
+	                    const std::vector<std::string>& attribs,
 	                    Descriptions& descriptions);
 	ProductionSiteDescr(const std::string& init_descname,
 	                    const LuaTable& t,
+	                    const std::vector<std::string>& attribs,
 	                    Descriptions& descriptions);
 
-	Building& create_object() const override;
+	[[nodiscard]] Building& create_object() const override;
 
 	// List of wares to register having economy checks. Parsed by the tribes during postload and must
 	// be nullptr after loading has finished
-	std::set<DescriptionIndex>* ware_demand_checks() const;
+	[[nodiscard]] std::set<DescriptionIndex>* ware_demand_checks() const;
 	// List of workers to register having economy checks. Parsed by the tribes during postload and
 	// must be nullptr after loading has finished
-	std::set<DescriptionIndex>* worker_demand_checks() const;
+	[[nodiscard]] std::set<DescriptionIndex>* worker_demand_checks() const;
 	// Clear ware and worker demand check info
 	void clear_demand_checks();
 
-	uint32_t nr_working_positions() const {
+	[[nodiscard]] uint32_t nr_working_positions() const {
 		uint32_t result = 0;
 		for (const auto& working_pos : working_positions()) {
 			result += working_pos.second;
 		}
 		return result;
 	}
-	const BillOfMaterials& working_positions() const {
+	[[nodiscard]] const BillOfMaterials& working_positions() const {
 		return working_positions_;
 	}
-	bool is_output_ware_type(const DescriptionIndex& i) const {
-		return output_ware_types_.count(i);
+	[[nodiscard]] bool is_output_ware_type(const DescriptionIndex& i) const {
+		return output_ware_types_.count(i) != 0u;
 	}
-	bool is_output_worker_type(const DescriptionIndex& i) const {
-		return output_worker_types_.count(i);
+	[[nodiscard]] bool is_output_worker_type(const DescriptionIndex& i) const {
+		return output_worker_types_.count(i) != 0u;
 	}
-	const BillOfMaterials& input_wares() const {
+	[[nodiscard]] const BillOfMaterials& input_wares() const {
 		return input_wares_;
 	}
-	const BillOfMaterials& input_workers() const {
+	[[nodiscard]] const BillOfMaterials& input_workers() const {
 		return input_workers_;
 	}
 	BillOfMaterials& mutable_input_wares() {
@@ -97,25 +103,25 @@ public:
 		return input_workers_;
 	}
 	using Output = std::set<DescriptionIndex>;
-	const Output& output_ware_types() const {
+	[[nodiscard]] const Output& output_ware_types() const {
 		return output_ware_types_;
 	}
-	const Output& output_worker_types() const {
+	[[nodiscard]] const Output& output_worker_types() const {
 		return output_worker_types_;
 	}
 	/// Map objects that this production site needs nearby according to attribute, without removing
 	/// them from the map
-	const std::set<std::pair<MapObjectType, MapObjectDescr::AttributeIndex>>&
+	[[nodiscard]] const std::set<std::pair<MapObjectType, MapObjectDescr::AttributeIndex>>&
 	needed_attributes() const {
 		return needed_attributes_;
 	}
 	/// Map objects that are collected from the map by this production site according to attribute
-	const std::set<std::pair<MapObjectType, MapObjectDescr::AttributeIndex>>&
+	[[nodiscard]] const std::set<std::pair<MapObjectType, MapObjectDescr::AttributeIndex>>&
 	collected_attributes() const {
 		return collected_attributes_;
 	}
 	/// Map objects that are placed on the map by this production site according to attribute
-	const std::set<std::pair<MapObjectType, MapObjectDescr::AttributeIndex>>&
+	[[nodiscard]] const std::set<std::pair<MapObjectType, MapObjectDescr::AttributeIndex>>&
 	created_attributes() const {
 		return created_attributes_;
 	}
@@ -130,15 +136,15 @@ public:
 
 	/// The resources that this production site needs to collect from the map, the max percent it can
 	/// achieve, and the chance when depleted
-	const std::map<std::string, CollectedResourceInfo>& collected_resources() const {
+	[[nodiscard]] const std::map<std::string, CollectedResourceInfo>& collected_resources() const {
 		return collected_resources_;
 	}
 	/// The resources that this production site will place on the map
-	const std::set<std::string>& created_resources() const {
+	[[nodiscard]] const std::set<std::string>& created_resources() const {
 		return created_resources_;
 	}
 	/// The bobs (critters) that this production site needs to collect from the map
-	const std::set<std::string>& collected_bobs() const {
+	[[nodiscard]] const std::set<std::string>& collected_bobs() const {
 		return collected_bobs_;
 	}
 	/// Set that this production site needs to collect the given bob from the map
@@ -146,7 +152,7 @@ public:
 		collected_bobs_.insert(bobname);
 	}
 	/// The bobs (critters or workers) that this production site will place on the map
-	const std::set<std::string>& created_bobs() const {
+	[[nodiscard]] const std::set<std::string>& created_bobs() const {
 		return created_bobs_;
 	}
 	/// Set that this production site will place the given bob on the map
@@ -154,11 +160,11 @@ public:
 		created_bobs_.insert(bobname);
 	}
 	/// The immovables that this production site needs to be nearby
-	const std::set<std::string>& needed_immovables() const {
+	[[nodiscard]] const std::set<std::string>& needed_immovables() const {
 		return needed_immovables_;
 	}
 	/// The immovables that this production site needs to collect from the map
-	const std::set<std::string>& collected_immovables() const {
+	[[nodiscard]] const std::set<std::string>& collected_immovables() const {
 		return collected_immovables_;
 	}
 	/// Set that this production site makes use of the given immovable nearby
@@ -170,7 +176,7 @@ public:
 		collected_immovables_.insert(immovablename);
 	}
 	/// The immovables that this production site will place on the map
-	const std::set<std::string>& created_immovables() const {
+	[[nodiscard]] const std::set<std::string>& created_immovables() const {
 		return created_immovables_;
 	}
 	/// Set that this production site will place the given immovable on the map
@@ -178,32 +184,46 @@ public:
 		created_immovables_.insert(immovablename);
 	}
 
-	const ProductionProgram* get_program(const std::string&) const;
+	[[nodiscard]] const ProductionProgram* get_program(const std::string&) const;
 	using Programs = std::map<std::string, std::unique_ptr<ProductionProgram>>;
-	const Programs& programs() const {
+	[[nodiscard]] const Programs& programs() const {
 		return programs_;
 	}
-	Programs& mutable_programs() {
+	[[nodiscard]] Programs& mutable_programs() {
 		return programs_;
 	}
 
-	const std::string& out_of_resource_title() const {
+	[[nodiscard]] bool is_infinite_production_useful() const {
+		return is_infinite_production_useful_;
+	}
+	void set_infinite_production_useful(const bool u) {
+		is_infinite_production_useful_ = u;
+	}
+
+	[[nodiscard]] bool has_ship_fleet_check() const {
+		return has_ship_fleet_check_;
+	}
+	[[nodiscard]] bool has_ferry_fleet_check() const {
+		return has_ferry_fleet_check_;
+	}
+
+	[[nodiscard]] const std::string& out_of_resource_title() const {
 		return out_of_resource_title_;
 	}
 
-	const std::string& out_of_resource_heading() const {
+	[[nodiscard]] const std::string& out_of_resource_heading() const {
 		return out_of_resource_heading_;
 	}
 
-	const std::string& out_of_resource_message() const {
+	[[nodiscard]] const std::string& out_of_resource_message() const {
 		return out_of_resource_message_;
 	}
 
-	const std::string& resource_not_needed_message() const {
+	[[nodiscard]] const std::string& resource_not_needed_message() const {
 		return resource_not_needed_message_;
 	}
 
-	uint32_t out_of_resource_productivity_threshold() const {
+	[[nodiscard]] uint32_t out_of_resource_productivity_threshold() const {
 		return out_of_resource_productivity_threshold_;
 	}
 
@@ -220,21 +240,21 @@ public:
 	void add_supported_by_productionsite(const std::string& productionsite);
 	/// Returns whether this production site competes with the given 'productionsite' for map
 	/// resources.
-	bool competes_with_productionsite(const std::string& productionsite) const;
+	[[nodiscard]] bool competes_with_productionsite(const std::string& productionsite) const;
 	/// Returns whether this production site creates map resources or objects that the given
 	/// 'productionsite' needs.
-	bool supports_productionsite(const std::string& productionsite) const;
+	[[nodiscard]] bool supports_productionsite(const std::string& productionsite) const;
 	/// Returns whether the given 'productionsite' creates map resources or objects that this
 	/// production site needs.
-	bool is_supported_by_productionsite(const std::string& productionsite) const;
+	[[nodiscard]] bool is_supported_by_productionsite(const std::string& productionsite) const;
 	/// Returns the production sites that need a map resource or object that this production site
 	/// will place on the map.
-	std::set<std::string> supported_productionsites() const {
+	[[nodiscard]] std::set<std::string> supported_productionsites() const {
 		return supported_productionsites_;
 	}
 	/// Returns the production sites that create a map resource or object that this production site
 	/// needs.
-	std::set<std::string> supported_by_productionsites() const {
+	[[nodiscard]] std::set<std::string> supported_by_productionsites() const {
 		return supported_by_productionsites_;
 	}
 
@@ -294,6 +314,7 @@ private:
 	std::set<std::string> collected_immovables_;
 	std::set<std::string> created_immovables_;
 	Programs programs_;
+	bool is_infinite_production_useful_;
 	std::string out_of_resource_title_;
 	std::string out_of_resource_heading_;
 	std::string out_of_resource_message_;
@@ -302,12 +323,15 @@ private:
 	std::set<std::string> competing_productionsites_;
 	std::set<std::string> supported_productionsites_;
 	std::set<std::string> supported_by_productionsites_;
+	bool has_ship_fleet_check_{false};
+	bool has_ferry_fleet_check_{false};
 
 	DISALLOW_COPY_AND_ASSIGN(ProductionSiteDescr);
 };
 
 class ProductionSite : public Building {
 	friend class MapBuildingdataPacket;
+	// TODO(Nordfriese): Refactor this. Many friend classes are bad style.
 	friend struct ProductionProgram::ActReturn;
 	friend struct ProductionProgram::ActReturn::WorkersNeedExperience;
 	friend struct ProductionProgram::ActCall;
@@ -321,6 +345,7 @@ class ProductionSite : public Building {
 	friend struct ProductionProgram::ActCheckSoldier;
 	friend struct ProductionProgram::ActTrain;
 	friend struct ProductionProgram::ActPlaySound;
+	friend struct ProductionProgram::ActRunScript;
 	friend struct ProductionProgram::ActConstruct;
 	MO_DESCR(ProductionSiteDescr)
 
@@ -335,19 +360,19 @@ public:
 	void set_stopped(bool);
 
 	struct WorkingPosition {
-		WorkingPosition(Request* const wr = nullptr, Worker* const w = nullptr)
+		explicit WorkingPosition(Request* const wr = nullptr, Worker* const w = nullptr)
 		   : worker_request(wr), worker(w) {
 		}
 		Request* worker_request;
 		OPtr<Worker> worker;
 	};
 
-	const std::vector<WorkingPosition>* working_positions() const {
+	[[nodiscard]] const std::vector<WorkingPosition>* working_positions() const {
 		return &working_positions_;
 	}
 
 	virtual bool has_workers(DescriptionIndex targetSite, Game& game);
-	uint8_t get_statistics_percent() {
+	[[nodiscard]] uint8_t get_statistics_percent() const {
 		return last_stat_percent_ / 10;
 	}
 
@@ -355,11 +380,11 @@ public:
 	// and sets actual_percent_ to new value
 	void update_actual_statistics(Duration, bool);
 
-	uint8_t get_actual_statistics() {
+	[[nodiscard]] uint8_t get_actual_statistics() const {
 		return actual_percent_ / 10;
 	}
 
-	const std::string& production_result() const {
+	[[nodiscard]] const std::string& production_result() const {
 		return production_result_;
 	}
 
@@ -370,7 +395,8 @@ public:
 		production_result_ = text;
 	}
 
-	InputQueue& inputqueue(DescriptionIndex, WareWorker, const Request*) override;
+	InputQueue&
+	inputqueue(DescriptionIndex, WareWorker, const Request*, uint32_t disambiguator_id) override;
 
 	bool init(EditorGameBase&) override;
 	void cleanup(EditorGameBase&) override;
@@ -387,13 +413,13 @@ public:
 	void set_economy(Economy*, WareWorker) override;
 
 	using InputQueues = std::vector<InputQueue*>;
-	const InputQueues& inputqueues() const {
+	[[nodiscard]] const InputQueues& inputqueues() const {
 		return input_queues_;
 	}
 
-	const std::vector<Worker*>& workers() const;
+	[[nodiscard]] const std::vector<Worker*>& workers() const;
 
-	bool can_start_working() const;
+	[[nodiscard]] bool can_start_working() const;
 
 	/// sends a message to the player e.g. if the building's resource can't be found
 	void notify_player(Game& game,
@@ -403,7 +429,7 @@ public:
 
 	void set_default_anim(const std::string&);
 
-	std::unique_ptr<const BuildingSettings> create_building_settings() const override;
+	[[nodiscard]] std::unique_ptr<const BuildingSettings> create_building_settings() const override;
 
 	// This function forces the productionsite to interrupt whatever it is doing ASAP,
 	// and start the specified program immediately afterwards. If that program expects
@@ -411,20 +437,35 @@ public:
 	void set_next_program_override(Game&, const std::string&, MapObject* extra_data);
 	// Returns `true` if `set_next_program_override()` has been called recently
 	// and the force-started program has not terminated yet.
-	bool has_forced_state() const;
+	[[nodiscard]] bool has_forced_state() const;
+
+	[[nodiscard]] bool infinite_production() const {
+		return infinite_production_;
+	}
+	void set_infinite_production(bool);
+
+	[[nodiscard]] const std::vector<ShipFleetYardInterface*>& get_ship_fleet_interfaces() const {
+		return ship_fleet_interfaces_;
+	}
+	[[nodiscard]] const std::vector<FerryFleetYardInterface*>& get_ferry_fleet_interfaces() const {
+		return ferry_fleet_interfaces_;
+	}
+	void remove_fleet_interface(EditorGameBase& egbase, const ShipFleetYardInterface* i);
+	void remove_fleet_interface(EditorGameBase& egbase, const FerryFleetYardInterface* i);
+	void init_yard_interfaces(EditorGameBase& egbase);
 
 protected:
 	void update_statistics_string(std::string* statistics) override;
 
 	void load_finish(EditorGameBase& egbase) override;
+	void postload(EditorGameBase& egbase) override;
 
-protected:
 	struct State {
-		const ProductionProgram* program;  ///< currently running program
-		size_t ip;                         ///< instruction pointer
-		ProgramResult phase;               ///< micro-step index (instruction dependent)
+		const ProductionProgram* program{nullptr};  ///< currently running program
+		size_t ip{0};                               ///< instruction pointer
+		ProgramResult phase{ProgramResult::kNone};  ///< micro-step index (instruction dependent)
 		enum StateFlags : uint32_t { kStateFlagIgnoreStopped = 1, kStateFlagHasExtraData = 2 };
-		uint32_t flags;  ///< pfXXX flags
+		uint32_t flags{0};  ///< pfXXX flags
 
 		/**
 		 * Instruction-dependent additional data.
@@ -434,8 +475,7 @@ protected:
 		Coords coord;
 		/*@}*/
 
-		State()
-		   : program(nullptr), ip(0), phase(ProgramResult::kNone), flags(0), coord(Coords::null()) {
+		State() : coord(Coords::null()) {
 		}
 	};
 
@@ -449,12 +489,12 @@ protected:
 	 */
 	virtual void find_and_start_next_program(Game&);
 
-	State& top_state() {
-		assert(stack_.size());
+	[[nodiscard]] State& top_state() {
+		assert(!stack_.empty());
 		return *stack_.rbegin();
 	}
-	State* get_state() {
-		return stack_.size() ? &*stack_.rbegin() : nullptr;
+	[[nodiscard]] State* get_state() {
+		return !stack_.empty() ? &*stack_.rbegin() : nullptr;
 	}
 	void program_act(Game&);
 
@@ -480,10 +520,10 @@ protected:
 		post_timer_ = t;
 	}
 
-protected:  // TrainingSite must have access to this stuff
+	// TrainingSite must have access to this stuff
 	std::vector<WorkingPosition> working_positions_;
 
-	int32_t fetchfromflag_;  ///< Number of wares to fetch from flag
+	int32_t fetchfromflag_{0};  ///< Number of wares to fetch from flag
 
 	/// If a program has ended with the result Failed or Skipped, that program may not
 	/// start again until a certain time has passed. This is a map from program
@@ -494,21 +534,27 @@ protected:  // TrainingSite must have access to this stuff
 	FailedSkippedPrograms failed_skipped_programs_;
 
 	using Stack = std::vector<State>;
-	Stack stack_;          ///<  program stack
-	bool program_timer_;   ///< execute next instruction based on pointer
-	Time program_time_;    ///< timer time
-	Duration post_timer_;  ///< Time to schedule after ends
+	Stack stack_;                ///<  program stack
+	bool program_timer_{false};  ///< execute next instruction based on pointer
+	Time program_time_{0U};      ///< timer time
+	Duration post_timer_{50U};   ///< Time to schedule after ends
 
 	BillOfMaterials produced_wares_;
 	BillOfMaterials recruited_workers_;
 	InputQueues input_queues_;  ///< input queues for all inputs
-	uint16_t last_stat_percent_;
+	uint16_t last_stat_percent_{0U};
 	// integer 0-10000000, to be divided by 10000 to get a percent, to avoid float (target range:
 	// 0-100)
-	uint32_t actual_percent_;  // basically this is percent * 10 to avoid floats
-	Time last_program_end_time;
-	bool is_stopped_;
-	std::string default_anim_;  // normally "idle", "empty", if empty mine.
+	uint32_t actual_percent_{0U};  // basically this is percent * 10 to avoid floats
+	Time last_program_end_time{0U};
+	bool is_stopped_{false};
+	bool infinite_production_{false};
+	std::string default_anim_{"idle"};  // normally "idle", "empty", if empty mine.
+
+	std::vector<ShipFleetYardInterface*> ship_fleet_interfaces_;
+	std::vector<FerryFleetYardInterface*> ferry_fleet_interfaces_;
+	std::unique_ptr<Notifications::Subscriber<NoteFieldTerrainChanged>>
+	   field_terrain_changed_subscriber_;
 
 private:
 	enum class Trend { kUnchanged, kRising, kFalling };
@@ -516,7 +562,7 @@ private:
 	std::string statistics_string_on_changed_statistics_;
 	std::string production_result_;  // hover tooltip text
 
-	int32_t main_worker_;
+	int32_t main_worker_{-1};
 
 	DISALLOW_COPY_AND_ASSIGN(ProductionSite);
 };
@@ -530,13 +576,12 @@ private:
 struct Input {
 	Input(const DescriptionIndex& Ware, uint8_t const Max) : ware_(Ware), max_(Max) {
 	}
-	~Input() {
-	}
+	~Input() = default;
 
-	DescriptionIndex ware() const {
+	[[nodiscard]] DescriptionIndex ware() const {
 		return ware_;
 	}
-	uint8_t max() const {
+	[[nodiscard]] uint8_t max() const {
 		return max_;
 	}
 
