@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2024 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -61,8 +61,9 @@ const uint32_t TrainingSite::training_state_multiplier_ = 12;
  */
 TrainingSiteDescr::TrainingSiteDescr(const std::string& init_descname,
                                      const LuaTable& table,
+                                     const std::vector<std::string>& attribs,
                                      Descriptions& descriptions)
-   : ProductionSiteDescr(init_descname, MapObjectType::TRAININGSITE, table, descriptions),
+   : ProductionSiteDescr(init_descname, MapObjectType::TRAININGSITE, table, attribs, descriptions),
      num_soldiers_(table.get_int("soldier_capacity")),
      max_stall_(table.get_int("trainer_patience")) {
 	// Read the range of levels that can update this building
@@ -109,7 +110,7 @@ TrainingSiteDescr::TrainingSiteDescr(const std::string& init_descname,
 				if (from_checksoldier.level >= checkme.level) {
 					throw GameDataError(
 					   "Trainingsite '%s' is trying to train a soldier attribute from level "
-					   "%d to %d, but the 'checksoldier' action's level must be lower "
+					   "%u to %u, but the 'checksoldier' action's level must be lower "
 					   "than the 'train' action's level in program '%s'",
 					   name().c_str(), from_checksoldier.level, checkme.level, program.first.c_str());
 				}
@@ -159,10 +160,9 @@ unsigned TrainingSiteDescr::get_min_level(const TrainingAttribute at) const {
 		return min_defense_;
 	case TrainingAttribute::kEvade:
 		return min_evade_;
-	case TrainingAttribute::kTotal:
+	default:
 		throw wexception("Unknown attribute value!");
 	}
-	NEVER_HERE();
 }
 
 /**
@@ -181,10 +181,9 @@ unsigned TrainingSiteDescr::get_max_level(const TrainingAttribute at) const {
 		return max_defense_;
 	case TrainingAttribute::kEvade:
 		return max_evade_;
-	case TrainingAttribute::kTotal:
+	default:
 		throw wexception("Unknown attribute value!");
 	}
-	NEVER_HERE();
 }
 
 /**
@@ -235,10 +234,9 @@ void TrainingSiteDescr::update_level(TrainingAttribute attrib,
 		max_evade_ = std::max(max_evade_, to_level);
 		train_evade_ = true;
 		return;
-	case TrainingAttribute::kTotal:
+	default:
 		throw wexception("Unknown attribute value!");
 	}
-	NEVER_HERE();
 }
 
 // TODO(sirver): This SoldierControl looks very similar to te one in
@@ -273,8 +271,6 @@ std::vector<Soldier*> TrainingSite::SoldierControl::associated_soldiers() const
 }
 
 void TrainingSite::SoldierControl::set_soldier_capacity(Quantity const capacity) {
-	assert(min_soldier_capacity() <= capacity);
-	assert(capacity <= max_soldier_capacity());
 	if (training_site_->capacity_ == capacity) {
 		return;  // Nothing to do
 	}
@@ -300,7 +296,7 @@ void TrainingSite::SoldierControl::set_soldier_capacity(Quantity const capacity)
 			training_site_->recent_capacity_increase_ = true;
 		}
 	}
-	training_site_->capacity_ = capacity;
+	training_site_->capacity_ = std::min(capacity, max_soldier_capacity());
 	training_site_->update_soldier_request(false);
 }
 
@@ -635,8 +631,9 @@ void TrainingSite::update_soldier_request(bool did_incorporate) {
 			recent_capacity_increase_ = false;
 		}
 
-		soldier_request_ = new Request(
-		   *this, owner().tribe().soldier(), TrainingSite::request_soldier_callback, wwWORKER);
+		soldier_request_ = new SoldierRequest(
+		   *this, owner().tribe().soldier(), TrainingSite::request_soldier_callback, wwWORKER,
+		   requesting_weak_trainees_ ? SoldierPreference::kRookies : SoldierPreference::kHeroes);
 
 		RequireOr r;
 
@@ -806,17 +803,17 @@ void TrainingSite::drop_stalled_soldiers(Game& /* game */) {
 					//  - is below maximum, and
 					//  - is not in a stalled state
 					// Check done separately for each art.
-					int32_t level = soldier->get_level(upgrade.attribute);
+					const unsigned level = soldier->get_level(upgrade.attribute);
 
 					// Below maximum -check
-					if (level > upgrade.max) {
+					if (static_cast<int32_t>(level) > upgrade.max) {
 						continue;
 					}
 
 					TypeAndLevel train_tl(upgrade.attribute, level);
 					TrainFailCount::iterator tstep = training_failure_count_.find(train_tl);
 					if (tstep == training_failure_count_.end()) {
-						log_warn("TrainingSite::drop_stalled_soldiers: training step %d,%d "
+						log_warn("TrainingSite::drop_stalled_soldiers: training step %u,%u "
 						         "not found in this school!\n",
 						         static_cast<unsigned int>(upgrade.attribute), level);
 						break;
