@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2024 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,7 @@
 
 #include "logic/map_objects/tribes/warehouse.h"
 
+#include <algorithm>
 #include <memory>
 
 #include "base/log.h"
@@ -31,15 +32,16 @@
 #include "economy/ship_fleet.h"
 #include "economy/warehousesupply.h"
 #include "economy/wares_queue.h"
-#include "graphic/text_layout.h"
 #include "logic/editor_game_base.h"
 #include "logic/game.h"
 #include "logic/map_objects/findbob.h"
 #include "logic/map_objects/findnode.h"
 #include "logic/map_objects/tribes/battle.h"
 #include "logic/map_objects/tribes/carrier.h"
+#include "logic/map_objects/tribes/militarysite.h"
 #include "logic/map_objects/tribes/requirements.h"
 #include "logic/map_objects/tribes/soldier.h"
+#include "logic/map_objects/tribes/trainingsite.h"
 #include "logic/map_objects/tribes/tribe_descr.h"
 #include "logic/map_objects/tribes/worker.h"
 #include "logic/message_queue.h"
@@ -55,17 +57,20 @@ constexpr int kFleeingUnitsCap = 500;
 // Goes through the list and removes all workers that are no longer in the
 // game.
 void remove_no_longer_existing_workers(Game& game, std::vector<OPtr<Worker>>* workers) {
-	for (std::vector<OPtr<Worker>>::iterator i = workers->begin(); i != workers->end(); ++i) {
+	std::vector<OPtr<Worker>>::iterator i = workers->begin();
+	while (i != workers->end()) {
 		if (i->get(game) == nullptr) {
-			workers->erase(i);
-			remove_no_longer_existing_workers(game, workers);
-			return;
+			i = workers->erase(i);
+		} else {
+			++i;
 		}
 	}
 }
 
 }  // namespace
 
+constexpr unsigned kDefendersKeptInside = 2;
+
 bool Warehouse::AttackTarget::can_be_attacked() const {
 	return warehouse_->descr().get_conquers() > 0;
 }
@@ -90,27 +95,22 @@ void Warehouse::AttackTarget::enemy_soldier_approaches(const Soldier& enemy) con
 		return;
 	}
 
-	DescriptionIndex const soldier_index = owner->tribe().soldier();
-	Requirements noreq;
-
-	if (warehouse_->count_workers(game, soldier_index, noreq, Match::kCompatible) == 0u) {
+	if (warehouse_->count_all_soldiers() <= kDefendersKeptInside) {
 		return;
 	}
 
-	Soldier& defender =
-	   dynamic_cast<Soldier&>(warehouse_->launch_worker(game, soldier_index, noreq));
+	Requirements noreq;
+	Soldier& defender = warehouse_->launch_soldier(game, noreq, true);
 	defender.start_task_defense(game, false);
 }
 
 AttackTarget::AttackResult Warehouse::AttackTarget::attack(Soldier* enemy) const {
 	Player* owner = warehouse_->get_owner();
 	Game& game = dynamic_cast<Game&>(owner->egbase());
-	DescriptionIndex const soldier_index = owner->tribe().soldier();
 	Requirements noreq;
 
-	if (warehouse_->count_workers(game, soldier_index, noreq, Match::kCompatible) != 0u) {
-		Soldier& defender =
-		   dynamic_cast<Soldier&>(warehouse_->launch_worker(game, soldier_index, noreq));
+	if (warehouse_->count_all_soldiers() != 0u) {
+		Soldier& defender = warehouse_->launch_soldier(game, noreq, true);
 		defender.start_task_defense(game, true);
 		enemy->send_signal(game, "sleep");
 		return AttackTarget::AttackResult::DefenderLaunched;
@@ -176,6 +176,8 @@ void WarehouseSupply::set_economy(Economy* const e, WareWorker type) {
 				}
 			}
 			break;
+		default:
+			NEVER_HERE();
 		}
 	}
 
@@ -197,6 +199,8 @@ void WarehouseSupply::set_economy(Economy* const e, WareWorker type) {
 				}
 			}
 			break;
+		default:
+			NEVER_HERE();
 		}
 		ec->add_supply(*this);
 	}
@@ -309,6 +313,13 @@ WareInstance& WarehouseSupply::launch_ware(Game& game, const Request& req) {
 
 /// Launch a ware as worker.
 Worker& WarehouseSupply::launch_worker(Game& game, const Request& req) {
+	if (req.get_index() == warehouse_->owner().tribe().soldier()) {
+		SoldierPreference pref = SoldierPreference::kAny;
+		if (upcast(const SoldierRequest, sr, &req); sr != nullptr) {
+			pref = sr->get_preference();
+		}
+		return warehouse_->launch_soldier(game, req.get_requirements(), false, pref);
+	}
 	return warehouse_->launch_worker(game, req.get_index(), req.get_requirements());
 }
 
@@ -324,8 +335,9 @@ Warehouse Building
  */
 WarehouseDescr::WarehouseDescr(const std::string& init_descname,
                                const LuaTable& table,
+                               const std::vector<std::string>& attribs,
                                Descriptions& descriptions)
-   : BuildingDescr(init_descname, MapObjectType::WAREHOUSE, table, descriptions) {
+   : BuildingDescr(init_descname, MapObjectType::WAREHOUSE, table, attribs, descriptions) {
 	heal_per_second_ = table.get_int("heal_per_second");
 
 	if (table.has_key("conquers")) {
@@ -340,13 +352,9 @@ WarehouseDescr::WarehouseDescr(const std::string& init_descname,
 
 std::vector<Soldier*> Warehouse::SoldierControl::present_soldiers() const {
 	std::vector<Soldier*> rv;
-	DescriptionIndex const soldier_index = warehouse_->owner().tribe().soldier();
-	IncorporatedWorkers::const_iterator sidx = warehouse_->incorporated_workers_.find(soldier_index);
-
-	if (sidx != warehouse_->incorporated_workers_.end()) {
-		const WorkerList& soldiers = sidx->second;
-		for (OPtr<Worker> temp_soldier : soldiers) {
-			rv.push_back(dynamic_cast<Soldier*>(temp_soldier.get(warehouse_->get_owner()->egbase())));
+	if (!warehouse_->incorporated_soldiers_.empty()) {
+		for (OPtr<Soldier> temp_soldier : warehouse_->incorporated_soldiers_) {
+			rv.push_back(temp_soldier.get(warehouse_->get_owner()->egbase()));
 		}
 	}
 	return rv;
@@ -360,7 +368,7 @@ std::vector<Soldier*> Warehouse::SoldierControl::stationed_soldiers() const {
 
 std::vector<Soldier*> Warehouse::SoldierControl::associated_soldiers() const {
 	std::vector<Soldier*> soldiers = stationed_soldiers();
-	Request* sr = warehouse_->soldier_request_.get_request();
+	Request* sr = warehouse_->soldier_request_manager_.get_request();
 	if (sr != nullptr) {
 		for (const Transfer* t : sr->get_transfers()) {
 			Soldier& s = dynamic_cast<Soldier&>(*t->get_worker());
@@ -392,20 +400,21 @@ void Warehouse::SoldierControl::drop_soldier(Soldier& /* soldier */) {
 }
 
 int Warehouse::SoldierControl::outcorporate_soldier(Soldier& soldier) {
-	DescriptionIndex const soldier_index = warehouse_->owner().tribe().soldier();
-	if (warehouse_->incorporated_workers_.count(soldier_index) != 0u) {
-		WorkerList& soldiers = warehouse_->incorporated_workers_[soldier_index];
-
-		WorkerList::iterator i = std::find(soldiers.begin(), soldiers.end(), &soldier);
-
-		soldiers.erase(i);
-		warehouse_->supply_->remove_workers(soldier_index, 1);
-	}
+	if (!warehouse_->incorporated_soldiers_.empty()) {
+		SoldierList::iterator i = std::find(warehouse_->incorporated_soldiers_.begin(),
+		                                    warehouse_->incorporated_soldiers_.end(), &soldier);
+		if (i != warehouse_->incorporated_soldiers_.end()) {
+			warehouse_->incorporated_soldiers_.erase(i);
+			warehouse_->supply_->remove_workers(warehouse_->owner().tribe().soldier(), 1);
+		}
 #ifndef NDEBUG
-	else {
-		throw wexception("outcorporate_soldier: soldier not in this warehouse!");
-	}
+		else {
+			throw wexception("outcorporate_soldier: soldier not in this warehouse!");
+		}
+	} else {
+		throw wexception("outcorporate_soldier: no soldiers in this warehouse!");
 #endif
+	}
 	return 0;
 }
 
@@ -418,7 +427,7 @@ Warehouse::Warehouse(const WarehouseDescr& warehouse_descr)
    : Building(warehouse_descr),
      attack_target_(this),
      soldier_control_(this),
-     soldier_request_(
+     soldier_request_manager_(
         *this,
         SoldierPreference::kAny /* no exchange by default */,
         Warehouse::request_soldier_callback,
@@ -552,12 +561,14 @@ bool Warehouse::init(EditorGameBase& egbase) {
 	Player* player = get_owner();
 
 	init_containers(*player);
-	warehouse_name_ = player->pick_warehousename(descr().get_isport());
+	warehouse_name_ =
+	   player->pick_warehousename(descr().get_isport() ? Player::WarehouseNameType::kPort :
+	                                                     Player::WarehouseNameType::kWarehouse);
 
 	set_seeing(true);
 
 	// Garrisons should be treated as more important than militarysites
-	set_priority(wwWORKER, player->tribe().soldier(), WarePriority::kHigh);
+	set_priority(wwWORKER, player->tribe().soldier(), WarePriority::kHigh, 0);
 
 	// Even though technically, a warehouse might be completely empty,
 	// we let warehouse see always for simplicity's sake (since there's
@@ -624,6 +635,11 @@ bool Warehouse::init(EditorGameBase& egbase) {
 void Warehouse::set_warehouse_name(const std::string& name) {
 	warehouse_name_ = name;
 	get_owner()->reserve_warehousename(name);
+
+	// Line breaks mess up the type icons in the census strings and push the garrison strings
+	// down where they're covered by the flag, so we replace spaces with non-breaking ones.
+	// The names will be richtext escaped for safety, so we can't use "&nbsp;".
+	replace_all(warehouse_name_, " ", " ");
 }
 
 void Warehouse::init_containers(const Player& player) {
@@ -753,6 +769,7 @@ void Warehouse::cleanup(EditorGameBase& egbase) {
 	// This will launch all workers including incorporated ones up to kFleeingUnitsCap and then empty
 	// the stock.
 	if (upcast(Game, game, &egbase)) {
+		set_desired_soldier_count(0);  // allow garrisoned soldiers to flee
 		const WareList& workers = get_workers();
 		for (DescriptionIndex id = 0; id < workers.get_nrwareids(); ++id) {
 			// If the game is running, have the workers flee the warehouse.
@@ -770,6 +787,7 @@ void Warehouse::cleanup(EditorGameBase& egbase) {
 		}
 	}
 	incorporated_workers_.clear();
+	incorporated_soldiers_.clear();
 
 	while (!planned_workers_.empty()) {
 		planned_workers_.back().cleanup();
@@ -829,20 +847,17 @@ void Warehouse::act(Game& game, uint32_t const data) {
 	if (next_military_act_ <= gametime) {
 		DescriptionIndex const soldier_index = owner().tribe().soldier();
 
-		if (incorporated_workers_.count(soldier_index) != 0u) {
-			WorkerList& soldiers = incorporated_workers_[soldier_index];
-
+		if (!incorporated_soldiers_.empty()) {
 			unsigned total_heal = descr().get_heal_per_second();
 			// Using an explicit iterator, as we plan to erase some
 			// of those guys
-			for (WorkerList::iterator it = soldiers.begin(); it != soldiers.end(); ++it) {
-				// This is a safe cast: we know only soldiers can land in this
-				// slot in the incorporated array
-				Soldier* soldier = dynamic_cast<Soldier*>(it->get(game));
+			for (SoldierList::iterator it = incorporated_soldiers_.begin();
+			     it != incorporated_soldiers_.end(); ++it) {
+				Soldier* soldier = it->get(game);
 
 				//  Soldier dead ...
 				if ((soldier == nullptr) || soldier->get_current_health() == 0) {
-					it = soldiers.erase(it);
+					it = incorporated_soldiers_.erase(it);
 					supply_->remove_workers(soldier_index, 1);
 					continue;
 				}
@@ -858,7 +873,7 @@ void Warehouse::act(Game& game, uint32_t const data) {
 
 	if (gametime > next_swap_soldiers_time_) {
 		next_swap_soldiers_time_ = gametime + kSoldierSwapTime;
-		soldier_request_.update();
+		soldier_request_manager_.update();
 	}
 
 	if (next_stock_remove_act_ <= gametime) {
@@ -893,7 +908,7 @@ void Warehouse::set_economy(Economy* const e, WareWorker type) {
 	}
 	supply_->set_economy(e, type);
 	Building::set_economy(e, type);
-	soldier_request_.set_economy(e, type);
+	soldier_request_manager_.set_economy(e, type);
 
 	for (const PlannedWorkers& pw : planned_workers_) {
 		for (Request* req : pw.requests) {
@@ -924,6 +939,10 @@ PlayerImmovable::Workers Warehouse::get_incorporated_workers() {
 			all_workers.push_back(worker.get(get_owner()->egbase()));
 		}
 	}
+	// Soldiers are stored separately
+	for (OPtr<Soldier> soldier : incorporated_soldiers_) {
+		all_workers.push_back(soldier.get(get_owner()->egbase()));
+	}
 	return all_workers;
 }
 
@@ -971,6 +990,10 @@ Quantity Warehouse::count_workers(const Game& game,
                                   DescriptionIndex worker_id,
                                   const Requirements& req,
                                   Match exact) {
+	if (worker_id == owner().tribe().soldier()) {
+		return count_soldiers(game, req);
+	}
+
 	Quantity sum = 0;
 
 	do {
@@ -996,9 +1019,26 @@ Quantity Warehouse::count_workers(const Game& game,
 	return sum;
 }
 
+Quantity Warehouse::count_soldiers(const Game& game, const Requirements& req) {
+	Quantity sum = incorporated_soldiers_.size();
+	assert(sum == supply_->stock_workers(owner().tribe().soldier()));
+
+	for (OPtr<Soldier> soldier : incorporated_soldiers_) {
+		if (!req.check(*soldier.get(game))) {
+			// This soldier doesn't match the requirements
+			--sum;
+		}
+	}
+
+	return sum;
+}
+
 /// Start a worker of a given type. The worker will
 /// be assigned a job by the caller.
 Worker& Warehouse::launch_worker(Game& game, DescriptionIndex worker_id, const Requirements& req) {
+	if (worker_id == owner().tribe().soldier()) {
+		return launch_soldier(game, req);
+	}
 	do {
 		if (supply_->stock_workers(worker_id) != 0u) {
 			uint32_t unincorporated = supply_->stock_workers(worker_id);
@@ -1049,6 +1089,111 @@ Worker& Warehouse::launch_worker(Game& game, DescriptionIndex worker_id, const R
 	throw wexception("Warehouse::launch_worker: worker does not actually exist");
 }
 
+Soldier& Warehouse::launch_soldier(Game& game,
+                                   const Requirements& req,
+                                   const bool defender,
+                                   const SoldierPreference pref) {
+	// counterpart to remove_no_longer_existing_workers()
+	std::vector<OPtr<Soldier>>::iterator it = incorporated_soldiers_.begin();
+	while (it != incorporated_soldiers_.end()) {
+		if (it->get(game) == nullptr) {
+			it = incorporated_soldiers_.erase(it);
+		} else {
+			++it;
+		}
+	}
+
+	if (incorporated_soldiers_.empty()) {
+		throw wexception("Warehouse::launch_soldier: no stored soldiers");
+	}
+
+	const DescriptionIndex soldier_index = owner().tribe().soldier();
+	assert(incorporated_soldiers_.size() == supply_->stock_workers(soldier_index));
+
+	// Make sure incorporated_soldiers_ is sorted after loading a game.
+	if (!soldiers_are_sorted_) {
+		std::sort(incorporated_soldiers_.begin(), incorporated_soldiers_.end(),
+		          [&game](OPtr<Soldier> a, OPtr<Soldier> b) {
+			          // We're sorting in decreasing order
+			          return a.get(game)->get_total_level() > b.get(game)->get_total_level();
+		          });
+		soldiers_are_sorted_ = true;
+	}
+
+	// Can't mix forward and reverse iterators, so we use indices :(
+	int i = 0;
+	int end = incorporated_soldiers_.size();
+
+	if (defender) {
+		// TODO(tothxa): This saves the strongest soldiers for the final battle.
+		//               Should it be the most preferred soldiers instead?
+		if (incorporated_soldiers_.size() > kDefendersKeptInside) {
+			i = kDefendersKeptInside;
+
+			// Try to send a fully healed soldier, fall back to highest remaining health
+			unsigned best_health = 0;
+			int best_health_index = -1;
+			Soldier* s = nullptr;
+			while (i < end) {
+				s = incorporated_soldiers_.at(i).get(game);
+				if (s->get_current_health() == s->get_max_health()) {
+					break;
+				}
+				if (s->get_current_health() > best_health) {
+					best_health = s->get_current_health();
+					best_health_index = i;
+				}
+				++i;
+			}
+			if (i == end) {
+				i = best_health_index;
+			}
+
+		} else {
+			i = incorporated_soldiers_.size() - 1;
+		}
+	} else {
+		assert(incorporated_soldiers_.size() > soldier_control_.soldier_capacity());
+		int step = 1;
+
+		if (pref == SoldierPreference::kHeroes) {
+			i = 0;
+			end = incorporated_soldiers_.size();
+		} else {
+			i = incorporated_soldiers_.size() - 1;
+			end = -1;
+			step = -1;
+		}
+		if (get_soldier_preference() != SoldierPreference::kAny) {
+			if (get_soldier_preference() == pref) {
+				i += step * soldier_control_.soldier_capacity();
+			} else {
+				end -= step * soldier_control_.soldier_capacity();
+			}
+		}
+		while (i != end) {
+			if (req.check(*incorporated_soldiers_.at(i).get(game))) {
+				break;
+			}
+			i += step;
+		}
+	}
+
+	if (i == end) {
+		throw wexception("Warehouse::launch_soldier: no stored soldier met the requirements at %s",
+		                 warehouse_name_.c_str());
+	}
+
+	Soldier* soldier = incorporated_soldiers_.at(i).get(game);
+	soldier->reset_tasks(game);   //  forget everything you did
+	soldier->set_location(this);  //  back in a economy
+	it = incorporated_soldiers_.begin();
+	it += i;
+	incorporated_soldiers_.erase(it);
+	supply_->remove_workers(soldier_index, 1);
+	return *soldier;
+}
+
 void Warehouse::incorporate_worker(EditorGameBase& egbase, Worker* w) {
 	assert(w != nullptr);
 	assert(w->get_owner() == get_owner());
@@ -1074,11 +1219,15 @@ void Warehouse::incorporate_worker(EditorGameBase& egbase, Worker* w) {
 		return;
 	}
 
-	// Incorporate the worker
-	if (incorporated_workers_.count(worker_index) == 0u) {
-		incorporated_workers_[worker_index] = WorkerList();
+	if (worker_index == owner().tribe().soldier()) {
+		incorporate_soldier_inner(egbase, dynamic_cast<Soldier*>(w));
+	} else {
+		// Incorporate the worker
+		if (incorporated_workers_.count(worker_index) == 0u) {
+			incorporated_workers_[worker_index] = WorkerList();
+		}
+		incorporated_workers_[worker_index].emplace_back(w);
 	}
-	incorporated_workers_[worker_index].push_back(w);
 
 	w->set_location(nullptr);  //  no longer in an economy
 
@@ -1089,6 +1238,21 @@ void Warehouse::incorporate_worker(EditorGameBase& egbase, Worker* w) {
 	}
 }
 
+void Warehouse::incorporate_soldier_inner(EditorGameBase& egbase, Soldier* soldier) {
+	const unsigned total_level = soldier->get_total_level();
+	if (total_level == 0 || incorporated_soldiers_.empty()) {
+		incorporated_soldiers_.emplace_back(soldier);
+		return;
+	}
+
+	// Keep them sorted in decreasing order of total level
+	auto it = incorporated_soldiers_.begin();
+	while (it != incorporated_soldiers_.end() && it->get(egbase)->get_total_level() > total_level) {
+		++it;
+	}
+	incorporated_soldiers_.emplace(it, soldier);
+}
+
 /// Create an instance of a ware and make sure it gets
 /// carried out of the warehouse.
 WareInstance& Warehouse::launch_ware(Game& game, DescriptionIndex const ware_index) {
@@ -1471,6 +1635,10 @@ void Warehouse::check_remove_stock(Game& game) {
 		if (get_worker_policy(widx) != StockPolicy::kRemove || (get_workers().stock(widx) == 0u)) {
 			continue;
 		}
+		if (widx == owner().tribe().soldier() &&
+		    get_workers().stock(widx) <= get_desired_soldier_count()) {
+			continue;
+		}
 
 		Worker& worker = launch_worker(game, widx, Requirements());
 		worker.start_task_leavebuilding(game, true);
@@ -1478,21 +1646,28 @@ void Warehouse::check_remove_stock(Game& game) {
 	}
 }
 
-InputQueue& Warehouse::inputqueue(DescriptionIndex index, WareWorker type, const Request* r) {
+InputQueue& Warehouse::inputqueue(DescriptionIndex index,
+                                  WareWorker type,
+                                  const Request* r,
+                                  uint32_t disambiguator_id) {
 	assert(portdock_ != nullptr);
 	assert(portdock_->expedition_bootstrap() != nullptr);
-	return r != nullptr ? portdock_->expedition_bootstrap()->inputqueue(*r) :
-	                      portdock_->expedition_bootstrap()->inputqueue(index, type, false);
+	return r != nullptr ?
+	          portdock_->expedition_bootstrap()->inputqueue(*r) :
+	          portdock_->expedition_bootstrap()->inputqueue(index, type, false, disambiguator_id);
 }
 
 void Warehouse::set_desired_soldier_count(Quantity q) {
-	desired_soldier_count_ = q;
-	soldier_request_.update();
+	q = std::min(q, soldier_control_.max_soldier_capacity());
+	if (q != desired_soldier_count_) {
+		desired_soldier_count_ = q;
+		soldier_request_manager_.update();
+	}
 }
 
 void Warehouse::set_soldier_preference(SoldierPreference p) {
-	soldier_request_.set_preference(p);
-	soldier_request_.update();
+	soldier_request_manager_.set_preference(p);
+	soldier_request_manager_.update();
 }
 
 void Warehouse::request_soldier_callback(Game& game,
@@ -1506,8 +1681,41 @@ void Warehouse::request_soldier_callback(Game& game,
 	wh.soldier_control_.incorporate_soldier(game, s);
 }
 
+std::string Warehouse::warehouse_census_string() const {
+	// U+2654 white chess king character
+	// "👑" U+1F451 crown character is missing from our font
+	static const std::string hq_fmt = "♔&nbsp;%s&nbsp;♔";
+
+	static const std::string port_fmt = "⚓&nbsp;%s&nbsp;⚓";  // U+2693 anchor character
+
+	// U+27F0 upwards quadruple arrow character (similar to a house)
+	// "📦" U+1F4E6 package character is missing from our font
+	static const std::string wh_fmt = "⟰&nbsp;%s&nbsp;⟰";
+
+	std::string icon_format;
+	if (descr().get_isport()) {
+		icon_format = port_fmt;
+	} else if (descr().get_conquers() > 0) {
+		icon_format = hq_fmt;
+	} else {
+		icon_format = wh_fmt;
+	}
+
+	return named_building_census_string(icon_format, get_warehouse_name());
+}
+
 void Warehouse::update_statistics_string(std::string* str) {
-	*str = richtext_escape(get_warehouse_name());
+	if (descr().get_conquers() > 0) {
+		// Port or HQ
+		if (get_desired_soldier_count() > 0) {
+			*str = soldier_control_.get_status_string(owner().tribe(), get_soldier_preference());
+		} else {
+			*str = "—";
+		}
+	} else {
+		// plain warehouse
+		str->clear();
+	}
 }
 
 std::unique_ptr<const BuildingSettings> Warehouse::create_building_settings() const {
