@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2024 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -66,13 +66,14 @@ Available actions are:
 - `callobject`_
 - `plant`_
 - `createbob`_
+- `terraform`_
 - `removeobject`_
 - `repeatsearch`_
 - `findresources`_
 - `scout`_
 - `playsound`_
 - `construct`_
-- `terraform`_
+- `script`_
 */
 
 const WorkerProgram::ParseMap WorkerProgram::parsemap_[] = {
@@ -94,6 +95,7 @@ const WorkerProgram::ParseMap WorkerProgram::parsemap_[] = {
    {"playsound", &WorkerProgram::parse_playsound},
    {"construct", &WorkerProgram::parse_construct},
    {"terraform", &WorkerProgram::parse_terraform},
+   {"script", &WorkerProgram::parse_script},
 
    {nullptr, nullptr}};
 
@@ -166,7 +168,7 @@ createware
  */
 void WorkerProgram::parse_createware(Worker::Action* act, const std::vector<std::string>& cmd) {
 	if (cmd.size() != 1) {
-		throw wexception("Usage: createware=<ware type>");
+		throw GameDataError("Usage: createware=<ware type>");
 	}
 
 	const DescriptionIndex ware_index = descriptions_.load_ware(cmd[0]);
@@ -279,16 +281,26 @@ void WorkerProgram::parse_breed(Worker::Action* act, const std::vector<std::stri
 findobject
 ^^^^^^^^^^
 .. function:: findobject=radius:\<distance\> [type:\<map_object_type\>] [attrib:\<attribute\>]
-   [no_notify]
+   [name:\<name\>] [no_notify]
 
    :arg int radius: Search for an object within the given radius around the worker.
-   :arg string type: The type of map object to search for. Defaults to ``immovable``.
+   :arg string type: The type of map object to search for.
+      The possible values are:
+
+      * ``immovable``: Only immovables with the given name or attribute are considered.
+      * ``bob``: Only critters and workers with the given name or attribute are considered.
+      * ``special``: Special objects like pinned notes might be considered.
+        (no consistency checks are implemented for this case)
+
+      Defaults to ``immovable``.
+
    :arg string attrib: The attribute that the map object should possess.
+   :arg string name: The internal name of the map object (since version 1.3)
    :arg empty no_notify: Do not send a message to the player if this step fails.
 
    Find and select an object based on a number of predicates, which can be specified
-   in arbitrary order. The object can then be used in other commands like ``walk``
-   or ``callobject``. Examples::
+   in arbitrary order. Note that the predicates ``attrib`` and ``name`` are mutually exclusive. The
+   object can then be used in other commands like ``walk`` or ``callobject``. Examples::
 
       cut_granite = {
          "findobject=attrib:rocks radius:6", -- Find rocks on the map within a radius of 6 from your
@@ -316,6 +328,7 @@ findobject
  * iparam2 = attribute predicate (if >= 0)
  * iparam3 = send message on failure (if != 0)
  * sparam1 = type
+ * sparam2 = name
  */
 void WorkerProgram::parse_findobject(Worker::Action* act, const std::vector<std::string>& cmd) {
 	act->function = &Worker::run_findobject;
@@ -323,6 +336,7 @@ void WorkerProgram::parse_findobject(Worker::Action* act, const std::vector<std:
 	act->iparam2 = -1;
 	act->iparam3 = 1;
 	act->sparam1 = "immovable";
+	act->sparam2.clear();
 
 	// Parse predicates
 	for (const std::string& argument : cmd) {
@@ -340,12 +354,33 @@ void WorkerProgram::parse_findobject(Worker::Action* act, const std::vector<std:
 			   NoteMapObjectDescription(item.second, NoteMapObjectDescription::LoadType::kAttribute));
 			act->iparam2 = MapObjectDescr::get_attribute_id(item.second);
 		} else if (item.first == "type") {
-			act->sparam1 = item.second;
+			if (item.second == "immovable" || item.second == "bob" || item.second == "special") {
+				act->sparam1 = item.second;
+			} else {
+				throw GameDataError("Invalid usage of 'type' predicate: Possible values are "
+				                    "'immovable', 'bob' and 'special'.");
+			}
+		} else if (item.first == "name") {
+			act->sparam2 = item.second;
 		} else {
 			throw GameDataError("Unknown findobject predicate %s", argument.c_str());
 		}
 	}
 
+	if (!act->sparam2.empty()) {
+		if (act->iparam2 >= 0) {
+			throw GameDataError("Invalid usage of findobject predicates: 'attrib' and 'name' are not "
+			                    "to be used together.");
+		}
+		if (act->sparam1 == "immovable" || act->sparam1 == "bob") {
+			Notifications::publish(NoteMapObjectDescription(
+			   act->sparam2, NoteMapObjectDescription::LoadType::kObject, false));
+			needed_named_map_objects_.insert(std::make_pair(
+			   act->sparam1 == "immovable" ? MapObjectType::IMMOVABLE : MapObjectType::BOB,
+			   act->sparam2));
+		}
+	}
+
 	if (act->iparam2 >= 0) {
 		needed_attributes_.insert(
 		   std::make_pair(act->sparam1 == "immovable" ? MapObjectType::IMMOVABLE : MapObjectType::BOB,
@@ -686,6 +721,11 @@ void WorkerProgram::parse_callobject(Worker::Action* act, const std::vector<std:
 	if (!needed_attributes_.empty()) {
 		collected_attributes_.insert(needed_attributes_.begin(), needed_attributes_.end());
 	}
+	// same for the mapobjects found by name
+	if (!needed_named_map_objects_.empty()) {
+		collected_named_map_objects_.insert(
+		   needed_named_map_objects_.begin(), needed_named_map_objects_.end());
+	}
 }
 
 /* RST
@@ -861,6 +901,11 @@ void WorkerProgram::parse_removeobject(Worker::Action* act,
 	if (!needed_attributes_.empty()) {
 		collected_attributes_.insert(needed_attributes_.begin(), needed_attributes_.end());
 	}
+	// same for the mapobjects found by name
+	if (!needed_named_map_objects_.empty()) {
+		collected_named_map_objects_.insert(
+		   needed_named_map_objects_.begin(), needed_named_map_objects_.end());
+	}
 }
 
 /* RST
@@ -1025,4 +1070,16 @@ void WorkerProgram::parse_construct(Worker::Action* act, const std::vector<std::
 
 	act->function = &Worker::run_construct;
 }
+
+/* RST
+script
+^^^^^^
+Runs a Lua function. See :ref:`map_object_programs_script`.
+*/
+void WorkerProgram::parse_script(Worker::Action* act, const std::vector<std::string>& cmd) {
+	RunScriptParameters parameters = MapObjectProgram::parse_act_script(cmd);
+
+	act->sparam1 = parameters.function;
+	act->function = &Worker::run_script;
+}
 }  // namespace Widelands
