@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,17 +31,29 @@ struct Message {
 	enum class Type : uint8_t {
 		kNoMessages,
 		kAllMessages,
+
 		kGameLogic,
+
 		kGeologists,
+
 		kScenario,
+
 		kSeafaring,
 		kEconomy,              // economy
 		kEconomySiteOccupied,  // economy
-		kWarfare,              // everything starting from here is warfare
+		kWarfare,              // warfare messages
 		kWarfareSiteDefeated,
 		kWarfareSiteLost,
 		kWarfareUnderAttack,
-		kTradeOfferReceived,
+		kWarfareEnd = kWarfareUnderAttack,  // end of warfare messages
+
+		kDiplomacyBegin,
+		kDiplomacy = kDiplomacyBegin,
+		kTrading,
+		kDiplomacyEnd = kTrading,
+
+		kEconomyLoadGame = 255,  // Only this type is allowed in game loading code,
+		                         // must not be used elsewhere.
 	};
 
 	/**
@@ -81,37 +93,37 @@ struct Message {
 	     status_(s) {
 	}
 
-	Message::Type type() const {
+	[[nodiscard]] Message::Type type() const {
 		return type_;
 	}
-	const std::string& sub_type() const {
+	[[nodiscard]] const std::string& sub_type() const {
 		return sub_type_;
 	}
-	const Time& sent() const {
+	[[nodiscard]] const Time& sent() const {
 		return sent_;
 	}
-	const std::string& title() const {
+	[[nodiscard]] const std::string& title() const {
 		return title_;
 	}
-	const std::string& icon_filename() const {
+	[[nodiscard]] const std::string& icon_filename() const {
 		return icon_filename_;
 	}
-	const Image* icon() const {
+	[[nodiscard]] const Image* icon() const {
 		return icon_;
 	}
-	const std::string& heading() const {
+	[[nodiscard]] const std::string& heading() const {
 		return heading_;
 	}
-	const std::string& body() const {
+	[[nodiscard]] const std::string& body() const {
 		return body_;
 	}
-	const Widelands::Coords& position() const {
+	[[nodiscard]] const Widelands::Coords& position() const {
 		return position_;
 	}
-	Widelands::Serial serial() const {
+	[[nodiscard]] Widelands::Serial serial() const {
 		return serial_;
 	}
-	Status status() const {
+	[[nodiscard]] Status status() const {
 		return status_;
 	}
 	Status set_status(Status const s) {
@@ -121,17 +133,27 @@ struct Message {
 	/**
 	 * Returns the main type for the message's sub type
 	 */
-	Message::Type message_type_category() const {
-		if (type_ >= Widelands::Message::Type::kWarfare) {
+	[[nodiscard]] Message::Type message_type_category() const {
+		if (type_ >= Widelands::Message::Type::kWarfare &&
+		    type_ <= Widelands::Message::Type::kWarfareEnd) {
 			return Widelands::Message::Type::kWarfare;
-
-		} else if (type_ >= Widelands::Message::Type::kEconomy &&
-		           type_ <= Widelands::Message::Type::kEconomySiteOccupied) {
+		}
+		if (type_ >= Widelands::Message::Type::kDiplomacyBegin &&
+		    type_ <= Widelands::Message::Type::kDiplomacyEnd) {
+			return Widelands::Message::Type::kDiplomacy;
+		}
+		if (type_ == Widelands::Message::Type::kEconomy ||
+		    type_ == Widelands::Message::Type::kEconomySiteOccupied ||
+		    type_ == Widelands::Message::Type::kEconomyLoadGame) {
 			return Widelands::Message::Type::kEconomy;
 		}
 		return type_;
 	}
 
+	[[nodiscard]] bool allowed_during_game_loading() const {
+		return type_ == Widelands::Message::Type::kEconomyLoadGame;
+	}
+
 private:
 	Message::Type type_;
 	const std::string sub_type_;
@@ -145,6 +167,7 @@ private:
 	Widelands::Serial serial_;  // serial to map object
 	Status status_;
 };
+
 }  // namespace Widelands
 
 #endif  // end of include guard: WL_LOGIC_MESSAGE_H
