@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2021-2022 by the Widelands Development Team
+ * Copyright (C) 2021-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,14 +30,15 @@ namespace AddOns {
 class MutableAddOn {
 public:
 	explicit MutableAddOn(const AddOnInfo& a);
-	virtual ~MutableAddOn() {
-	}
+	virtual ~MutableAddOn() = default;
 	// Creates an addon with its type matching its category
 	static std::unique_ptr<MutableAddOn> create_mutable_addon(const AddOnInfo& a);
 	void update_info(const std::string& descname,
 	                 const std::string& author,
 	                 const std::string& description,
-	                 const std::string& version);
+	                 const std::string& version,
+	                 const std::string& min_wl_version,
+	                 const std::string& max_wl_version);
 	using ProgressFunction = std::function<void(size_t)>;
 	void set_callbacks(const ProgressFunction& init, const ProgressFunction& progress) {
 		callback_init_ = init;
@@ -46,31 +47,40 @@ public:
 	// May throw a WLWarning, if it fails
 	virtual bool write_to_disk();
 
-	const std::string& get_internal_name() const {
+	[[nodiscard]] const std::string& get_internal_name() const {
 		return internal_name_;
 	}
-	const std::string& get_descname() const {
+	[[nodiscard]] const std::string& get_descname() const {
 		return descname_;
 	}
-	const std::string& get_description() const {
+	[[nodiscard]] const std::string& get_description() const {
 		return description_;
 	}
-	const std::string& get_author() const {
+	[[nodiscard]] const std::string& get_author() const {
 		return author_;
 	}
-	const std::string& get_version() const {
+	[[nodiscard]] const std::string& get_version() const {
 		return version_;
 	}
-	const std::string& get_min_wl_version() const {
+	[[nodiscard]] const std::string& get_min_wl_version() const {
 		return min_wl_version_;
 	}
-	const std::string& get_max_wl_version() const {
+	[[nodiscard]] const std::string& get_max_wl_version() const {
 		return max_wl_version_;
 	}
 	void set_version(const std::string& version) {
 		version_ = version;
 	}
-	AddOnCategory get_category() const {
+	void set_min_wl_version(const std::string& min_wl_version) {
+		min_wl_version_ = min_wl_version;
+	}
+	void set_max_wl_version(const std::string& max_wl_version) {
+		max_wl_version_ = max_wl_version;
+	}
+	void set_force_sync_safe(bool fss) {
+		force_sync_safe_ = fss;
+	}
+	[[nodiscard]] AddOnCategory get_category() const {
 		return category_;
 	}
 
@@ -81,15 +91,21 @@ protected:
 	void cleanup_temp_dir();
 	size_t do_recursively_copy_file_or_directory(const std::string& source,
 	                                             const std::string& dest,
-	                                             const bool dry_run);
+	                                             bool dry_run);
 
-	std::string internal_name_, descname_, description_, author_, version_, min_wl_version_,
-	   max_wl_version_;
+	std::string internal_name_;
+	std::string descname_;
+	std::string description_;
+	std::string author_;
+	std::string version_;
+	std::string min_wl_version_;
+	std::string max_wl_version_;
 	AddOnCategory category_;
 	/*
 	 * The `sync_safe` property is not stored here because all new or updated add-ons
 	 * should by default be marked as sync-unsafe until approved by a reviewer.
 	 */
+	bool force_sync_safe_;
 
 	std::string directory_, backup_path_;
 
@@ -124,12 +140,15 @@ public:
 		return &tree_;
 	}
 
+	static const AddOnVersion kNoVersionRequirement;
+	AddOnVersion detect_min_wl_version(const DirectoryTree* start = nullptr) const;
+
 	std::map<std::string, unsigned> count_all_dirnames(const DirectoryTree* start = nullptr) const;
 
 	void set_dirname(const std::string& dir, const std::string& name) {
 		dirnames_[dir] = name;
 	}
-	std::string get_dirname(const std::string& dir) const {
+	[[nodiscard]] std::string get_dirname(const std::string& dir) const {
 		const auto it = dirnames_.find(dir);
 		return it != dirnames_.end() ? it->second : "";
 	}
@@ -190,6 +209,16 @@ private:
 	CampaignInfo metadata_;
 };
 
+class MapGenAddon : public MutableAddOn {
+public:
+	using MutableAddOn::MutableAddOn;
+};
+
+class UIPluginAddon : public MutableAddOn {
+public:
+	using MutableAddOn::MutableAddOn;
+};
+
 class WinCondAddon : public MutableAddOn {
 public:
 	using MutableAddOn::MutableAddOn;
