@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007-2024 by the Widelands Development Team
+ * Copyright (C) 2007-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,7 +24,8 @@
 #include "base/random.h"
 #include "base/time_string.h"
 #include "base/wexception.h"
-#include "build_info.h"
+#include "commands/cmd_replay_sync_read.h"
+#include "commands/cmd_replay_sync_write.h"
 #include "game_io/game_loader.h"
 #include "game_io/game_preload_packet.h"
 #include "io/filesystem/filesystem.h"
@@ -35,9 +36,7 @@
 #include "logic/game.h"
 #include "logic/game_controller.h"
 #include "logic/game_data_error.h"
-#include "logic/playercommand.h"
 #include "logic/save_handler.h"
-#include "ui_basic/messagebox.h"
 #include "wui/interactive_base.h"
 
 namespace Widelands {
@@ -53,64 +52,9 @@ static inline void delete_temp_file(const std::string& temp_file) {
 // File format definitions
 constexpr uint32_t kReplayMagic = 0x2E21A102;
 constexpr uint8_t kCurrentPacketVersion = 4;
-constexpr Duration kSyncInterval(200);
 
 enum { pkt_end = 2, pkt_playercommand = 3, pkt_syncreport = 4 };
 
-class CmdReplaySyncRead : public Command {
-public:
-	CmdReplaySyncRead(const Time& init_duetime, crypto::MD5Checksum hash)
-	   : Command(init_duetime), hash_(hash) {
-	}
-
-	[[nodiscard]] QueueCommandTypes id() const override {
-		return QueueCommandTypes::kReplaySyncRead;
-	}
-
-	void execute(Game& game) override {
-		if (reported_desync_for_ == &game) {
-			// We already know there was a desync
-			return;
-		}
-
-		const crypto::MD5Checksum myhash = game.get_sync_hash();
-
-		if (hash_ != myhash) {
-			reported_desync_for_ = &game;
-			log_err_time(game.get_gametime(),
-			             "REPLAY: Lost synchronization at time %u\n"
-			             "I have:     %s\n"
-			             "Replay has: %s\n",
-			             duetime().get(), myhash.str().c_str(), hash_.str().c_str());
-
-			// In case syncstream logging is on, save it for analysis
-			game.save_syncstream(true);
-
-			// There has to be a better way to do this.
-			game.game_controller()->set_desired_speed(0);
-
-			UI::WLMessageBox m(
-			   game.get_ibase(), UI::WindowStyle::kWui, _("Desync"),
-			   format(_("The replay has desynced and the game was paused.\n"
-			            "You are probably watching a replay created with another version of "
-			            "Widelands, which is not supported.\n\n"
-			            "If you are certain that the replay was created with the same version "
-			            "of Widelands, %1$s, please report this problem as a bug.\n"
-			            "You will find related messages in the standard output (stdout.txt on "
-			            "Windows). Please add this information to your report."),
-			          build_ver_details()),
-			   UI::WLMessageBox::MBoxType::kOk);
-			m.run<UI::Panel::Returncodes>();
-		}
-	}
-
-private:
-	crypto::MD5Checksum hash_;
-
-	static const Game* reported_desync_for_;
-};
-const Game* CmdReplaySyncRead::reported_desync_for_(nullptr);
-
 /**
  * Load the savegame part of the given replay and open the command log.
  */
@@ -237,28 +181,6 @@ bool ReplayReader::end_of_replay() {
 	return cmdlog_ == nullptr;
 }
 
-/**
- * Command / timer that regularly inserts synchronization hashes into
- * the replay.
- */
-class CmdReplaySyncWrite : public Command {
-public:
-	explicit CmdReplaySyncWrite(const Time& init_duetime) : Command(init_duetime) {
-	}
-
-	[[nodiscard]] QueueCommandTypes id() const override {
-		return QueueCommandTypes::kReplaySyncWrite;
-	}
-
-	void execute(Game& game) override {
-		if (ReplayWriter* const rw = game.get_replaywriter()) {
-			rw->send_sync(game.get_sync_hash());
-
-			game.enqueue_command(new CmdReplaySyncWrite(duetime() + kSyncInterval));
-		}
-	}
-};
-
 /**
  * Start a replay at the given filename (the caller must add the suffix).
  *
@@ -304,7 +226,7 @@ ReplayWriter::ReplayWriter(Game& game, const std::string& filename)
 	verb_log_info("Done reloading the game from replay");
 	delete_temp_file(temp_savegame);
 
-	game.enqueue_command(new CmdReplaySyncWrite(game.get_gametime() + kSyncInterval));
+	game.enqueue_command(new CmdReplaySyncWrite(game.get_gametime() + kReplaySyncInterval));
 
 	game.rng().write_state(*cmdlog_);
 }
