@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015-2022 by the Widelands Development Team
+ * Copyright (C) 2015-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,19 +22,15 @@
 
 #include <SDL_timer.h>
 
+#include "commands/cmd_replay_end.h"
 #include "logic/game.h"
+#include "logic/playersmanager.h"
 #include "logic/replay.h"
-#include "ui_basic/messagebox.h"
 #include "wui/interactive_base.h"
 
-ReplayGameController::ReplayGameController(Widelands::Game& game, const std::string& filename)
-   : game_(game),
-     lastframe_(SDL_GetTicks()),
-     time_(game_.get_gametime()),
-     speed_(1000),
-     paused_(false) {
-	game_.set_game_controller(std::shared_ptr<ReplayGameController>(this));
-	replayreader_.reset(new Widelands::ReplayReader(game_, filename));
+ReplayGameController::ReplayGameController(Widelands::Game& game)
+   : game_(game), lastframe_(SDL_GetTicks()), time_(game_.get_gametime()) {
+	replayreader_.reset(new Widelands::ReplayReader(game_, game_.replay_filename()));
 }
 
 void ReplayGameController::think() {
@@ -59,8 +55,9 @@ void ReplayGameController::think() {
 		}
 
 		if (replayreader_->end_of_replay()) {
+			time_ = game_.get_gametime();
 			replayreader_.reset(nullptr);
-			game_.enqueue_command(new CmdReplayEnd(time_ = game_.get_gametime()));
+			game_.enqueue_command(new Widelands::CmdReplayEnd(time_));
 		}
 	}
 }
@@ -97,20 +94,15 @@ void ReplayGameController::set_paused(bool const paused) {
 	paused_ = paused;
 }
 
-void ReplayGameController::CmdReplayEnd::execute(Widelands::Game& game) {
-	game.game_controller()->set_desired_speed(0);
-
-	// Need to pull this out into a variable to make the includes script happy
-	InteractiveBase* i = game.get_ibase();
-	assert(i);
-	UI::WLMessageBox mmb(i, UI::WindowStyle::kWui, _("End of Replay"),
-	                     _("The end of the replay has been reached and the game has "
-	                       "been paused. You may unpause the game and continue watching "
-	                       "if you want to."),
-	                     UI::WLMessageBox::MBoxType::kOk);
-	mmb.run<UI::Panel::Returncodes>();
-}
-
-Widelands::QueueCommandTypes ReplayGameController::CmdReplayEnd::id() const {
-	return Widelands::QueueCommandTypes::kReplayEnd;
+void ReplayGameController::report_result(uint8_t p_nr,
+                                         Widelands::PlayerEndResult result,
+                                         const std::string& info) {
+	Widelands::PlayerEndStatus pes;
+	Widelands::Player* player = game_.get_player(p_nr);
+	assert(player != nullptr);
+	pes.player = player->player_number();
+	pes.time = game_.get_gametime();
+	pes.result = result;
+	pes.info = info;
+	game_.player_manager()->add_player_end_status(pes);
 }
