@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015-2022 by the Widelands Development Team
+ * Copyright (C) 2015-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,16 +21,16 @@
 
 #include <memory>
 
-#include "logic/cmd_queue.h"
+#include "commands/command.h"
 #include "logic/game_controller.h"
 
 namespace Widelands {
 class ReplayReader;
-}
+}  // namespace Widelands
 
 class ReplayGameController : public GameController {
 public:
-	ReplayGameController(Widelands::Game& game, const std::string& filename);
+	explicit ReplayGameController(Widelands::Game& game);
 
 	void think() override;
 
@@ -39,24 +39,22 @@ public:
 	GameController::GameType get_game_type() override;
 	uint32_t real_speed() override;
 	uint32_t desired_speed() override;
-	void set_desired_speed(uint32_t const speed) override;
+	void set_desired_speed(uint32_t speed) override;
 	bool is_paused() override;
-	void set_paused(bool const paused) override;
+	void set_paused(bool paused) override;
+	void
+	report_result(uint8_t p_nr, Widelands::PlayerEndResult result, const std::string& info) override;
+	void set_write_replay(bool /* replay */) override {
+		NEVER_HERE();
+	}
 
 private:
-	struct CmdReplayEnd : public Widelands::Command {
-		explicit CmdReplayEnd(const Time& init_duetime) : Widelands::Command(init_duetime) {
-		}
-		void execute(Widelands::Game& game) override;
-		Widelands::QueueCommandTypes id() const override;
-	};
-
 	Widelands::Game& game_;
 	std::unique_ptr<Widelands::ReplayReader> replayreader_;
 	int32_t lastframe_;
 	Time time_;
-	uint32_t speed_;
-	bool paused_;
+	uint32_t speed_{1000};
+	bool paused_{false};
 };
 
 #endif  // end of include guard: WL_LOGIC_REPLAY_GAME_CONTROLLER_H
