@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,26 +28,12 @@
 #include "game_io/game_saver.h"
 #include "io/filesystem/filesystem.h"
 #include "io/filesystem/filesystem_exceptions.h"
-#include "logic/filesystem_constants.h"
 #include "logic/game.h"
 #include "logic/game_controller.h"
 #include "logic/generic_save_handler.h"
 #include "wlapplication_options.h"
 #include "wui/interactive_base.h"
 
-SaveHandler::SaveHandler()
-   : next_save_realtime_(0),
-     last_save_realtime_(0),
-     initialized_(false),
-     allow_saving_(true),
-     save_requested_(false),
-     saving_next_tick_(false),
-     autosave_filename_(kAutosavePrefix),
-     fs_type_(FileSystem::ZIP),
-     autosave_interval_in_ms_(kDefaultAutosaveInterval * 60 * 1000),
-     number_of_rolls_(5) {
-}
-
 bool SaveHandler::roll_save_files(const std::string& filename, std::string* const error) const {
 	int32_t rolls = 0;
 	std::string filename_previous;
@@ -115,8 +101,9 @@ bool SaveHandler::check_next_tick(Widelands::Game& game, uint32_t realtime) cons
 		return false;  // no autosave or not due, yet
 	}
 
-	// check if game is paused (in any way)
-	if (game.game_controller()->is_paused_or_zero_speed()) {
+	// Prevents unnecessary autosaves while the game is paused, but allows the first autosave
+	// during the pause if there had been progress before pausing.
+	if (next_save_min_gametime_ > game.get_gametime()) {
 		return false;
 	}
 
@@ -138,7 +125,8 @@ void SaveHandler::think(Widelands::Game& game) {
 	}
 
 	const uint32_t realtime = SDL_GetTicks();
-	initialize(realtime);
+	initialize(game, realtime);
+	bool force_skip = false;
 
 	// Are we saving now?
 	if (saving_next_tick_ || save_requested_) {
@@ -157,17 +145,25 @@ void SaveHandler::think(Widelands::Game& game) {
 			save_filename_ = "";
 		} else {
 			// Autosave ...
-			save_success = roll_save_files(filename, &error);
-			if (save_success) {
-				filename = format("%s_00", autosave_filename_);
-				verb_log_info_time(game.get_gametime(), "Autosave: saving as %s\n", filename.c_str());
+			if (skip_when_inactive_ && UI::Panel::time_of_last_user_activity() < last_save_realtime_) {
+				verb_log_info_time(game.get_gametime(), "Autosave: Skipping due to user inactivity");
+				last_save_realtime_ = realtime;
+				force_skip = true;
+				game.get_ibase()->log_message(_("Saving skipped"));
+			} else {
+				save_success = roll_save_files(filename, &error);
+				if (save_success) {
+					filename = format("%s_00", autosave_filename_);
+					verb_log_info_time(
+					   game.get_gametime(), "Autosave: saving as %s\n", filename.c_str());
+				}
 			}
 		}
 
-		if (save_success) {
+		if (save_success && !force_skip) {
 			// Saving now (always overwrite file)
 			std::string complete_filename = create_file_name(kSaveDir, filename);
-			save_success = save_game(game, complete_filename, &error);
+			save_success = save_game(game, complete_filename, std::nullopt, &error);
 		}
 		if (!save_success) {
 			log_err_time(game.get_gametime(), "Autosave: ERROR! - %s\n", error.c_str());
@@ -182,10 +178,13 @@ void SaveHandler::think(Widelands::Game& game) {
 		// This prevents us from going into endless autosave cycles if the save
 		// should take longer than the autosave interval.
 		next_save_realtime_ = SDL_GetTicks() + autosave_interval_in_ms_;
+		next_save_min_gametime_ = game.get_gametime() + autosave_gametime_interval_;
 
-		verb_log_info_time(
-		   game.get_gametime(), "Autosave: save took %d ms\n", SDL_GetTicks() - realtime);
-		game.get_ibase()->log_message(_("Game saved"));
+		if (!force_skip) {
+			verb_log_info_time(
+			   game.get_gametime(), "Autosave: save took %u ms\n", SDL_GetTicks() - realtime);
+			game.get_ibase()->log_message(_("Game saved"));
+		}
 	} else {
 		saving_next_tick_ = check_next_tick(game, realtime);
 	}
@@ -194,7 +193,7 @@ void SaveHandler::think(Widelands::Game& game) {
 /**
  * Lazy intialisation on first call.
  */
-void SaveHandler::initialize(uint32_t realtime) {
+void SaveHandler::initialize(Widelands::Game& game, uint32_t realtime) {
 	if (initialized_) {
 		return;
 	}
@@ -202,11 +201,14 @@ void SaveHandler::initialize(uint32_t realtime) {
 	fs_type_ = get_config_bool("nozip", false) ? FileSystem::DIR : FileSystem::ZIP;
 
 	autosave_interval_in_ms_ = get_config_int("autosave", kDefaultAutosaveInterval * 60) * 1000;
+	autosave_gametime_interval_ = Duration(autosave_interval_in_ms_ / 2);
 
 	next_save_realtime_ = realtime + autosave_interval_in_ms_;
 	last_save_realtime_ = realtime;
+	next_save_min_gametime_ = game.get_gametime() + autosave_gametime_interval_;
 
 	number_of_rolls_ = get_config_int("rolling_autosave", 5);
+	skip_when_inactive_ = get_config_bool("skip_autosave_on_inactivity", true);
 
 	initialized_ = true;
 }
@@ -240,6 +242,7 @@ std::string SaveHandler::create_file_name(const std::string& dir,
  */
 bool SaveHandler::save_game(Widelands::Game& game,
                             const std::string& complete_filename,
+                            std::optional<FileSystem::Type> fstype,
                             std::string* const error_str) {
 	ScopedTimer save_timer("SaveHandler::save_game() took %ums", true);
 
@@ -249,7 +252,7 @@ bool SaveHandler::save_game(Widelands::Game& game,
 		   Widelands::GameSaver gs(fs, game);
 		   gs.save();
 	   },
-	   complete_filename, fs_type_);
+	   complete_filename, fstype.has_value() ? fstype.value() : fs_type_);
 	gsh.save();
 	last_save_realtime_ = SDL_GetTicks();
 
