@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -52,7 +52,7 @@ struct MapSaver {
 private:
 	EditorGameBase& egbase_;
 	FileSystem& fs_;
-	MapObjectSaver* mos_;
+	MapObjectSaver* mos_{nullptr};
 };
 }  // namespace Widelands
 
