@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2024 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -216,7 +216,7 @@ load_s2mf_section(FileRead& fr, int32_t const width, int32_t const height) {
 	}
 	int32_t const size = fr.signed_32();
 	if (size != dw * dh) {
-		throw wexception("expected %u but found %u", dw * dh, size);
+		throw wexception("expected %d but found %d", dw * dh, size);
 	}
 
 	if (dw < width || dh < height) {
@@ -247,8 +247,9 @@ std::string get_world_name(S2MapLoader::WorldType world) {
 		return "blackland";
 	case S2MapLoader::WorldType::kWinterland:
 		return "winterland";
+	default:
+		throw wexception("Invalid world type %d", static_cast<int>(world));
 	}
-	NEVER_HERE();
 }
 
 /// Returns S2 terrain index into (pre one-world) terrain names. Those are then
@@ -1027,7 +1028,7 @@ void S2MapLoader::load_s2mf(Widelands::EditorGameBase& egbase) {
 					break;
 
 				default:
-					log_warn("S2Map: Unknown immovable %d", static_cast<uint32_t>(codon));
+					log_warn("S2Map: Unknown immovable %u", static_cast<uint32_t>(codon));
 					break;
 				}  // end switch (codon)
 				break;
@@ -1052,7 +1053,7 @@ void S2MapLoader::load_s2mf(Widelands::EditorGameBase& egbase) {
 		log_info("-> Player %u: ", p);
 
 		Widelands::Coords starting_pos = map_.get_starting_pos(p);
-		if (!starting_pos) {
+		if (!starting_pos.valid()) {
 			//  Do not throw exception, else map will not be loadable in the
 			//  editor. Player initialization will keep track of wrong starting
 			//  positions.
