@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -68,7 +68,7 @@ WidelandsMapLoader::~WidelandsMapLoader() {  // NOLINT
 }
 
 /**
- * Preloads a map so that the map class returns valid data for all it's
+ * Preloads a map so that the map class returns valid data for all its
  * get_info() functions (width, nrplayers..)
  */
 int32_t WidelandsMapLoader::preload_map(bool const scenario, AddOns::AddOnsList* addons) {
@@ -83,8 +83,10 @@ int32_t WidelandsMapLoader::preload_map(bool const scenario, AddOns::AddOnsList*
 
 		if (addons != nullptr) {
 			// first, clear all world add-ons…
+			std::vector<std::shared_ptr<AddOns::AddOnInfo>> desired_world_addons;
 			for (auto it = addons->begin(); it != addons->end();) {
 				if ((*it)->category == AddOns::AddOnCategory::kWorld) {
+					desired_world_addons.push_back(*it);
 					it = addons->erase(it);
 				} else {
 					++it;
@@ -105,6 +107,9 @@ int32_t WidelandsMapLoader::preload_map(bool const scenario, AddOns::AddOnsList*
 						}
 						assert(pair.first->category == AddOns::AddOnCategory::kWorld);
 						addons->push_back(pair.first);
+						desired_world_addons.erase(std::remove(desired_world_addons.begin(),
+						                                       desired_world_addons.end(), pair.first),
+						                           desired_world_addons.end());
 						break;
 					}
 				}
@@ -114,6 +119,8 @@ int32_t WidelandsMapLoader::preload_map(bool const scenario, AddOns::AddOnsList*
 					                    AddOns::version_to_string(requirement.second).c_str());
 				}
 			}
+			// Reenable additional desired world add-ons
+			addons->insert(addons->end(), desired_world_addons.begin(), desired_world_addons.end());
 		}
 	}
 	{
@@ -199,7 +206,7 @@ int32_t WidelandsMapLoader::load_map_complete(EditorGameBase& egbase,
 	// PRELOAD DATA BEGIN
 	auto set_progress_message = [is_editor](const std::string& text, unsigned step) {
 		Notifications::publish(UI::NoteLoadingMessage(
-		   format(_("Loading map: %1$s (%2$u/%3$d)"), text, step, (is_editor ? 9 : 23))));
+		   format(_("Loading map: %1$s (%2$u/%3$d)"), text, step, (is_editor ? 9 : 24))));
 	};
 
 	set_progress_message(_("Elemental data"), 1);
@@ -366,11 +373,10 @@ int32_t WidelandsMapLoader::load_map_complete(EditorGameBase& egbase,
 		{
 			const Field& fields_end = map()[map().max_index()];
 			for (Field* field = &map()[0]; field < &fields_end; ++field) {
-				if (BaseImmovable* const imm = field->get_immovable()) {
-					if (upcast(Building const, building, imm)) {
-						if (field != &map()[building->get_position()]) {
-							continue;  //  not the building's main position
-						}
+				if (BaseImmovable* const imm = field->get_immovable(); imm != nullptr) {
+					if (imm->descr().type() >= MapObjectType::BUILDING &&
+					    field != &map()[dynamic_cast<Building*>(imm)->get_position()]) {
+						continue;  //  not the building's main position
 					}
 					imm->load_finish(egbase);
 				}
@@ -439,6 +445,23 @@ int32_t WidelandsMapLoader::load_map_complete(EditorGameBase& egbase,
 
 	map_.ensure_resource_consistency(egbase.descriptions());
 
+	if (!is_editor) {
+		verb_log_info("Fourth phase loading Map Objects ... ");
+		set_progress_message(_("Postloading map objects"), 24);
+		{
+			const Field& fields_end = map()[map().max_index()];
+			for (Field* field = &map()[0]; field < &fields_end; ++field) {
+				if (BaseImmovable* const imm = field->get_immovable(); imm != nullptr) {
+					if (imm->descr().type() >= MapObjectType::BUILDING &&
+					    field != &map()[dynamic_cast<Building*>(imm)->get_position()]) {
+						continue;  //  not the building's main position
+					}
+					imm->postload(egbase);
+				}
+			}
+		}
+	}
+
 	set_state(State::kLoaded);
 
 	return 0;
