@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2024 by the Widelands Development Team
+ * Copyright (C) 2006-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,6 +24,7 @@
 #include "economy/flag.h"
 #include "logic/filesystem_constants.h"
 #include "logic/game_controller.h"
+#include "logic/map_objects/tribes/ship.h"
 #include "logic/map_objects/tribes/tribe_descr.h"
 #include "logic/message.h"
 #include "logic/objective.h"
@@ -32,7 +33,8 @@
 #include "logic/player_end_result.h"
 #include "logic/playersmanager.h"
 #include "scripting/globals.h"
-#include "scripting/lua_map.h"
+#include "scripting/map/lua_field.h"
+#include "scripting/map/lua_tribe_description.h"
 #include "wlapplication_options.h"
 #include "wui/interactive_player.h"
 #include "wui/story_message_box.h"
@@ -115,6 +117,13 @@ const MethodType<LuaPlayer> LuaPlayer::Methods[] = {
    METHOD(LuaPlayer, get_produced_wares_count),
    METHOD(LuaPlayer, set_attack_forbidden),
    METHOD(LuaPlayer, is_attack_forbidden),
+   METHOD(LuaPlayer, cancel_trade),
+   METHOD(LuaPlayer, reject_trade),
+   METHOD(LuaPlayer, retract_trade),
+   METHOD(LuaPlayer, propose_trade_extension),
+   METHOD(LuaPlayer, accept_trade_extension),
+   METHOD(LuaPlayer, reject_trade_extension),
+   METHOD(LuaPlayer, retract_trade_extension),
    {nullptr, nullptr},
 };
 const PropertyType<LuaPlayer> LuaPlayer::Properties[] = {
@@ -132,6 +141,7 @@ const PropertyType<LuaPlayer> LuaPlayer::Properties[] = {
    PROP_RW(LuaPlayer, see_all),
    PROP_RW(LuaPlayer, allow_additional_expedition_items),
    PROP_RW(LuaPlayer, hidden_from_general_statistics),
+   PROP_RO(LuaPlayer, ai_type),
    {nullptr, nullptr, nullptr},
 };
 
@@ -232,7 +242,11 @@ int LuaPlayer::get_resigned(lua_State* L) {
 int LuaPlayer::get_end_result(lua_State* L) {
 	const Widelands::PlayerEndStatus* p =
 	   get_egbase(L).player_manager()->get_player_end_status(player_number());
-	lua_pushinteger(L, static_cast<int>(p->result));
+	if (p == nullptr) {
+		lua_pushinteger(L, 255);
+	} else {
+		lua_pushinteger(L, static_cast<int>(p->result));
+	}
 	return 1;
 }
 
@@ -366,6 +380,24 @@ int LuaPlayer::set_hidden_from_general_statistics(lua_State* L) {
 	return 0;
 }
 
+/* RST
+   .. attribute:: ai_type
+
+      .. versionadded:: 1.2
+
+      (RO) Type of the AI controlling this player
+      ("normal", "weak", "very_weak", "empty"; "" if human controlled only)
+
+      This information is reliable for multi player games. For single player games,
+      the value for an ai player may also be "random" or even "".
+*/
+int LuaPlayer::get_ai_type(lua_State* L) {
+	Widelands::Game& game = get_game(L);
+	const Widelands::Player& p = get(L, game);
+	lua_pushstring(L, p.get_ai());
+	return 1;
+}
+
 /*
  ==========================================================
  LUA METHODS
@@ -1075,15 +1107,15 @@ int LuaPlayer::switchplayer(lua_State* L) {
 }
 
 /* RST
-   .. method:: produced_wares_count(what)
+   .. method:: get_produced_wares_count(what)
 
       Returns count of wares produced by the player up to now.
 
-      :arg what: This can be either :const:`"all"` or a single name of a ware or an :class`array`
+      :arg what: This can be either :const:`"all"` or a single name of a ware or an :class:`array`
          of ware names.
       :type what: :class:`string` or :class:`array`
-      :returns: If a single ware name is given, integer is returned, otherwise a :class:`table`
-         is returned.
+      :returns: If a single ware name is given, :class:`integer` is returned,
+         otherwise a :class:`table` is returned.
       :rtype: :class:`integer` or :class:`table`
 */
 int LuaPlayer::get_produced_wares_count(lua_State* L) {
@@ -1151,6 +1183,151 @@ int LuaPlayer::set_attack_forbidden(lua_State* L) {
 	return 0;
 }
 
+/* RST
+   .. method:: cancel_trade(id)
+
+      .. versionadded:: 1.3
+
+      Cancel the trade agreement with the provided ID.
+
+      Only active trade agreements can be cancelled.
+      Either of the two players between whom the agreement exists may cancel it at any time.
+
+      :arg id: Unique ID of the trade to cancel.
+      :type id: :class:`integer`
+
+      :see also: :attr:`wl.Game.trades`
+*/
+int LuaPlayer::cancel_trade(lua_State* L) {
+	Widelands::Game& game = get_game(L);
+	game.cancel_trade(luaL_checkinteger(L, 2), false, &get(L, game));
+	return 0;
+}
+
+/* RST
+   .. method:: reject_trade(id)
+
+      .. versionadded:: 1.3
+
+      Reject the proposed trade with the provided ID.
+
+      Only proposed trade offers can be rejected.
+      Only the recipient of the offer may reject it.
+
+      :arg id: Unique ID of the trade to reject.
+      :type id: :class:`integer`
+
+      :see also: :attr:`wl.Game.trades`
+*/
+int LuaPlayer::reject_trade(lua_State* L) {
+	get_game(L).reject_trade(luaL_checkinteger(L, 2));
+	return 0;
+}
+
+/* RST
+   .. method:: retract_trade(id)
+
+      .. versionadded:: 1.3
+
+      Retract the proposed trade with the provided ID.
+
+      Only proposed trade offers can be retracted.
+      Only the player who initiated the offer may retract it.
+
+      :arg id: Unique ID of the trade to retract.
+      :type id: :class:`integer`
+
+      :see also: :attr:`wl.Game.trades`
+*/
+int LuaPlayer::retract_trade(lua_State* L) {
+	get_game(L).retract_trade(luaL_checkinteger(L, 2));
+	return 0;
+}
+
+/* RST
+   .. method:: propose_trade_extension(id, batches)
+
+      .. versionadded:: 1.3
+
+      Propose to extend the trade with the given ID by the given number of batches.
+
+      :arg id: Unique ID of the trade to propose to extend.
+      :type id: :class:`integer`
+      :arg batches: Number of batches to add to the trade.
+         Maximum 100. Use :const:`-1` to make the trade indefinite.
+      :type batches: :class:`integer`
+
+      :see also: :attr:`wl.Game.trade_extension_proposals`
+*/
+int LuaPlayer::propose_trade_extension(lua_State* L) {
+	get_game(L).propose_trade_extension(
+	   player_number(), luaL_checkinteger(L, 2), luaL_checkinteger(L, 3));
+	return 0;
+}
+
+/* RST
+   .. method:: accept_trade_extension(id, batches)
+
+      .. versionadded:: 1.3
+
+      Accept the proposal to extend the trade with the given ID by the given number of batches.
+
+      :arg id: Unique ID of the trade to propose to extend.
+      :type id: :class:`integer`
+      :arg batches: Number of batches to add to the trade.
+         Maximum 100. Use :const:`-1` to make the trade indefinite.
+      :type batches: :class:`integer`
+
+      :see also: :attr:`wl.Game.trade_extension_proposals`
+*/
+int LuaPlayer::accept_trade_extension(lua_State* L) {
+	get_game(L).accept_trade_extension(
+	   player_number(), luaL_checkinteger(L, 2), luaL_checkinteger(L, 3));
+	return 0;
+}
+
+/* RST
+   .. method:: reject_trade_extension(id, batches)
+
+      .. versionadded:: 1.3
+
+      Reject the proposal to extend the trade with the given ID by the given number of batches.
+
+      :arg id: Unique ID of the trade to propose to extend.
+      :type id: :class:`integer`
+      :arg batches: Number of batches to add to the trade.
+         Maximum 100. Use :const:`-1` to make the trade indefinite.
+      :type batches: :class:`integer`
+
+      :see also: :attr:`wl.Game.trade_extension_proposals`
+*/
+int LuaPlayer::reject_trade_extension(lua_State* L) {
+	get_game(L).reject_trade_extension(
+	   player_number(), luaL_checkinteger(L, 2), luaL_checkinteger(L, 3));
+	return 0;
+}
+
+/* RST
+   .. method:: retract_trade_extension(id, batches)
+
+      .. versionadded:: 1.3
+
+      Retract the proposal to extend the trade with the given ID by the given number of batches.
+
+      :arg id: Unique ID of the trade to propose to extend.
+      :type id: :class:`integer`
+      :arg batches: Number of batches to add to the trade.
+         Maximum 100. Use :const:`-1` to make the trade indefinite.
+      :type batches: :class:`integer`
+
+      :see also: :attr:`wl.Game.trade_extension_proposals`
+*/
+int LuaPlayer::retract_trade_extension(lua_State* L) {
+	get_game(L).retract_trade_extension(
+	   player_number(), luaL_checkinteger(L, 2), luaL_checkinteger(L, 3));
+	return 0;
+}
+
 /*
  ==========================================================
  C METHODS
@@ -1534,6 +1711,8 @@ int LuaInboxMessage::get_status(lua_State* L) {
 	case Widelands::Message::Status::kArchived:
 		lua_pushstring(L, "archived");
 		break;
+	default:
+		NEVER_HERE();
 	}
 	return 1;
 }
