@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2024 by the Widelands Development Team
+ * Copyright (C) 2006-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,7 +29,7 @@ class LuaTable;
 // Provides an interface to call and execute Lua Code.
 class LuaInterface {
 public:
-	LuaInterface();
+	explicit LuaInterface(bool is_main_menu = false);
 	virtual ~LuaInterface();
 
 	// Interpret the given string, will throw 'LuaError' on any error.
@@ -41,6 +41,10 @@ public:
 	// Returns an empty table.
 	std::unique_ptr<LuaTable> empty_table();
 
+	lua_State* L() {
+		return lua_state_;
+	}
+
 protected:
 	lua_State* lua_state_;
 };
