@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2022 by the Widelands Development Team
+ * Copyright (C) 2006-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -72,11 +72,12 @@
  */
 class LunaClass {
 public:
-	virtual ~LunaClass() {
-	}
+	virtual ~LunaClass() = default;
 
 	CLANG_DIAG_RESERVED_IDENTIFIER_OFF
+	// NOLINTNEXTLINE(bugprone-reserved-identifier,cert-dcl37-c,cert-dcl51-cpp)
 	virtual void __persist(lua_State*) = 0;
+	// NOLINTNEXTLINE(bugprone-reserved-identifier,cert-dcl37-c,cert-dcl51-cpp)
 	virtual void __unpersist(lua_State*) = 0;
 	CLANG_DIAG_RESERVED_IDENTIFIER_ON
 
@@ -115,8 +116,9 @@ void register_class(lua_State* const L,
 	register_properties_in_metatable<T, T>(L);
 	register_methods_in_metatable<T, T>(L);
 
-	if (!return_metatable)
+	if (!return_metatable) {
 		lua_pop(L, 1);  // remove the Metatable
+	}
 }
 /**
  * Makes the first class a children of the second. Make sure that T is really a
