@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,6 +17,8 @@
 
 #include "ui_basic/slider.h"
 
+#include <cassert>
+#include <limits>
 #include <memory>
 
 #include <SDL_mouse.h>
@@ -52,6 +54,7 @@ namespace UI {
  * \param bar_size Length of the cursor move.
  */
 Slider::Slider(Panel* const parent,
+               const std::string& name,
                const int32_t x,
                const int32_t y,
                const uint32_t w,
@@ -68,6 +71,7 @@ Slider::Slider(Panel* const parent,
                const int32_t bar_size)
    : Panel(parent,
            style == SliderStyle::kFsMenu ? PanelStyle::kFsMenu : PanelStyle::kWui,
+           name,
            x,
            y,
            w,
@@ -76,19 +80,17 @@ Slider::Slider(Panel* const parent,
      min_value_(min_value),
      max_value_(max_value),
      value_(value),
-     relative_move_(0),
-     highlighted_(false),
-     pressed_(false),
      enabled_(enabled),
      cursor_style_(style),
      x_gap_(x_gap),
      y_gap_(y_gap),
      bar_size_(bar_size),
-     cursor_size_(cursor_size),
-     cursor_fixed_height_(-1) {
+     cursor_size_(cursor_size) {
 	set_thinks(false);
+	set_snap_target(true);
 	set_can_focus(enabled_);
 	calculate_cursor_position();
+	calculate_big_step();
 }
 
 inline const UI::PanelStyleInfo& Slider::cursor_style() const {
@@ -106,9 +108,11 @@ void Slider::set_value(int32_t new_value) {
 }
 
 void Slider::calculate_cursor_position() {
-	if (max_value_ == min_value_) {
-		cursor_pos_ = min_value_;
-	} else if (value_ == min_value_) {
+	assert(max_value_ >= min_value_);
+	assert(value_ >= min_value_);
+	assert(max_value_ >= value_);
+
+	if (value_ == min_value_) {
 		cursor_pos_ = 0;
 	} else if (value_ == max_value_) {
 		cursor_pos_ = get_bar_size();
@@ -117,6 +121,44 @@ void Slider::calculate_cursor_position() {
 	}
 }
 
+void Slider::calculate_big_step() {
+	assert(max_value_ >= min_value_);
+
+	// Check integer overflow
+	assert(max_value_ - min_value_ >= 0);
+
+	int32_t range = max_value_ - min_value_;
+
+	// Number of steps between the values that can be set with the numeric keys by handle_key()
+	constexpr int32_t kNumkeySteps = 8;  // 9 - 1
+
+	// The position of the value within the range is visualised by sliders, so stepping
+	// over small ranges looks better if instead of fixing the big step size we make it
+	// step from min or max to middle (2 steps from min to max or vice versa).
+	// Let's allow this from "big" step size of 2.
+	constexpr int32_t kMinTwoStep = 4;  // 2 * 2
+
+	// So we may have 1, 2 or 8 steps. Let's make the whole thing powers of 2.
+	constexpr int32_t kMediumNumSteps = 4;
+
+	if (range < kMinTwoStep) {
+		big_step_ = range;  // to min/max
+	} else {
+		int32_t num_steps = range / ChangeBigStep::kSmallRange;
+		if (num_steps >= kNumkeySteps) {
+			num_steps = kNumkeySteps;
+		} else if (num_steps >= kMediumNumSteps) {
+			num_steps = kMediumNumSteps;
+		} else {
+			num_steps = 2;
+		}
+		big_step_ = range / num_steps;
+	}
+
+	assert(max_value_ < std::numeric_limits<int32_t>::max() - big_step_);
+	assert(min_value_ > std::numeric_limits<int32_t>::min() + big_step_);
+}
+
 void Slider::layout() {
 	Panel::layout();
 	calculate_cursor_position();
@@ -129,11 +171,16 @@ void Slider::layout() {
  */
 void Slider::set_max_value(int32_t new_max) {
 	assert(min_value_ <= new_max);
-	if (max_value_ != new_max) {
-		calculate_cursor_position();
+	if (max_value_ == new_max) {
+		return;
 	}
 	max_value_ = new_max;
-	set_value(value_);
+	calculate_big_step();
+	if (value_ > max_value_) {
+		set_value(max_value_);
+	} else {
+		calculate_cursor_position();
+	}
 }
 
 /**
@@ -143,11 +190,16 @@ void Slider::set_max_value(int32_t new_max) {
  */
 void Slider::set_min_value(int32_t new_min) {
 	assert(max_value_ >= new_min);
-	if (min_value_ != new_min) {
-		calculate_cursor_position();
+	if (min_value_ == new_min) {
+		return;
 	}
 	min_value_ = new_min;
-	set_value(value_);
+	calculate_big_step();
+	if (value_ < min_value_) {
+		set_value(min_value_);
+	} else {
+		calculate_cursor_position();
+	}
 }
 
 /**
@@ -241,13 +293,21 @@ void Slider::set_highlighted(bool highlighted) {
 
 bool Slider::handle_key(bool down, SDL_Keysym code) {
 	if (down && enabled_) {
-		switch (get_keyboard_change(code, false)) {
+		switch (get_keyboard_change(code)) {
+		case ChangeType::kNone:
+			break;
 		case ChangeType::kPlus:
 			set_value(get_value() + 1);
 			return true;
 		case ChangeType::kMinus:
 			set_value(get_value() - 1);
 			return true;
+		case ChangeType::kBigPlus:
+			set_value(get_value() + big_step_);
+			return true;
+		case ChangeType::kBigMinus:
+			set_value(get_value() - big_step_);
+			return true;
 		case ChangeType::kSetMax:
 			set_value(get_max_value());
 			return true;
@@ -255,7 +315,7 @@ bool Slider::handle_key(bool down, SDL_Keysym code) {
 			set_value(0);
 			return true;
 		default:
-			break;
+			NEVER_HERE();
 		}
 
 		int32_t num = -1;
@@ -293,6 +353,13 @@ bool Slider::handle_mousewheel(int32_t x, int32_t y, uint16_t modstate) {
 		return true;
 	}
 
+	// Try big step
+	change = get_mousewheel_change(MousewheelHandlerConfigID::kChangeValueBig, x, y, modstate);
+	if (change != 0) {
+		set_value(get_value() + change * big_step_);
+		return true;
+	}
+
 	return false;
 }
 
@@ -564,9 +631,8 @@ bool VerticalSlider::handle_mousepress(const uint8_t btn, int32_t x, int32_t y)
 		cursor_pressed(y);
 		return true;
 	}
-	if (y >= get_y_gap() && y <= static_cast<int32_t>(get_h()) - get_y_gap() &&
-	    x >= get_x_gap() - 2 &&
-	    x < static_cast<int32_t>(get_w()) - get_x_gap() + 2) {  //  click on bar
+	if (y >= get_y_gap() && y <= get_h() - get_y_gap() && x >= get_x_gap() - 2 &&
+	    x < get_w() - get_x_gap() + 2) {  //  click on bar
 		bar_pressed(y, get_y_gap());
 		return true;
 	}
@@ -574,6 +640,7 @@ bool VerticalSlider::handle_mousepress(const uint8_t btn, int32_t x, int32_t y)
 }
 
 DiscreteSlider::DiscreteSlider(Panel* const parent,
+                               const std::string& name,
                                const int32_t x,
                                const int32_t y,
                                const uint32_t w,
@@ -586,6 +653,7 @@ DiscreteSlider::DiscreteSlider(Panel* const parent,
                                const bool enabled)
    : Panel(parent,
            init_style == SliderStyle::kFsMenu ? PanelStyle::kFsMenu : PanelStyle::kWui,
+           name,
            x,
            y,
            w,
@@ -593,6 +661,7 @@ DiscreteSlider::DiscreteSlider(Panel* const parent,
            tooltip_text),
      style_(init_style),
      slider(this,
+            "discrete_slider_impl",
             // here, we take into account the h_gap introduced by HorizontalSlider
             w / (2 * labels_in.size()) - cursor_size / 2,
             0,
