@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2022 by the Widelands Development Team
+ * Copyright (C) 2003-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,8 @@
 
 #include "ui_basic/tabpanel.h"
 
+#include <algorithm>
+
 #include <SDL_mouse.h>
 
 #include "graphic/font_handler.h"
@@ -57,8 +59,7 @@ Tab::Tab(TabPanel* const tab_parent,
          const Image* init_pic,
          const std::string& tooltip_text,
          Panel* const contents)
-   : NamedPanel(
-        tab_parent, s, name, x, 0, kTabPanelButtonHeight, kTabPanelButtonHeight, tooltip_text),
+   : Panel(tab_parent, s, name, x, 0, kTabPanelButtonHeight, kTabPanelButtonHeight, tooltip_text),
      parent(tab_parent),
      id(tab_id),
      pic(init_pic),
@@ -123,15 +124,16 @@ bool Tab::handle_mousepress(uint8_t /*btn*/, int32_t /*x*/, int32_t /*y*/) {
  * Initialize an empty TabPanel. We use width == 0 as an indicator that the size hasn't been set
  * yet.
  */
-TabPanel::TabPanel(Panel* const parent, UI::TabPanelStyle style)
+TabPanel::TabPanel(Panel* const parent, UI::TabPanelStyle style, const std::string& name)
    : Panel(parent,
            style == TabPanelStyle::kFsMenu ? PanelStyle::kFsMenu : PanelStyle::kWui,
+           name,
            0,
            0,
            0,
            0),
      tab_style_(style),
-     active_(0),
+
      highlight_(kNotFound),
      background_style_(style) {
 	set_can_focus(true);
@@ -272,6 +274,11 @@ void TabPanel::update_desired_size() {
 		h += panelh;
 	}
 
+	// size of tab headers
+	if (!tabs_.empty()) {
+		w = std::max(w, tabs_.back()->get_x() + tabs_.back()->get_w());
+	}
+
 	set_desired_size(w, h);
 
 	// This is not redundant, because even if all this doesn't change our
@@ -286,8 +293,9 @@ void TabPanel::update_desired_size() {
 uint32_t TabPanel::add(const std::string& name,
                        const std::string& title,
                        Panel* const panel,
-                       const std::string& tooltip_text) {
-	return add_tab(name, title, nullptr, tooltip_text, panel);
+                       const std::string& tooltip_text,
+                       int index) {
+	return add_tab(name, title, nullptr, tooltip_text, panel, index);
 }
 
 /**
@@ -296,8 +304,9 @@ uint32_t TabPanel::add(const std::string& name,
 uint32_t TabPanel::add(const std::string& name,
                        const Image* pic,
                        Panel* const panel,
-                       const std::string& tooltip_text) {
-	return add_tab(name, "", pic, tooltip_text, panel);
+                       const std::string& tooltip_text,
+                       int index) {
+	return add_tab(name, "", pic, tooltip_text, panel, index);
 }
 
 /** Common adding function for textual and pictorial tabs. */
@@ -305,16 +314,26 @@ uint32_t TabPanel::add_tab(const std::string& name,
                            const std::string& title,
                            const Image* pic,
                            const std::string& tooltip_text,
-                           Panel* panel) {
+                           Panel* panel,
+                           int index) {
 	assert(panel);
 	assert(panel->get_parent() == this);
 
-	size_t id = tabs_.size();
+	const size_t id = index < 0 ? tabs_.size() : index;
+	if (active_ >= id && !tabs_.empty()) {
+		++active_;
+	}
 	int32_t x = id > 0 ? tabs_[id - 1]->get_x() + tabs_[id - 1]->get_w() : 0;
-	tabs_.push_back(
+	tabs_.insert(
+	   tabs_.begin() + id,
 	   new Tab(this, panel_style_, id, x,
 	           tab_style_ == TabPanelStyle::kFsMenu ? FontStyle::kFsMenuLabel : FontStyle::kWuiLabel,
 	           name, title, pic, tooltip_text, panel));
+	const int tabw = tabs_.at(id)->get_w();
+	for (size_t i = id + 1; i < tabs_.size(); ++i) {
+		Tab* t = tabs_.at(i);
+		t->set_pos(Vector2i(t->get_x() + tabw, t->get_y()));
+	}
 
 	// Add a margin if there is a border
 	if (tab_style_ == UI::TabPanelStyle::kFsMenu) {
@@ -418,16 +437,15 @@ void TabPanel::draw(RenderTarget& dst) {
 			// Scale the image down if needed, but keep the ratio.
 			constexpr int kMaxImageSize = kTabPanelButtonHeight - 2 * kTabPanelImageMargin;
 			double image_scale =
-			   std::min(1., std::min(static_cast<double>(kMaxImageSize) / tabs_[idx]->pic->width(),
-			                         static_cast<double>(kMaxImageSize) / tabs_[idx]->pic->height()));
+			   std::min({1., static_cast<double>(kMaxImageSize) / tabs_[idx]->pic->width(),
+			             static_cast<double>(kMaxImageSize) / tabs_[idx]->pic->height()});
 
 			uint16_t picture_width = image_scale * tabs_[idx]->pic->width();
 			uint16_t picture_height = image_scale * tabs_[idx]->pic->height();
 			dst.blitrect_scale(
 			   Rectf(x + (kTabPanelButtonHeight - picture_width) / 2.f,
 			         (kTabPanelButtonHeight - picture_height) / 2.f, picture_width, picture_height),
-			   tabs_[idx]->pic, Recti(0, 0, tabs_[idx]->pic->width(), tabs_[idx]->pic->height()), 1,
-			   BlendMode::UseAlpha);
+			   tabs_[idx]->pic, tabs_[idx]->pic->rect(), 1.f, BlendMode::UseAlpha);
 		} else if (tabs_[idx]->rendered_title != nullptr) {
 			tabs_[idx]->rendered_title->draw(
 			   dst, Vector2i(x + kTabPanelTextMargin,
