@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2021-2024 by the Widelands Development Team
+ * Copyright (C) 2021-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -39,31 +39,55 @@ inline std::string safe_richtext_message(std::string body) {
 }
 
 void uninstall(AddOnsCtrl* ctrl, std::shared_ptr<AddOns::AddOnInfo> info, const bool local) {
+	const bool is_map = ends_with(info->internal_name, ".map");
+
 	if (!matches_keymod(SDL_GetModState(), KMOD_CTRL)) {
 		UI::WLMessageBox w(
 		   &ctrl->get_topmost_forefather(), UI::WindowStyle::kFsMenu, _("Uninstall"),
-		   safe_richtext_message(
-		      format(local ? _("Are you certain that you want to uninstall this add-on?\n\n"
-		                       "%1$s\n"
-		                       "by %2$s\n"
-		                       "Version %3$s\n"
-		                       "Category: %4$s\n"
-		                       "%5$s\n\n"
-		                       "Note that this add-on can not be downloaded again from the server.") :
-		                     _("Are you certain that you want to uninstall this add-on?\n\n"
-		                       "%1$s\n"
-		                       "by %2$s\n"
-		                       "Version %3$s\n"
-		                       "Category: %4$s\n"
-		                       "%5$s"),
-		             info->descname(), info->author(), AddOns::version_to_string(info->version),
-		             AddOns::kAddOnCategories.at(info->category).descname(), info->description())),
+		   is_map ?
+		      safe_richtext_message(
+		         format(local ? _("Are you certain that you want to uninstall this map?\n\n"
+		                          "%1$s\n"
+		                          "by %2$s\n"
+		                          "%3$s\n\n"
+		                          "Note that this map can not be downloaded again from the server.") :
+		                        _("Are you certain that you want to uninstall this map?\n\n"
+		                          "%1$s\n"
+		                          "by %2$s\n"
+		                          "%3$s"),
+		                info->descname(), info->author(), info->description())) :
+		      safe_richtext_message(format(
+		         local ? _("Are you certain that you want to uninstall this add-on?\n\n"
+		                   "%1$s\n"
+		                   "by %2$s\n"
+		                   "Version %3$s\n"
+		                   "Category: %4$s\n"
+		                   "%5$s\n\n"
+		                   "Note that this add-on can not be downloaded again from the server.") :
+		                 _("Are you certain that you want to uninstall this add-on?\n\n"
+		                   "%1$s\n"
+		                   "by %2$s\n"
+		                   "Version %3$s\n"
+		                   "Category: %4$s\n"
+		                   "%5$s"),
+		         info->descname(), info->author(), AddOns::version_to_string(info->version),
+		         AddOns::kAddOnCategories.at(info->category).descname(), info->description())),
 		   UI::WLMessageBox::MBoxType::kOkCancel);
 		if (w.run<UI::Panel::Returncodes>() != UI::Panel::Returncodes::kOk) {
 			return;
 		}
 	}
 
+	if (is_map) {
+		// Maps only have this one file and no translations currently
+		g_fs->fs_unlink(kMapsDir + FileSystem::file_separator() + kDownloadedMapsDir +
+		                FileSystem::file_separator() + info->map_file_name);
+		ctrl->clear_cache_for_map(info->internal_name);
+		ctrl->update_dependency_errors();
+		ctrl->rebuild_maps();
+		return;
+	}
+
 	// Delete the add-on…
 	g_fs->fs_unlink(kAddOnDir + FileSystem::file_separator() + info->internal_name);
 
@@ -81,7 +105,17 @@ void uninstall(AddOnsCtrl* ctrl, std::shared_ptr<AddOns::AddOnInfo> info, const
 				AddOns::update_ui_theme(AddOns::UpdateThemeAction::kAutodetect);
 				ctrl->get_topmost_forefather().template_directory_changed();
 			}
-			return ctrl->rebuild(true);
+
+			if (info->category == AddOns::AddOnCategory::kUIPlugin) {
+				ctrl->fsmm().reinit_plugins();
+			}
+
+			ctrl->clear_cache_for_installed(info->internal_name);
+			ctrl->clear_cache_for_browse(info->internal_name);
+			ctrl->update_dependency_errors();
+			ctrl->rebuild_installed();
+			ctrl->rebuild_browse();
+			return;
 		}
 	}
 	NEVER_HERE();
@@ -114,6 +148,38 @@ std::string required_wl_version_and_sync_safety_string(std::shared_ptr<AddOns::A
 	return result;
 }
 
+std::string assemble_map_description_text(const std::string& descr,
+                                          const std::string& hint,
+                                          const std::string& comment) {
+	std::string result;
+
+	if (!descr.empty()) {
+		result += format(
+		   "<vspace gap=%d><p>%s<br>%s</p>", kRowButtonSpacing,
+		   g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelHeading)
+		      .as_font_tag(_("Description:")),
+		   g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph).as_font_tag(descr));
+	}
+
+	if (!hint.empty()) {
+		result += format(
+		   "<vspace gap=%d><p>%s<br>%s</p>", kRowButtonSpacing,
+		   g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelHeading)
+		      .as_font_tag(_("Hint:")),
+		   g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph).as_font_tag(hint));
+	}
+
+	if (!comment.empty()) {
+		result += format("<vspace gap=%d><p>%s<br>%s</p>", kRowButtonSpacing,
+		                 g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelHeading)
+		                    .as_font_tag(_("Comment by uploader:")),
+		                 g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelParagraph)
+		                    .as_font_tag(comment));
+	}
+
+	return result;
+}
+
 }  // namespace
 
 InstalledAddOnRow::InstalledAddOnRow(Panel* parent,
@@ -199,13 +265,20 @@ InstalledAddOnRow::InstalledAddOnRow(Panel* parent,
 				                                              "images/ui_basic/checkbox_checked.png" :
 				                                              "images/ui_basic/checkbox_empty.png"));
 				toggle_enabled_.set_tooltip(pair.second ? _("Disable") : _("Enable"));
+
 				if (pair.first->category == AddOns::AddOnCategory::kTheme) {
 					AddOns::update_ui_theme(pair.second ? AddOns::UpdateThemeAction::kEnableArgument :
 					                                      AddOns::UpdateThemeAction::kAutodetect,
 					                        pair.first->internal_name);
 					get_topmost_forefather().template_directory_changed();
+				} else if (pair.first->category == AddOns::AddOnCategory::kUIPlugin) {
+					ctrl->fsmm().reinit_plugins();
 				}
-				return ctrl->rebuild(true);
+
+				ctrl->clear_cache_for_installed(info->internal_name);
+				ctrl->update_dependency_errors();
+				ctrl->rebuild_installed();
+				return;
 			}
 		}
 		NEVER_HERE();
@@ -259,6 +332,11 @@ void RemoteAddOnRow::draw(RenderTarget& r) {
 	r.brighten_rect(Recti(0, 0, get_w(), get_h()), -20);
 }
 
+void MapRow::draw(RenderTarget& r) {
+	UI::Panel::draw(r);
+	r.brighten_rect(Recti(0, 0, get_w(), get_h()), -20);
+}
+
 RemoteAddOnRow::RemoteAddOnRow(Panel* parent,
                                AddOnsCtrl* ctrl,
                                std::shared_ptr<AddOns::AddOnInfo> info,
@@ -418,8 +496,14 @@ RemoteAddOnRow::RemoteAddOnRow(Panel* parent,
 				return;
 			}
 		}
+
 		ctrl->install_or_upgrade(info_, false);
-		ctrl->rebuild(true);
+
+		ctrl->clear_cache_for_installed(info_->internal_name);
+		ctrl->clear_cache_for_browse(info_->internal_name);
+		ctrl->update_dependency_errors();
+		ctrl->rebuild_installed();
+		ctrl->rebuild_browse();
 	});
 #ifdef NDEBUG
 	install_.set_enabled(info->matches_widelands_version());
@@ -448,7 +532,12 @@ RemoteAddOnRow::RemoteAddOnRow(Panel* parent,
 			}
 		}
 		ctrl->install_or_upgrade(info, !full_upgrade_possible_);
-		ctrl->rebuild(true);
+
+		ctrl->clear_cache_for_installed(info->internal_name);
+		ctrl->clear_cache_for_browse(info->internal_name);
+		ctrl->update_dependency_errors();
+		ctrl->rebuild_installed();
+		ctrl->rebuild_browse();
 	});
 #ifdef NDEBUG
 	upgrade_.set_enabled(info->matches_widelands_version());
@@ -547,8 +636,158 @@ void RemoteAddOnRow::layout() {
 	                                   4 * kRowButtonSize - bottom_row_right_.get_h()));
 }
 
-bool RemoteAddOnRow::upgradeable() const {
-	return upgrade_.enabled();
+MapRow::MapRow(Panel* parent,
+               AddOnsCtrl* ctrl,
+               std::shared_ptr<AddOns::AddOnInfo> info,
+               bool installed)
+   : UI::Panel(parent,
+               UI::PanelStyle::kFsMenu,
+               format("map_row_%s", info->internal_name),
+               0,
+               0,
+               kRowButtonSize,
+               7 * kRowButtonSize + kRowButtonSpacing),
+     info_(info),
+     minimap_(this, UI::PanelStyle::kFsMenu, "minimap", info_->icon),
+     install_(this,
+              "install",
+              0,
+              0,
+              24,
+              24,
+              UI::ButtonStyle::kFsMenuSecondary,
+              g_image_cache->get("images/ui_basic/continue.png"),
+              _("Install")),
+     uninstall_(this,
+                "uninstall",
+                0,
+                0,
+                24,
+                24,
+                UI::ButtonStyle::kFsMenuSecondary,
+                g_image_cache->get("images/wui/menus/exit.png"),
+                _("Uninstall")),
+     interact_(this,
+               "interact",
+               0,
+               0,
+               24,
+               24,
+               UI::ButtonStyle::kFsMenuSecondary,
+               "…",
+               _("Comments and Votes")),
+     txt_(this,
+          "description",
+          0,
+          0,
+          24,
+          24,
+          UI::PanelStyle::kFsMenu,
+          format("<rt><p>%s</p><p>%s%s</p><p>%s</p>%s</rt>",
+                 format(_("%1$s %2$s"),
+                        g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelHeading)
+                           .as_font_tag(info->descname()),
+                        g_style_manager->font_style(UI::FontStyle::kItalic)
+                           .as_font_tag(format(_("(%s)"), info->internal_name))),
+                 g_style_manager->font_style(UI::FontStyle::kItalic)
+                    .as_font_tag(info->author() == info->upload_username ?
+                                    format(_("by %s"), info->author()) :
+                                    format(_("by %1$s (uploaded by %2$s)"),
+                                           info->author(),
+                                           info->upload_username)),
+                 required_wl_version_and_sync_safety_string(info),
+                 g_style_manager->font_style(UI::FontStyle::kFsMenuInfoPanelHeading)
+                    .as_font_tag(format(_("Size: %1$u×%2$u • Max Players: %3$u • %4$s"),
+                                        info->map_width,
+                                        info->map_height,
+                                        info->map_nr_players,
+                                        info->map_world_name.empty() ?
+                                           _("One World") :
+                                           i18n::translate(info->map_world_name))),
+                 assemble_map_description_text(
+                    info->description(), info->map_hint(), info->map_uploader_comment()))),
+     bottom_row_(
+        this,
+        UI::PanelStyle::kFsMenu,
+        "bottom_row",
+        UI::FontStyle::kFsTooltip,
+        0,
+        0,
+        0,
+        0,
+        format(
+           /** TRANSLATORS: Timestamp · Filesize · Download count · Average rating ·
+              Number of comments */
+           _("%1$s   %2$s   ⬇ %3$u   ★ %4$s   “” %5$u"),
+           time_string(info->upload_timestamp),
+           filesize_string(info->total_file_size),
+           info->download_count,
+           (info->number_of_votes() != 0u ? format_l("%.2f", info->average_rating()) : "–"),
+           info->user_comments.size()),
+        UI::Align::kLeft) {
+	interact_.sigclicked.connect([ctrl, info]() {
+		RemoteInteractionWindow m(*ctrl, info);
+		m.run<UI::Panel::Returncodes>();
+	});
+	uninstall_.sigclicked.connect(
+	   [ctrl, this]() { uninstall(ctrl, info_, !ctrl->is_remote(info_->internal_name)); });
+	install_.sigclicked.connect([ctrl, this]() {
+		// No need to confirm for maps
+		ctrl->install_map(info_);
+
+		ctrl->clear_cache_for_map(info_->internal_name);
+		ctrl->rebuild_maps();
+	});
+#ifdef NDEBUG
+	install_.set_enabled(info->matches_widelands_version());
+#endif
+
+	if (installed) {
+		install_.set_enabled(false);
+	} else {
+		uninstall_.set_enabled(false);
+	}
+
+	bottom_row_.set_handle_mouse(true);
+	bottom_row_.set_tooltip(format(
+	   "%s<br>%s<br>%s<br>%s",
+	   format(ngettext("Total size: %u byte", "Total size: %u bytes", info->total_file_size),
+	          info->total_file_size),
+	   format(ngettext("%u download", "%u downloads", info->download_count), info->download_count),
+	   (info->number_of_votes() != 0u ?
+	       format_l(ngettext("Average rating: %1$.3f (%2$u vote)",
+	                         "Average rating: %1$.3f (%2$u votes)", info->number_of_votes()),
+	                info->average_rating(), info->number_of_votes()) :
+	       _("No votes yet")),
+	   format(ngettext("%u comment", "%u comments", info->user_comments.size()),
+	          info->user_comments.size())));
+	set_can_focus(true);
+	layout();
+}
+
+void MapRow::layout() {
+	UI::Panel::layout();
+	if (get_w() <= 6 * kRowButtonSize) {
+		// size not yet set
+		return;
+	}
+
+	set_desired_size(get_w(), 7 * kRowButtonSize + kRowButtonSpacing);
+
+	minimap_.set_size(6 * kRowButtonSize, 6 * kRowButtonSize);
+	install_.set_size(kRowButtonSize, kRowButtonSize);
+	uninstall_.set_size(kRowButtonSize, kRowButtonSize);
+	interact_.set_size(kRowButtonSize, kRowButtonSize);
+	bottom_row_.set_size(get_w() - 3 * kRowButtonSize - 3 * kRowButtonSpacing, bottom_row_.get_h());
+	txt_.set_size(get_w() - 6 * kRowButtonSize - kRowButtonSpacing, 6 * kRowButtonSize);
+
+	bottom_row_.set_pos(Vector2i(0, 7 * kRowButtonSize - bottom_row_.get_h()));
+	interact_.set_pos(Vector2i(get_w() - kRowButtonSize, 6 * kRowButtonSize + kRowButtonSpacing));
+	uninstall_.set_pos(
+	   Vector2i(interact_.get_x() - kRowButtonSize - kRowButtonSpacing, interact_.get_y()));
+	install_.set_pos(
+	   Vector2i(uninstall_.get_x() - kRowButtonSize - kRowButtonSpacing, interact_.get_y()));
+	txt_.set_pos(Vector2i(6 * kRowButtonSize + kRowButtonSpacing, 0));
 }
 
 }  // namespace AddOnsUI
