@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2024 by the Widelands Development Team
+ * Copyright (C) 2004-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,8 @@
 
 #include "ui_fsmenu/internet_lobby.h"
 
+#include <memory>
+
 #include "base/i18n.h"
 #include "base/random.h"
 #include "build_info.h"
@@ -475,9 +477,9 @@ void InternetLobby::clicked_joingame() {
 		const std::pair<NetAddress, NetAddress>& ips = InternetGaming::ref().ips();
 
 		try {
-			running_game_.reset(new GameClient(capsule_, running_game_, ips,
-			                                   InternetGaming::ref().get_local_clientname(), true,
-			                                   opengames_list_.get_selected().name));
+			running_game_ = std::make_shared<GameClient>(capsule_, running_game_, ips,
+			                                             InternetGaming::ref().get_local_clientname(),
+			                                             true, opengames_list_.get_selected().name);
 		} catch (const std::exception& e) {
 			running_game_.reset();
 			UI::WLMessageBox mbox(&capsule_.menu(), UI::WindowStyle::kFsMenu, _("Network Error"),
@@ -538,9 +540,8 @@ void InternetLobby::clicked_hostgame() {
 
 	// Start our relay host
 	try {
-		running_game_.reset(new GameHost(&capsule_, running_game_,
-		                                 InternetGaming::ref().get_local_clientname(), tribeinfos_,
-		                                 true));
+		running_game_ = std::make_shared<GameHost>(
+		   &capsule_, running_game_, InternetGaming::ref().get_local_clientname(), tribeinfos_, true);
 	} catch (const std::exception& e) {
 		running_game_.reset();
 		UI::WLMessageBox mbox(&capsule_.menu(), UI::WindowStyle::kFsMenu, _("Network Error"),
