@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2022 by the Widelands Development Team
+ * Copyright (C) 2004-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,16 +25,16 @@
 #include "network/internet_gaming.h"
 #include "ui_basic/box.h"
 #include "ui_basic/button.h"
-#include "ui_basic/editbox.h"
 #include "ui_basic/listselect.h"
 #include "ui_basic/table.h"
 #include "ui_basic/textarea.h"
+#include "ui_basic/textinput.h"
 #include "ui_fsmenu/menu.h"
 #include "wui/game_chat_panel.h"
 
 namespace Widelands {
 struct TribeBasicInfo;
-}
+}  // namespace Widelands
 namespace FsMenu {
 class InternetLobby : public TwoColumnsBasicNavigationMenu {
 public:
@@ -85,7 +85,7 @@ private:
 	UI::EditBox servername_;
 	UI::Button hostgame_;
 
-	uint32_t prev_clientlist_len_;
+	uint32_t prev_clientlist_len_{1000};
 	FxId new_client_fx_;
 
 	// Login information
