@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2024 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,9 +29,11 @@
 #include "base/random.h"
 #include "base/time_string.h"
 #include "base/warning.h"
+#include "base/wexception.h"
 #include "build_info.h"
 #include "editor/editorinteractive.h"
 #include "graphic/graphic.h"
+#include "graphic/graphic_functions.h"
 #include "graphic/style_manager.h"
 #include "graphic/text_layout.h"
 #include "logic/filesystem_constants.h"
@@ -59,8 +61,7 @@
 
 namespace FsMenu {
 
-constexpr uint32_t kInitialFadeoutDelay = 2500;
-constexpr uint32_t kInitialFadeoutDuration = 4000;
+constexpr uint32_t kSplashFadeoutDuration = 2000;
 constexpr uint32_t kImageExchangeInterval = 20000;
 constexpr uint32_t kImageExchangeDuration = 2500;
 
@@ -96,11 +97,17 @@ int16_t MainMenu::calc_desired_window_height(const UI::Window::WindowLayoutID id
 }
 
 int16_t MainMenu::calc_desired_window_x(const UI::Window::WindowLayoutID id) {
-	return (get_w() - calc_desired_window_width(id)) / 2 - UI::Window::kVerticalBorderThickness;
+	const UI::WindowStyleInfo& wsi = g_style_manager->window_style(UI::WindowStyle::kFsMenu);
+	return (get_w() - (calc_desired_window_width(id) + wsi.left_border_thickness() +
+	                   wsi.right_border_thickness())) /
+	       2;
 }
 
 int16_t MainMenu::calc_desired_window_y(const UI::Window::WindowLayoutID id) {
-	return (get_h() - calc_desired_window_height(id)) / 2 - UI::Window::kTopBorderThickness;
+	const UI::WindowStyleInfo& wsi = g_style_manager->window_style(UI::WindowStyle::kFsMenu);
+	return (get_h() - (calc_desired_window_height(id) + wsi.top_border_thickness() +
+	                   wsi.bottom_border_thickness())) /
+	       2;
 }
 
 MainMenu::MainMenu(const bool skip_init)
@@ -238,6 +245,7 @@ MainMenu::MainMenu(const bool skip_init)
 
 	if (!skip_init) {
 		init_time_ = SDL_GetTicks();
+		splash_state_ = SplashState::kSplash;
 		set_button_visibility(false);
 	}
 
@@ -250,6 +258,8 @@ MainMenu::MainMenu(const bool skip_init)
 	layout();
 
 	initialization_complete();
+
+	reinit_plugins();
 }
 
 void MainMenu::main_loop() {
@@ -259,10 +269,20 @@ void MainMenu::main_loop() {
 			return;  // We only get here though normal termination by the user.
 		} catch (const WLWarning& e) {
 			// WLWarning is reserved for bad circumstances that are (most likely) not a bug.
+			// But they still should make the regression test suite fail.
+			if (g_fail_on_errors) {
+				log_warn("Something went wrong. The error message is:\n%s", e.what());
+				abort();
+			}
 			show_messagebox(e.title(), e.what());
 		} catch (const std::exception& e) {
 			// This is the outermost wrapper within the GUI and should only very rarely be reached.
 			// Most likely we got here through a bug in Widelands.
+			if (g_fail_on_errors) {
+				log_err("An error has occured. The error message is:\n%s", e.what());
+				log_err("Please report this problem to help us improve Widelands.");
+				abort();
+			}
 			show_messagebox(
 			   _("Error!"),
 			   format(
@@ -301,7 +321,6 @@ void MainMenu::update_template() {
 	editor_.set_min_lineheight(dropdowns_lineheight);
 	replay_.set_min_lineheight(dropdowns_lineheight);
 
-	splashscreen_ = g_image_cache->get("loadscreens/splash.jpg");
 	title_image_ = g_image_cache->get("loadscreens/logo.png");
 
 	images_.clear();
@@ -522,7 +541,8 @@ void MainMenu::set_labels() {
 	   UI::PanelStyle::kFsMenu));
 
 	version_.set_text(
-	   /** TRANSLATORS: %1$s = version string and build type  ("Debug" or "Release") */
+	   /** TRANSLATORS: %1$s = version string and build type  ("Debug", "Release", or
+	      "RelWithDebInfo") */
 	   format(_("Version %1$s"), build_ver_details()));
 	copyright_.set_text(
 	   /** TRANSLATORS: Placeholders are the copyright years */
@@ -543,13 +563,46 @@ void MainMenu::set_button_visibility(const bool v) {
 	clock_.set_visible(v);
 }
 
+void MainMenu::end_splashscreen() {
+	assert(splash_state_ == SplashState::kSplash);
+	verb_log_info("Initiating splash screen fade out");
+	if (g_sh->current_songset() != Songset::kMenu) {
+		// mix the intro down during the splash fade out phase, then draw() will start the menu music
+		// in the menu fade in phase
+		g_sh->stop_music(kSplashFadeoutDuration);
+	}
+	splash_state_ = SplashState::kSplashFadeOut;
+	init_time_ = SDL_GetTicks();
+}
+
 void MainMenu::abort_splashscreen() {
+	verb_log_info("Splash screen ended");
+	splash_state_ = SplashState::kDone;
 	init_time_ = kNoSplash;
+	last_image_exchange_time_ = SDL_GetTicks();
+}
+
+void MainMenu::think() {
+	UI::Panel::think();
+	plugin_actions_->think();
+}
+
+void MainMenu::reinit_plugins() {
+	lua_.reset(new LuaFsMenuInterface(this));
+	plugin_actions_.reset(
+	   new PluginActions(this, [this](const std::string& cmd) { lua_->interpret_string(cmd); }));
+
+	for (const auto& pair : AddOns::g_addons) {
+		if (pair.second && pair.first->category == AddOns::AddOnCategory::kUIPlugin) {
+			lua_->run_script(kAddOnDir + FileSystem::file_separator() + pair.first->internal_name +
+			                 FileSystem::file_separator() + "init.lua");
+		}
+	}
 }
 
 bool MainMenu::handle_mousepress(uint8_t /*btn*/, int32_t /*x*/, int32_t /*y*/) {
-	if (init_time_ != kNoSplash) {
-		init_time_ = kNoSplash;
+	if (splash_state_ != SplashState::kDone) {
+		abort_splashscreen();
 		return true;
 	}
 	return false;
@@ -577,13 +630,19 @@ bool MainMenu::handle_key(const bool down, const SDL_Keysym code) {
 		return true;
 	}
 
-	if (down) {
-		bool fell_through = false;
-		if (init_time_ != kNoSplash) {
-			init_time_ = kNoSplash;
-			fell_through = true;
+	if (splash_state_ != SplashState::kDone && down) {
+		abort_splashscreen();
+		if (matches_shortcut(KeyboardShortcut::kMainMenuQuit, code)) {
+			// don't initiate quitting in this case
+			return true;
 		}
+	}
 
+	if (plugin_actions_->check_keyboard_shortcut_action(code, down)) {
+		return true;
+	}
+
+	if (down) {
 		auto check_match_shortcut = [this, &code](KeyboardShortcut k, MenuTarget t) {
 			if (matches_shortcut(k, code)) {
 				action(t);
@@ -654,10 +713,8 @@ bool MainMenu::handle_key(const bool down, const SDL_Keysym code) {
 			return true;
 		}
 		if (matches_shortcut(KeyboardShortcut::kMainMenuQuit, code)) {
-			if (!fell_through) {
-				exit();
-				return true;
-			}
+			exit();
+			return true;
 		}
 		if (matches_shortcut(KeyboardShortcut::kMainMenuContinuePlaying, code)) {
 			if (!filename_for_continue_playing_.empty()) {
@@ -686,16 +743,6 @@ bool MainMenu::handle_key(const bool down, const SDL_Keysym code) {
 	return UI::Panel::handle_key(down, code);
 }
 
-inline Rectf MainMenu::image_pos(const Image& i, const bool crop) {
-	return UI::fit_image(i.width(), i.height(), get_w(), get_h(), crop);
-}
-
-static inline void
-do_draw_image(RenderTarget& r, const Rectf& dest, const Image& img, const float opacity) {
-	r.blitrect_scale(
-	   dest, &img, Recti(0, 0, img.width(), img.height()), opacity, BlendMode::UseAlpha);
-}
-
 inline float MainMenu::calc_opacity(const uint32_t time) const {
 	return last_image_ == draw_image_ ?
 	          1.f :
@@ -704,38 +751,45 @@ inline float MainMenu::calc_opacity(const uint32_t time) const {
 }
 
 /*
- * The four phases of the animation:
- *   1) Show the splash image with full opacity on a black background for `kInitialFadeoutDelay`
- *   2) Show the splash image semi-transparent on a black background for `kInitialFadeoutDuration`
- *   3) Show the background & menu semi-transparent for `kInitialFadeoutDuration`
- *   4) Show the background & menu with full opacity indefinitely
- * We skip straight to the last phase 4 if we are returning from some other FsMenu screen.
+ * The four phases of the splash screen are:
+ *   1) SplashState::kSplash:
+ *        We start with the splash image shown with full opacity on a black background.
+ *        We initiate fade out when the intro music ends.
+ *   2) SplashState::kSplashFadeOut:
+ *        Show the splash image semi-transparent on a black background for `kSplashFadeoutDuration`
+ *   3) SplashState::kMenuFadeIn:
+ *        Show the background & menu semi-transparent for `kSplashFadeoutDuration`
+ *   4) SplashState::kDone:
+ *        Show the background & menu with full opacity indefinitely
+ *
+ * Phases 1 and 2 are handled by draw_overlay().
+ * Phase 3 is handled by draw() and draw_overlay() together.
+ * Phase 4 is handled by draw() alone.
+ * Stepping through the phases is handled by draw_overlay().
+ *
+ * We skip straight to phase 4 if we are returning from some other FsMenu screen or if a key is
+ * pressed or the mouse is clicked.
  */
 
 void MainMenu::draw(RenderTarget& r) {
-	UI::Panel::draw(r);
-	r.fill_rect(Recti(0, 0, get_w(), get_h()), RGBAColor(0, 0, 0, 255));
-
-	const uint32_t time = SDL_GetTicks();
-	assert(init_time_ == kNoSplash || time >= init_time_);
-
-	if (init_time_ != kNoSplash &&
-	    time - init_time_ < kInitialFadeoutDelay + kInitialFadeoutDuration) {
-		// still in splash phase
+	if (splash_state_ == SplashState::kSplash || splash_state_ == SplashState::kSplashFadeOut) {
+		// Handled by draw_overlay(). The actual menu is not visible in these states.
 		return;
 	}
 
-	// Sanitize phase info and button visibility
-	if (init_time_ != kNoSplash &&
-	    time - init_time_ > kInitialFadeoutDelay + 2 * kInitialFadeoutDuration) {
-		init_time_ = kNoSplash;
-	}
-	if (init_time_ == kNoSplash ||
-	    time - init_time_ > kInitialFadeoutDelay + kInitialFadeoutDuration) {
-		set_button_visibility(true);
+	// TODO(tothxa): This shouldn't be in draw(), but we don't have think(), nor a single
+	//               entry point.
+	// Reset the songset when a game, replay or editing session returns.
+	if (g_sh->current_songset() != Songset::kMenu) {
+		g_sh->change_music(Songset::kMenu, 500);
 	}
 
+	UI::Panel::draw(r);
+	r.fill_rect(Recti(0, 0, get_w(), get_h()), RGBAColor(0, 0, 0, 255));
+	set_button_visibility(true);
+
 	// Exchange stale background images
+	const uint32_t time = SDL_GetTicks();
 	assert(time >= last_image_exchange_time_);
 	if (time - last_image_exchange_time_ > kImageExchangeInterval) {
 		last_image_ = draw_image_;
@@ -751,13 +805,13 @@ void MainMenu::draw(RenderTarget& r) {
 		float opacity = 1.f;
 
 		if (time - last_image_exchange_time_ < kImageExchangeDuration) {
-			const Image& img = *g_image_cache->get(images_[last_image_]);
+			const Image* img = g_image_cache->get(images_[last_image_]);
 			opacity = calc_opacity(time);
-			do_draw_image(r, image_pos(img), img, 1.f - opacity);
+			r.blit_fit(img, true, 1.f - opacity);
 		}
 
-		const Image& img = *g_image_cache->get(images_[draw_image_]);
-		do_draw_image(r, image_pos(img), img, opacity);
+		const Image* img = g_image_cache->get(images_[draw_image_]);
+		r.blit_fit(img, true, opacity);
 	}
 
 	{  // Darken button boxes
@@ -783,29 +837,61 @@ void MainMenu::draw(RenderTarget& r) {
 
 	// Widelands logo
 	const Rectf rect = title_pos();
-	do_draw_image(
-	   r, Rectf(rect.x + rect.w * 0.2f, rect.y + rect.h * 0.2f, rect.w * 0.6f, rect.h * 0.6f),
-	   *title_image_, 1.f);
+	const Rectf dest(rect.x + rect.w * 0.2f, rect.y + rect.h * 0.2f, rect.w * 0.6f, rect.h * 0.6f);
+	r.blitrect_scale(dest, title_image_, title_image_->rect(), 1.0f, BlendMode::UseAlpha);
 }
 
 void MainMenu::draw_overlay(RenderTarget& r) {
-	if (init_time_ == kNoSplash) {
+	if (splash_state_ == SplashState::kDone) {
 		// overlays are needed only during the first three phases
 		return;
 	}
+
 	const uint32_t time = SDL_GetTicks();
 
-	if (time - init_time_ < kInitialFadeoutDelay + kInitialFadeoutDuration) {
-		const float opacity = time - init_time_ > kInitialFadeoutDelay ?
-		                         1.f - static_cast<float>(time - init_time_ - kInitialFadeoutDelay) /
-		                                  kInitialFadeoutDuration :
-		                         1.f;
-		do_draw_image(r, image_pos(*splashscreen_, false), *splashscreen_, opacity);
+	assert(init_time_ != kNoSplash && time >= init_time_);
+
+	float progress = 0.0f;
+
+	if (splash_state_ == SplashState::kSplash) {
+		// When the intro music ends, the event handler in wlapplication.cc starts the main menu
+		// music. We use that to detect when it's time to end the splash screen by default.
+		// We can't set up a notification, because the main menu may not be created before it ends
+		// if the startup is extremely slow for some reason.
+		const bool intro_is_playing = (g_sh->current_songset() == Songset::kIntro) &&
+		                              (g_sh->is_sound_audible(SoundType::kMusic)) &&
+		                              Mix_PlayingMusic() != 0;
+
+		if (!intro_is_playing) {
+			end_splashscreen();
+		}
+	} else if (time - init_time_ > kSplashFadeoutDuration) {
+		// The next step is due
+		if (splash_state_ == SplashState::kMenuFadeIn ||
+		    (time - init_time_ > 2 * kSplashFadeoutDuration)) {
+			abort_splashscreen();
+			return;
+		}
+
+		assert(splash_state_ == SplashState::kSplashFadeOut);
+		init_time_ = time;
+		last_image_exchange_time_ = time;
+		splash_state_ = SplashState::kMenuFadeIn;
+
 	} else {
-		const unsigned opacity =
-		   255 - 255.f * (time - init_time_ - kInitialFadeoutDelay - kInitialFadeoutDuration) /
-		            kInitialFadeoutDuration;
-		r.fill_rect(Recti(0, 0, get_w(), get_h()), RGBAColor(0, 0, 0, opacity), BlendMode::Default);
+		// We're in the middle of phase 2 or 3
+		progress = static_cast<float>(time - init_time_) / kSplashFadeoutDuration;
+	}
+
+	if (splash_state_ != SplashState::kMenuFadeIn) {
+		// TODO(tothxa): Some dynamic content would be nice to entertain the user while the
+		//               intro music is playing
+		const std::string footer =
+		   splash_state_ == SplashState::kSplash ? _("Click for the main menu") : "";
+		draw_splashscreen(r, footer, 1.0f - progress);
+	} else {
+		const unsigned alpha = 255 - 255.f * progress;  // fade in of menu = fade out of overlay
+		r.fill_rect(Recti(0, 0, get_w(), get_h()), RGBAColor(0, 0, 0, alpha), BlendMode::Default);
 	}
 }
 
@@ -979,7 +1065,6 @@ void MainMenu::action(const MenuTarget t) {
 			break;
 		}
 		menu_capsule_.clear_content();
-		g_sh->change_music(Songset::kIngame, 1000);
 		new NetSetupLAN(menu_capsule_);
 		break;
 	case MenuTarget::kMetaserver: {
@@ -1003,7 +1088,6 @@ void MainMenu::action(const MenuTarget t) {
 			get_config_string("password_sha1", password_);
 		}
 
-		g_sh->change_music(Songset::kIngame, 1000);
 		new InternetLobby(menu_capsule_, nickname_, password_, register_, tribeinfos);
 	} break;
 
