@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2024 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -50,6 +50,8 @@ namespace {
 constexpr int kDropdownFullscreen = -2;
 constexpr int kDropdownMaximized = -1;
 
+constexpr int kDropdownFollowMouse = -1;
+
 // Locale identifiers can look like this: ca_ES@valencia.UTF-8
 // The contents of 'selected_locale' will be changed to match the 'current_locale'
 void find_selected_locale(std::string* selected_locale, const std::string& current_locale) {
@@ -147,12 +149,28 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
                           UI::DropdownType::kTextual,
                           UI::PanelStyle::kFsMenu,
                           UI::ButtonStyle::kFsMenuMenu),
+     display_dropdown_(&box_interface_vbox_,
+                       "dropdown_display",
+                       0,
+                       0,
+                       100,  // 100 is arbitrary, will be resized in layout().
+                       50,
+                       24,
+                       pgettext("multiscreen", "Display"),
+                       UI::DropdownType::kTextual,
+                       UI::PanelStyle::kFsMenu,
+                       UI::ButtonStyle::kFsMenuMenu),
      sdl_cursor_(&box_interface_,
                  UI::PanelStyle::kFsMenu,
                  "sdl_cursor",
                  Vector2i::zero(),
-                 _("Use system mouse cursor"),
-                 "",
+                 /** TRANSLATORS: short description for the sdl_cursor option */
+                 _("Let the system draw the mouse cursor"),
+                 /** TRANSLATORS: tooltip text for the sdl_cursor option */
+                 _("If in doubt, leave this enabled. When disabled, cursor updates may be slow "
+                   "and the cursor appears frozen during long operations. Disable it only if "
+                   "the cursor doesn’t appear right, or if you want it to be visible in "
+                   "screenshots or screencasts."),
                  0),
      tooltip_accessibility_mode_(&box_interface_,
                                  UI::PanelStyle::kFsMenu,
@@ -408,6 +426,7 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
 	// Interface
 	box_interface_vbox_.add(&language_dropdown_, UI::Box::Resizing::kFullSize);
 	box_interface_vbox_.add(&resolution_dropdown_, UI::Box::Resizing::kFullSize);
+	box_interface_vbox_.add(&display_dropdown_, UI::Box::Resizing::kFullSize);
 	// TODO(tothxa): Replace with infinite space if box layouting quirks get fixed
 	box_interface_vbox_.add(&translation_padding_, UI::Box::Resizing::kFullSize);
 	// box_interface_vbox_.add_inf_space();
@@ -510,6 +529,7 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
 
 	// Fill in data
 	// Interface options
+	add_displays(opt);
 	add_screen_resolutions(opt);
 
 	sdl_cursor_.set_state(opt.sdl_cursor);
@@ -556,6 +576,26 @@ Options::Options(MainMenu& fsmm, OptionsCtrl::OptionsStruct opt)
 	}
 	initialization_complete();
 }
+
+void Options::add_displays(const OptionsCtrl::OptionsStruct& opt) {
+	display_dropdown_.add(
+	   /** TRANSLATORS: Entry in the display selection dropdown */
+	   pgettext("multiscreen", "Follow mouse"), kDropdownFollowMouse, nullptr, opt.display < 0);
+	for (int i = 0; i < SDL_GetNumVideoDisplays(); ++i) {
+		SDL_Rect r;
+		if (SDL_GetDisplayBounds(i, &r) == 0) {
+			display_dropdown_.add(
+			   /** TRANSLATORS: Display index and virtual coordinates, e.g, '#0 (0, 0, 1920, 1080)'*/
+			   format(_("#%1% (%2%, %3%, %4%, %5%)"), i, r.x, r.y, r.w, r.h), i, nullptr,
+			   opt.display == i);
+		}
+	}
+	if (!display_dropdown_.has_selection()) {
+		display_dropdown_.select(kDropdownFollowMouse);
+	}
+	display_dropdown_.set_enabled(SDL_GetNumVideoDisplays() > 1);
+}
+
 void Options::add_screen_resolutions(const OptionsCtrl::OptionsStruct& opt) {
 	ScreenResolution current_res = {g_gr->get_window_mode_xres(), g_gr->get_window_mode_yres()};
 
@@ -615,7 +655,8 @@ void Options::layout() {
 		language_dropdown_.set_height(tabs_.get_h() - language_dropdown_.get_y() - buth -
 		                              3 * kPadding);
 
-		const int min_h = language_dropdown_.get_h() + resolution_dropdown_.get_h() + 2 * kPadding;
+		const int min_h = language_dropdown_.get_h() + resolution_dropdown_.get_h() +
+		                  display_dropdown_.get_h() + 3 * kPadding;
 		const int half_w = (tab_panel_width - 3 * kPadding) / 2;
 
 		// Make initial value big enough to avoid needing a scrollbar
@@ -676,15 +717,17 @@ void Options::add_languages_to_list(const std::string& current_locale) {
 
 		// We start with the locale directory so we can pick up locales
 		// that don't have a configuration file yet.
-		std::unique_ptr<FileSystem> fs(&FileSystem::create(i18n::get_localedir()));
+		std::unique_ptr<FileSystem> fs(&FileSystem::create(i18n::get_localedir() + "/widelands"));
 		FilenameSet files = fs->list_directory(".");
 
-		for (const std::string& localename : files) {  // Begin scan locales directory
+		for (std::string localename : files) {  // Begin scan locales directory
 			const char* path = localename.c_str();
 			if ((strcmp(FileSystem::fs_filename(path), ".") == 0) ||
-			    (strcmp(FileSystem::fs_filename(path), "..") == 0) || !fs->is_directory(path)) {
+			    (strcmp(FileSystem::fs_filename(path), "..") == 0) ||
+			    FileSystem::filename_ext(path) != ".po") {
 				continue;
 			}
+			localename = FileSystem::filename_without_ext(path);
 
 			try {  // Begin read locale from table
 				std::unique_ptr<LuaTable> table = all_locales->get_table(localename);
@@ -834,6 +877,9 @@ OptionsCtrl::OptionsStruct Options::get_values() {
 			os_.yres = res.yres;
 		}
 	}
+	if (display_dropdown_.has_selection()) {
+		os_.display = display_dropdown_.get_selected();
+	}
 	os_.sdl_cursor = sdl_cursor_.get_state();
 	os_.tooltip_accessibility_mode = tooltip_accessibility_mode_.get_state();
 
@@ -892,9 +938,10 @@ void OptionsCtrl::handle_menu() {
 	MenuTarget i = opt_dialog_->run<MenuTarget>();
 	if (i != MenuTarget::kBack) {
 		save_options();
-		g_gr->set_fullscreen(opt_dialog_->get_values().fullscreen);
+		int display = opt_dialog_->get_values().display;
+		g_gr->set_fullscreen(opt_dialog_->get_values().fullscreen, display);
 		if (opt_dialog_->get_values().maximized) {
-			g_gr->set_maximized(true);
+			g_gr->set_maximized(true, display);
 		} else if (!opt_dialog_->get_values().fullscreen && !opt_dialog_->get_values().maximized) {
 			g_gr->change_resolution(
 			   opt_dialog_->get_values().xres, opt_dialog_->get_values().yres, true);
@@ -911,6 +958,7 @@ void OptionsCtrl::handle_menu() {
 OptionsCtrl::OptionsStruct OptionsCtrl::options_struct(uint32_t active_tab) {
 	OptionsStruct opt;
 	// Interface options
+	opt.display = opt_section_.get_int("display", kDropdownFollowMouse);
 	opt.xres = opt_section_.get_int("xres", kDefaultResolutionW);
 	opt.yres = opt_section_.get_int("yres", kDefaultResolutionH);
 	opt.maximized = opt_section_.get_bool("maximized", false);
@@ -960,6 +1008,7 @@ void OptionsCtrl::save_options() {
 	OptionsCtrl::OptionsStruct opt = opt_dialog_->get_values();
 
 	// Interface options
+	opt_section_.set_int("display", opt.display);
 	opt_section_.set_int("xres", opt.xres);
 	opt_section_.set_int("yres", opt.yres);
 	opt_section_.set_bool("maximized", opt.maximized);
@@ -1003,6 +1052,7 @@ void OptionsCtrl::save_options() {
 	g_mouse_cursor->set_use_sdl(opt_dialog_->get_values().sdl_cursor);
 	i18n::set_locale(opt.language);
 	UI::g_fh->reinitialize_fontset(i18n::get_locale());
+	WLApplication::get().init_plugin_shortcuts();  // To update the descnames
 
 	// Sound options
 	g_sh->save_config();
