@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2024 by the Widelands Development Team
+ * Copyright (C) 2006-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,6 +29,7 @@
 #include <regex>
 
 #include <SDL.h>
+#include <SDL_mouse.h>
 #include <SDL_ttf.h>
 #ifdef __APPLE__
 #include <mach-o/dyld.h>
@@ -53,6 +54,7 @@
 #include "graphic/default_resolution.h"
 #include "graphic/font_handler.h"
 #include "graphic/graphic.h"
+#include "graphic/graphic_functions.h"
 #include "graphic/mouse_cursor.h"
 #include "graphic/style_manager.h"
 #include "graphic/text/font_set.h"
@@ -379,9 +381,8 @@ WLApplication::WLApplication(int const argc, char const* const* const argv)
 	parse_commandline(argc, argv);  // throws ParameterError, handled by main.cc
 
 	setup_homedir();
-	init_settings();
-	datadir_ = g_fs->canonicalize_name(datadir_);
-	datadir_for_testing_ = g_fs->canonicalize_name(datadir_for_testing_);
+
+	init_settings();  // Also sets up the filesystems and language support.
 
 	set_initializer_thread();
 
@@ -389,20 +390,7 @@ WLApplication::WLApplication(int const argc, char const* const* const argv)
 	signal(SIGUSR1, toggle_verbose);
 #endif
 
-	log_info("Adding directory: %s\n", datadir_.c_str());
-	g_fs->add_file_system(&FileSystem::create(datadir_));
-
-	if (!datadir_for_testing_.empty()) {
-		log_info("Adding directory: %s\n", datadir_for_testing_.c_str());
-		g_fs->add_file_system(&FileSystem::create(datadir_for_testing_));
-	}
-
-	init_language();  // search paths must already be set up
 	changedir_on_mac();
-	cleanup_replays();
-	cleanup_ai_files();
-	cleanup_temp_files();
-	cleanup_temp_backups();
 
 #ifndef SDL_BYTEORDER
 	log_info("Byte order: unknown, assuming little-endian\n");
@@ -420,13 +408,97 @@ WLApplication::WLApplication(int const argc, char const* const* const argv)
 		exit(2);
 	}
 
+	// Start intro music before splashscreen: it takes slightly less time,
+	// and the music starts with some delay
+	g_sh = new SoundHandler();
+	g_sh->register_songs("music", Songset::kIntro);
+	g_sh->register_songs("music", Songset::kMenu);
+	g_sh->change_music(get_config_bool("play_intro_music", true) ? Songset::kIntro : Songset::kMenu);
+
+	g_gr = new Graphic();
+	g_gr->initialize(
+	   get_config_bool("debug_gl_trace", false) ? Graphic::TraceGl::kYes : Graphic::TraceGl::kNo,
+	   get_config_int("display", -1), get_config_int("xres", kDefaultResolutionW),
+	   get_config_int("yres", kDefaultResolutionH), get_config_bool("fullscreen", false),
+	   get_config_bool("maximized", false));
+
+	{
+		// The window manager may resize the window on creation, so we have to handle resize events
+		// first to be able to draw the splash screen in the right size. This also creates mousemove
+		// events that we may mess up when we set the cursor.
+		// We throw away everything else, hopefully we don't have much yet...
+		// Window event and mouse cursor handling needs g_gr.
+
+		SDL_PumpEvents();
+
+		// Known harmless events we'd drop anyway
+		SDL_FlushEvent(SDL_AUDIODEVICEADDED);
+		SDL_FlushEvent(SDL_TEXTEDITING);  // reported on windows
+		// end of ignored events
+
+		SDL_Event ev;
+		int ignored = 0;
+		int handled = 0;
+
+		while (SDL_PollEvent(&ev) != 0) {
+			if (ev.type == SDL_WINDOWEVENT) {
+				handle_window_event(ev);
+				++handled;
+			} else if (ev.type == SDL_MOUSEMOTION) {
+				mouse_position_ = Vector2i(ev.motion.x, ev.motion.y);
+			} else {
+				verb_log_dbg("Ignoring SDL event 0x%04x", ev.type);
+				++ignored;
+			}
+		}
+		if (ignored > 0) {
+			log_warn("Ignored %d non-mousemove SDL events at start up", ignored);
+		}
+		if (handled > 0) {
+			// Initial creation already creates some events
+			verb_log_info("Handled %d SDL window events at start up", handled);
+		}
+	}
+
+	init_mouse_cursor();
+
+	// Prepare for drawing splash screen
+
+	/*****
+	 * These could be moved later if we decide to show some graphic (an hourglass?) instead
+	 * of the text label in the initial splash screen to draw it faster.
+	 */
+	if (TTF_Init() == -1) {
+		log_err("True Type library did not initialize: %s\n", TTF_GetError());
+		exit(2);
+	}
+
+	UI::g_fh = UI::create_fonthandler(
+	   g_image_cache, i18n::get_locale());  // This will create the fontset, so loading it first.
+	verb_log_info("Font handler created");
+
+	set_template_dir("");
+	verb_log_info("Loaded default styles");
+	/*
+	 * End of text rendering dependencies
+	 *****/
+
+	// The initital splash screen is only drawn once, it doesn't get updates until the main menu
+	// overrides it. Normally it shouldn't take more than a few seconds.
+	RenderTarget* r = g_gr->get_render_target();
+	draw_splashscreen(*r, _("Loading…"), 1.0f);
+	g_gr->refresh();
+	verb_log_info("Splash screen shown");
+
+	// This was taken out of Graphic::initialize() to allow showing the splashscreen earlier.
+	// This is one of the slowest parts of the start up.
+	g_gr->rebuild_texture_atlas();
+
 	// Try to detect configurations with inverted horizontal scroll
-	const char* sdl_video = SDL_GetCurrentVideoDriver();
-	assert(sdl_video != nullptr);
 	SDL_version sdl_ver = {SDL_MAJOR_VERSION, SDL_MINOR_VERSION, SDL_PATCHLEVEL};
 	SDL_GetVersion(&sdl_ver);
-	// Keep cursor in window while dragging
-	SDL_SetHint(SDL_HINT_MOUSE_RELATIVE_MODE_WARP, "1");
+	const char* sdl_video = SDL_GetCurrentVideoDriver();
+	assert(sdl_video != nullptr);
 	bool sdl_scroll_x_bug = false;
 
 	// SDL version < 2.0 is not supported, >= 2.1 will have the changes
@@ -444,41 +516,37 @@ WLApplication::WLApplication(int const argc, char const* const* const argv)
 		update_mousewheel_settings();
 	}
 
-	g_gr = new Graphic();
+	// Keep cursor in window while dragging
+	SDL_SetHint(SDL_HINT_MOUSE_RELATIVE_MODE_WARP, "1");
 
-	if (TTF_Init() == -1) {
-		log_err("True Type library did not initialize: %s\n", TTF_GetError());
-		exit(2);
-	}
+	verb_log_info("Cleaning up temporary files");
+	cleanup_replays();
+	cleanup_ai_files();
+	cleanup_temp_files();
+	cleanup_temp_backups();
 
-	UI::g_fh = UI::create_fonthandler(
-	   g_image_cache, i18n::get_locale());  // This will create the fontset, so loading it first.
-
-	g_gr->initialize(
-	   get_config_bool("debug_gl_trace", false) ? Graphic::TraceGl::kYes : Graphic::TraceGl::kNo,
-	   get_config_int("xres", kDefaultResolutionW), get_config_int("yres", kDefaultResolutionH),
-	   get_config_bool("fullscreen", false), get_config_bool("maximized", false));
-
-	g_mouse_cursor = new MouseCursor();
-	g_mouse_cursor->initialize(get_config_bool("sdl_cursor", true));
-
-	g_sh = new SoundHandler();
-
-	g_sh->register_songs("music", Songset::kIntro);
-	g_sh->register_songs("music", Songset::kMenu);
+	verb_log_info("Loading songsets");
 	g_sh->register_songs("music", Songset::kIngame);
 	g_sh->register_songs("music", Songset::kCustom);
 
 	UI::ColorChooser::read_favorites_settings();
 
-	set_template_dir("");
+	verb_log_info("Initializing Add-Ons");
 	initialize_g_addons();
+	verb_log_info("Done initializing Add-Ons");
 
 	// Register the click sound for UI::Panel.
 	// We do it here to ensure that the sound handler has been created first, and we only want to
 	// register it once.
 	UI::Panel::register_click();
 
+	// Mark playlist config values as used to prevent errors with the options system
+	// for these auto-generated config keys
+	Section* sec = get_config_section_ptr("songs");
+	if (sec != nullptr) {
+		sec->mark_values();
+	}
+
 	// Make sure we didn't forget to read any global option
 	check_config_used();
 
@@ -493,6 +561,8 @@ WLApplication::WLApplication(int const argc, char const* const* const argv)
 		log_info("Developer tools are enabled.");
 		g_script_console_history.load(kScriptConsoleHistoryFile);
 	}
+
+	verb_log_info("WLApplication created");
 }
 
 /**
@@ -518,6 +588,9 @@ WLApplication::~WLApplication() {
 		g_script_console_history.save(kScriptConsoleHistoryFile);
 	}
 
+	// To be proper, release our textdomain
+	i18n::release_textdomain();
+
 	assert(UI::g_fh);
 	delete UI::g_fh;
 	UI::g_fh = nullptr;
@@ -537,7 +610,62 @@ WLApplication::~WLApplication() {
 	SDL_Quit();
 }
 
+void WLApplication::init_mouse_cursor() {
+	// Fix mouse_position_ initialisation when we don't have mousemove events at startup.
+	// This is actually only needed by soft mode, but we do it for SDL mode too to make the
+	// cursor appear at the right position if the user has to use the keyboard to turn SDL
+	// mode off.
+	if (mouse_position_ == Vector2i::zero()) {
+		// Initialize the mouse position to the current one.
+		// Unfortunately we have to do it the hard way, because SDL_GetMouseState() doesn't work
+		// right until we had a mousemove event.
+		int mouse_global_x;
+		int mouse_global_y;
+		int window_x;
+		int window_y;
+		SDL_Window* sdl_window = g_gr->get_sdlwindow();
+		SDL_GetWindowPosition(sdl_window, &window_x, &window_y);
+		SDL_GetGlobalMouseState(&mouse_global_x, &mouse_global_y);
+		mouse_position_.x = mouse_global_x - window_x;
+		mouse_position_.y = mouse_global_y - window_y;
+
+		// Fix SDL's internal notion of the relative cursor position too by generating some motion
+		// events. Must be done before g_mouse_cursor->initialize().
+		// TODO(tothxa): I don't know why, but all these steps seem to be necessary on my system to
+		//               fix the case in soft mode when the mouse is first moved while it is hidden.
+		//               Without these, it is resumed at the position where it was hidden.
+		SDL_PumpEvents();
+		SDL_WarpMouseInWindow(sdl_window, mouse_position_.x - 1, mouse_position_.y - 1);
+		SDL_PumpEvents();
+		SDL_Delay(2);  // 1 tick doesn't work
+		SDL_WarpMouseInWindow(sdl_window, mouse_position_.x, mouse_position_.y);
+		SDL_PumpEvents();
+	}
+
+	if (!get_config_bool("sdl_cursor", true)) {
+		// Set system's "waiting" mouse cursor in case setting our own cursor in
+		// g_mouse_cursor->initialize() fails. Maybe it works, maybe not.
+		SDL_Cursor* tmp_cursor = SDL_CreateSystemCursor(SDL_SYSTEM_CURSOR_WAIT);
+		SDL_SetCursor(tmp_cursor);
+		SDL_FreeCursor(tmp_cursor);
+	}
+
+	// The cursor initialization itself
+	// We always init it in SDL mode to have a chance of a working cursor in the splash screen,
+	// before we start refreshing the screen ourselves.
+	// TODO(tothxa): May not work if the system doesn't support color cursors? What happens then?
+	//               No cursor, default system cursor or error?
+	//               Remove argument if this works out. And move initialize() into the constructor?
+	g_mouse_cursor = new MouseCursor();
+	g_mouse_cursor->initialize(true);
+
+	// TODO(tothxa): Do this in g_mouse_cursor->initialize() or the constructor too?
+	g_mouse_cursor->change_wait(true);
+}
+
 void WLApplication::initialize_g_addons() {
+	init_plugin_shortcuts();
+
 	AddOns::g_addons.clear();
 	if (g_fs->is_directory(kAddOnDir)) {
 		std::set<std::string> found;
@@ -656,7 +784,7 @@ static void init_one_player_from_template(unsigned p,
 	}
 	if (!found_init) {
 		throw wexception(
-		   "Invalid starting condition '%s' for player %d", init_script_name.c_str(), p + 1);
+		   "Invalid starting condition '%s' for player %u", init_script_name.c_str(), p + 1);
 	}
 }
 
@@ -815,12 +943,18 @@ void WLApplication::run() {
 
 	FsMenu::MainMenu menu(game_type_ != GameType::kNone);
 
+	// This is actually the last step of initialization, postponed from init_mouse_cursor().
+	// FsMenu::MainMenu() takes a few seconds and we will only start refreshing the screen
+	// by the chosen option below.
+	if (!get_config_bool("sdl_cursor", true)) {
+		g_mouse_cursor->set_use_sdl(false);
+	}
+
 	check_crash_reports(menu);
 
 	switch (game_type_) {
 	case GameType::kEditor: {
 		bool success = false;
-		g_sh->change_music(Songset::kIngame);
 		if (filename_.empty()) {
 			success = EditorInteractive::run_editor(&menu, EditorInteractive::Init::kDefault);
 		} else {
@@ -831,8 +965,9 @@ void WLApplication::run() {
 				} else {
 					const std::string message = _("Widelands could not find the last edited map.");
 					log_err("%s\n", message.c_str());
-
-					g_sh->change_music(Songset::kMenu);
+					if (g_fail_on_errors) {
+						abort();
+					}
 					menu.show_messagebox(_("No Last Edited Map"), message);
 					have_filename = false;
 				}
@@ -854,7 +989,6 @@ void WLApplication::run() {
 		Widelands::Game game;
 		std::string title;
 		std::string message;
-		g_sh->change_music(Songset::kIngame);
 		try {
 			bool start_replay = (game_type_ == GameType::kReplay);
 			if (use_last(filename_)) {
@@ -887,16 +1021,20 @@ void WLApplication::run() {
 		if (!message.empty()) {
 			log_err("%s\n", message.c_str());
 			game.full_cleanup();
-			g_sh->change_music(Songset::kMenu);
+			if (g_fail_on_errors) {
+				abort();
+			}
 			menu.show_messagebox(title, message);
 			menu.main_loop();
 		}
 	} break;
 
 	case GameType::kScenario: {
-		g_sh->change_music(Songset::kIngame);
 		Widelands::Game game;
 		try {
+			if (scenario_difficulty_ != Widelands::kScenarioDifficultyNotSet) {
+				game.set_scenario_difficulty(scenario_difficulty_);
+			}
 			game.run_splayer_scenario_direct({filename_}, script_to_run_);
 		} catch (const std::exception& e) {
 			emergency_save(&menu, game, e.what());
@@ -904,17 +1042,10 @@ void WLApplication::run() {
 	} break;
 
 	case GameType::kFromTemplate: {
-		g_sh->change_music(Songset::kIngame);
 		init_and_run_game_from_template(menu);
 	} break;
 
 	default:
-		// TODO(tothxa): This is actually never heard due to the next one replacing it immediately.
-		//               Can we delete this, and the song file too?
-		g_sh->change_music(Songset::kIntro);
-
-		g_sh->change_music(Songset::kMenu, 1000);
-
 		menu.main_loop();
 	}
 
@@ -948,21 +1079,12 @@ bool WLApplication::poll_event(SDL_Event& ev) const {
 
 	case SDL_USEREVENT: {
 		if (ev.user.code == CHANGE_MUSIC) {
-			/* Notofication from the SoundHandler that a song has finished playing.
+			/* Notification from the SoundHandler that a song has finished playing.
 			 * Usually, another song from the same songset will be started.
-			 * There is a special case for the intro screen's music: only one song will be
-			 * played. If the user has not clicked the mouse or pressed escape when the song
-			 * finishes, Widelands will automatically go on to the main menu.
+			 * There is a special case for the intro music: it will only be played once.
 			 */
 			assert(!SoundHandler::is_backend_disabled());
-			if (g_sh->current_songset() == "intro") {
-				// Special case for splashscreen: there, only one song is ever played
-				SDL_Event new_event;
-				new_event.type = SDL_KEYDOWN;
-				new_event.key.state = SDL_PRESSED;
-				new_event.key.keysym.sym = SDLK_ESCAPE;
-				SDL_PushEvent(&new_event);
-			} else {
+			if (g_sh->current_songset() != Songset::kIntro) {
 				g_sh->change_music();
 			}
 		}
@@ -974,6 +1096,27 @@ bool WLApplication::poll_event(SDL_Event& ev) const {
 	return true;
 }
 
+void WLApplication::handle_window_event(SDL_Event& ev) {
+	assert(ev.type == SDL_WINDOWEVENT);
+	switch (ev.window.event) {
+	case SDL_WINDOWEVENT_RESIZED:
+		// Do not save the new size to config at this point to avoid saving sizes that
+		// result from maximization etc. Save at shutdown instead.
+		if (!g_gr->fullscreen()) {
+			g_gr->change_resolution(ev.window.data1, ev.window.data2, false);
+		}
+		break;
+	case SDL_WINDOWEVENT_MAXIMIZED:
+		set_config_bool("maximized", true);
+		break;
+	case SDL_WINDOWEVENT_RESTORED:
+		set_config_bool("maximized", g_gr->maximized());
+		break;
+	default:
+		break;
+	}
+}
+
 bool WLApplication::handle_key(bool down, const SDL_Keycode& keycode, const int modifiers) {
 	if (!down || !handle_key_enabled_) {
 		return false;
@@ -1064,26 +1207,13 @@ void WLApplication::handle_input(InputCallback const* cb) {
 			}
 			break;
 		case SDL_WINDOWEVENT:
-			switch (ev.window.event) {
-			case SDL_WINDOWEVENT_RESIZED:
-				// Do not save the new size to config at this point to avoid saving sizes that
-				// result from maximization etc. Save at shutdown instead.
-				if (!g_gr->fullscreen()) {
-					g_gr->change_resolution(ev.window.data1, ev.window.data2, false);
-				}
-				break;
-			case SDL_WINDOWEVENT_MAXIMIZED:
-				set_config_bool("maximized", true);
-				break;
-			case SDL_WINDOWEVENT_RESTORED:
-				set_config_bool("maximized", g_gr->maximized());
-				break;
-			}
+			handle_window_event(ev);
 			break;
 		case SDL_QUIT:
 			should_die_ = true;
 			break;
-		default:;
+		default:
+			break;
 		}
 	}
 
@@ -1187,6 +1317,20 @@ void WLApplication::set_mouse_lock(const bool locked) {
 	}
 }
 
+/** Register the datadir and the datadir for testing (if any) with the global filesystem wrapper. */
+void WLApplication::init_filesystems() {
+	datadir_ = g_fs->canonicalize_name(datadir_);
+	datadir_for_testing_ = g_fs->canonicalize_name(datadir_for_testing_);
+
+	log_info("Adding data directory: %s\n", datadir_.c_str());
+	g_fs->add_file_system(&FileSystem::create(datadir_));
+
+	if (!datadir_for_testing_.empty()) {
+		log_info("Adding testing directory: %s\n", datadir_for_testing_.c_str());
+		g_fs->add_file_system(&FileSystem::create(datadir_for_testing_));
+	}
+}
+
 /**
  * Read the config file, parse the commandline and give all other internal
  * parameters sensible default values
@@ -1234,7 +1378,7 @@ void WLApplication::init_language() {
 	if (!localedir_.empty()) {
 		i18n::set_localedir(g_fs->canonicalize_name(localedir_));
 	} else {
-		i18n::set_localedir(g_fs->canonicalize_name(datadir_ + "/locale"));
+		i18n::set_localedir(g_fs->canonicalize_name(datadir_ + "/i18n/translations"));
 	}
 
 	// If locale dir is not a directory, barf. We can handle it not being there tough.
@@ -1254,19 +1398,91 @@ void WLApplication::init_language() {
 
 	// Initialize locale and grab "widelands" textdomain
 	i18n::init_locale();
-	i18n::grab_textdomain("widelands", i18n::get_localedir().c_str());
+	i18n::grab_textdomain("widelands", i18n::get_localedir());
 
 	// Set locale corresponding to selected language
 	std::string language = get_config_string("language", "");
 	i18n::set_locale(language);
 }
 
+void WLApplication::init_plugin_shortcuts() {
+	LuaInterface lua;
+	for (const std::string& name : g_fs->list_directory(kAddOnDir)) {
+		std::string path = name;
+		path += FileSystem::file_separator();
+		path += kAddOnKeyboardShortcutsFile;
+		if (g_fs->file_exists(path)) {
+			try {
+				std::unique_ptr<LuaTable> all_definitions(lua.run_script(path));
+				for (const auto& table : all_definitions->array_entries<std::unique_ptr<LuaTable>>()) {
+					std::string internal_name;
+					try {
+						internal_name = table->get_string("internal_name");
+						std::string descname = table->get_string("descname");
+
+						std::set<KeyboardShortcutScope> scopes;
+						std::unique_ptr<LuaTable> scopes_table = table->get_table("scopes");
+						for (const std::string& scope : scopes_table->array_entries<std::string>()) {
+							if (scope == "global") {
+								scopes.insert(KeyboardShortcutScope::kGlobal);
+							} else if (scope == "game") {
+								scopes.insert(KeyboardShortcutScope::kGame);
+							} else if (scope == "editor") {
+								scopes.insert(KeyboardShortcutScope::kEditor);
+							} else if (scope == "main_menu") {
+								scopes.insert(KeyboardShortcutScope::kMainMenu);
+							} else {
+								throw WLWarning("", "Invalid scope '%s'", scope.c_str());
+							}
+						}
+						if (scopes.empty()) {
+							throw WLWarning("", "No scopes");
+						}
+
+						std::string keycode_name = table->get_string("keycode");
+						SDL_Keycode default_shortcut = SDL_GetKeyFromName(keycode_name.c_str());
+						if (default_shortcut == SDLK_UNKNOWN) {
+							throw WLWarning("", "Invalid keycode '%s'", keycode_name.c_str());
+						}
+
+						int default_mods = 0;
+						if (table->has_key("mods")) {
+							std::unique_ptr<LuaTable> mods_table = table->get_table("mods");
+							for (const std::string& mod : mods_table->array_entries<std::string>()) {
+								if (mod == "ctrl" || mod == "control") {
+									default_mods |= KMOD_CTRL;
+								} else if (mod == "shift") {
+									default_mods |= KMOD_SHIFT;
+								} else if (mod == "alt") {
+									default_mods |= KMOD_ALT;
+								} else if (mod == "gui" || mod == "super" || mod == "meta" ||
+								           mod == "cmd" || mod == "command" || mod == "windows") {
+									default_mods |= KMOD_GUI;
+								} else {
+									throw WLWarning("", "Invalid modifier '%s'", mod.c_str());
+								}
+							}
+						}
+
+						create_replace_shortcut(
+						   internal_name, descname, scopes, keysym(default_shortcut, default_mods));
+					} catch (const std::exception& e) {
+						log_err("Error in plugin keyboard shortcut definition in '%s': '%s': %s",
+						        path.c_str(), internal_name.c_str(), e.what());
+					}
+				}
+			} catch (const std::exception& e) {
+				log_err("Error reading plugin keyboard shortcut definitions from '%s': %s",
+				        path.c_str(), e.what());
+			}
+		}
+	}
+}
+
 /**
  * Remember the last settings: write them into the config file
  */
 void WLApplication::shutdown_settings() {
-	// To be proper, release our textdomain
-	i18n::release_textdomain();
 	write_config();
 }
 
@@ -1453,31 +1669,21 @@ void WLApplication::handle_commandline_parameters() {
 			}
 
 			if (sep_pos != build_id().size() || 0 != text.compare(0, sep_pos, build_id())) {
-				return std::string("Incorrect version string part");
-			}
-
-			text = text.substr(sep_pos + (text.at(sep_pos) == '\r' ? 2 : 1));
-			sep_pos = text.find_first_of("\n\r");
-			if (sep_pos == std::string::npos) {
-				return std::string("Malformed two-liner version string");
-			}
-
-			if (sep_pos != build_type().size() || 0 != text.compare(0, sep_pos, build_type())) {
-				return std::string("Incorrect type string part");
+				return std::string("Incorrect version string");
 			}
 		} catch (const std::exception& e) {
 			return std::string(e.what());
 		}
 		return std::string();
 	};
-	bool found = false;
+	bool found_datadir = false;
 	if (OptionalParameter datadir_option = get_commandline_option_value("datadir");
 	    datadir_option.has_value()) {
 		datadir_ = *datadir_option;
 
 		const std::string err = checkdatadirversion(datadir_);
-		found = err.empty();
-		if (!found) {
+		found_datadir = err.empty();
+		if (!found_datadir) {
 			log_err("Invalid explicit datadir '%s': %s", datadir_.c_str(), err.c_str());
 		}
 	} else {
@@ -1488,7 +1694,7 @@ void WLApplication::handle_commandline_parameters() {
 			datadir_ = INSTALL_DATADIR;
 			const std::string err = checkdatadirversion(datadir_);
 			if (err.empty()) {
-				found = true;
+				found_datadir = true;
 			} else {
 				wrong_candidates.emplace_back(datadir_, err);
 			}
@@ -1496,7 +1702,7 @@ void WLApplication::handle_commandline_parameters() {
 
 		// Next, pick the first applicable XDG path.
 #ifdef USE_XDG
-		if (!found) {
+		if (!found_datadir) {
 			for (const auto& datadir : FileSystem::get_xdgdatadirs()) {
 				RealFSImpl dir(datadir);
 				if (dir.is_directory(datadir + "/widelands")) {
@@ -1504,7 +1710,7 @@ void WLApplication::handle_commandline_parameters() {
 
 					const std::string err = checkdatadirversion(datadir_);
 					if (err.empty()) {
-						found = true;
+						found_datadir = true;
 						break;
 					}
 					wrong_candidates.emplace_back(datadir_, err);
@@ -1514,17 +1720,17 @@ void WLApplication::handle_commandline_parameters() {
 #endif
 
 		// Finally, try a relative datadir.
-		if (!found) {
+		if (!found_datadir) {
 			datadir_ = get_executable_directory() + FileSystem::file_separator() + INSTALL_DATADIR;
 			const std::string err = checkdatadirversion(datadir_);
 			if (err.empty()) {
-				found = true;
+				found_datadir = true;
 			} else {
 				wrong_candidates.emplace_back(datadir_, err);
 			}
 		}
 
-		if (!found) {
+		if (!found_datadir) {
 			log_err("Unable to detect the datadir. Please specify a datadir explicitly\n"
 			        "with the --datadir command line option. Tried the following %d path(s):",
 			        static_cast<int>(wrong_candidates.size()));
@@ -1533,22 +1739,30 @@ void WLApplication::handle_commandline_parameters() {
 			}
 		}
 	}
-	if (found && !is_absolute_path(datadir_)) {
+	if (found_datadir && !is_absolute_path(datadir_)) {
 		try {
 			datadir_ = absolute_path_if_not_windows(FileSystem::get_working_directory() +
 			                                        FileSystem::file_separator() + datadir_);
 		} catch (const WException& e) {
 			log_err("Error parsing datadir: %s\n", e.what());
-			found = false;
+			found_datadir = false;
 		}
 	}
 
+	if (OptionalParameter testdir = get_commandline_option_value("datadir_for_testing");
+	    testdir.has_value()) {
+		datadir_for_testing_ = *testdir;
+	}
+
 	if (OptionalParameter lang = get_commandline_option_value("language"); lang.has_value()) {
 		set_config_string("language", *lang);
 	}
-	if (found) {
-		init_language();  // do this now to have translated command line help
-	}
+
+	init_filesystems();
+
+	// Do this now to have translated command line help.
+	init_language();
+
 	// Set up list of valid command line options and their translated help texts
 	fill_parameter_vector();
 
@@ -1559,11 +1773,6 @@ void WLApplication::handle_commandline_parameters() {
 		   format(_("Unknown command line parameter: %s\nMaybe a '=' is missing?"), *err));
 	}
 
-	if (OptionalParameter testdir = get_commandline_option_value("datadir_for_testing");
-	    testdir.has_value()) {
-		datadir_for_testing_ = *testdir;
-	}
-
 	// TODO(tothxa): These were checked before datadir and locale were set up, but don't seem to be
 	//               used during detecting and setting them up. Let's see if anything breaks if we
 	//               move them here.
@@ -1574,6 +1783,31 @@ void WLApplication::handle_commandline_parameters() {
 		g_fail_on_lua_error = true;
 	}
 
+	if (OptionalParameter msg_timeout = get_commandline_option_value("messagebox-timeout");
+	    msg_timeout.has_value()) {
+		int64_t t;
+		if (!to_long(msg_timeout.value(), &t)) {
+			throw ParameterError(
+			   CmdLineVerbosity::None,
+			   format(_("Non-integer value for command line parameter --messagebox-timeout=%s"),
+			          msg_timeout.value()));
+		}
+		if (t <= 0) {
+			throw ParameterError(
+			   CmdLineVerbosity::None,
+			   ("Value for command line parameter --messagebox-timeout must be positive."));
+		}
+
+		g_message_box_timeout = 1000 * t;
+		if (g_message_box_timeout / 1000 != t) {
+			g_message_box_timeout = 0;
+			throw ParameterError(
+			   CmdLineVerbosity::None,
+			   format(_("Value is out of range for command line parameter --messagebox-timeout=%s"),
+			          msg_timeout.value()));
+		}
+	}
+
 	// Mutually exclusive options.
 	// These would be better as e.g. "--use_zip=[true|false]", but then we'd have to negate the
 	// boolean value stored in the string, but trueWords and falseWords are hidden in io/profile.cc.
@@ -1591,6 +1825,10 @@ void WLApplication::handle_commandline_parameters() {
 
 	// *** End of moved checks ***
 
+	if (check_commandline_flag("fail-on-errors")) {
+		g_fail_on_errors = true;
+	}
+
 	if (check_commandline_flag("verbose")) {
 		g_verbose = true;
 	}
@@ -1622,6 +1860,27 @@ void WLApplication::handle_commandline_parameters() {
 		}
 	}
 
+	if (OptionalParameter difficulty = get_commandline_option_value("difficulty");
+	    difficulty.has_value()) {
+		if (game_type_ != GameType::kScenario) {
+			throw ParameterError(
+			   CmdLineVerbosity::None,
+			   ("Command line parameter --difficulty can only be used with --scenario=..."));
+		}
+		int64_t d;
+		if (!to_long(difficulty.value(), &d)) {
+			throw ParameterError(
+			   CmdLineVerbosity::None,
+			   format(_("Non-integer value for command line parameter --difficulty=%s"),
+			          difficulty.value()));
+		}
+		if (d <= 0) {
+			throw ParameterError(CmdLineVerbosity::None,
+			                     ("Value for command line parameter --difficulty must be positive."));
+		}
+		scenario_difficulty_ = d;
+	}
+
 	if (OptionalParameter val = get_commandline_option_value("script"); val.has_value()) {
 		script_to_run_ = *val;
 		if (script_to_run_.back() == '/') {
@@ -1713,7 +1972,7 @@ void WLApplication::handle_commandline_parameters() {
 		}
 	}
 
-	if (!found) {
+	if (!found_datadir) {
 		throw ParameterError(CmdLineVerbosity::None);  // datadir error already printed
 	}
 }
@@ -1739,6 +1998,9 @@ void WLApplication::emergency_save(UI::Panel* panel,
 
 	if (Widelands::UnhandledVersionError::is_unhandled_version_error(error)) {
 		// It's an incompatible savegame. Don't ask for a bug report, don't bother trying to save.
+		if (g_fail_on_errors) {
+			abort();
+		}
 		if (panel != nullptr) {
 			UI::WLMessageBox m(panel, UI::WindowStyle::kFsMenu, _("Incompatible"), error,
 			                   UI::WLMessageBox::MBoxType::kOk);
@@ -1754,6 +2016,9 @@ void WLApplication::emergency_save(UI::Panel* panel,
 		        "  Please add this information to your report.\n",
 		        build_ver_details().c_str());
 	}
+	if (g_fail_on_errors) {
+		abort();
+	}
 	log_err("  If desired, Widelands attempts to create an emergency savegame.\n"
 	        "  It is often – though not always – possible to load it and continue playing.\n"
 	        "##############################");
@@ -1969,7 +2234,7 @@ bool WLApplication::redirect_output(std::string path) {
 	setvbuf(stdout, nullptr, _IOLBF, BUFSIZ);
 
 	/* No buffering */
-	setbuf(stderr, nullptr);
+	setbuf(stderr, nullptr);  // NOLINT(bugprone-unsafe-functions)
 
 	redirected_stdio_ = true;
 	return true;
@@ -2004,7 +2269,6 @@ void WLApplication::check_crash_reports(FsMenu::MainMenu& menu) {
 		log_info("- %s", filename.c_str());
 	}
 
-	g_sh->change_music(Songset::kMenu);
 	menu.abort_splashscreen();
 	FsMenu::CrashReportWindow reporter(menu, crashes);
 	reporter.run<UI::Panel::Returncodes>();
