@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2022 by the Widelands Development Team
+ * Copyright (C) 2012-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,13 +20,18 @@
 #define WL_WLAPPLICATION_OPTIONS_H
 
 #include <map>
+#include <vector>
 
 #include <SDL_keyboard.h>
 
 #include "io/profile.h"
 
+/* Command line options */
+extern bool g_allow_script_console;
+extern bool g_write_syncstreams;
+
 /*
- * Further explanations for all functions and its return values
+ * Further explanations for all functions and their return values
  * can be found in io/profile.cc
  */
 
@@ -45,11 +50,10 @@ Section& get_config_section(const std::string&);
 Section* get_config_section_ptr(const std::string&);
 bool get_config_bool(const std::string& name, bool dflt);
 bool get_config_bool(const std::string& section, const std::string& name, bool dflt);
-int32_t get_config_int(const std::string& name, const int32_t dflt = 0);
+int32_t get_config_int(const std::string& name, int32_t dflt = 0);
 int32_t get_config_int(const std::string& section, const std::string& name, int32_t dflt);
 uint32_t get_config_natural(const std::string& name, uint32_t dflt);
-uint32_t
-get_config_natural(const std::string& section, const std::string& name, const uint32_t dflt);
+uint32_t get_config_natural(const std::string& section, const std::string& name, uint32_t dflt);
 std::string get_config_string(const std::string& name, const std::string& dflt);
 std::string
 get_config_string(const std::string& section, const std::string& name, const std::string& dflt);
@@ -102,6 +106,8 @@ enum class KeyboardShortcut : uint16_t {
 	kMainMenuEditorLoad,
 	kMainMenuContinueEditing,
 	kMainMenuReplay,
+	kMainMenuLoadReplay,
+	kMainMenuReplayLast,
 	kMainMenuOptions,
 	kMainMenuAddons,
 	kMainMenuAbout,
@@ -109,39 +115,84 @@ enum class KeyboardShortcut : uint16_t {
 	kMainMenu_End = kMainMenuQuit,
 
 	kCommon_Begin = kMainMenu_End + 1,
-	kCommonFullscreen = kCommon_Begin,
-	kCommonScreenshot,
+
+	// These will get different descriptions for the in-game and in-editor help
+	kCommonLoad = kCommon_Begin,
+	kCommonSave,
+	kCommonExit,
+	kCommonEncyclopedia,
+
+	// These can be inserted as one block in the in-game and in-editor help
+	kCommonGeneral_Begin,
+	kCommonZoomIn = kCommonGeneral_Begin,
+	kCommonZoomOut,
+	kCommonZoomReset,
+	kCommonContextMenu,
 	kCommonTextCut,
 	kCommonTextCopy,
 	kCommonTextPaste,
 	kCommonSelectAll,
 	kCommonDeleteItem,
 	kCommonTooltipAccessibilityMode,
-	kCommonEncyclopedia,
+	kCommonFullscreen,
+	kCommonScreenshot,
+	kCommonChangeMusic,
+	kCommonGeneral_End = kCommonChangeMusic,
+
+	// These are only shown in the help in debug builds
+	kCommonDebugConsole,
+	kCommonCheatMode,
+
+	// These will be moved to related items in the in-game and in-editor help
 	kCommonBuildhelp,
 	kCommonMinimap,
-	kCommonZoomIn,
-	kCommonZoomOut,
-	kCommonZoomReset,
 	kCommonQuicknavPrev,
 	kCommonQuicknavNext,
 	kCommon_End = kCommonQuicknavNext,
 
 	kEditor_Begin = kCommon_End + 1,
-	kEditorMenu = kEditor_Begin,
-	kEditorSave,
-	kEditorLoad,
-	kEditorUndo,
-	kEditorRedo,
-	kEditorTools,
-	kEditorInfo,
-	kEditorPlayers,
-	kEditorToolHistory,
+	kEditorNewMap = kEditor_Begin,
+	kEditorNewRandomMap,
+	kEditorLoad,  // alias of kCommonLoad
+	kEditorSave,  // alias of kCommonSave
+	kEditorUploadMap,
+	kEditorMapOptions,
+	kEditorExit,               // alias of kCommonExit
+	kEditorHelp,               // alias of kCommonEncyclopedia
+	kEditorShowhideBuildhelp,  // alias of kCommonBuildhelp
+	kEditorShowhideMaximumBuildhelp,
+	kEditorShowhideHeightHeatMap,
 	kEditorShowhideGrid,
+	kEditorShowhideOceans,
 	kEditorShowhideImmovables,
 	kEditorShowhideCritters,
 	kEditorShowhideResources,
-	kEditorShowhideMaximumBuildhelp,
+	kEditorMinimap,           // alias of kCommonMinimap
+	kEditor_Special_MapMove,  // special entries for map movement with arrow keys
+	kEditorQuicknavPrev,      // alias of kCommonQuicknavPrev
+	kEditorQuicknavNext,      // alias of kCommonQuicknavNext
+	kEditorMain_End = kEditorQuicknavNext,
+
+	// This will be a separate section in the help
+	kEditorTools_Begin = kEditorMain_End + 1,
+	kEditorTools = kEditorTools_Begin,
+	kEditorChangeHeight,
+	kEditorRandomHeight,
+	kEditorTerrain,
+	kEditorImmovables,
+	kEditorAnimals,
+	kEditorResources,
+	kEditorPortSpaces,
+	kEditorInfo,
+	kEditorMapOrigin,
+	kEditorMapSize,
+	kEditorPlayers,
+	kEditorToolHistory,
+	kEditorUndo,
+	kEditorRedo,
+	kEditorTools_End = kEditorRedo,
+
+	// These will be grouped to one line in the in-editor help
 	kEditorToolsize1,
 	kEditorToolsize2,
 	kEditorToolsize3,
@@ -152,52 +203,67 @@ enum class KeyboardShortcut : uint16_t {
 	kEditorToolsize8,
 	kEditorToolsize9,
 	kEditorToolsize10,
-	kEditor_End = kEditorToolsize10,
+
+	kEditorToolgap0,
+	kEditorToolgap10,
+	kEditorToolgap20,
+	kEditorToolgap30,
+	kEditorToolgap40,
+	kEditorToolgap50,
+	kEditorToolgap60,
+	kEditorToolgap70,
+	kEditorToolgap80,
+	kEditorToolgap90,
+
+	kEditor_End = kEditorToolgap90,
 
 	kInGame_Begin = kEditor_End + 1,
-	kInGameSave = kInGame_Begin,
-	kInGameLoad,
-	kInGameChat,
+	kInGameSoundOptions = kInGame_Begin,
+	kInGameSave,  // alias of kCommonSave
+	kInGameLoad,  // alias of kCommonLoad
+	kInGameRestart,
+	kInGameExit,          // alias of kCommonExit
+	kInGameEncyclopedia,  // alias of kCommonEncyclopedia
+
 	kInGameMessages,
 	kInGameObjectives,
 	kInGameDiplomacy,
-	kInGameShowhideCensus,
-	kInGameShowhideStats,
-	kInGameShowhideSoldiers,
-	kInGameShowhideBuildings,
-	kInGameShowhideWorkareas,
+	kInGameChat,
+
 	kInGameStatsGeneral,
 	kInGameStatsWares,
 	kInGameStatsBuildings,
 	kInGameStatsStock,
 	kInGameStatsSoldiers,
 	kInGameStatsSeafaring,
-	kInGamePause,
+
 	kInGameSpeedUp,
 	kInGameSpeedUpSlow,
 	kInGameSpeedUpFast,
 	kInGameSpeedDown,
 	kInGameSpeedDownSlow,
 	kInGameSpeedDownFast,
+	kInGamePause,
 	kInGameSpeedReset,
+
+	kInGameShowhideBuildhelp,  // alias of kCommonBuildhelp
+	kInGameShowhideCensus,
+	kInGameShowhideStats,
+	kInGameShowhideSoldiers,
+	kInGameShowhideBuildings,
+	kInGameShowhideWorkareas,
+
+	kInGameMinimap,           // alias of kCommonMinimap
+	kInGame_Special_MapMove,  // special entries for map movement with arrow keys
 	kInGameScrollToHQ,
-	kInGameMessagesGoto,
-	kInGameMessagesFilterAll,
-	kInGameMessagesFilterGeologists,
-	kInGameMessagesFilterEconomy,
-	kInGameMessagesFilterSeafaring,
-	kInGameMessagesFilterWarfare,
-	kInGameMessagesFilterScenario,
-	kInGameSeafaringstatsGotoShip,
-	kInGameSeafaringstatsWatchShip,
-	kInGameSeafaringstatsOpenShipWindow,
-	kInGameSeafaringstatsOpenShipWindowAndGoto,
-	kInGameSeafaringstatsFilterAll,
-	kInGameSeafaringstatsFilterIdle,
-	kInGameSeafaringstatsFilterShipping,
-	kInGameSeafaringstatsFilterExpWait,
-	kInGameSeafaringstatsFilterExpScout,
-	kInGameSeafaringstatsFilterExpPortspace,
+	kInGameQuicknavPrev,       // alias of kCommonQuicknavPrev
+	kInGameQuicknavNext,       // alias of kCommonQuicknavNext
+	kInGame_Special_Quicknav,  // special entries with grouped quicknav set/goto shortcuts
+	kInGameQuicknavGUI,
+	kInGamePinnedNote,
+	kInGameMain_End = kInGamePinnedNote,
+
+	// These will be grouped to two lines in the Encyclopedia
 	kInGameQuicknavSet1,
 	kInGameQuicknavGoto1,
 	kInGameQuicknavSet2,
@@ -216,7 +282,36 @@ enum class KeyboardShortcut : uint16_t {
 	kInGameQuicknavGoto8,
 	kInGameQuicknavSet9,
 	kInGameQuicknavGoto9,
-	kInGame_End = kInGameQuicknavGoto9,
+
+	// These have their own sections in the Encyclopedia
+
+	kInGameMessages_Begin,
+	kInGameMessagesFilterAll = kInGameMessages_Begin,
+	kInGameMessagesFilterGeologists,
+	kInGameMessagesFilterEconomy,
+	kInGameMessagesFilterSeafaring,
+	kInGameMessagesFilterWarfare,
+	kInGameMessagesFilterScenario,
+	kInGameMessagesFilterDiplomacy,
+	kInGameMessagesGoto,
+	kInGameMessages_End = kInGameMessagesGoto,
+
+	kInGameSeafaringstats_Begin = kInGameMessages_End + 1,
+	kInGameSeafaringstatsFilterAll = kInGameSeafaringstats_Begin,
+	kInGameSeafaringstatsFilterIdle,
+	kInGameSeafaringstatsFilterShipping,
+	kInGameSeafaringstatsFilterExpWait,
+	kInGameSeafaringstatsFilterExpScout,
+	kInGameSeafaringstatsFilterExpPortspace,
+	kInGameSeafaringstatsFilterWarship,
+	kInGameSeafaringstatsFilterRefitting,
+	kInGameSeafaringstatsGotoShip,
+	kInGameSeafaringstatsWatchShip,
+	kInGameSeafaringstatsOpenShipWindow,
+	kInGameSeafaringstatsOpenShipWindowAndGoto,
+	kInGameSeafaringstats_End = kInGameSeafaringstatsOpenShipWindowAndGoto,
+
+	kInGame_End = kInGameSeafaringstats_End,
 
 	kFastplace_Begin = kInGame_End + 1,
 	kFastplace_End = kFastplace_Begin + 127,  // Arbitrary limit of 128 fastplace shortcuts.
@@ -224,11 +319,44 @@ enum class KeyboardShortcut : uint16_t {
 	k_End = kFastplace_End
 };
 
+KeyboardShortcut operator+(const KeyboardShortcut& id, int i);
+KeyboardShortcut& operator++(KeyboardShortcut& id);
+uint16_t operator-(const KeyboardShortcut& a, const KeyboardShortcut& b);
+
+enum class KeyboardShortcutScope {
+	kGlobal,  // special value that intersects with all other scopes
+
+	kMainMenu,
+	kEditor,
+	kGame,
+};
+
+/**
+ * Check whether the given shortcut can be used for setting and retrieving the
+ * actual key combination.
+ *
+ * This function should be used when iterating over KeyboardShortcut ranges.
+ *
+ * Not "real" shortcuts are aliases and special entries, that are only used for
+ * help generation to simplify inserting common shortcuts and other related
+ * help entries at the right place.
+ */
+bool is_real(KeyboardShortcut id);
+
+bool is_developer_tool(KeyboardShortcut id);
+
 /** Check whether a given shortcut is reserved for a fastplace shortcut slot. */
 inline bool is_fastplace(const KeyboardShortcut id) {
 	return id >= KeyboardShortcut::kFastplace_Begin && id <= KeyboardShortcut::kFastplace_End;
 }
 
+void create_replace_shortcut(const std::string& name,
+                             const std::string& descname,
+                             const std::set<KeyboardShortcutScope>& scopes,
+                             SDL_Keysym default_shortcut);
+KeyboardShortcut get_highest_used_keyboard_shortcut();
+std::vector<std::string> get_all_keyboard_shortcut_names();
+
 /**
  * Change a keyboard shortcut.
  * @param id ID of the shortcut to change.
@@ -242,6 +370,20 @@ bool set_shortcut(KeyboardShortcut id, SDL_Keysym code, KeyboardShortcut* confli
 /** Look up the keysym assigned to a given shortcut ID. */
 SDL_Keysym get_shortcut(KeyboardShortcut);
 
+/** Get the formatted help of the current in game keyboard shortcuts including headers. */
+std::string get_ingame_shortcut_help();
+
+struct FastplaceShortcut {
+	std::string hotkey;
+	std::string building;
+};
+
+/** Get the current fastplace shortcuts for tribe. */
+std::vector<FastplaceShortcut> get_active_fastplace_shortcuts(const std::string& tribe);
+
+/** Get the formatted help of the current editor keyboard shortcuts including headers. */
+std::string get_editor_shortcut_help();
+
 /** Look up the hardcoded default keysym for a given shortcut ID. */
 SDL_Keysym get_default_shortcut(KeyboardShortcut);
 
@@ -273,12 +415,13 @@ std::string to_string(KeyboardShortcut);
 
 /** Get the shortcut ID from an internal shortcut name. Throws an exception for invalid names. */
 KeyboardShortcut shortcut_from_string(const std::string&);
+bool shortcut_exists(const std::string&);
 
 /**
  * Generate a human-readable description of a keyboard shortcut.
  * Return value will either be an empty string or have a trailing "+".
  */
-std::string keymod_string_for(const uint16_t modstate, const bool rt_escape = true);
+std::string keymod_string_for(uint16_t modstate, bool rt_escape = true);
 std::string shortcut_string_for(SDL_Keysym, bool rt_escape);
 std::string shortcut_string_for(KeyboardShortcut, bool rt_escape);
 
@@ -295,21 +438,32 @@ void init_fastplace_default_shortcuts(
 /** Clear a shortcut. */
 void unset_shortcut(KeyboardShortcut);
 
+inline SDL_Keysym keysym(const SDL_Keycode c, uint16_t mod = 0) {
+	return SDL_Keysym{SDL_GetScancodeFromKey(c), c, mod, 0};
+}
+
+// Default step sizes for changing value of spinbox, slider, etc. on PgUp/PgDown or Ctrl+mousewheel
+namespace ChangeBigStep {
+constexpr int32_t kSmallRange = 3;
+constexpr int32_t kMediumRange = 5;
+constexpr int32_t kWideRange = 10;
+}  // namespace ChangeBigStep
+
 // Return values for changing value of spinbox, slider, etc.
 enum class ChangeType : int32_t {
 	kSetMin = std::numeric_limits<int32_t>::min(),  // set value to minimum
 	                                                //     -- keys: Home, Ctrl + decrease keys
-	kBigMinus = -10,                                // decrease by big step -- key: PageDown
+	kBigMinus = -ChangeBigStep::kWideRange,         // decrease by big step -- key: PageDown
 	kMinus = -1,                                    // decrease  -- keys: Left, Down, Minus
 	kNone = 0,                                      // no change -- all other keys
 	kPlus = 1,                                      // increase  -- keys: Right, Up, Plus
-	kBigPlus = 10,                                  // increase by big step -- key: PageUp
+	kBigPlus = ChangeBigStep::kWideRange,           // increase by big step -- key: PageUp
 	kSetMax = std::numeric_limits<int32_t>::max()   // set value to maximum
 	                                                //     -- keys: End, Ctrl + increase keys
 };
 
 // Helper function for spinbox, slider, etc. handle_key(...)
-ChangeType get_keyboard_change(SDL_Keysym, bool enable_big_step = false);
+ChangeType get_keyboard_change(SDL_Keysym);
 
 /*
  * Sets the directory where to read/write kConfigFile.
