@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2024 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,22 +22,24 @@
 #include "economy/economy.h"
 #include "graphic/font_handler.h"
 #include "logic/map_objects/tribes/building.h"
+#include "logic/map_objects/tribes/market.h"
 #include "logic/map_objects/tribes/ship.h"
 #include "logic/player.h"
 #include "logic/playersmanager.h"
 #include "ui_basic/box.h"
 #include "ui_basic/checkbox.h"
+#include "ui_basic/infinite_spinner.h"
 #include "ui_basic/multilinetextarea.h"
 #include "ui_basic/window.h"
 #include "wui/interactive_player.h"
 
+constexpr int kPadding = 8;
+
 struct ActionConfirm : public UI::Window {
 	ActionConfirm(InteractivePlayer& parent,
 	              const std::string& windowtitle,
 	              const std::string& message,
-	              Widelands::MapObject* map_object,
-	              // May be "", in which case no checkbox will be added
-	              const std::string& checkbox = "");
+	              Widelands::MapObject* map_object);
 
 	InteractivePlayer& iaplayer() const {
 		return dynamic_cast<InteractivePlayer&>(*get_parent());
@@ -47,7 +49,7 @@ struct ActionConfirm : public UI::Window {
 
 protected:
 	Widelands::ObjectPointer object_;
-	UI::Checkbox* checkbox_{nullptr};
+	UI::Box* custom_content_box_{nullptr};
 };
 
 /**
@@ -84,6 +86,9 @@ struct DismantleConfirm : public ActionConfirm {
 
 	void think() override;
 	void ok() override;
+
+private:
+	UI::Checkbox* checkbox_{nullptr};
 };
 
 /**
@@ -102,6 +107,8 @@ private:
 	// Do not make this a reference - it is a stack variable in the caller
 	const Widelands::DescriptionIndex id_;
 	bool still_under_construction_;
+
+	UI::Checkbox* checkbox_{nullptr};
 };
 
 /**
@@ -134,20 +141,84 @@ struct ResignConfirm : public ActionConfirm {
 	void ok() override;
 };
 
+/**
+ * Confirmation dialog box for cancelling a trade.
+ */
+struct CancelTradeConfirm : public ActionConfirm {
+	explicit CancelTradeConfirm(InteractivePlayer& parent, Widelands::TradeID trade_id);
+
+	void think() override;
+	void ok() override;
+
+private:
+	Widelands::TradeID trade_id_;
+};
+
+/**
+ * Dialog box for extending a trade by a configurable number of batches.
+ */
+struct TradeExtensionDialog : public ActionConfirm {
+	explicit TradeExtensionDialog(InteractivePlayer& parent, Widelands::TradeID trade_id);
+
+	void think() override;
+	void ok() override;
+
+private:
+	Widelands::TradeID trade_id_;
+
+	UI::InfiniteSpinner batches_;
+};
+
+/**
+ * Confirmation dialog box for unpausing a trade.
+ */
+struct UnpauseTradeConfirm : public ActionConfirm {
+	explicit UnpauseTradeConfirm(InteractivePlayer& parent,
+	                             Widelands::Market& market,
+	                             Widelands::TradeID trade_id);
+
+	void think() override;
+	void ok() override;
+
+private:
+	Widelands::OPtr<Widelands::Market> market_;
+	Widelands::TradeID trade_id_;
+};
+
+/**
+ * Confirmation dialog box with an arbitrary title, body, and callback function.
+ */
+struct GenericCallbackConfirm : public ActionConfirm {
+	explicit GenericCallbackConfirm(InteractivePlayer& parent,
+	                                Widelands::MapObject* object,
+	                                const std::string& title,
+	                                const std::string& body,
+	                                std::function<void()> callback);
+
+	void think() override;
+	void ok() override;
+
+private:
+	std::function<void()> callback_;
+};
+
 ActionConfirm::ActionConfirm(InteractivePlayer& parent,
                              const std::string& windowtitle,
                              const std::string& message,
-                             Widelands::MapObject* map_object,
-                             const std::string& checkbox)
+                             Widelands::MapObject* map_object)
    : UI::Window(
         &parent, UI::WindowStyle::kWui, "building_action_confirm", 0, 0, 200, 120, windowtitle),
      object_(map_object) {
-	const int padding = 6;
+	UI::Box* wrapper_box =
+	   new UI::Box(this, UI::PanelStyle::kWui, "wrapper_box", 0, 0, UI::Box::Horizontal);
 	UI::Box* main_box =
-	   new UI::Box(this, UI::PanelStyle::kWui, "main_box", padding, padding, UI::Box::Vertical);
+	   new UI::Box(wrapper_box, UI::PanelStyle::kWui, "main_box", 0, 0, UI::Box::Vertical);
 	UI::Box* button_box =
 	   new UI::Box(main_box, UI::PanelStyle::kWui, "buttons_box", 0, 0, UI::Box::Horizontal);
 
+	custom_content_box_ =
+	   new UI::Box(main_box, UI::PanelStyle::kWui, "box", 0, 0, UI::Box::Horizontal);
+
 	UI::MultilineTextarea* textarea = new UI::MultilineTextarea(
 	   main_box, "message", 0, 0, 200, 74, UI::PanelStyle::kWui, message, UI::Align::kCenter,
 	   UI::MultilineTextarea::ScrollMode::kNoScrolling);
@@ -161,27 +232,26 @@ ActionConfirm::ActionConfirm(InteractivePlayer& parent,
 	                  g_image_cache->get("images/wui/menu_abort.png"));
 	cancelbtn->sigclicked.connect([this]() { die(); });
 
-	button_box->add(
-	   UI::g_fh->fontset()->is_rtl() ? okbtn : cancelbtn, UI::Box::Resizing::kFillSpace);
-	button_box->add_space(2 * padding);
-	button_box->add(
-	   UI::g_fh->fontset()->is_rtl() ? cancelbtn : okbtn, UI::Box::Resizing::kFillSpace);
-	main_box->add(textarea);
-	if (!checkbox.empty()) {
-		checkbox_ =
-		   new UI::Checkbox(main_box, UI::PanelStyle::kWui, "checkbox", Vector2i(0, 0), checkbox);
-		// tooltip and initial state will be set by the subclass constructor
-		main_box->add_space(padding);
-		main_box->add(checkbox_, UI::Box::Resizing::kFullSize);
-	}
-	main_box->add_space(1.5 * padding);
-	main_box->add(button_box, UI::Box::Resizing::kFullSize);
-	button_box->set_size(textarea->get_w(), okbtn->get_h());
-	main_box->set_size(
-	   textarea->get_w(), textarea->get_h() + button_box->get_h() + 1.5 * padding +
-	                         (checkbox_ != nullptr ? checkbox_->get_h() + padding : 0));
-	set_inner_size(main_box->get_w() + 2 * padding, main_box->get_h() + 2 * padding);
+	button_box->add_space(kPadding);
+	button_box->add(UI::g_fh->fontset()->is_rtl() ? okbtn : cancelbtn, UI::Box::Resizing::kAlign,
+	                UI::Align::kCenter);
+	button_box->add_space(2 * kPadding);
+	button_box->add(UI::g_fh->fontset()->is_rtl() ? cancelbtn : okbtn, UI::Box::Resizing::kAlign,
+	                UI::Align::kCenter);
+	button_box->add_space(kPadding);
 
+	main_box->add(textarea, UI::Box::Resizing::kExpandBoth);
+	main_box->add_space(kPadding);
+	main_box->add(custom_content_box_, UI::Box::Resizing::kExpandBoth);
+	main_box->add_space(kPadding);
+	main_box->add(button_box, UI::Box::Resizing::kAlign, UI::Align::kCenter);
+	main_box->add_space(kPadding);
+
+	wrapper_box->add_space(kPadding);
+	wrapper_box->add(main_box, UI::Box::Resizing::kExpandBoth);
+	wrapper_box->add_space(kPadding);
+
+	set_center_panel(wrapper_box);
 	set_z(UI::Panel::ZOrder::kConfirmation);
 	center_to_parent();
 	cancelbtn->center_mouse();
@@ -261,13 +331,18 @@ DismantleConfirm::DismantleConfirm(InteractivePlayer& parent, Widelands::Buildin
    : ActionConfirm(parent,
                    _("Dismantle building?"),
                    _("Do you really want to dismantle this building?"),
-                   &building,
-                   should_allow_preserving_wares(building.descr()) ? _("Preserve wares") : "") {
-	if (checkbox_ != nullptr) {
+                   &building) {
+	if (should_allow_preserving_wares(building.descr())) {
+		checkbox_ = new UI::Checkbox(custom_content_box_, UI::PanelStyle::kWui, "checkbox",
+		                             Vector2i(0, 0), _("Preserve wares"));
+		custom_content_box_->add(checkbox_, UI::Box::Resizing::kFullSize);
+
 		checkbox_->set_tooltip(_("Any wares left in the building will be dropped out by the builder, "
 		                         "increasing the dismantling time"));
 		checkbox_->set_state(true);
 	}
+
+	initialization_complete();
 }
 
 /*
@@ -321,15 +396,21 @@ EnhanceConfirm::EnhanceConfirm(InteractivePlayer& parent,
               /** TRANSLATORS: Warning message when player wants to enhance a military building */
               _("Be careful if the enemy is near!")) :
            _("Do you really want to enhance this building?"),
-        &building,
-        should_allow_preserving_wares(building.descr()) ? _("Preserve wares") : ""),
+        &building),
      id_(id),
      still_under_construction_(still_under_construction) {
-	if (checkbox_ != nullptr) {
+
+	if (should_allow_preserving_wares(building.descr())) {
+		checkbox_ = new UI::Checkbox(custom_content_box_, UI::PanelStyle::kWui, "checkbox",
+		                             Vector2i(0, 0), _("Preserve wares"));
+		custom_content_box_->add(checkbox_, UI::Box::Resizing::kFullSize);
+
 		checkbox_->set_tooltip(_("Any wares left in the building will be dropped out by the builder, "
 		                         "increasing the enhancing time"));
 		checkbox_->set_state(true);
 	}
+
+	initialization_complete();
 }
 
 /*
@@ -342,8 +423,8 @@ void EnhanceConfirm::think() {
 	upcast(Widelands::Building, building, object_.get(egbase));
 
 	if ((building == nullptr) || !iaplayer().can_act(building->owner().player_number()) ||
-	    !(still_under_construction_ ||
-	      ((building->get_playercaps() & Widelands::Building::PCap_Enhancable) != 0u))) {
+	    (!still_under_construction_ &&
+	     ((building->get_playercaps() & Widelands::Building::PCap_Enhancable) == 0u))) {
 		die();
 	}
 }
@@ -479,6 +560,155 @@ void ResignConfirm::ok() {
 	die();
 }
 
+/**
+ * Create the panels for confirmation.
+ */
+CancelTradeConfirm::CancelTradeConfirm(InteractivePlayer& parent, Widelands::TradeID trade_id)
+   : ActionConfirm(
+        parent, _("Cancel?"), _("Do you really want to cancel this trade agreement?"), nullptr),
+     trade_id_(trade_id) {
+	// Nothing special to do
+}
+
+/**
+ * Make sure the trade still exists.
+ */
+void CancelTradeConfirm::think() {
+	if (!iaplayer().game().has_trade(trade_id_)) {
+		die();
+	}
+}
+
+/**
+ * The "Ok" button was clicked, so issue the command for cancelling the trade.
+ */
+void CancelTradeConfirm::ok() {
+	iaplayer().game().send_player_trade_action(
+	   iaplayer().player_number(), trade_id_, Widelands::TradeAction::kCancel, 0, 0);
+	die();
+}
+
+/**
+ * Create the panels for configuration.
+ */
+TradeExtensionDialog::TradeExtensionDialog(InteractivePlayer& parent, Widelands::TradeID trade_id)
+   : ActionConfirm(parent,
+                   _("Extend Trade"),
+                   _("Select by how many batches you want to extend this trade:"),
+                   nullptr),
+     trade_id_(trade_id),
+     batches_(custom_content_box_,
+              "batches",
+              UI::PanelStyle::kWui,
+              _("Toggle indefinite trade"),
+              _("Batches:"),
+              1,
+              1,
+              Widelands::kMaxBatches) {
+	custom_content_box_->add(&batches_, UI::Box::Resizing::kFullSize);
+
+	initialization_complete();
+}
+
+/**
+ * Make sure the trade still exists.
+ */
+void TradeExtensionDialog::think() {
+	if (!iaplayer().game().has_trade(trade_id_)) {
+		die();
+	}
+}
+
+/**
+ * The "Ok" button was clicked, so issue the command for proposing to extend the trade.
+ */
+void TradeExtensionDialog::ok() {
+	iaplayer().game().send_player_extend_trade(
+	   iaplayer().player_number(), trade_id_, Widelands::TradeAction::kExtend,
+	   batches_.is_infinite() ? Widelands::kInfiniteTrade : batches_.get_value());
+	die();
+}
+
+/**
+ * Create the panels for confirmation.
+ */
+UnpauseTradeConfirm::UnpauseTradeConfirm(InteractivePlayer& parent,
+                                         Widelands::Market& market,
+                                         Widelands::TradeID trade_id)
+   : ActionConfirm(parent,
+                   _("Unpause?"),
+                   _("Do you really want to unpause this trade?\n\nDoing so will reset all queues "
+                     "to their maximum capacity."),
+                   &market),
+     market_(&market),
+     trade_id_(trade_id) {
+	// Nothing special to do
+}
+
+/**
+ * Make sure the trade still exists.
+ */
+void UnpauseTradeConfirm::think() {
+	if (!iaplayer().game().has_trade(trade_id_)) {
+		die();
+	}
+}
+
+/**
+ * The "Ok" button was clicked, so issue the command for resuming the trade.
+ */
+void UnpauseTradeConfirm::ok() {
+	Widelands::Game& game = iaplayer().game();
+	MutexLock m(MutexLock::ID::kObjects);
+
+	if (game.has_trade(trade_id_)) {
+		Widelands::Market* market = market_.get(game);
+		assert(market != nullptr);
+		const Widelands::Market::TradeOrder& order = *market->trade_orders().at(trade_id_);
+		for (const auto& pair : order.wares_queues_) {
+			game.send_player_set_input_max_fill(*market, pair.second->get_index(),
+			                                    pair.second->get_type(), pair.second->get_max_size(),
+			                                    false, trade_id_);
+		}
+		game.send_player_set_input_max_fill(*market, order.carriers_queue_->get_index(),
+		                                    order.carriers_queue_->get_type(),
+		                                    order.carriers_queue_->get_max_size(), false, trade_id_);
+		game.send_player_trade_action(iaplayer().player_number(), trade_id_,
+		                              Widelands::TradeAction::kResume, market->serial(), 0);
+	}
+
+	die();
+}
+
+/**
+ * Create the panels for confirmation.
+ */
+GenericCallbackConfirm::GenericCallbackConfirm(InteractivePlayer& parent,
+                                               Widelands::MapObject* object,
+                                               const std::string& title,
+                                               const std::string& body,
+                                               std::function<void()> callback)
+   : ActionConfirm(parent, title, body, object), callback_(callback) {
+	// Nothing special to do
+}
+
+/**
+ * Make sure the building still exists, if we have one.
+ */
+void GenericCallbackConfirm::think() {
+	if (object_.is_set() && object_.get(iaplayer().egbase()) == nullptr) {
+		die();
+	}
+}
+
+/**
+ * The "Ok" button was clicked, so invoke the callback function.
+ */
+void GenericCallbackConfirm::ok() {
+	callback_();
+	die();
+}
+
 /**
  * Create a BulldozeConfirm window.
  * No further action is required by the caller: any action necessary to actually
@@ -547,3 +777,25 @@ void show_ship_cancel_expedition_confirm(InteractivePlayer& player, Widelands::S
 void show_resign_confirm(InteractivePlayer& player) {
 	new ResignConfirm(player);
 }
+
+void show_cancel_trade_confirm(InteractivePlayer& player, Widelands::TradeID trade_id) {
+	new CancelTradeConfirm(player, trade_id);
+}
+
+void show_trade_extension_dialog(InteractivePlayer& player, Widelands::TradeID trade_id) {
+	new TradeExtensionDialog(player, trade_id);
+}
+
+void show_resume_trade_confirm(InteractivePlayer& player,
+                               Widelands::Market& market,
+                               Widelands::TradeID trade_id) {
+	new UnpauseTradeConfirm(player, market, trade_id);
+}
+
+void show_generic_callback_confirm(InteractivePlayer& player,
+                                   Widelands::MapObject* object,
+                                   const std::string& title,
+                                   const std::string& body,
+                                   std::function<void()> callback) {
+	new GenericCallbackConfirm(player, object, title, body, callback);
+}
