@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2024 by the Widelands Development Team
+ * Copyright (C) 2010-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,16 +19,26 @@
 #ifndef WL_WUI_ACTIONCONFIRM_H
 #define WL_WUI_ACTIONCONFIRM_H
 
+#include <functional>
+
 #include "logic/widelands.h"
 
 class InteractivePlayer;
 
 namespace Widelands {
 class Building;
+class MapObject;
+class Market;
 struct PlayerImmovable;
 struct Ship;
 }  // namespace Widelands
 
+void show_generic_callback_confirm(InteractivePlayer& player,
+                                   Widelands::MapObject* object,
+                                   const std::string& title,
+                                   const std::string& body,
+                                   std::function<void()> callback);
+
 // Building confirm windows
 void show_bulldoze_confirm(InteractivePlayer& player,
                            Widelands::Building& building,
@@ -49,4 +59,13 @@ void show_ship_cancel_expedition_confirm(InteractivePlayer& player, Widelands::S
 // Diplomacy confirm windows
 void show_resign_confirm(InteractivePlayer& player);
 
+// Trade confirm windows
+void show_cancel_trade_confirm(InteractivePlayer& player, Widelands::TradeID trade_id);
+
+void show_resume_trade_confirm(InteractivePlayer& player,
+                               Widelands::Market& market,
+                               Widelands::TradeID trade_id);
+
+void show_trade_extension_dialog(InteractivePlayer& player, Widelands::TradeID trade_id);
+
 #endif  // end of include guard: WL_WUI_ACTIONCONFIRM_H
