@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2024 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -188,7 +188,7 @@ std::vector<Widelands::OPtr<Widelands::Bob>> AttackWindow::get_max_attackers() {
 		}
 
 		// Player can't see any part of the building, so it can't be attacked
-		// This is the same check as done later on in send_player_enemyflagaction()
+		// This is the same check as done later on in send_player_attack()
 		if (!sees) {
 			return result_vector;
 		}
@@ -520,8 +520,8 @@ void AttackWindow::act_attack() {
 
 		iplayer_.map_view()->mouse_to_field(target_coordinates_, MapView::Transition::Jump);
 	} else if (Widelands::Building* building = get_building(); building != nullptr) {
-		iplayer_.game().send_player_enemyflagaction(building->base_flag(), iplayer_.player_number(),
-		                                            attack_panel_.soldiers(), get_allow_conquer());
+		iplayer_.game().send_player_attack(building->base_flag(), iplayer_.player_number(),
+		                                   attack_panel_.soldiers(), get_allow_conquer());
 		iplayer_.map_view()->mouse_to_field(building->get_position(), MapView::Transition::Jump);
 	} else if (Widelands::Ship* ship = get_ship(); ship != nullptr) {
 		for (Widelands::Serial serial : attack_panel_.soldiers()) {
@@ -760,8 +760,7 @@ void AttackPanel::ListOfSoldiers::draw(RenderTarget& dst) {
 					dst.blitrect_scale(Rectf((column + kOffset) * attack_box_->icon_w_,
 					                         (row + kOffset) * attack_box_->icon_h_,
 					                         attack_box_->icon_w_ * kSize, attack_box_->icon_h_ * kSize),
-					                   anchor, Recti(0, 0, anchor->width(), anchor->height()), kAlpha,
-					                   BlendMode::Default);
+					                   anchor, anchor->rect(), kAlpha, BlendMode::Default);
 				}
 			}
 		}
