@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2024 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -304,7 +304,7 @@ void BuildingStatisticsMenu::init(int last_selected_tab) {
 	std::vector<Widelands::DescriptionIndex> buildings_to_add[kNoOfBuildingTabs];
 	// Add the player's own tribe's buildings.
 	for (Widelands::DescriptionIndex index : tribe.buildings()) {
-		if (own_building_is_valid(player, index, map_allows_seafaring, map_allows_waterways)) {
+		if (own_tribe_building_is_valid(player, index, map_allows_seafaring, map_allows_waterways)) {
 			buildings_to_add[find_tab_for_building(*tribe.get_building_descr(index))].push_back(index);
 		}
 	}
@@ -378,10 +378,13 @@ void BuildingStatisticsMenu::init(int last_selected_tab) {
 	initialization_complete();
 }
 
-bool BuildingStatisticsMenu::own_building_is_valid(const Widelands::Player& player,
-                                                   Widelands::DescriptionIndex index,
-                                                   bool map_allows_seafaring,
-                                                   bool map_allows_waterways) const {
+bool BuildingStatisticsMenu::own_tribe_building_is_valid(const Widelands::Player& player,
+                                                         Widelands::DescriptionIndex index,
+                                                         bool map_allows_seafaring,
+                                                         bool map_allows_waterways) const {
+	if (!player.tribe().has_building(index)) {
+		return false;
+	}
 	const Widelands::BuildingDescr& descr = *player.tribe().get_building_descr(index);
 
 	if (!descr.is_useful_on_map(map_allows_seafaring, map_allows_waterways) &&
@@ -392,10 +395,8 @@ bool BuildingStatisticsMenu::own_building_is_valid(const Widelands::Player& play
 	    descr.type() == Widelands::MapObjectType::DISMANTLESITE) {
 		return false;
 	}
-	// Only add allowed buildings or buildings that are owned by the player.
-	return (
-	   (player.is_building_type_allowed(index) && (descr.is_buildable() || descr.is_enhanced())) ||
-	   !player.get_building_statistics(index).empty());
+	// Only add allowed buildings of the player's tribe
+	return (player.is_building_type_allowed(index) && (descr.is_buildable() || descr.is_enhanced()));
 }
 
 bool BuildingStatisticsMenu::foreign_tribe_building_is_valid(
@@ -439,7 +440,7 @@ void BuildingStatisticsMenu::update_building_list() {
 	const bool map_allows_waterways = iplayer().game().map().get_waterway_max_length() >= 2;
 	for (Widelands::DescriptionIndex index = 0; index < nr_building_types_; ++index) {
 		const bool should_have_this_building =
-		   own_building_is_valid(player, index, map_allows_seafaring, map_allows_waterways) ||
+		   own_tribe_building_is_valid(player, index, map_allows_seafaring, map_allows_waterways) ||
 		   foreign_tribe_building_is_valid(player, index);
 		const bool has_this_building = building_buttons_[index] != nullptr;
 		if (should_have_this_building != has_this_building) {
@@ -602,6 +603,9 @@ void BuildingStatisticsMenu::jump_building(JumpTarget target, bool reverse) {
 		}
 		break;
 	}
+
+	default:
+		NEVER_HERE();
 	}
 
 	if (found) {
@@ -619,7 +623,7 @@ void BuildingStatisticsMenu::think() {
 	// Update statistics
 	const Time& gametime = iplayer().game().get_gametime();
 
-	if (was_minimized_ || (gametime - lastupdate_) > kUpdateTimeInGametimeMs) {
+	if (!is_minimal() && (was_minimized_ || (gametime - lastupdate_) > kUpdateTimeInGametimeMs)) {
 		update_building_list();
 		update();
 		lastupdate_ = gametime;
