@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,15 +30,11 @@
 #include "ui_basic/unique_window.h"
 #include "wui/interactive_player.h"
 
-namespace {
-
-constexpr int kNoOfBuildingTabs = 5;
-
-}  // namespace
-
 /// This window shows statistics for all the buildings that the player owns.
 /// It also allows to jump through buildings on the map.
 struct BuildingStatisticsMenu : public UI::UniqueWindow {
+	static constexpr int kNoOfBuildingTabs = 5;
+
 	BuildingStatisticsMenu(InteractivePlayer&, UI::UniqueWindow::Registry&);
 	~BuildingStatisticsMenu() override;
 
@@ -65,10 +61,10 @@ private:
 	void init(int last_selected_tab = 0);
 
 	/// Whether a building that is used by the player's tribe should be added
-	bool own_building_is_valid(const Widelands::Player& player,
-	                           Widelands::DescriptionIndex index,
-	                           bool map_allows_seafaring,
-	                           bool map_allows_waterways) const;
+	bool own_tribe_building_is_valid(const Widelands::Player& player,
+	                                 Widelands::DescriptionIndex index,
+	                                 bool map_allows_seafaring,
+	                                 bool map_allows_waterways) const;
 	/// Whether a building that isn't used by the player's tribe should be added
 	bool foreign_tribe_building_is_valid(const Widelands::Player& player,
 	                                     Widelands::DescriptionIndex index) const;
@@ -79,7 +75,6 @@ private:
 	void update_building_list();
 
 	/// Adds a button for the building type belonging to the id and descr to the tab.
-	/// Returns true when a new row needs to be created.
 	void
 	add_button(Widelands::DescriptionIndex id, const Widelands::BuildingDescr& descr, UI::Box* row);
 
@@ -124,7 +119,7 @@ private:
 	std::vector<UI::Textarea*> productivity_labels_;
 
 	/// At which percent to deem buildings as unproductive
-	int low_production_;
+	int low_production_{33};
 
 	UI::Box hbox_owned_, hbox_construction_, hbox_unproductive_;
 	UI::Textarea label_name_, label_owned_, label_construction_, label_unproductive_,
@@ -134,18 +129,18 @@ private:
 	UI::SpinBox unproductive_threshold_;
 
 	/// The building type we are currently navigating
-	Widelands::DescriptionIndex current_building_type_;
+	Widelands::DescriptionIndex current_building_type_{Widelands::INVALID_INDEX};
 	/// The last building that was jumped to
-	int32_t last_building_index_;
+	int32_t last_building_index_{0};
 	/// The type of last building that was jumped to
-	Widelands::DescriptionIndex last_building_type_;
+	Widelands::DescriptionIndex last_building_type_{Widelands::INVALID_INDEX};
 	/// The last time the information in this Panel got updated
-	Time lastupdate_;
+	Time lastupdate_{0U};
 	/// Whether the window was minimized the last time that think() was executed
-	bool was_minimized_;
+	bool was_minimized_{false};
 
 	/// Whether a building has been selected
-	bool has_selection_;
+	bool has_selection_{false};
 
 	/// The total number of building types available for all the tribes
 	const Widelands::DescriptionIndex nr_building_types_;
