@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2024 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -45,6 +45,7 @@ static const char pic_stock_policy_button_remove[] =
 static const char pic_decrease_capacity[] = "images/wui/buildings/menu_down_train.png";
 static const char pic_increase_capacity[] = "images/wui/buildings/menu_up_train.png";
 constexpr uint16_t kSoldierCapacityDisplayWidth = 145;
+constexpr int8_t kButtonSize = 34;
 
 ConstructionSiteWindow::FakeWaresDisplay::FakeWaresDisplay(UI::Panel* parent,
                                                            bool can_act,
@@ -104,6 +105,8 @@ void ConstructionSiteWindow::FakeWaresDisplay::draw_ware(RenderTarget& dst,
 	case Widelands::StockPolicy::kNormal:
 		// No icon for the normal policy
 		return;
+	default:
+		NEVER_HERE();
 	}
 	assert(pic);
 	dst.blit(ware_position(ware), pic);
@@ -151,6 +154,19 @@ void ConstructionSiteWindow::build_wares_tab(Widelands::ConstructionSite* constr
 	ensure_box_can_hold_input_queues(box);
 	add_wares_queues(construction_site, box);
 
+	if (ibase()->can_act(construction_site->owner().player_number())) {
+		UI::Box& builder_caps =
+		   *new UI::Box(&box, UI::PanelStyle::kWui, "builder_caps_box", 0, 0, UI::Box::Horizontal);
+		builder_caps.add_inf_space();
+		UI::Button& evict_button = *new UI::Button(
+		   &builder_caps, "evict", 0, 0, kButtonSize, kButtonSize, UI::ButtonStyle::kWuiMenu,
+		   g_image_cache->get("images/wui/buildings/menu_drop_soldier.png"),
+		   _("Send the builder away"));
+		evict_button.sigclicked.connect([this]() { evict_builder(); });
+		builder_caps.add(&evict_button);
+		box.add(&builder_caps, UI::Box::Resizing::kFullSize);
+	}
+
 	get_tabs()->add("wares", g_image_cache->get(pic_tab_wares), &box, _("Building materials"));
 }
 
@@ -343,7 +359,6 @@ void ConstructionSiteWindow::build_settings_tab(Widelands::ConstructionSite* con
 		                UI::Box::Horizontal);
 		settings_box->add(&soldier_preference_box, UI::Box::Resizing::kAlign, UI::Align::kCenter);
 		// TODO(Nordfriese): Use box layout
-		constexpr int kButtonSize = 32;
 		Panel& soldier_preference_panel =
 		   *new Panel(&soldier_preference_box, UI::PanelStyle::kWui, "soldier_preference_panel", 0, 0,
 		              kButtonSize * 3, kButtonSize);
@@ -386,6 +401,10 @@ void ConstructionSiteWindow::build_settings_tab(Widelands::ConstructionSite* con
 		}
 		settings_box->add_space(8);
 	} break;
+	case Widelands::MapObjectType::MARKET: {
+		// Nothing to do currently.
+		nothing_added = true;
+	} break;
 	default:
 		NEVER_HERE();
 	}
@@ -426,6 +445,24 @@ void ConstructionSiteWindow::change_policy(Widelands::WareWorker ww, Widelands::
 	}
 }
 
+void ConstructionSiteWindow::evict_builder() {
+	Widelands::ConstructionSite* construction_site = construction_site_.get(ibase()->egbase());
+
+	if (construction_site == nullptr) {
+		return;
+	}
+	std::vector<Widelands::Worker*> workers = construction_site->get_workers();
+	if (workers.empty()) {
+		return;
+	}
+	if (game_ != nullptr) {
+		Widelands::Worker* builder = workers.front();
+		game_->send_player_evict_worker(*builder);
+	} else {
+		NEVER_HERE();
+	}
+}
+
 /*
 ===============
 Make sure the window is redrawn when necessary.
@@ -591,6 +628,8 @@ bool ConstructionSoldierCapacityBox::handle_key(bool down, SDL_Keysym code) {
 		case ChangeType::kSetMin:
 			set_current(min_);
 			return true;
+		default:
+			NEVER_HERE();
 		}
 	}
 	return UI::Box::handle_key(down, code);
