@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2024 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,11 +20,12 @@
 
 #include "base/time_string.h"
 #include "base/wexception.h"
+#include "commands/cmd_message_set_status_archived.h"
+#include "commands/cmd_message_set_status_read.h"
 #include "graphic/text_layout.h"
 #include "logic/game_data_error.h"
 #include "logic/message_queue.h"
 #include "logic/player.h"
-#include "logic/playercommand.h"
 #include "wlapplication_options.h"
 #include "wui/interactive_player.h"
 
@@ -76,27 +77,47 @@ GameMessageMenu::GameMessageMenu(InteractivePlayer& plr, UI::UniqueWindow::Regis
 	list->focus();
 
 	// Buttons for message types
-	geologistsbtn_ = new UI::Button(this, "filter_geologists_messages", kPadding, kPadding,
-	                                kButtonSize, kButtonSize, UI::ButtonStyle::kWuiSecondary,
+	filter_box_ = new UI::Box(this, UI::PanelStyle::kWui, "filter_box", kPadding, kPadding,
+	                          UI::Box::Horizontal, 0, 0, kPadding);
+
+	geologistsbtn_ = new UI::Button(filter_box_, "filter_geologists_messages", 0, 0, kButtonSize,
+	                                kButtonSize, UI::ButtonStyle::kWuiSecondary,
 	                                g_image_cache->get("images/wui/fieldaction/menu_geologist.png"));
 
-	economybtn_ = new UI::Button(this, "filter_economy_messages", 2 * kPadding + kButtonSize,
-	                             kPadding, kButtonSize, kButtonSize, UI::ButtonStyle::kWuiSecondary,
+	economybtn_ = new UI::Button(filter_box_, "filter_economy_messages", 0, 0, kButtonSize,
+	                             kButtonSize, UI::ButtonStyle::kWuiSecondary,
 	                             g_image_cache->get("images/wui/stats/genstats_nrwares.png"));
 
-	seafaringbtn_ =
-	   new UI::Button(this, "filter_seafaring_messages", 3 * kPadding + 2 * kButtonSize, kPadding,
-	                  kButtonSize, kButtonSize, UI::ButtonStyle::kWuiSecondary,
-	                  g_image_cache->get("images/wui/buildings/start_expedition.png"));
+	seafaringbtn_ = new UI::Button(filter_box_, "filter_seafaring_messages", 0, 0, kButtonSize,
+	                               kButtonSize, UI::ButtonStyle::kWuiSecondary,
+	                               g_image_cache->get("images/wui/buildings/start_expedition.png"));
 
-	warfarebtn_ = new UI::Button(this, "filter_warfare_messages", 4 * kPadding + 3 * kButtonSize,
-	                             kPadding, kButtonSize, kButtonSize, UI::ButtonStyle::kWuiSecondary,
+	warfarebtn_ = new UI::Button(filter_box_, "filter_warfare_messages", 0, 0, kButtonSize,
+	                             kButtonSize, UI::ButtonStyle::kWuiSecondary,
 	                             g_image_cache->get("images/wui/messages/messages_warfare.png"));
 
-	scenariobtn_ = new UI::Button(this, "filter_scenario_messages", 5 * kPadding + 4 * kButtonSize,
-	                              kPadding, kButtonSize, kButtonSize, UI::ButtonStyle::kWuiSecondary,
+	scenariobtn_ = new UI::Button(filter_box_, "filter_scenario_messages", 0, 0, kButtonSize,
+	                              kButtonSize, UI::ButtonStyle::kWuiSecondary,
 	                              g_image_cache->get("images/wui/menus/objectives.png"));
 
+	diplomacybtn_ = new UI::Button(filter_box_, "filter_diplomacy_messages", 0, 0, kButtonSize,
+	                               kButtonSize, UI::ButtonStyle::kWuiSecondary,
+	                               g_image_cache->get("images/wui/menus/diplomacy.png"));
+
+	filter_box_->add(geologistsbtn_);
+	filter_box_->add(economybtn_);
+	filter_box_->add(seafaringbtn_);
+	filter_box_->add(warfarebtn_);
+	filter_box_->add(scenariobtn_);
+	filter_box_->add(diplomacybtn_);
+
+	{
+		int w;
+		int h;
+		filter_box_->get_desired_size(&w, &h);
+		filter_box_->set_size(w, h);
+	}
+
 	message_filter_ = Widelands::Message::Type::kAllMessages;
 	set_filter_messages_tooltips();
 	// End: Buttons for message types
@@ -150,6 +171,8 @@ GameMessageMenu::GameMessageMenu(InteractivePlayer& plr, UI::UniqueWindow::Regis
 	   [this]() { filter_messages(Widelands::Message::Type::kWarfare); });
 	scenariobtn_->sigclicked.connect(
 	   [this]() { filter_messages(Widelands::Message::Type::kScenario); });
+	diplomacybtn_->sigclicked.connect(
+	   [this]() { filter_messages(Widelands::Message::Type::kDiplomacy); });
 	list->selected.connect([this](uint32_t a) { selected(a); });
 	list->double_clicked.connect([this](uint32_t a) { double_clicked(a); });
 	archivebtn_->sigclicked.connect([this]() { archive_or_restore(); });
@@ -358,7 +381,7 @@ void GameMessageMenu::selected(uint32_t const t) {
 				game.send_player_command(new Widelands::CmdMessageSetStatusRead(
 				   game.get_gametime(), player.player_number(), id));
 			}
-			centerviewbtn_->set_enabled(bool(message->position()));
+			centerviewbtn_->set_enabled(message->position().valid());
 			message_body.set_text(as_message(message->heading(), message->body()));
 			update_archive_button_tooltip();
 			return;
@@ -422,6 +445,10 @@ bool GameMessageMenu::handle_key(bool down, SDL_Keysym code) {
 			filter_messages(Widelands::Message::Type::kScenario);
 			return true;
 		}
+		if (matches_shortcut(KeyboardShortcut::kInGameMessagesFilterDiplomacy, code)) {
+			filter_messages(Widelands::Message::Type::kDiplomacy);
+			return true;
+		}
 	}
 
 	return UI::UniqueWindow::handle_key(down, code);
@@ -448,6 +475,8 @@ void GameMessageMenu::archive_or_restore() {
 			game.send_player_command(new Widelands::CmdMessageSetStatusRead(
 			   game.get_gametime(), plnum, MessageId(selected_record)));
 			break;
+		default:
+			NEVER_HERE();
 		}
 	}
 }
@@ -457,7 +486,7 @@ void GameMessageMenu::center_view() {
 	assert(selection < list->size());
 	if (Message const* const message =
 	       iplayer().player().messages()[MessageId((*list)[selection])]) {
-		assert(message->position());
+		assert(message->position().valid());
 		iplayer().map_view()->scroll_to_field(message->position(), MapView::Transition::Smooth);
 	}
 }
@@ -484,15 +513,19 @@ void GameMessageMenu::filter_messages(Widelands::Message::Type const msgtype) {
 	case Widelands::Message::Type::kScenario:
 		toggle_filter_messages_button(*scenariobtn_, msgtype);
 		break;
+	case Widelands::Message::Type::kDiplomacy:
+		toggle_filter_messages_button(*diplomacybtn_, msgtype);
+		break;
 
 	case Widelands::Message::Type::kNoMessages:
 	case Widelands::Message::Type::kAllMessages:
 	case Widelands::Message::Type::kGameLogic:
 	case Widelands::Message::Type::kEconomySiteOccupied:
+	case Widelands::Message::Type::kTrading:
 	case Widelands::Message::Type::kWarfareSiteDefeated:
 	case Widelands::Message::Type::kWarfareSiteLost:
 	case Widelands::Message::Type::kWarfareUnderAttack:
-	case Widelands::Message::Type::kTradeOfferReceived:
+	case Widelands::Message::Type::kEconomyLoadGame:
 		set_filter_messages_tooltips();
 		message_filter_ = Widelands::Message::Type::kAllMessages;
 		geologistsbtn_->set_perm_pressed(false);
@@ -500,7 +533,11 @@ void GameMessageMenu::filter_messages(Widelands::Message::Type const msgtype) {
 		seafaringbtn_->set_perm_pressed(false);
 		warfarebtn_->set_perm_pressed(false);
 		scenariobtn_->set_perm_pressed(false);
+		diplomacybtn_->set_perm_pressed(false);
 		break;
+
+	default:
+		NEVER_HERE();
 	}
 	think();
 }
@@ -520,6 +557,7 @@ void GameMessageMenu::toggle_filter_messages_button(UI::Button& button,
 		seafaringbtn_->set_perm_pressed(false);
 		warfarebtn_->set_perm_pressed(false);
 		scenariobtn_->set_perm_pressed(false);
+		diplomacybtn_->set_perm_pressed(false);
 		button.set_perm_pressed(true);
 		message_filter_ = msgtype;
 
@@ -561,6 +599,11 @@ void GameMessageMenu::set_filter_messages_tooltips() {
 	   _("Show scenario messages only"),
 	   shortcut_string_for(KeyboardShortcut::kInGameMessagesFilterScenario, true),
 	   UI::PanelStyle::kWui));
+	diplomacybtn_->set_tooltip(as_tooltip_text_with_hotkey(
+	   /** TRANSLATORS: Tooltip in the messages window */
+	   _("Show diplomacy messages only"),
+	   shortcut_string_for(KeyboardShortcut::kInGameMessagesFilterDiplomacy, true),
+	   UI::PanelStyle::kWui));
 }
 
 /**
@@ -572,6 +615,9 @@ std::string GameMessageMenu::display_message_type_icon(const Widelands::Message&
 		return "images/wui/fieldaction/menu_geologist.png";
 	case Widelands::Message::Type::kEconomy:
 		return "images/wui/stats/genstats_nrwares.png";
+	case Widelands::Message::Type::kDiplomacy:
+	case Widelands::Message::Type::kTrading:
+		return "images/wui/menus/diplomacy.png";
 	case Widelands::Message::Type::kSeafaring:
 		return "images/wui/buildings/start_expedition.png";
 	case Widelands::Message::Type::kWarfare:
@@ -586,10 +632,11 @@ std::string GameMessageMenu::display_message_type_icon(const Widelands::Message&
 	case Widelands::Message::Type::kWarfareSiteDefeated:
 	case Widelands::Message::Type::kWarfareSiteLost:
 	case Widelands::Message::Type::kWarfareUnderAttack:
-	case Widelands::Message::Type::kTradeOfferReceived:
+	case Widelands::Message::Type::kEconomyLoadGame:
 		return "images/wui/messages/message_new.png";
+	default:
+		NEVER_HERE();
 	}
-	NEVER_HERE();
 }
 
 void GameMessageMenu::toggle_mode() {
@@ -609,6 +656,8 @@ void GameMessageMenu::toggle_mode() {
 		togglemodebtn_->set_pic(g_image_cache->get("images/wui/messages/message_archived.png"));
 		togglemodebtn_->set_tooltip(_("Show Archive"));
 		break;
+	default:
+		NEVER_HERE();
 	}
 	update_archive_button_tooltip();
 }
@@ -653,6 +702,8 @@ void GameMessageMenu::update_archive_button_tooltip() {
 			button_tooltip = _("Archive selected message");
 		}
 		break;
+	default:
+		NEVER_HERE();
 	}
 	archivebtn_->set_tooltip(as_tooltip_text_with_hotkey(
 	   button_tooltip, shortcut_string_for(KeyboardShortcut::kCommonDeleteItem, true),
