@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2020-2024 by the Widelands Development Team
+ * Copyright (C) 2020-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,6 +23,7 @@
 #include <SDL_timer.h>
 
 #include "base/time_string.h"
+#include "commands/cmd_message_set_status_archived.h"
 #include "graphic/font_handler.h"
 #include "graphic/text_layout.h"
 #include "logic/message_queue.h"
@@ -61,8 +62,9 @@ MessagePreview::MessagePreview(InfoPanel* i, const Widelands::Message* m, Widela
 }
 
 inline bool MessagePreview::message_still_exists() const {
-	return !(id_.operator bool()) || (owner_.message_queue_ == nullptr) ||
-	       (owner_.message_queue_->count(id_.value()) != 0u);
+	return !id_.valid() || (owner_.message_queue_ == nullptr) ||
+	       (owner_.message_queue_->count(id_.value()) != 0u &&
+	        message_->status() != Widelands::Message::Status::kArchived);
 }
 
 void MessagePreview::think() {
@@ -107,12 +109,21 @@ void MessagePreview::draw(RenderTarget& r) {
 bool MessagePreview::handle_mousepress(const uint8_t button, int32_t /* x */, int32_t /* y */) {
 	switch (button) {
 	case SDL_BUTTON_LEFT:  // center view
-		if ((message_ != nullptr) && (message_->position().operator bool())) {
-			owner_.ibase_.map_view()->scroll_to_field(
-			   message_->position(), MapView::Transition::Smooth);
+		if (message_ != nullptr) {
+			if (message_->position().valid()) {
+				owner_.ibase_.map_view()->scroll_to_field(
+				   message_->position(), MapView::Transition::Smooth);
+			} else if (owner_.iplayer_ != nullptr &&
+			           message_->message_type_category() == Widelands::Message::Type::kDiplomacy) {
+				owner_.iplayer_->diplomacy_.create();
+			}
 		}
 		break;
-	case SDL_BUTTON_MIDDLE:  // hide message
+	case SDL_BUTTON_MIDDLE:  // hide and delete message
+		if (owner_.iplayer_ != nullptr && message_ != nullptr) {
+			owner_.iplayer_->game().send_player_command(new Widelands::CmdMessageSetStatusArchived(
+			   owner_.iplayer_->game().get_gametime(), owner_.iplayer_->player_number(), id_));
+		}
 		owner_.pop_message(this);
 		break;
 	case SDL_BUTTON_RIGHT: {  // open message menu
@@ -477,6 +488,15 @@ void InfoPanel::think() {
 	}
 	if ((iplayer_ != nullptr) && (message_queue_ == nullptr)) {
 		message_queue_ = &iplayer_->player().messages();
+
+		// Check for unexpired messages and messages generated during loading the game
+		const uint32_t gametime = iplayer_->game().get_gametime().get();
+		const Time oldest_to_show(
+		   gametime > kMessagePreviewMaxLifetime ? gametime - kMessagePreviewMaxLifetime : 0u);
+		while (last_message_id_->value() > 0 &&
+		       (*message_queue_)[*last_message_id_]->sent() > oldest_to_show) {
+			*last_message_id_ = Widelands::MessageId(last_message_id_->value() - 1);
+		}
 	}
 
 	while ((message_queue_ != nullptr) &&
@@ -497,11 +517,11 @@ void InfoPanel::think() {
 		if (static_cast<uint8_t>(p->get_z()) < static_cast<uint8_t>(UI::Panel::ZOrder::kInfoPanel) &&
 		    p->get_x() < snap_target_panel_.get_w() &&
 		    (on_top_ ? (p->get_y() < snap_target_panel_.get_y() + snap_target_panel_.get_h()) :
-                     (p->get_y() + p->get_h() > snap_target_panel_.get_y()))) {
+		               (p->get_y() + p->get_h() > snap_target_panel_.get_y()))) {
 			if (UI::Window* w = dynamic_cast<UI::Window*>(p); w != nullptr && !w->moved_by_user()) {
 				w->set_pos(Vector2i(
 				   w->get_x(), on_top_ ? snap_target_panel_.get_y() + snap_target_panel_.get_h() :
-                                     snap_target_panel_.get_y() - w->get_h()));
+				                         snap_target_panel_.get_y() - w->get_h()));
 			}
 		}
 	}
@@ -563,8 +583,8 @@ void InfoPanel::layout() {
 	snap_target_panel_.set_pos(Vector2i(0, toggle_mode_.get_y()));
 	snap_target_panel_.set_size((display_mode_ == UI::ToolbarDisplayMode::kMinimized ||
 	                             display_mode_ == UI::ToolbarDisplayMode::kOnMouse_Hidden) ?
-                                  toggle_mode_.get_w() :
-                                  w,
+	                               toggle_mode_.get_w() :
+	                               w,
 	                            toggle_mode_.get_h());
 }
 
@@ -574,8 +594,8 @@ void InfoPanel::draw(RenderTarget& r) {
 	}
 
 	const int h = display_mode_ == UI::ToolbarDisplayMode::kOnMouse_Hidden ?
-                    kSpacing :
-                    UI::main_toolbar_button_size();
+	                 kSpacing :
+	                 UI::main_toolbar_button_size();
 	r.brighten_rect(Recti(0, on_top_ ? 0 : get_h() - h, get_w(), h), -100);
 
 	r.draw_rect(Recti(0, on_top_ ? h : get_h() - h - 1, get_w(), 1), RGBColor(0, 0, 0));
