@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2024 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -36,7 +36,6 @@
 #include "graphic/graphic.h"
 #include "graphic/render_queue.h"
 #include "graphic/rendertarget.h"
-#include "logic/cmd_queue.h"
 #include "logic/game.h"
 #include "logic/game_controller.h"
 #include "logic/game_data_error.h"
@@ -71,6 +70,7 @@
 #include "wui/game_objectives_menu.h"
 #include "wui/info_panel.h"
 #include "wui/mapviewpixelfunctions.h"
+#include "wui/marketwindow.h"
 #include "wui/militarysitewindow.h"
 #include "wui/minimap.h"
 #include "wui/seafaring_statistics_menu.h"
@@ -159,6 +159,8 @@ InteractiveBase::InteractiveBase(EditorGameBase& the_egbase, Section& global_s,
                        UI::PanelStyle::kWui,
                        UI::ButtonStyle::kWuiPrimary),
      quick_navigation_(&map_view_),
+     plugin_actions_(
+        this, [this](const std::string& cmd) { egbase().lua().interpret_string(cmd); }),
      minimap_registry_(the_egbase.is_game()),
      workareas_cache_(nullptr),
      egbase_(the_egbase),
@@ -284,43 +286,19 @@ UI::Box* InteractiveBase::toolbar() {
 	return &toolbar_.box;
 }
 
-void InteractiveBase::add_plugin_timer(const std::string& action,
-                                       uint32_t interval,
-                                       bool failsafe) {
-	plugin_timers_.emplace_back(action, interval, failsafe);
-}
-
 void InteractiveBase::add_plugin_menu() {
 	plugins_dropdown_.set_image(g_image_cache->get("images/plugin.png"));
 	toolbar()->add(&plugins_dropdown_);
 	plugins_dropdown_.selected.connect(
-	   [this] { plugin_action(plugins_dropdown_.get_selected(), true); });
-}
-
-bool InteractiveBase::plugin_action(const std::string& action, bool failsafe) {
-	try {
-		egbase().lua().interpret_string(action);
-		return true;
-	} catch (const LuaError& e) {
-		if (!failsafe || g_fail_on_lua_error) {
-			log_err("FATAL: Lua error in plugin: %s", e.what());
-			throw;
-		}
-
-		log_err("Lua error in plugin: %s", e.what());
-		UI::WLMessageBox m(this, UI::WindowStyle::kWui, _("Plugin Error"),
-		                   format_l(_("Error when running plugin:\n%s"), e.what()),
-		                   UI::WLMessageBox::MBoxType::kOk);
-		m.run<UI::Panel::Returncodes>();
-		return false;
-	}
+	   [this] { plugin_actions_.plugin_action(plugins_dropdown_.get_selected(), true); });
 }
 
 void InteractiveBase::add_toolbar_plugin(const std::string& action,
                                          const std::string& icon,
                                          const std::string& label,
-                                         const std::string& tt) {
-	plugins_dropdown_.add(label, action, g_image_cache->get(icon), false, tt);
+                                         const std::string& tt,
+                                         const std::string& hotkey) {
+	plugins_dropdown_.add(label, action, g_image_cache->get(icon), false, tt, hotkey);
 	finalize_toolbar();
 }
 
@@ -397,6 +375,8 @@ void InteractiveBase::mapview_menu_selected(MapviewMenuEntry entry) {
 		map_view()->reset_zoom();
 		mapviewmenu_.toggle();
 	} break;
+	default:
+		NEVER_HERE();
 	}
 }
 
@@ -687,6 +667,7 @@ WorkareasEntry InteractiveBase::get_workarea_overlay(const Widelands::Map& map,
 		wa_index = 0;
 		break;
 	default:
+		// Should be impossible, checked by TribeDescr finalizing code
 		throw wexception(
 		   "Encountered unexpected WorkareaInfo size %i", static_cast<int>(workarea_info->size()));
 	}
@@ -979,19 +960,7 @@ void InteractiveBase::think() {
 		it = wanted_building_windows_.erase(it);
 	}
 
-	const uint32_t time = SDL_GetTicks();
-	for (auto plugin = plugin_timers_.begin(); plugin != plugin_timers_.end();) {
-		if (time >= plugin->next_run) {
-			plugin->next_run = time + plugin->interval;
-			if (!plugin_action(plugin->action, plugin->failsafe)) {
-				// In case of an error, remove it from the queue
-				log_err("Unregistering defective plugin timer");
-				plugin = plugin_timers_.erase(plugin);
-				continue;
-			}
-		}
-		++plugin;
-	}
+	plugin_actions_.think();
 }
 
 double InteractiveBase::average_fps() const {
@@ -1039,8 +1008,8 @@ void InteractiveBase::blit_overlay(RenderTarget* dst,
                                    float opacity) {
 	const Recti pixel_perfect_rect =
 	   Recti(position - hotspot * scale, image->width() * scale, image->height() * scale);
-	dst->blitrect_scale(pixel_perfect_rect.cast<float>(), image,
-	                    Recti(0, 0, image->width(), image->height()), opacity, BlendMode::UseAlpha);
+	dst->blitrect_scale(
+	   pixel_perfect_rect.cast<float>(), image, image->rect(), opacity, BlendMode::UseAlpha);
 }
 
 void InteractiveBase::blit_field_overlay(RenderTarget* dst,
@@ -1400,7 +1369,7 @@ void InteractiveBase::finish_build_road() {
 	if (road_building_mode_->type == RoadBuildingType::kWaterway &&
 	    length > egbase().map().get_waterway_max_length()) {
 		log_warn_time(egbase().get_gametime(),
-		              "Refusing to finish waterway building: length is %" PRIuS " but limit is %d\n",
+		              "Refusing to finish waterway building: length is %" PRIuS " but limit is %u\n",
 		              length, egbase().map().get_waterway_max_length());
 	} else if (length != 0u) {
 		upcast(Game, g, &egbase());
@@ -1691,9 +1660,9 @@ void InteractiveBase::road_building_add_overlay(const Widelands::CoordPath& path
 		//  can't build on robusts
 		const Widelands::BaseImmovable* imm = map.get_immovable(neighb);
 		if ((imm != nullptr) && imm->get_size() >= Widelands::BaseImmovable::SMALL &&
-		    (!((dynamic_cast<const Widelands::Flag*>(imm) != nullptr) ||
-		       ((dynamic_cast<const Widelands::RoadBase*>(imm) != nullptr) &&
-		        ((caps & Widelands::BUILDCAPS_FLAG) != 0))))) {
+		    ((dynamic_cast<const Widelands::Flag*>(imm) == nullptr) &&
+		     ((dynamic_cast<const Widelands::RoadBase*>(imm) == nullptr) ||
+		      ((caps & Widelands::BUILDCAPS_FLAG) == 0)))) {
 			continue;
 		}
 		if (path.get_index(neighb) >= 0) {
@@ -1812,7 +1781,12 @@ UI::UniqueWindow* InteractiveBase::show_building_window(const Widelands::Coords&
 			                    avoid_fastclick, workarea_preview_wanted);
 		};
 		break;
-	// TODO(sirver,trading): Add UI for market.
+	case Widelands::MapObjectType::MARKET:
+		registry.open_window = [this, &registry, building, avoid_fastclick, workarea_preview_wanted] {
+			new MarketWindow(*this, registry, *dynamic_cast<Widelands::Market*>(building),
+			                 avoid_fastclick, workarea_preview_wanted);
+		};
+		break;
 	default:
 		log_err_time(egbase().get_gametime(), "Unable to show window for building '%s', type '%s'.\n",
 		             building->descr().name().c_str(), to_string(building->descr().type()).c_str());
@@ -1877,6 +1851,10 @@ void InteractiveBase::broadcast_cheating_message(const std::string& code,
 }
 
 bool InteractiveBase::handle_key(bool const down, SDL_Keysym const code) {
+	if (plugin_actions_.check_keyboard_shortcut_action(code, down)) {
+		return true;
+	}
+
 	if (quick_navigation_.handle_key(down, code)) {
 		return true;
 	}
@@ -1895,6 +1873,12 @@ bool InteractiveBase::handle_key(bool const down, SDL_Keysym const code) {
 		return true;
 	}
 
+	if (matches_shortcut(KeyboardShortcut::kCommonChangeMusic, code)) {
+		// request soundhandler to change music
+		g_sh->change_music();
+		return true;
+	}
+
 	if (down) {
 		if (matches_shortcut(KeyboardShortcut::kCommonBuildhelp, code)) {
 			toggle_buildhelp();
