@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,6 +27,7 @@
 #include "ui_basic/unique_window.h"
 #include "wui/general_statistics_menu.h"
 #include "wui/interactive_base.h"
+#include "wui/stock_menu.h"
 
 class InteractiveGameBase : public InteractiveBase {
 public:
@@ -34,25 +35,26 @@ public:
 	                    Section& global_s,
 	                    bool multiplayer,
 	                    ChatProvider* chat_provider);
-	~InteractiveGameBase() override {
-	}
+	~InteractiveGameBase() override = default;
 	Widelands::Game* get_game() const override;
 	Widelands::Game& game() const override;
 
+	void think() override;
+
 	// Only the 'InteractiveGameBase' has all information of what should be
 	// drawn into a map_view (i.e. which overlays are available). The
 	// 'WatchWindow' does not have this information, but needs to draw
 	// 'map_views', hence this function.
 	virtual void draw_map_view(MapView* given_map_view, RenderTarget* dst) = 0;
 
-	void set_sel_pos(Widelands::NodeAndTriangle<> const center) override;
+	void set_sel_pos(Widelands::NodeAndTriangle<> center) override;
 
 	virtual void node_action(const Widelands::NodeAndTriangle<>& node_and_triangle) = 0;
 
 	void show_watch_window(Widelands::Bob&);
 
-	bool try_show_ship_window();
-	bool is_multiplayer() {
+	bool try_show_ship_windows();
+	bool is_multiplayer() const {
 		return multiplayer_;
 	}
 
@@ -83,6 +85,7 @@ protected:
 	// Adds the gamespeedmenu_ to the toolbar
 	void add_gamespeed_menu();
 
+	void add_diplomacy_menu();
 	// Adds a chat toolbar button and registers the chat console window
 	void add_chat_ui();
 
@@ -100,7 +103,7 @@ public:
 
 		GeneralStatisticsMenu::Registry stats_general;
 		UI::UniqueWindow::Registry stats_wares;
-		UI::UniqueWindow::Registry stats_stock;
+		StockMenu::Registry stats_stock;
 		UI::UniqueWindow::Registry stats_buildings;
 		UI::UniqueWindow::Registry stats_soldiers;
 		UI::UniqueWindow::Registry stats_seafaring;
@@ -108,6 +111,8 @@ public:
 		UI::UniqueWindow::Registry help;
 	} menu_windows_;
 
+	UI::UniqueWindow::Registry diplomacy_;
+
 protected:
 	UI::UniqueWindow::Registry chat_;
 	bool multiplayer_;
@@ -119,12 +124,13 @@ protected:
 	UI::UniqueWindow::Registry game_summary_;
 	UI::UniqueWindow::Registry client_disconnected_;
 
+	const Image* grid_marker_pic_;
+	const Image* special_coords_marker_pic_;
+
 private:
 	// For referencing the items in mainmenu_
 	enum class MainMenuEntry {
-#ifndef NDEBUG  //  only in debug builds
 		kScriptConsole,
-#endif
 		kOptions,
 		kSaveMap,
 		kLoadMap,
@@ -132,6 +138,10 @@ private:
 		kExitGame
 	};
 
+	int32_t pause_on_inactivity_;
+	bool can_restart_;
+	void handle_restart(bool force = false);
+
 	// For referencing the items in gamespeedmenu_
 	enum class GameSpeedEntry { kIncrease, kDecrease, kPause };
 
