@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2022 by the Widelands Development Team
+ * Copyright (C) 2006-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,7 +30,7 @@ MapData::MapData(const std::string& init_filename,
                  const std::string& init_author,
                  const MapData::MapType& init_maptype,
                  const MapData::DisplayType& init_displaytype)
-   : filename(init_filename),
+   : filenames({init_filename}),
      name(init_localized_name),
      localized_name(init_localized_name),
      authors(init_author),
@@ -39,13 +39,14 @@ MapData::MapData(const std::string& init_filename,
      height(0),
      maptype(init_maptype),
      displaytype(init_displaytype) {
+	assert(!filenames.empty());
 }
 
 MapData::MapData(const Widelands::Map& map,
                  const std::string& init_filename,
                  const MapData::MapType& init_maptype,
                  const MapData::DisplayType& init_displaytype)
-   : MapData(init_filename,
+   : MapData({init_filename},
              _("No Name"),
              map.get_author().empty() ? _("No Author") : map.get_author(),
              init_maptype,
@@ -66,6 +67,7 @@ MapData::MapData(const Widelands::Map& map,
 	height = map.get_height();
 	suggested_teams = map.get_suggested_teams();
 	tags = map.get_tags();
+	minimum_required_widelands_version = map.version().minimum_required_widelands_version;
 
 	if (maptype == MapData::MapType::kScenario) {
 		tags.insert("scenario");
@@ -93,8 +95,8 @@ bool MapData::compare_names(const MapData& other) const {
 	std::string other_name;
 	switch (displaytype) {
 	case MapData::DisplayType::kFilenames:
-		this_name = filename;
-		other_name = other.filename;
+		this_name = filenames.at(0);
+		other_name = other.filenames.at(0);
 		break;
 
 	case MapData::DisplayType::kMapnames:
@@ -106,6 +108,9 @@ bool MapData::compare_names(const MapData& other) const {
 		this_name = localized_name;
 		other_name = other.localized_name;
 		break;
+
+	default:
+		NEVER_HERE();
 	}
 
 	// If there is no width, we have a directory - we want them first.
@@ -133,10 +138,18 @@ bool MapData::compare_size(const MapData& other) const {
 	if (width == other.width && height == other.height) {
 		return compare_names(other);
 	}
-	if (width != other.width) {
-		return width < other.width;
+
+	if ((width * height) != (other.width * other.height)) {
+		return (width * height) < (other.width * other.height);
 	}
-	return height < other.height;
+
+	return width < other.width;
+}
+
+void MapData::add(const MapData& md) {
+	assert(maptype == MapType::kDirectory);
+	assert(md.maptype == MapType::kDirectory);
+	filenames.insert(filenames.end(), md.filenames.begin(), md.filenames.end());
 }
 
 // static
@@ -157,7 +170,7 @@ MapData MapData::create_parent_dir(const std::string& current_dir) {
 			}
 		}
 	}
-	return MapData(filename, parent_name());
+	return MapData({filename}, parent_name());
 }
 
 // static
@@ -175,19 +188,19 @@ MapData MapData::create_empty_dir(const std::string& current_dir) {
 // static
 MapData MapData::create_directory(const std::string& directory) {
 	std::string localized_name = FileSystem::fs_filename(directory.c_str());
-	if (directory == "maps/MP_Scenarios") {
+	if (directory == kMultiPlayerScenarioDirFull) {
 		/** TRANSLATORS: Directory name for MP Scenarios in map selection */
 		localized_name = _("Multiplayer Scenarios");
-	} else if (directory == "maps/SP_Scenarios") {
+	} else if (directory == kSinglePlayerScenarioDirFull) {
 		/** TRANSLATORS: Directory name for SP Scenarios in map selection */
 		localized_name = _("Singleplayer Scenarios");
-	} else if (directory == "maps/My_Maps") {
+	} else if (directory == kMyMapsDirFull) {
 		/** TRANSLATORS: Directory name for user maps in map selection */
 		localized_name = _("My Maps");
-	} else if (directory == "maps/Downloaded") {
+	} else if (directory == kDownloadedMapsDirFull) {
 		/** TRANSLATORS: Directory name for downloaded maps in map selection */
 		localized_name = _("Downloaded Maps");
-	} else if (directory.compare(0, kAddOnDir.size(), kAddOnDir) == 0) {
+	} else if (starts_with(directory, kAddOnDir)) {
 		std::string addon = directory.substr(kAddOnDir.size() + 1);
 		addon = addon.substr(0, addon.find('/'));
 		std::unique_ptr<i18n::GenericTextdomain> td(AddOns::create_textdomain_for_addon(addon));
@@ -204,5 +217,5 @@ MapData MapData::create_directory(const std::string& directory) {
 			}
 		}
 	}
-	return MapData(directory, localized_name);
+	return MapData({directory}, localized_name);
 }
