@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2024 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -168,9 +168,12 @@ bool MapDetails::update(const MapData& mapdata, bool localize_mapname, bool rend
 		std::string description = as_heading(authors_heading, style_);
 		description += as_content(mapdata.authors.get_names(), style_);
 
-		std::vector<std::string> tags;
-		for (const auto& tag : mapdata.tags) {
-			tags.push_back(localize_tag(tag).displayname);
+		std::vector<std::string> tags(mapdata.tags.size());
+		{
+			size_t i = 0;
+			for (const auto& tag : mapdata.tags) {
+				tags.at(i++) = localize_tag(tag).displayname;
+			}
 		}
 		std::sort(tags.begin(), tags.end());
 		description += as_heading(_("Tags"), style_);
