@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,7 +24,7 @@
 static char const* pic_tab_military = "images/wui/buildings/menu_tab_military.png";
 
 MilitarySiteWindow::MilitarySiteWindow(InteractiveBase& parent,
-                                       UI::UniqueWindow::Registry& reg,
+                                       BuildingWindow::Registry& reg,
                                        Widelands::MilitarySite& ms,
                                        bool avoid_fastclick,
                                        bool workarea_preview_wanted)
