@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2022 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,12 +18,14 @@
 
 #include "wui/savegameloader.h"
 
+#include <memory>
+
 #include "base/i18n.h"
-#include "base/string.h"
 #include "base/time_string.h"
 #include "game_io/game_loader.h"
 #include "io/filesystem/layered_filesystem.h"
-#include "logic/filesystem_constants.h"
+#include "logic/game.h"
+#include "logic/replay.h"
 
 SavegameLoader::SavegameLoader(Widelands::Game& game) : game_(game) {
 }
@@ -37,33 +39,40 @@ std::vector<SavegameData> SavegameLoader::load_files(const std::string& director
 	return loaded_games;
 }
 
-std::string SavegameLoader::get_savename(const std::string& gamefilename) const {
-	return gamefilename;
-}
-
 void SavegameLoader::load(const std::string& to_be_loaded,
                           std::vector<SavegameData>& loaded_games) const {
 	if (g_fs->is_directory(to_be_loaded)) {
-		try {
-			load_savegame_from_directory(to_be_loaded, loaded_games);
-		} catch (const std::exception&) {
+		bool success = false;
+		if (is_valid_savegame(to_be_loaded)) {
+			try {
+				load_savegame_from_directory(to_be_loaded, loaded_games);
+				success = true;
+			} catch (...) {
+				// We'll try it as a subdirectory then
+			}
+		}
+		if (!success) {
 			// loading failed, so this is actually a normal directory
 			add_sub_dir(to_be_loaded, loaded_games);
 		}
-	} else {
+	} else if (is_valid_savegame(to_be_loaded)) {
 		load_savegame_from_file(to_be_loaded, loaded_games);
 	}
 }
 
 void SavegameLoader::load_savegame_from_directory(const std::string& gamefilename,
-                                                  std::vector<SavegameData>& loaded_games) const {
-
+                                                  std::vector<SavegameData>& loaded_games,
+                                                  const bool load_for_replay) const {
+	Widelands::ReplayfileSavegameExtractor converter(gamefilename);
 	Widelands::GamePreloadPacket gpdp;
 	SavegameData gamedata(gamefilename);
 
-	Widelands::GameLoader gl(gamefilename, game_);
+	Widelands::GameLoader gl(converter.file(), game_);
 	gl.preload_game(gpdp);
 	gamedata.gametype = gpdp.get_gametype();
+	if (load_for_replay && converter.is_replay()) {
+		gamedata.gametype = GameController::GameType::kReplay;
+	}
 	if (!is_valid_gametype(gamedata)) {
 		return;
 	}
@@ -74,19 +83,18 @@ void SavegameLoader::load_savegame_from_directory(const std::string& gamefilenam
 }
 
 void SavegameLoader::load_savegame_from_file(const std::string& gamefilename,
-                                             std::vector<SavegameData>& loaded_games) const {
-	std::string savename = get_savename(gamefilename);
-
-	if (!g_fs->file_exists(savename) || !ends_with(savename, kSavegameExtension)) {
-		return;
-	}
-
+                                             std::vector<SavegameData>& loaded_games,
+                                             const bool load_for_replay) const {
 	Widelands::GamePreloadPacket gpdp;
 	SavegameData gamedata(gamefilename);
 	try {
-		Widelands::GameLoader gl(savename, game_);
+		Widelands::ReplayfileSavegameExtractor converter(gamefilename);
+		Widelands::GameLoader gl(converter.file(), game_);
 		gl.preload_game(gpdp);
 		gamedata.gametype = gpdp.get_gametype();
+		if (load_for_replay && converter.is_replay()) {
+			gamedata.gametype = GameController::GameType::kReplay;
+		}
 		if (!is_valid_gametype(gamedata)) {
 			return;
 		}
@@ -202,16 +210,20 @@ void SavegameLoader::add_sub_dir(const std::string& gamefilename,
 ReplayLoader::ReplayLoader(Widelands::Game& game) : SavegameLoader(game) {
 }
 
-bool ReplayLoader::is_valid_gametype(const SavegameData& /*gamedata*/) const {
-	return true;  // TODO(jmoerschbach): why?? what is the purpose of
-	              // GameController::GameType::kReplay? return gamedata.is_replay(); <-- should be
-	              // this, right?!
+bool ReplayLoader::is_valid_gametype(const SavegameData& gamedata) const {
+	return gamedata.is_replay();
 }
 
-std::string ReplayLoader::get_savename(const std::string& gamefilename) const {
-	std::string savename = gamefilename;
-	savename += kSavegameExtension;
-	return savename;
+void ReplayLoader::load_savegame_from_directory(const std::string& gamefilename,
+                                                std::vector<SavegameData>& loaded_games,
+                                                const bool /* load_for_replay */) const {
+	SavegameLoader::load_savegame_from_directory(gamefilename, loaded_games, true);
+}
+
+void ReplayLoader::load_savegame_from_file(const std::string& gamefilename,
+                                           std::vector<SavegameData>& loaded_games,
+                                           const bool /* load_for_replay */) const {
+	SavegameLoader::load_savegame_from_file(gamefilename, loaded_games, true);
 }
 
 MultiPlayerLoader::MultiPlayerLoader(Widelands::Game& game) : SavegameLoader(game) {
@@ -235,3 +247,37 @@ EverythingLoader::EverythingLoader(Widelands::Game& game) : SavegameLoader(game)
 bool EverythingLoader::is_valid_gametype(const SavegameData& /*gamedata*/) const {
 	return true;
 }
+
+std::optional<SavegameData> newest_saved_game_or_replay(bool find_replay) {
+	std::unique_ptr<Widelands::Game> game = nullptr;
+	try {
+		game.reset(new Widelands::Game());
+	} catch (...) {
+		// We'll just return nothing then
+	}
+	if (game == nullptr) {
+		return std::nullopt;
+	}
+
+	std::vector<SavegameData> games;
+	if (find_replay) {
+		ReplayLoader loader(*game);
+		games = loader.load_files(kReplayDir);
+	} else {
+		SinglePlayerLoader loader(*game);
+		games = loader.load_files(kSaveDir);
+	}
+
+	SavegameData* newest = nullptr;
+	for (SavegameData& data : games) {
+		if (!data.is_directory() &&  // shouldn't these be filtered out already by loader?
+		    (find_replay ? data.is_replay() : data.is_singleplayer()) &&
+		    (newest == nullptr || newest->compare_save_time(data))) {
+			newest = &data;
+		}
+	}
+	if (newest == nullptr) {
+		return std::nullopt;
+	}
+	return *newest;
+}
