@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2024 by the Widelands Development Team
+ * Copyright (C) 2017-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,7 +25,6 @@
 #include "logic/game.h"
 #include "logic/game_data_error.h"
 #include "logic/player.h"
-#include "logic/playercommand.h"
 #include "ui_basic/box.h"
 #include "wlapplication_options.h"
 #include "wui/interactive_player.h"
@@ -245,8 +244,9 @@ SeafaringStatisticsMenu::SeafaringStatisticsMenu(InteractivePlayer& plr,
 			   case Widelands::NoteShip::Action::kLost:
 				   remove_ship(note.ship->serial());
 				   return;
+			   default:
+				   NEVER_HERE();
 			   }
-			   NEVER_HERE();
 		   }
 	   });
 
@@ -272,8 +272,9 @@ SeafaringStatisticsMenu::status_to_string(SeafaringStatisticsMenu::ShipFilterSta
 		return pgettext("ship_state", "Being Refitted");
 	case SeafaringStatisticsMenu::ShipFilterStatus::kAll:
 		return "All";  // The user shouldn't see this, so we don't localize
+	default:
+		NEVER_HERE();
 	}
-	NEVER_HERE();
 }
 
 const Image*
@@ -304,6 +305,8 @@ SeafaringStatisticsMenu::status_to_image(SeafaringStatisticsMenu::ShipFilterStat
 	case SeafaringStatisticsMenu::ShipFilterStatus::kAll:
 		filename = "images/wui/ship/ship_scout_ne.png";
 		break;
+	default:
+		NEVER_HERE();
 	}
 	return g_image_cache->get(filename);
 }
@@ -343,8 +346,12 @@ SeafaringStatisticsMenu::create_shipinfo(const Widelands::Ship& ship) const {
 			case Widelands::ShipStates::kSinkAnimation:
 				status = ShipFilterStatus::kAll;
 				break;
+			default:
+				NEVER_HERE();
 			}
 			break;
+		default:
+			NEVER_HERE();
 		}
 	}
 	return std::unique_ptr<const ShipInfo>(new ShipInfo(ship.get_shipname(), status, ship.serial()));
@@ -542,6 +549,8 @@ void SeafaringStatisticsMenu::filter_ships(ShipFilterStatus status) {
 		warship_btn_.set_perm_pressed(false);
 		refitting_btn_.set_perm_pressed(false);
 		break;
+	default:
+		NEVER_HERE();
 	}
 	fill_table();
 }
