@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2024 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -538,6 +538,8 @@ void SoldierList::think() {
 	case Widelands::MapObjectType::SHIP: {
 		upcast(const Widelands::Ship, ship, object);
 		soldier_preference_.set_state(static_cast<uint8_t>(ship->get_soldier_preference()), false);
+		soldierpanel_.set_visible((ship->get_ship_type() == Widelands::ShipType::kWarship) &&
+		                          !ship->is_refitting());
 		break;
 	}
 	case Widelands::MapObjectType::TRAININGSITE:
