@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2024 by the Widelands Development Team
+ * Copyright (C) 2002-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,10 +19,12 @@
 #include "wui/story_message_box.h"
 
 #include <SDL_mouse.h>
+#include <SDL_timer.h>
 
 #include "logic/game_controller.h"
 #include "logic/save_handler.h"
 #include "ui_basic/button.h"
+#include "ui_basic/messagebox.h"
 #include "ui_basic/multilinetextarea.h"
 #include "wui/interactive_player.h"
 
@@ -90,7 +92,8 @@ StoryMessageBox::StoryMessageBox(Widelands::Game* game,
                 _("Return to the main menu")),
      desired_speed_(game->game_controller()->desired_speed()),
      modal_(modal),
-     game_(game) {
+     game_(game),
+     start_time_(SDL_GetTicks()) {
 
 	// Pause the game
 	game_->game_controller()->set_desired_speed(0);
@@ -203,3 +206,12 @@ bool StoryMessageBox::handle_key(bool down, SDL_Keysym code) {
 	}
 	return UI::Window::handle_key(down, code);
 }
+
+void StoryMessageBox::think() {
+	Window::think();
+
+	if (is_modal() && g_message_box_timeout > 0 &&
+	    SDL_GetTicks() > start_time_ + g_message_box_timeout) {
+		clicked_ok();
+	}
+}
