@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2020-2022 by the Widelands Development Team
+ * Copyright (C) 2020-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,9 +27,7 @@ class InfoPanel;
 /// A horizontal menu bar embellished with background graphics
 class MainToolbar : public UI::Panel {
 public:
-	MainToolbar(InfoPanel&);
-
-	static constexpr uint8_t kButtonSize = 33;
+	explicit MainToolbar(InfoPanel&);
 
 	/// Sets the actual size and position of the toolbar
 	void finalize();
@@ -39,13 +37,14 @@ public:
 	/// A row of buttons and dropdown menus
 	UI::Box box;
 
-	bool on_top, draw_background;
+	bool on_top{false};
+	bool draw_background{false};
 
 private:
 	/// The set of background images
 	ToolbarImageset imageset_;
 	/// How often the left and right images get repeated, calculated from the width of the box
-	int repeat_;
+	int repeat_{0};
 };
 
 #endif  // end of include guard: WL_WUI_TOOLBAR_H
