@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2024 by the Widelands Development Team
+ * Copyright (C) 2003-2025 by the Widelands Development Team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -35,6 +35,7 @@
 #include "logic/map_objects/tribes/ware_descr.h"
 #include "logic/map_objects/tribes/worker.h"
 #include "logic/player.h"
+#include "ui_basic/button.h"
 #include "ui_basic/window.h"
 
 constexpr int kWareMenuInfoSize = 12;
@@ -81,8 +82,6 @@ AbstractWaresDisplay::AbstractWaresDisplay(
 		   std::make_pair(index, std::make_pair(RGBAColor(0, 0, 0, 0), nullptr)));
 	}
 
-	curware_.set_text(_("Stock"));
-
 	graphic_resolution_changed_subscriber_ = Notifications::subscribe<GraphicResolutionChanged>(
 	   [this](const GraphicResolutionChanged& /* note */) { recalc_desired_size(true); });
 
@@ -122,6 +121,11 @@ void AbstractWaresDisplay::set_vgap(int32_t gap, bool relayout) {
 	recalc_desired_size(relayout);
 }
 
+void AbstractWaresDisplay::set_min_free_vertical_space(int32_t s, bool relayout) {
+	min_free_vertical_space_ = s;
+	recalc_desired_size(relayout);
+}
+
 void AbstractWaresDisplay::recalc_desired_size(bool relayout) {
 	relayout_icons_order_coords();
 
@@ -370,8 +374,9 @@ const Widelands::TribeDescr::WaresOrder& AbstractWaresDisplay::icons_order() con
 		return tribe_.wares_order();
 	case Widelands::wwWORKER:
 		return tribe_.workers_order();
+	default:
+		NEVER_HERE();
 	}
-	NEVER_HERE();
 }
 
 const WaresOrderCoords& AbstractWaresDisplay::icons_order_coords() const {
@@ -566,12 +571,16 @@ bool AbstractWaresDisplay::ware_selected(Widelands::DescriptionIndex ware) const
 }
 
 // Wares hiding
-void AbstractWaresDisplay::hide_ware(Widelands::DescriptionIndex ware) {
-	if (hidden_[ware]) {
-		return;
+void AbstractWaresDisplay::set_hidden(Widelands::DescriptionIndex ware, bool hide) {
+	if (hidden_[ware] != hide) {
+		hidden_[ware] = hide;
+		if (hide) {
+			selected_[ware] = false;
+		} else {
+			last_ware_details_cache_update_ = 0;
+		}
+		need_texture_update_ = true;
 	}
-	hidden_[ware] = true;
-	need_texture_update_ = true;
 }
 
 bool AbstractWaresDisplay::is_ware_hidden(Widelands::DescriptionIndex ware) const {
@@ -669,20 +678,20 @@ static const char* unit_suffixes[] = {
    "%1%",
    /** TRANSLATORS: This is a large number with a suffix (e.g. 50k = 50,000). */
    /** TRANSLATORS: Space is limited, use only 1 letter for the suffix and no whitespace. */
-   _("%1%k"),
+   gettext_noop("%1%k"),
    /** TRANSLATORS: This is a large number with a suffix (e.g. 5M = 5,000,000). */
    /** TRANSLATORS: Space is limited, use only 1 letter for the suffix and no whitespace. */
-   _("%1%M"),
+   gettext_noop("%1%M"),
    /** TRANSLATORS: This is a large number with a suffix (e.g. 5G = 5,000,000,000). */
    /** TRANSLATORS: Space is limited, use only 1 letter for the suffix and no whitespace. */
-   _("%1%G")};
+   gettext_noop("%1%G")};
 std::string get_amount_string(uint32_t amount, bool cutoff1k) {
 	uint8_t size = 0;
 	while (amount >= ((size != 0u) || cutoff1k ? 1000 : 10000)) {
 		amount /= 1000;
 		size++;
 	}
-	return format(unit_suffixes[size], amount);
+	return format(_(unit_suffixes[size]), amount);
 }
 
 uint32_t StockMenuWaresDisplay::amount_of(const Widelands::DescriptionIndex ware) {
@@ -710,28 +719,113 @@ std::string WaresDisplay::info_for_ware(Widelands::DescriptionIndex ware) {
 	return get_amount_string(amount_of(ware));
 }
 
+UI::Box& TradeProposalWaresDisplay::create(UI::Panel* parent,
+                                           const Widelands::TribeDescr& tribe,
+                                           const std::string& heading,
+                                           int spacing,
+                                           TradeProposalWaresDisplay** result_pointer) {
+	UI::Box& box = *new UI::Box(parent, UI::PanelStyle::kWui, "vbox", 0, 0, UI::Box::Vertical);
+	TradeProposalWaresDisplay* impl = new TradeProposalWaresDisplay(&box, tribe);
+
+	box.add(new UI::Textarea(&box, UI::PanelStyle::kWui, "label",
+	                         UI::FontStyle::kWuiInfoPanelHeading, heading, UI::Align::kCenter),
+	        UI::Box::Resizing::kFullSize);
+	box.add_space(spacing);
+	box.add(impl, UI::Box::Resizing::kExpandBoth);
+
+	*result_pointer = impl;
+	return box;
+}
+
+TradeProposalWaresDisplay::TradeProposalWaresDisplay(UI::Panel* parent,
+                                                     const Widelands::TribeDescr& tribe)
+   : WaresDisplay(parent, 0, 0, tribe, Widelands::wwWARE, true) {
+	set_other(nullptr);
+	set_min_free_vertical_space(430, true);
+}
+
+uint32_t TradeProposalWaresDisplay::amount_of(Widelands::DescriptionIndex di) {
+	const auto it = config_.find(di);
+	return it != config_.end() ? it->second : 0;
+}
+
+void TradeProposalWaresDisplay::set_zero() {
+	config_.clear();
+	for (Widelands::DescriptionIndex di : tribe().wares()) {
+		unselect_ware(di);
+	}
+}
+
+void TradeProposalWaresDisplay::set_other(const Widelands::TribeDescr* other) {
+	for (auto it = config_.begin(); it != config_.end();) {
+		if (other == nullptr || !other->has_ware(it->first)) {
+			it = config_.erase(it);
+		} else {
+			++it;
+		}
+	}
+
+	for (Widelands::DescriptionIndex di : tribe().wares()) {
+		set_hidden(di, other == nullptr || !other->has_ware(di));
+	}
+}
+
+void TradeProposalWaresDisplay::change(const int delta) {
+	for (Widelands::DescriptionIndex di : tribe().wares()) {
+		if (ware_selected(di)) {
+			auto it = config_.find(di);
+			if (it == config_.end()) {
+				if (delta > 0) {
+					config_.emplace(di, delta);
+				}
+			} else {
+				if (it->second + delta > 0) {
+					it->second += delta;
+				} else {
+					config_.erase(it);
+				}
+			}
+		}
+	}
+}
+
+bool TradeProposalWaresDisplay::anything_selected() const {
+	return std::any_of(
+	   config_.begin(), config_.end(),
+	   [](const std::pair<Widelands::DescriptionIndex, int>& pair) { return pair.second > 0; });
+}
+
+Widelands::BillOfMaterials TradeProposalWaresDisplay::get_selection() const {
+	Widelands::BillOfMaterials bill;
+	for (const auto& pair : config_) {
+		if (pair.second > 0) {
+			bill.emplace_back(pair.first, pair.second);
+		}
+	}
+	return bill;
+}
+
+void TradeProposalWaresDisplay::draw(RenderTarget& dst) {
+	dst.brighten_rect(Recti(0, 0, get_inner_w(), get_inner_h()), -32);
+	WaresDisplay::draw(dst);
+}
+
 std::string waremap_to_richtext(const Widelands::TribeDescr& tribe,
                                 const std::map<Widelands::DescriptionIndex, uint8_t>& map) {
 	std::string ret;
 
-	std::map<Widelands::DescriptionIndex, uint8_t>::const_iterator c;
-
-	Widelands::TribeDescr::WaresOrder::iterator i;
-	std::vector<Widelands::DescriptionIndex>::iterator j;
-	Widelands::TribeDescr::WaresOrder order = tribe.wares_order();
-
 	const UI::WareInfoStyleInfo& style =
 	   g_style_manager->ware_info_style(UI::WareInfoStyle::kNormal);
 
-	for (i = order.begin(); i != order.end(); ++i) {
-		for (j = i->begin(); j != i->end(); ++j) {
-			if ((c = map.find(*j)) != map.end()) {
+	for (const auto& column : tribe.wares_order()) {
+		for (Widelands::DescriptionIndex di : column) {
+			if (const auto ware_amount = map.find(di); ware_amount != map.end()) {
 				ret += "<div width=30 padding=2><p align=center>"
 				       "<div width=26 background=" +
 				       style.icon_background().hex_value() + "><p align=center><img src=\"" +
-				       tribe.get_ware_descr(c->first)->icon_filename() +
+				       tribe.get_ware_descr(ware_amount->first)->icon_filename() +
 				       "\"></p></div><div width=26 background=" + style.info_background().hex_value() +
-				       "><p>" + style.info_font().as_font_tag(get_amount_string(c->second)) +
+				       "><p>" + style.info_font().as_font_tag(get_amount_string(ware_amount->second)) +
 				       "</p></div></p></div>";
 			}
 		}
