@@ -41,7 +41,9 @@ wl.Descriptions():new_productionsite_type {
          descname = _("felling trees"),
          actions = {
             "callworker=harvest",
-            "sleep=duration:20s"
+            "sleep=duration:20s",
+            "script=barbarians_custom_building_program_counter_increment",
+            "callworker=remove_pinned_note",
          }
       },
    },
@@ -53,3 +55,8 @@ wl.Descriptions():new_productionsite_type {
       productivity_threshold = 60
    },
 }
+
+function barbarians_custom_building_program_counter_increment(site)
+   assert_equal("barbarians_custom_building", site.descr.name)
+   barbarians_custom_building_program_counter = (barbarians_custom_building_program_counter or 0) + 1
+end
