@@ -40,7 +40,21 @@ wl.Descriptions():new_worker_type {
          "callobject=fall",
          "animate=idle duration:2s",
          "createware=custom_ware",
+         "script=barbarians_custom_worker_program_counter_increment",
+         "return"
+      },
+      remove_pinned_note = {
+         "findobject=type:special name:pinned_note radius:5",
+         "walk=object",
+         "animate=hacking duration:15s",
+         "removeobject",
+         "animate=idle duration:2s",
          "return"
       }
    },
 }
+
+function barbarians_custom_worker_program_counter_increment(worker)
+   assert_equal("barbarians_custom_worker", worker.descr.name)
+   barbarians_custom_worker_program_counter = (barbarians_custom_worker_program_counter or 0) + 1
+end
