@@ -18,7 +18,7 @@ wl.Descriptions():new_worker_type {
          -- These won't be played in the test suite, just testing the table structure.
          sound_effect = {
             path = "sound/hammering/hammering",
-            priority = 64
+            priority = "25%"
          },
       },
       walk = {
