@@ -1,6 +1,6 @@
-include "scripting/lunit.lua"
 include "scripting/coroutine.lua"
 include "scripting/infrastructure.lua"
+include "test/scripting/lunit.lua"
 include "test/scripting/stable_save.lua"
 
 game = wl.Game()
