@@ -497,10 +497,13 @@ def apply_fixes(args, tmpdir):
 
 # List of files with a non-zero return code.
 failed_files = []
+progress_counter = 0
+total_files = 0
 
 def run_tidy(tmpdir, build_path, quiet, queue, lock):
     """Takes filenames out of queue and runs clang-tidy on them."""
     global failed_files
+    global progress_counter
     while True:
         name, command, dir = queue.get()
         hash = None
@@ -541,7 +544,8 @@ def run_tidy(tmpdir, build_path, quiet, queue, lock):
         with lock:
             if tidy_error:
                 failed_files.append(name)
-            sys.stderr.write('\n' + ' '.join(invocation) + '\n')
+            progress_counter += 1
+            sys.stderr.write('\n' + f'{progress_counter}/{total_files}: ' + ' '.join(invocation) + '\n')
             sys.stderr.flush()
             if len(output) > 0:
                 sys.stdout.write(output + '\n')
@@ -630,7 +634,7 @@ def main():
                             'which can be applied with clang-apply-replacements.')
     parser.add_argument('-j', type=int, default=0,
                         help='number of tidy instances to be run in parallel.')
-    parser.add_argument('files', nargs='*', default=['.*'],
+    parser.add_argument('-files', nargs='*', default=['.*'],
                         help='files to be processed (regex on path)')
     parser.add_argument('-fix', action='store_true', help='apply fix-its')
     parser.add_argument('-format', action='store_true', help='Reformat code '
@@ -741,6 +745,9 @@ def main():
         if file_name_re.search(name) and not 'src/third_party' in name:
             files.append((name, entry['command'], entry['directory']))
 
+    global total_files
+    total_files = len(files)
+
     return_code = 0
     lock = threading.Lock()
     task_queue = queue.Queue(max_task)
